package ch.ige.adr.client;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.TitledBorder;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdresseDelegate;
import ch.ige.adr.vo.PostOrtAVO;
import ch.ige.adrhelp.HelpUtil;
import ch.ige.main.client.XxMessage;
import ch.ige.util.client.IgeTable;

/**
 * Dialog für die Auwahl einer PLZ bzw. eines Ortes
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: PlzOrtDialog.java,v 1.21 2008/04/28 15:32:34 galli Exp $
 */
public class PlzOrtDialog extends JDialog
{
  // Logging
  private static final Logger LOG = Logger.getLogger( PlzOrtDialog.class );

  // Applikationsmeldungen
  protected XxMessage xxMessage = null;

  protected JPanel pnlPlzOrt = new JPanel();

  protected BorderLayout borderLayout1 = new BorderLayout();

  protected JPanel pnlButtons = new JPanel();

  protected JPanel pnlCenter = new JPanel();

  protected JScrollPane scpPlzOrt = new JScrollPane();

  protected PlzOrtTableModel tableModel = new PlzOrtTableModel();

  protected JTable tblPlzOrt = new IgeTable();

  protected Box box2;

  protected JButton btnOk = new JButton();

  protected JButton btnCancel = new JButton();

  protected Component component1;

  protected BorderLayout borderLayout2 = new BorderLayout();

  protected String[] plzOrt = null;

  /**
   * Dialog erstellen
   * @param owner Aufrufender Dialog
   */
  public PlzOrtDialog( Dialog owner )
  {
    super( owner );
    try
    {
      xxMessage = XxMessage.getInstance();
      jbInit();
      pack();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( this, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE );
    }
  }

  /**
   * Dialog erstellen
   */
  public PlzOrtDialog()
  {
    this( (Dialog) null );
  }

  /**
   * Komponente initialisieren
   * @throws Exception
   */
  void jbInit() throws Exception
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.setRows( 1 );
    box2 = Box.createHorizontalBox();
    component1 = Box.createGlue();
    this.setTitle( "Auswahl PLZ und Ort" );
    this.setModal( true );
    pnlPlzOrt.setLayout( borderLayout1 );
    pnlButtons.setLayout( gridLayout );
    tblPlzOrt.setPreferredScrollableViewportSize( new Dimension( 300, 150 ) );
    tblPlzOrt.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyPressed( KeyEvent e )
      {
        tblPlzOrt_keyPressed( e );
      }
    } );
    pnlPlzOrt.setMinimumSize( new Dimension( 400, 250 ) );
    pnlPlzOrt.setPreferredSize( new Dimension( 400, 250 ) );
    pnlCenter.setMinimumSize( new Dimension( 30, 36 ) );
    pnlCenter.setPreferredSize( new Dimension( 30, 36 ) );
    pnlCenter.setLayout( borderLayout2 );
    pnlButtons.setMinimumSize( new Dimension( 300, 29 ) );
    pnlButtons.setPreferredSize( new Dimension( 300, 29 ) );
    pnlButtons.add( box2, null );
    btnOk.setAlignmentX( (float) 1.0 );
    btnOk.setMaximumSize( new Dimension( 100, 27 ) );
    btnOk.setMinimumSize( new Dimension( 100, 27 ) );
    btnOk.setPreferredSize( new Dimension( 100, 27 ) );
    btnOk.setMnemonic( 'O' );
    btnOk.setText( "OK" );
    btnOk.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnOk_actionPerformed( e );
      }
    } );
    btnCancel.setAlignmentX( (float) 1.0 );
    btnCancel.setMaximumSize( new Dimension( 100, 27 ) );
    btnCancel.setMinimumSize( new Dimension( 100, 27 ) );
    btnCancel.setPreferredSize( new Dimension( 100, 27 ) );
    btnCancel.setMnemonic( 'A' );
    btnCancel.setText( "Abbrechen" );
    btnCancel.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnCancel_actionPerformed( e );
      }
    } );
    scpPlzOrt.setMinimumSize( new Dimension( 30, 26 ) );
    scpPlzOrt.setPreferredSize( new Dimension( 30, 26 ) );
    pnlPlzOrt.add( pnlCenter, BorderLayout.CENTER );
    pnlCenter.add( scpPlzOrt, BorderLayout.CENTER );
    scpPlzOrt.getViewport().add( tblPlzOrt );
    pnlPlzOrt.setBorder( new TitledBorder( BorderFactory.createLineBorder( new Color( 153, 153, 153 ), 1 ), "PLZ und Ort" ) );
    getContentPane().add( pnlPlzOrt );
    pnlPlzOrt.add( pnlButtons, BorderLayout.SOUTH );
    box2.add( component1, null );
    box2.add( btnOk, null );
    box2.add( btnCancel, null );
    tblPlzOrt.setModel( tableModel );
    // Spaltenbreite der Tabelle
    tblPlzOrt.getColumnModel().getColumn( 0 ).setPreferredWidth( 80 );
    tblPlzOrt.getColumnModel().getColumn( 1 ).setPreferredWidth( 300 );

    tblPlzOrt.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblPlzOrt_MouseListener( e );
      }
    } );
    getRootPane().setDefaultButton( btnOk );

  }

  /**
   * PLZ und Ortschaften lesen
   * @param plz Postleitzahl
   * @param ort Ortschaft
   */
  void read( String plz, String ort )
  {
    if ( plz == null )
    {
      plz = "";
    }
    if ( ort == null )
    {
      ort = "";
    }
    try
    {
      // Suchen
      ArrayList al = AdresseDelegate.getInstance().findPostOrt( plz, ort );
      PostOrtAVO[] postOrtAVO = new PostOrtAVO[al.size()];
      postOrtAVO = (PostOrtAVO[]) al.toArray( postOrtAVO );
      tableModel.setData( postOrtAVO );
      if ( postOrtAVO.length > 0 )
      {
        tblPlzOrt.setRowSelectionInterval( 0, 0 );
      }
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( this, ex, "Fehler", JOptionPane.ERROR_MESSAGE );
      return;
    }

  }

  /**
   * Plz-Dialog anzeigen.
   * @param owner Aufrufender Dialog
   * @param plz Postleitzahl
   * @param ort Ortschaft
   * @param eineAnzeigen </p>
   * Falls nur ein Eintrag vorhanden: true - anzeigen, false - nicht anzeigen
   * @return String[0] - Postleitzahl; String[1] - Ortschaft
   * <p>
   * String.length == 0 - Postleitzahl bzw. Ortschaft exisiert nicht
   */
  public static String[] showPlzOrt( Dialog owner, String plz, String ort, boolean eineAnzeigen )
  {
    PlzOrtDialog frame = new PlzOrtDialog( owner );
    frame.read( plz, ort );
    if ( frame.tableModel.getRowCount() == 0 )
    {
      // Plz bzw. Ort existiert nicht
      return new String[0];
    }
    else if ( frame.tableModel.getRowCount() == 1 && !eineAnzeigen )
    {
      // Eine Plz bzw. ein Ort gefunden und soll *nicht* angezeigt,
      // sondern sofort zurückgegeben werden.
      frame.plzOrt = new String[2];
      frame.plzOrt[0] = (String) frame.tableModel.getValueAt( 0, 0 );
      frame.plzOrt[1] = (String) frame.tableModel.getValueAt( 0, 1 );
      return frame.plzOrt;
    }
    frame.pack();
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension frameSize = frame.getSize();
    frame.setLocation( ( screenSize.width - frameSize.width ) / 2, ( screenSize.height - frameSize.height ) / 2 );
    frame.setVisible( true );
    return frame.plzOrt;
  }

  /**
   * Abbrechen Knopf wurde geklickt.
   * @param e Auslösendes Event
   */
  void btnCancel_actionPerformed( ActionEvent e )
  {
    this.setVisible( false );
    this.dispose();
  }

  /**
   * OK Knopf wurde geklickt.
   * @param e Auslösendes Event
   */
  void btnOk_actionPerformed( ActionEvent e )
  {
    int row = tblPlzOrt.getSelectedRows()[0];
    plzOrt = new String[2];
    plzOrt[0] = (String) tableModel.getValueAt( row, 0 );
    plzOrt[1] = (String) tableModel.getValueAt( row, 1 );
    this.setVisible( false );
    this.dispose();
  }

  //********** Pane-spezifische Methoden ***********************************

  /**
   * Maus wurde geklickt.
   * @param e Auslösendes Event
   */
  void tblPlzOrt_MouseListener( MouseEvent e )
  {
    // bei Doppelclick OK
    if ( e.getClickCount() == 2 )
    {
      btnOk_actionPerformed( null );
    }
  }

  /**
   * Enter-Taste in der Tabelle löst den DefaultButton aus.
   * @param e KeyEvent
   */
  protected void tblPlzOrt_keyPressed( KeyEvent e )
  {
    if ( e.getKeyCode() == KeyEvent.VK_ENTER )
    {
      getRootPane().getDefaultButton().doClick();
      e.consume();
    }
  }

  /**
   * initialisiert javahelp
   */
  public void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this.getRootPane(), "adrErfassen.auswahlplzort" );
    HelpUtil.getInstance().enableHelp( tblPlzOrt, "adrErfassen.auswahlplzort" );
  }
}
