package ch.ige.adr.client;

import java.awt.Dialog;
import java.awt.Frame;
import java.util.Date;
import java.util.StringTokenizer;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdrConstants;
import ch.ige.adr.server.AdresseDelegate;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.adrhelp.HelpUtil;
import ch.ige.auth.IGESubject;
import ch.ige.authz.client.AuthMain;
import ch.ige.main.client.WizardMediator;
import ch.ige.main.client.XxMessage;
import ch.ige.util.exception.ExistsException;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;

/**
 * Wizard für das Erfassen und Mutieren der Registeradresse
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: RegadrWizard.java,v 1.37 2008/04/28 15:32:34 galli Exp $
 */
public class RegadrWizard extends WizardMediator
{
  // Logging
  private static final Logger LOG = Logger.getLogger( RegadrWizard.class );

  // Applikationsmeldungen
  protected XxMessage xxMessage = null;

  protected boolean finish = false;

  protected boolean edit = false; // true - editRegadr(), false - newRegadr aufgerufen

  protected int regadrNr = 0;

  protected PnlRegadr pnlRegadr = new PnlRegadr();

  protected PnlAdresse pnlAdresse = new PnlAdresse();

  /**
   * Konstruktor
   * @param parent aufrufendes Frame
   */
  public RegadrWizard( Frame parent )
  {
    super( parent );
    try
    {
      xxMessage = XxMessage.getInstance();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( parent, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    RegadrWizardInit();
  }

  /**
   * Konstruktor
   * @param parent aufrufender Dialog
   */
  public RegadrWizard( Dialog parent )
  {
    super( parent );
    try
    {
      xxMessage = XxMessage.getInstance();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( parent, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    RegadrWizardInit();
  }

  /**
   * Initialisierung
   */
  protected void RegadrWizardInit()
  {
    pnlRegadr.addObserver( this );
    alPanel.add( new WizardMediator.WizardPanel( pnlRegadr, true ) );
    alPanel.add( new WizardMediator.WizardPanel( pnlAdresse, true ) );
    maxPanel = alPanel.size();
    title = "Register- und Versandadresse erfassen";
  }

  /**
   * Registeradresse erfassen
   * @return Registeradressnummer
   */
  public Integer newRegadr()
  {
    edit = false;
    RegadrVO regadrVO = new RegadrVO();
    regadrVO.setBlandCd( "CH" );
    pnlRegadr.setRegadr( regadrVO, edit );
    init();
    return finish ? new Integer( regadrNr ) : null;
  }

  /**
   * Registeradresse mutieren
   * @param regadrNr Registeradressnummer
   * @return true = fertiggestellt, false = abgebrochen
   */
  public boolean editRegadr( int regadrNr )
  {
    edit = true;
    try
    {
      RegadrVO regadrVO = AdresseDelegate.getInstance().getRegadr( regadrNr );
      pnlRegadr.setRegadr( regadrVO, edit );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    init();
    return finish;
  }

  /**
   * Fertigstellen
   * @return true = Fertigstellen erfolgreich, false = nicht erfolgreich
   */
  public boolean finish()
  {
    if ( pos == 0 )
    {
      // Fertigstellen wurde auf dem Panel Registeradresse geklickt.
      // Nur Registeradresse speichern
      if ( !pnlRegadr.verifyInput() )
      {
        return false;
      }
      RegadrVO regadrVO = pnlRegadr.getRegadr();
      if ( !checkSaveRegadr( regadrVO ) )
      {
        return false;
      }
      finish = true;
      try
      {
        if ( edit )
        {
          AdresseDelegate.getInstance().setRegadr( regadrVO );
        }
        else
        {
          regadrNr = AdresseDelegate.getInstance().createRegadr( regadrVO );
        }
      }
      catch ( Exception ex )
      {
        finish = false;
        LOG.error( ex );
        String message = xxMessage.getMessage( ex );
        JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
      }
    }
    else
    {
      // Fertigstellen wurde auf dem Panel Versandadresse geklickt.
      // Register- und Versandadresse speichern.
      if ( !pnlAdresse.verifyInput() )
      {
        return false;
      }
      RegadrVO regadrVO = pnlRegadr.getRegadr();
      VersadrVO versadrVO = pnlAdresse.getVersadr();
      // Default für Mehrwertsteuerklassifikation setzen
      if ( versadrVO.getLandCdOrEmptyStr().equals( "CH" ) || versadrVO.getLandCdOrEmptyStr().equals( AdrConstants.LAND_LI ) )
      {
        // voller Steuersatz (Inland)
        versadrVO.setTaxClass( "1" );
      }
      else
      {
        // steuerbefreit (Ausland)
        versadrVO.setTaxClass( "0" );
      }

      finish = true;
      try
      {
        IGESubject subject = AuthMain.getInstance().getSubject();
        versadrVO.setEmployeeId( subject.getEmployeeID() );
        versadrVO.setErstBen( subject.getInitials() );
        versadrVO.setErstDat( new Date() );
        if ( edit )
        {
          // Registeradresse mutieren und Versandadresse kreieren.
          // Die Versandadresse kann nicht mit der Registeradresse
          // zusammen mutiert werden.
          AdresseDelegate.getInstance().setRegadr( regadrVO, versadrVO );
        }
        else
        {
          // Register- und Versandadresse kreieren.
          regadrNr = AdresseDelegate.getInstance().createRegadr( regadrVO, versadrVO );
        }
      }
      catch ( ExistsException ex )
      {
        // Diese Registeradresse existiert bereits.
        String message = xxMessage.getMessage( 1037 );
        JOptionPane.showMessageDialog( dlgWizard, message, "Information", JOptionPane.INFORMATION_MESSAGE );
        return true;

      }
      catch ( Exception ex )
      {
        LOG.error( ex );
        String message = xxMessage.getMessage( ex );
        JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
      }
    }
    return true;
  }

  /**
   * Vorwärts
   * @return true = Vorwärts erfolgreich, false = nicht erfolgreich
   */
  public boolean forward()
  {
    if ( !pnlRegadr.verifyInput() )
    {
      return false;
    }
    RegadrVO regadrVO = pnlRegadr.getRegadr();
    if ( !checkSaveRegadr( regadrVO ) )
    {
      return false;
    }
    VersadrVO versadrVO = new VersadrVO();
    String[] zeile = new String[] { "", "", "", "", "" };
    int i = 0;
    StringTokenizer st = new StringTokenizer( regadrVO.getRegNameOrEmptyString(), "\n" );
    String restVorher = ""; // Rest der vorangehenden Zeile
    while ( i < 5 && st.hasMoreTokens() )
    {
      zeile[i] = restVorher + st.nextToken();
      if ( zeile[i].length() > 40 )
      {
        // Zeile länger als 40 Zeichen, Rest auf nächste Zeile schieben
        int pos = zeile[i].substring( 0, 40 ).lastIndexOf( ' ' );
        if ( pos == -1 )
        {
          pos = 40;
        }
        restVorher = zeile[i].substring( pos + 1 ) + " ";
        zeile[i] = zeile[i].substring( 0, pos );
      }
      else
      {
        restVorher = "";
      }
      i++;
    }
    if ( restVorher.length() > 0 )
    {
      zeile[i] = restVorher;
      restVorher = "";
      i++;
    }
    // Strasse mindestens auf Zeile 2
    if ( i < 2 )
    {
      i = 2;
    }
    st = new StringTokenizer( regadrVO.getRegStrasseOrEmptyString(), "\n" );
    while ( i < 5 && st.hasMoreTokens() )
    {
      zeile[i] = restVorher + st.nextToken();
      if ( zeile[i].length() > 40 )
      {
        // Zeile länger als 40 Zeichen, Rest auf nächste Zeile schieben
        int pos = zeile[i].substring( 0, 40 ).lastIndexOf( ' ' );
        if ( pos == -1 )
        {
          pos = 40;
        }
        restVorher = zeile[i].substring( pos + 1 ) + " ";
        zeile[i] = zeile[i].substring( 0, pos );
      }
      else
      {
        restVorher = "";
      }
      i++;
    }
    versadrVO.setName( zeile[0] );
    versadrVO.setZeile1( zeile[1] );
    versadrVO.setZeile2( zeile[2] );
    versadrVO.setZeile3( zeile[3] );
    versadrVO.setZeile4( zeile[4] );
    // Falls die Zeichen vor dem ersten " " alles Ziffern sind,
    // werden diese als Postleitzahl übernommen.
    String plz = "";
    String ort = "";
    String regOrt = regadrVO.getRegOrtOrEmptyString();
    int pos = regOrt.indexOf( '\n' );
    if ( pos >= 0 )
    {
      regOrt = regOrt.substring( 0, pos );
    }
    pos = regOrt.indexOf( ' ' );
    if ( pos == -1 )
    {
      pos = regOrt.length();
    }
    plz = regOrt.substring( 0, pos );
    try
    {
      // Mit Postleitzahl
      Integer.parseInt( plz );
      if ( pos < regOrt.length() )
      {
        ort = regOrt.substring( pos + 1, regOrt.length() );
      }
    }
    catch ( NumberFormatException ex )
    {
      // Keine Postleitzahl
      plz = "";
      ort = regOrt;
    }
    versadrVO.setPlz( plz );
    versadrVO.setOrt( ort );
    try
    {
      versadrVO.setLandCd( AdresseDelegate.getInstance().getLandCd( regadrVO.getBlandCd() ) );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    versadrVO.setSuchbegr( regadrVO.getUpVerzName() );
    pnlAdresse.setVersadr( versadrVO );
    return true;
  }

  /**
   * Prüft, ob es eine andere Registeradresse (andere regadrNr) mit gleichen
   * regName, regStrasse, regOrt und blandCd gibt, und ob sie gespeichert
   * werden soll.
   * @param regadrVO Registeradresse
   * @return true - Die Registeradresse soll gespeichert werden. <br>
   * false - Die Registeradresse existiert bereits und soll nicht gespeichert werden.
   *
   */
  protected boolean checkSaveRegadr( RegadrVO regadrVO )
  {
    RegadrExistsVO regadrExistsVO = new RegadrExistsVO();
    regadrExistsVO.blandCd = regadrVO.getBlandCd();
    regadrExistsVO.regName = regadrVO.getRegName();
    regadrExistsVO.regStrasse = regadrVO.getRegStrasse();
    regadrExistsVO.regOrt = regadrVO.getRegOrt();
    int regadrNr = 0;
    try
    {
      regadrNr = AdresseDelegate.getInstance().getRegadr( regadrExistsVO );
      if ( regadrNr != regadrVO.getRegadrNr() && regadrNr != 0 )
      {
        // Diese Registeradresse existiert bereits mit der Nummer !P1
        String message = xxMessage.getMessage( 1039, "" + regadrNr );
        Object[] options = { "OK", "Abbrechen" };
        int selectedOption = JOptionPane.showOptionDialog( null, message, "Information", JOptionPane.DEFAULT_OPTION, JOptionPane.INFORMATION_MESSAGE, null,
            options, options[1] );
        return ( selectedOption == 0 );
      }
      return true;
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
      return false;
    }
  }

  /**
   * initialisiert javahelp
   */
  public void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( dlgWizard.getRootPane(), "adrErfassen.regadrerfassen" );
  }
}
