package ch.ige.adr.server;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.EJBException;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

import ch.ige.adr.client.EmailVerifier;
import ch.ige.adr.server.dao.AdresseDAO;
import ch.ige.adr.server.dao.AdresseSapDAO;
import ch.ige.adr.server.dao.EmailadrDAO;
import ch.ige.adr.vo.AdresseSapVO;
import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.AdresseVO;
import ch.ige.adr.vo.EmailadrVO;
import ch.ige.ses.server.sapctrl.SapDelegateAdresse;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.vo.VersadrVO;

/**
 * Business Object für Adressen
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseBO.java,v 1.35 2007/07/03 15:20:54 nydegger Exp $
 */

public class AdresseBO
{
  private static final Logger LOG = Logger.getLogger(AdresseBO.class);
  private AdresseXxIdentNumberLocal xxIdentNumberLocal = null;

  /**
   * constructor
   * @param xxIdentNumberLocal AdresseXxIdentNumberLocal
   */
  public AdresseBO(AdresseXxIdentNumberLocal xxIdentNumberLocal)
  {
    this.xxIdentNumberLocal = xxIdentNumberLocal;
  }

  /**
   * Kreiert eine Versandadresse.
   * @param versadrVO Value Object der Versandadresse
   * @return Adressnummer
   */
  public int createVersadr(VersadrVO versadrVO)
  {
    try
    {
      versadrVO.setAdrNr(xxIdentNumberLocal.getNextId(AdrConstants.ADR_NR));

      // Adresse schreiben
      AdresseVO adresseVO = new AdresseVO();
      adresseVO.setAdrNr(versadrVO.getAdrNr());
      adresseVO.setSuchbegr(versadrVO.getSuchbegr());
      adresseVO.setSprachCd(versadrVO.getSprachCd());
      adresseVO.setAnrCd(versadrVO.getAnrCd());
      adresseVO.setName(versadrVO.getName());
      adresseVO.setZeile1(versadrVO.getZeile1());
      adresseVO.setZeile2(versadrVO.getZeile2());
      adresseVO.setZeile3(versadrVO.getZeile3());
      adresseVO.setZeile4(versadrVO.getZeile4());
      adresseVO.setLandCd(versadrVO.getLandCd());
      adresseVO.setPfPlz(versadrVO.getPfPlz());
      adresseVO.setPlz(versadrVO.getPlz());
      adresseVO.setOrt(versadrVO.getOrt());
      adresseVO.setTelefon(versadrVO.getTelefon());
      //  telex; wird nicht mehr verwendet
      adresseVO.setFax(versadrVO.getFax());
      adresseVO.setErstDat(versadrVO.getErstDat());
      adresseVO.setErstBen(versadrVO.getErstBen());
      adresseVO.setGeaendDat(versadrVO.getGeaendDat());
      adresseVO.setGeaendBen(versadrVO.getGeaendBen());
      adresseVO.setUVersion(versadrVO.getUVersion());
      adresseVO.setPlzZusatz(versadrVO.getPlzZusatz());
      adresseVO.setGesperrt(versadrVO.getGesperrt());
      adresseVO.setAdrNrNeu(versadrVO.getAdrNrNeu());
      adresseVO.setVersion(versadrVO.getVersion());
      new AdresseDAO().insert(adresseVO);
      // Email-Adresse schreiben
      if (versadrVO.getEmailNameDomainOrEmptyStr().length() > 0)
      {
        int pos = versadrVO.getEmailNameDomain().indexOf('@');
        if(EmailVerifier.verifyEmail(versadrVO.getEmailNameDomain()) == false)
        {
          throw new IllegalArgumentException("Ungültige Email-Adresse");
        }
        EmailadrVO emailadrVO = new EmailadrVO();
        emailadrVO.setEmailadrNr(xxIdentNumberLocal.getNextId(AdrConstants.EMAILADR));
        emailadrVO.setAdrNr(versadrVO.getAdrNr());
        emailadrVO.setEmailName(versadrVO.getEmailNameDomainOrEmptyStr().substring(0, pos));
        emailadrVO.setEmailDomain(versadrVO.getEmailNameDomain().substring(pos + 1));
        new EmailadrDAO().insert(emailadrVO);
      }
      AdresseSapVO adresseSapVO = new AdresseSapVO();
      adresseSapVO.adrNr = versadrVO.getAdrNr();
      adresseSapVO.custGroup = versadrVO.getCustGroup();
      adresseSapVO.taxClass = versadrVO.getTaxClass();
      adresseSapVO.custGrp1 = versadrVO.getCustGrp1();
      adresseSapVO.version = versadrVO.getAdresseSapVersion();
      new AdresseSapDAO().insert(adresseSapVO);
      if (Sap.isEnabled())
      {
        // Kopie in SAP ablegen
        SapDelegateAdresse.getInstance().create(versadrVO);
      }
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
    return versadrVO.getAdrNr();
  }

  /**
   * Gibt das Value Object der Versandadresse zurück.
   * @param adrNr Adressnummer
   * @return Value Object der Versandadresse
   */
  public VersadrVO getVersadr(int adrNr)
  {
    try
    {
      // Adresse lesen
      AdresseVO adresseVO = new AdresseDAO().select(adrNr);
      if (adresseVO == null)
      {
        throw new EJBException("MNR=1303, getVersadr");
      }
      VersadrVO versadrVO = new VersadrVO();
      versadrVO.setAdrNr(adresseVO.getAdrNr());
      versadrVO.setSuchbegr(adresseVO.getSuchbegr());
      versadrVO.setSprachCd(adresseVO.getSprachCd());
      versadrVO.setAnrCd(adresseVO.getAnrCd());
      versadrVO.setName(adresseVO.getName());
      versadrVO.setZeile1(adresseVO.getZeile1());
      versadrVO.setZeile2(adresseVO.getZeile2());
      versadrVO.setZeile3(adresseVO.getZeile3());
      versadrVO.setZeile4(adresseVO.getZeile4());
      versadrVO.setLandCd(adresseVO.getLandCd());
      versadrVO.setPlz(adresseVO.getPlz());
      versadrVO.setPlzZusatz(adresseVO.getPlz());
      versadrVO.setOrt(adresseVO.getOrt());
      versadrVO.setTelefon(adresseVO.getTelefon());
      //  telex; wird nicht mehr verwendet
      versadrVO.setFax(adresseVO.getFax());
      versadrVO.setErstDat(adresseVO.getErstDat());
      versadrVO.setErstBen(adresseVO.getErstBen());
      versadrVO.setGeaendDat(adresseVO.getGeaendDat());
      versadrVO.setGeaendBen(adresseVO.getGeaendBen());
      versadrVO.setUVersion(adresseVO.getUVersion());
      versadrVO.setPlzZusatz(adresseVO.getPlzZusatz());
      versadrVO.setGesperrt(adresseVO.getGesperrt());
      versadrVO.setAdrNrNeu(adresseVO.getAdrNrNeu());
      versadrVO.setVersion(adresseVO.getVersion());
      // Email-Adresse lesen
      EmailadrVO emailadrVO = new EmailadrDAO().selectByAdrNr(adrNr);
      if (emailadrVO != null)
      {
        versadrVO.setEmailadrNr(emailadrVO.getEmailadrNr());
        versadrVO.setEmailNameDomain(emailadrVO.getEmailName() + '@' + emailadrVO.getEmailDomain());
        versadrVO.setEmailVersion(emailadrVO.getVersion());
      }
      // AdresseSap lesen
      AdresseSapVO adresseSapVO = new AdresseSapDAO().select(adrNr);
      if (adresseSapVO != null)
      {
        versadrVO.setCustGroup(adresseSapVO.custGroup);
        versadrVO.setTaxClass(adresseSapVO.taxClass);
        versadrVO.setCustGrp1(adresseSapVO.custGrp1);
        versadrVO.setAdresseSapVersion(adresseSapVO.version);
      }
      return versadrVO;
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Liest alle Adr-Nummern aus der Tabelle Adresse
   * @return ArrayList mit Integer-Objekten mit den Adressnummern
   * @param startAdrNr Anfangs-Adressnummer
   */
  public ArrayList getAdrNr(int startAdrNr)
  {
    try
    {
      return new AdresseDAO().selectAdrNr(startAdrNr);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Setzt die Versandadresse auf den Wert des Value Objects.
   * @param versadrVO Versandadresse
   */
  public void setVersadr(VersadrVO versadrVO)
  {
    AdresseVO adresseVO = new AdresseVO();
    adresseVO.setAdrNr(versadrVO.getAdrNr());
    adresseVO.setSuchbegr(versadrVO.getSuchbegr());
    adresseVO.setSprachCd(versadrVO.getSprachCd());
    adresseVO.setAnrCd(versadrVO.getAnrCd());
    adresseVO.setName(versadrVO.getName());
    adresseVO.setZeile1(versadrVO.getZeile1());
    adresseVO.setZeile2(versadrVO.getZeile2());
    adresseVO.setZeile3(versadrVO.getZeile3());
    adresseVO.setZeile4(versadrVO.getZeile4());
    adresseVO.setLandCd(versadrVO.getLandCd());
    adresseVO.setPfPlz(versadrVO.getPfPlz());
    adresseVO.setPlz(versadrVO.getPlz());
    adresseVO.setOrt(versadrVO.getOrt());
    adresseVO.setTelefon(versadrVO.getTelefon());
    //  telex; wird nicht mehr verwendet
    adresseVO.setFax(versadrVO.getFax());
    adresseVO.setErstDat(versadrVO.getErstDat());
    adresseVO.setErstBen(versadrVO.getErstBen());
    adresseVO.setGeaendDat(versadrVO.getGeaendDat());
    adresseVO.setGeaendBen(versadrVO.getGeaendBen());
    adresseVO.setUVersion(Crescendo.getNextUVersion(versadrVO.getUVersion()));
    adresseVO.setPlzZusatz(versadrVO.getPlzZusatz());
    adresseVO.setGesperrt(versadrVO.getGesperrt());
    adresseVO.setAdrNrNeu(versadrVO.getAdrNrNeu());
    adresseVO.setVersion(versadrVO.getVersion()+ 1);
    try
    {
      // Adresse schreiben
      if (new AdresseDAO().update(adresseVO) != 1)
      {
        throw new EJBException("MNR=1303, setVersadr");
      }
      // Email-Adresse schreiben
      if (versadrVO.getEmailNameDomainOrEmptyStr().length() > 0)
      {
        // mit Email-Adresse
        int pos = versadrVO.getEmailNameDomain().indexOf('@');
        LOG.debug(versadrVO.getEmailNameDomain());
        if (EmailVerifier.verifyEmail(versadrVO.getEmailNameDomainOrEmptyStr()) == false)
        {
          throw new IllegalArgumentException("Ungültige Email-Adresse");
        }

        EmailadrVO emailadrVO = new EmailadrVO();
        emailadrVO.setEmailadrNr(versadrVO.getEmailadrNr());
        emailadrVO.setAdrNr(versadrVO.getAdrNr());
        emailadrVO.setEmailName(versadrVO.getEmailNameDomain().substring(0, pos));
        emailadrVO.setEmailDomain(versadrVO.getEmailNameDomain().substring(pos + 1));
        emailadrVO.setVersion(versadrVO.getEmailVersion());
        if (versadrVO.getEmailadrNr() == 0)
        {
          // neue Email-Adresse einfügen
          emailadrVO.setEmailadrNr(xxIdentNumberLocal.getNextId(AdrConstants.EMAILADR));
          new EmailadrDAO().insert(emailadrVO);
        }
        else
        {
          // existierende Email-Adresse überschreiben
          emailadrVO.setVersion(emailadrVO.getVersion() + 1);
          new EmailadrDAO().update(emailadrVO);
        }
      }
      else
      {
        // ohne Email-Adresse
        if (versadrVO.getEmailadrNr() != 0)
        {
          // existierende Email-Adresse löschen
          new EmailadrDAO().delete(versadrVO.getEmailadrNr());
        }
      }
      AdresseSapVO adresseSapVO = new AdresseSapVO();
      adresseSapVO.adrNr = versadrVO.getAdrNr();
      adresseSapVO.custGroup = versadrVO.getCustGroup();
      adresseSapVO.taxClass = versadrVO.getTaxClass();
      adresseSapVO.custGrp1 = versadrVO.getCustGrp1();
      adresseSapVO.version = versadrVO.getAdresseSapVersion() + 1;
      new AdresseSapDAO().update(adresseSapVO);
      if (Sap.isEnabled())
      {
        // Kopie in SAP updaten
        SapDelegateAdresse.getInstance().update(versadrVO);
      }
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Löscht die Versandadresse
   * @param adrNr Adressnummer
   */
  public void removeVersadr(int adrNr)
  {
    try
    {
      if (new AdresseDAO().delete(adrNr) != 1)
      {
        throw new EJBException("MNR=1303, removeVersadr");
      }
      if (Sap.isEnabled())
      {
        // Kopie in SAP löschen
        SapDelegateAdresse.getInstance().delete(adrNr);
      }
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
  }

  /**
   * Sperrt die Versandadresse
   * @param adrNr Adressnummer
   * @param gesperrt true - sperren, false - Sperrung aufheben
   * @param geaendBen Kürzel des Benuzter
   */
  public void blockVersadr(int adrNr, boolean gesperrt, String geaendBen)
  {
    try
    {
      // Adresse lesen
      AdresseVO adresseVO = new AdresseDAO().select(adrNr);
      if (adresseVO == null)
      {
        throw new EJBException("MNR=1303, blockVersadr");
      }
      if (gesperrt)
      {
        adresseVO.setGesperrt(1);
      }
      else
      {
        adresseVO.setGesperrt(0);
      }
      adresseVO.setGeaendDat(new java.util.Date());
      adresseVO.setGeaendBen(geaendBen);
      adresseVO.setUVersion(Crescendo.getNextUVersion(adresseVO.getUVersion()));
      adresseVO.setVersion(adresseVO.getVersion() + 1);
      if (new AdresseDAO().update(adresseVO) != 1)
      {
        throw new EJBException("MNR=1303, blockVersadr");
      }
      if (Sap.isEnabled())
      {
        // Kopie in SAP sperren bzw Sperrung aufheben
        if (gesperrt)
        {
          // Kopie in SAP sperren
          SapDelegateAdresse.getInstance().block(adrNr);
        }
        else
        {
          // Kopie in SAP, Sperrung aufheben
          SapDelegateAdresse.getInstance().unblock(adrNr);
        }
      }
    }
    catch (Exception ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }

  }
  /**
   * Findet alle Adressen, auf welche die Selektionskriterien zutreffen.
   * @param adresseSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit AdresseAVO-Objekten mit den gefundenen Adressen
   */
  public ArrayList findVersadr(AdresseSelKritVO adresseSelKritVO)
  {
    if (adresseSelKritVO.suchmodusName.equals(AdresseSelKritVO.SUCHMODUS_INDEX))
    {
      // Bei Indexsuche Name normalisieren
      adresseSelKritVO.name = PostOrtNorm.getOrtNorm(adresseSelKritVO.name, true);
      if (adresseSelKritVO.name.equals("."))
      {
        adresseSelKritVO.name = "";
      }
    }
    // Strasse immer normalisieren, da es nur Index gibt
    adresseSelKritVO.strasse = PostOrtNorm.getOrtNorm(adresseSelKritVO.strasse, true);
    if (adresseSelKritVO.strasse.equals("."))
    {
      adresseSelKritVO.strasse = "";
    }
    if (adresseSelKritVO.suchmodusOrt.equals(AdresseSelKritVO.SUCHMODUS_INDEX))
    {
      // Bei Indexsuche Name normalisieren
      adresseSelKritVO.ort = PostOrtNorm.getOrtNorm(adresseSelKritVO.ort, true);
      if (adresseSelKritVO.ort.equals("."))
      {
        adresseSelKritVO.ort = "";
      }
    }
    if (adresseSelKritVO.adrNr == 0 &&
        adresseSelKritVO.suchbegr.equals("") &&
        adresseSelKritVO.name.equals("") &&
        adresseSelKritVO.strasse.equals("") &&
        adresseSelKritVO.landCd.equals("") &&
        adresseSelKritVO.plz.equals("") &&
        adresseSelKritVO.ort.equals(""))
    {
      throw new EJBException("MNR=1908, findVersadr");
    }
    try
    {
      return new AdresseDAO().select(adresseSelKritVO);
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseBO.class);
      throw new EJBException(ex);
    }
  }
}


class Sap
{
  static boolean enabled = true;
  static
  {
    try
    {
      // Naming-Kontext ermitteln
      Context ctx = new InitialContext();
      // JNDI-Namen nachschlagen
      Boolean sapDisabled = (Boolean) ctx.lookup( "java:comp/env/sapDisabled" );
      enabled = !sapDisabled.booleanValue();
    }
    catch ( Exception ex )
    {
      /** do nothing */
    }
  }

  /**
   * Singleton
   */
  private Sap()
  {
    /** not much action here */
  }

  /**
   * @return true wenn enabled
   */
  public static boolean isEnabled()
  {
    return enabled;
  }
}
