package ch.ige.adr.server;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import ch.ige.util.server.DBHelper;

/**
 * Vergibt den nächsten Primärschlüssel
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 *
 * @author  A. Rüegsegger
 * @version $Id: AdresseXxIdentNumberBean.java,v 1.15 2007/07/03 15:20:54 nydegger Exp $
 * <!-- begin-user-doc --> A generated session bean <!-- end-user-doc --> * 
 * <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *  name="AdresseXxIdentNumber"  
 *  description="An EJB named AdresseXxIdentNumber"
 *  display-name="AdresseXxIdentNumber"
 *  view-type="local"
 *  jndi-name="AdresseXxIdentNumber"
 *  local-jndi-name="AdresseXxIdentNumberLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 *            
 * @ejb.interface
 *  local-class="ch.ige.adr.server.AdresseXxIdentNumberLocal"
 * @ejb.home
 *  local-class="ch.ige.adr.server.AdresseXxIdentNumberLocalHome"
 * @ejb.transaction
 *   type="RequiresNew"
 * <!-- end-xdoclet-definition --> 
 */
public class AdresseXxIdentNumberBean implements SessionBean
{
  private final int MAX_RETRY = 50;
  private SessionContext sessionContext;


  /**
   * ejbCreate
   */
  public void ejbCreate()
  {
    /**
     * ejbCreate()
     */
  }
  /**
   * ejbRemove 
   */
  public void ejbRemove()
  {
    /**
     * ejbRemove 
     */
  }
  /**
   * ejbActivate 
   */
  public void ejbActivate()
  {
    /**
     * ejbActivate 
     */
  }
  /**
   * ejbPassivate 
   */
  public void ejbPassivate()
  {
    /**
     * ejbPassivate 
     */
  }
  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  /**
   * Vergibt den nächsten Primärschlüssel
   * Diese Funktion muss in einer eigenen Transaktion ablaufen,
   * damit Tabelle nur möglichst kurze Zeit gelockt wird
   * @param name Name des Primärschlüssels. Konstanten aus XxIdentNumber.
   * @return nächster Primärschlüssel
   * <!-- begin-xdoclet-definition -->
  * @ejb.interface-method
  * <!-- end-xdoclet-definition -->
 */
  public int getNextId(String name)
  {
    Connection con = null;
    PreparedStatement psSelect = null;
    PreparedStatement psUpdate = null;
    ResultSet rs = null;
    String query = null;

    int wert = 0;
    int version = 0;
    int counter = 0;

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);

      do
      {
        psSelect = con.prepareStatement(
            "SELECT wert, version " +
            "FROM xx_ident_number " +
            "WHERE ident = 'bag' AND name = ? AND code = 0");
        psSelect.setString(1, name);
        rs = psSelect.executeQuery();

        if(rs.next())
        {
          wert = rs.getInt("wert");
          version = rs.getInt("version");
        }
        else
        {
          throw new EJBException("Name = " + name + "in Tabelle XX_IDENT_NUMBER nicht gefunden");
        }

        query = "UPDATE xx_ident_number " +
                "SET wert = wert + sequenz, " +
                "version = ? " +
                "WHERE ident = 'bag' AND name = ? AND code = 0";
        psUpdate = con.prepareStatement(query);
        psUpdate.setInt(1, version + 1);
        psUpdate.setString(2, name);
      }
      while ( psUpdate.executeUpdate() != 1 && ++counter < MAX_RETRY );

      if (counter < MAX_RETRY)
      {
        return wert;
      }
      else
      {
        throw new EJBException("Probleme bei Update xx_ident_number Query: " +
                                query + " version = " + version + ", name = " + name);
      }
    }
    catch (SQLException ex)
    {
      throw new EJBException(ex);
    }
    finally
    {
      DBHelper.close(psSelect, psUpdate, con);
    }
  }
}
