package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Datenbankzugriff auf Tabelle adresse_ptt_pend.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: AdressePttPendDAO.java,v 1.12 2006/12/11 15:17:28 nydegger Exp $
 */
public class AdressePttPendDAO
{

  /**
   * Fügt einen Datensatz in die Tabelle adresse_ptt_pend ein<br>
   * @param adrNr  AdresseNr
   * @param adrOrt Ort
   * @throws SQLException Unerwarteter Fehler
   */
  public void insert(int adrNr, String adrOrt) throws SQLException
  {
    String            query = null;
    Connection        con   = null;
    PreparedStatement ps    = null;

    try
    {
      query ="INSERT INTO adresse_ptt_pend (" +
             "adr_nr, status, ort_norm, erst_dat, erst_ben, version) " +
             "VALUES(?,?,?,?,?,?)";

      con  = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps = con.prepareStatement("INSERT INTO adresse_ptt_pend (adr_nr, status, ort_norm, erst_dat, erst_ben, version)VALUES(?,?,?,?,?,?)");
      ps.setInt(1, adrNr);
      ps.setInt(2, 1);
      ps.setString(3, adrOrt);
      ps.setDate(4, new java.sql.Date(new java.util.Date().getTime()));
      ps.setString(5, "");
      ps.setInt(6, 0);

      ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdressePttPendDAO.class, "adrNr = " + adrNr + " adrOrt " + adrOrt);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Löscht alle Datensätze in der Tabelle adresse_ptt_pend.
   * @return Anzahl der gelöschten Adressen
   * @exception SQLException
   */
  public int delete() throws SQLException
  {
    int rowCount;
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "DELETE FROM adresse_ptt_pend";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdressePttPendDAO.class, "query = " + query);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }
}
