package ch.ige.adr.server.dao.test;

import java.util.ArrayList;
import java.util.GregorianCalendar;

import junit.framework.TestCase;
import ch.ige.adr.server.dao.AdresseDAO;
import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.AdresseVO;
import ch.ige.util.server.DBHelper;

/**
 * Test der Klasse AdresseDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: AdresseDAOTest.java,v 1.20 2007/07/03 15:20:54 nydegger Exp $
 */

public class AdresseDAOTest extends TestCase
{
  private AdresseDAO adrDAO = new AdresseDAO();
  private AdresseVO adrVO = null;

  /**
   * @param s
   */
  public AdresseDAOTest(String s)
  {
    super(s);
    DBHelper.setRunOnServer( false );
  }

  protected void setUp()
  {
//    adrVO
    adrVO = new AdresseVO();
    adrVO.setAdrNr(1);
    adrVO.setSuchbegr("TEST");
    adrVO.setSprachCd("D");
    adrVO.setAnrCd("01");
    adrVO.setName("Muster");
    adrVO.setZeile1("Musterstrasse 12");
    adrVO.setZeile2("Mustergebiet");
    adrVO.setZeile3("Beispielhof");
    adrVO.setZeile4("Kiesgrube");
    adrVO.setLandCd("CH");
    adrVO.setPfPlz("3150");
    adrVO.setPlz("3177");
    adrVO.setOrt("Laupen");
    adrVO.setTelefon("111");
    adrVO.setFax("222");
    adrVO.setErstDat(new GregorianCalendar(2000, 11, 12).getTime());
    adrVO.setErstBen("stn");
    adrVO.setOrt("Laupen");
    adrVO.setGeaendBen("stn");
    adrVO.setUVersion("!");
    adrVO.setPlzZusatz("2");
    adrVO.setGesperrt(0);
    adrVO.setAdrNrNeu(1);
    adrVO.setVersion(0);
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle adresse ein
   */
  public void test01Insert()
  {
    System.out.println("Test INSERT");

    try
    {
      adrDAO.insert(adrVO);
      // Testen ob eingefügt
      adrVO = adrDAO.select(1);
      if (adrVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("adrVO existiert ", adrVO != null);
      assertEquals(" ", adrVO.getTelefon(), "111");
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Testet die Suche im Ortsabgleich Modus
   */
  public void _test02SearchOrtsabgleichModus()
  {
    System.out.println("Test SEARCH Modus Ortsabgleich");

    try
    {
//      adrDAO.se

      adrDAO.insert(adrVO);
      // Testen ob eingefügt
      adrVO = adrDAO.select(1);
      if (adrVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("adrVO existiert ", adrVO != null);
      assertEquals(" ", adrVO.getTelefon(), "111");
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert einen Datensatz
   */
//  public void testUpdate()
//  {
//    System.out.println("Test UPDATE ");
//    AdresseDAO adrDAO = new AdresseDAO();
//    AdresseVO adrVO = new AdresseVO();
//
//    try {
//      artextVO = artextDAO.select("100000", "D", "xxx");
//      if (artextVO == null)
//      {
//        System.out.println("UPDATE: Datensatz existiert nicht");
//      }
//      else
//      {
//        artextVO.text = "update";
//        artextVO.version++;
//        int retVal = artextDAO.update(artextVO);
//        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
//        //prüfen ob Änderung gespeichert
//        artextVO = artextDAO.select("100000", "D", "xxx");
//        assertEquals("text: ", artextVO.text, "update");
//      }
//    }
//    catch(Exception e) {
//      e.printStackTrace();
//      System.err.println("Exception thrown:  "+e);
//    }
//  }


  
  
  /**
   *  Selektiert Datensatze aus der Tabelle adresse
   */
  public void test03Select()
  {
    System.out.println("Test SELECT ");

    try
    {
      ArrayList al = adrDAO.select("Stefan Nydegger", "Laupen BE");
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Select mit Name und Ort, keine Daten gefunden", al.size() > 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle adresse
   */
  public void test04Delete()
  {
    System.out.println("Test DELETE");

    try
    {
      int retVal = adrDAO.delete(1);
      assertEquals("Anzahl gelöschte Datensätze ", retVal, 1);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
  
  /**
   *  Selektiert Datensatze aus der Tabelle adresse
   */
  public void test05Select()
  {
    System.out.println("Test SELECT mit AdrSelKritVO");

    try
    {
      AdresseSelKritVO vo = new AdresseSelKritVO();
      vo.name = "Stefan Nydegger";
      vo.ort = "Laupen BE";
      vo.suchmodusName = AdresseSelKritVO.SUCHMODUS_VOLLTEXT;
      vo.suchmodusOrt = AdresseSelKritVO.SUCHMODUS_VOLLTEXT;
      ArrayList al = adrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Select mit Name und Ort, keine Daten gefunden", al.size() > 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
  
  /**                                                                                                                                                                                                                                        
   *  Selektiert Datensatze aus der Tabelle adresse
   */
  public void test06Select()
  {
    System.out.println("Test SELECT mit AdrSelKritVO");

    try
    {
      // Textindex mit Name
      AdresseSelKritVO vo = new AdresseSelKritVO();
      vo.name = "Stefan Nydegger";
      vo.suchmodusName = AdresseSelKritVO.SUCHMODUS_VOLLTEXT;
      ArrayList al = adrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
   
      assertTrue("Volltextsuche nach name, keine Daten gefunden", al.size() > 0);

      vo.name = "Stefan Nydegger";
      vo.suchmodusName = AdresseSelKritVO.SUCHMODUS_INDEX;
      al = adrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Indexsuche nach Name , keine Daten gefunden", al.size() > 0);
   
      vo.name = "Stefan Nydegger";
      vo.suchmodusName = AdresseSelKritVO.SUCHMODUS_INDEX;
      vo.strasse = "Eigerweg 9";
      al = adrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Indexsuche nach Name UND Strasse , keine Daten gefunden", al.size() > 0);
      
      vo.name = "Stefan Nydegger";
      vo.suchmodusName = AdresseSelKritVO.SUCHMODUS_INDEX;
      vo.strasse = "Eigerweg 9";
      vo.ort = "Laupen BE";
      vo.suchmodusOrt = AdresseSelKritVO.SUCHMODUS_INDEX;
      al = adrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Indexsuche nach Name UND Ort , keine Daten gefunden", al.size() > 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
  /**
   *  Selektiert Datensatze aus der Tabelle adresse
   */
  public void test07Select()
  {
    System.out.println("Test SELECT mit ArrayList von Suchworten");
    ArrayList searchAl = new ArrayList();
    searchAl.add( "Stefan");
    searchAl.add( "Nydegger" );
    searchAl.add( "Eigerweg" );
    searchAl.add( "9" );
    searchAl.add( "Laupen" );
    searchAl.add( "BE" );
    
    try
    {
      ArrayList al = adrDAO.select(searchAl);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Select mit Name und Ort, keine Daten gefunden", al.size() > 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
  
  
}
