package ch.ige.adr.server.dao.test;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import junit.framework.TestCase;
import ch.ige.adr.server.dao.AdresseDAO;
import ch.ige.adr.server.dao.RegadrDAO;
import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.AdresseVO;
import ch.ige.adr.vo.RegadrExistsVO;
import ch.ige.adr.vo.RegadrSelKritVO;
import ch.ige.util.server.DBHelper;

/**
 * Test der Klasse ArtextDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: RegadrDAOTest.java,v 1.13 2007/09/28 07:54:00 nydegger Exp $
 */

public class RegadrDAOTest extends TestCase
{

  private AdresseVO adrVO = null;

  /**
   * @param s
   */
  public RegadrDAOTest(String s)
  {
    super(s);
    DBHelper.setRunOnServer( false );
  }

  protected void setUp()
  {
//    adrVO
    adrVO = new AdresseVO();
    adrVO.setAdrNr(1);
    adrVO.setSuchbegr("TEST");
    adrVO.setSprachCd("D");
    adrVO.setAnrCd("H");
    adrVO.setName("Muster");
    adrVO.setZeile1("Musterstrasse 12");
    adrVO.setZeile2("Mustergebiet");
    adrVO.setZeile3("Beispielhof");
    adrVO.setZeile4("Kiesgrube");
    adrVO.setLandCd("CH");
    adrVO.setPfPlz("3150");
    adrVO.setPlz("3177");
    adrVO.setOrt("Laupen");
    adrVO.setTelefon("111");
    adrVO.setFax("222");
    adrVO.setErstDat(new GregorianCalendar(2000, 11, 12).getTime());
    adrVO.setErstBen("stn");
    adrVO.setOrt("Laupen");
    adrVO.setGeaendBen("stn");
    adrVO.setUVersion("!");
    adrVO.setPlzZusatz("2");
    adrVO.setGesperrt(0);
    adrVO.setAdrNrNeu(1);
    adrVO.setVersion(0);

    // falls vorhanden wird addresse gelösche
    try
    {
      int retVal = new AdresseDAO().delete(1);
    }
    catch (SQLException ex)
    {
      fail();
    }
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle adresse ein
   */
  public void _test01Insert() {
    System.out.println("Test INSERT");
    AdresseDAO adrDAO = new AdresseDAO();

    try
    {
      adrDAO.insert(adrVO);
      // Testen ob eingefügt
      adrVO = adrDAO.select(1);
      if (adrVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("adrVO existiert ", adrVO != null);
      assertEquals(" ", adrVO.getTelefon(), "111");
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  testet die Suche nach einer bestehenden Registeradresse
   */
  public void _test03SelectExistRegadr() {
    System.out.println("Test test03SelectExistRegadr");
    RegadrExistsVO regadrExistsVO = new RegadrExistsVO();
    RegadrDAO regAdrDAO = new RegadrDAO();

    regadrExistsVO.blandCd = "JP";
    regadrExistsVO.regName = "Kometani, Haruyuki";
    regadrExistsVO.regOrt = "Chiyoda, \nTokyo 100-8310";
    regadrExistsVO.regStrasse = "c/o Mitsubishi Denki K.K., \n2-3, Marunouchi 2-chome";

    try
    {
      int adrNr = regAdrDAO.selectRegadrNr(regadrExistsVO);
      System.out.println("adrNr " + adrNr);
      assertTrue(adrNr > 0);
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /**
   *  testet die Suche nach einer bestehenden Registeradresse
   */
  public void test04SelectExistRegadr()
  {
    System.out.println( "Test test04SelectExistRegadr" );
    RegadrExistsVO regadrExistsVO = new RegadrExistsVO();
    RegadrDAO regAdrDAO = new RegadrDAO();

    regadrExistsVO.blandCd = "CH";
    regadrExistsVO.regName = "Fritz Nydegger";
    regadrExistsVO.regStrasse = "Lerchenfeldstrasse 54";
    regadrExistsVO.regOrt = "3603 Thun";

    try
    {
      int adrNr = regAdrDAO.selectRegadrNr( regadrExistsVO );

      System.out.println( "adrNr " + adrNr );
      assertTrue( adrNr > 0 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
    }
  }
  
  /**                                                                                                                                                                                                                                        
   *  Selektiert Datensatze aus der Tabelle adresse
   */
  public void test06Select()
  {
    System.out.println("Test SELECT mit AdrSelKritVO");

    try
    {
      RegadrDAO regAdrDAO = new RegadrDAO();
      // Textindex mit reg_name
      RegadrSelKritVO vo = new RegadrSelKritVO();

      vo.regName = "Fey Consulting AG";
      vo.suchmodusVolltext = true;
      ArrayList al = regAdrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertEquals("Volltextsuche nach name, keine Daten gefunden", al.size(), 2);

      vo.regName = "Fey Consulting AG";
      vo.suchmodusVolltext = false;
      al = regAdrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Indexsuche nach Name , keine Daten gefunden", al.size() > 0);
   
      vo.regName = "Fey Consulting AG";
      vo.regOrt = "8132 Egg b. Zürich";
      vo.suchmodusVolltext = true;
      al = regAdrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertEquals("Volltextsuche nach Name UND Ort , keine Daten gefunden", al.size() ,1);
      
      vo.regName = "Fey Consulting AG";
      vo.regOrt = "8132 Egg b. Zürich";
      vo.suchmodusVolltext = false;
      al = regAdrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertEquals("Indexsuche nach Name UND Ort , keine Daten gefunden", al.size() ,1);
      
      vo.upVerzName = "FEY CONSULTING AG";
      vo.regName = "";
      vo.regOrt = "";
      vo.suchmodusVolltext = false;
      vo.nurMitVersAdr = false;
      al = regAdrDAO.select(vo);
      System.out.println("Anzahl Adressen " + al.size());
      assertTrue("Indexsuche nach Name UND Ort , keine Daten gefunden", al.size()> 1);
      
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert einen Datensatz
   */
//  public void testUpdate()
//  {
//    System.out.println("Test UPDATE ");
//    AdresseDAO adrDAO = new AdresseDAO();
//    AdresseVO adrVO = new AdresseVO();
//
//    try {
//      artextVO = artextDAO.select("100000", "D", "xxx");
//      if (artextVO == null)
//      {
//        System.out.println("UPDATE: Datensatz existiert nicht");
//      }
//      else
//      {
//        artextVO.text = "update";
//        artextVO.version++;
//        int retVal = artextDAO.update(artextVO);
//        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
//        //prüfen ob Änderung gespeichert
//        artextVO = artextDAO.select("100000", "D", "xxx");
//        assertEquals("text: ", artextVO.text, "update");
//      }
//    }
//    catch(Exception e) {
//      e.printStackTrace();
//      System.err.println("Exception thrown:  "+e);
//    }
//  }


 /**
   * Löscht einen Datensatz aus der Tabelle adresse
   */
//   public void test02Delete()
//   {
//     System.out.println("Test DELETE");
//     AdresseDAO adrDAO = new AdresseDAO();
//     try
//     {
//       int retVal = adrDAO.delete(1);
//       assertEquals("Anzahl gelöschte Datensätze ", retVal, 1);
//     }
//     catch(Exception e)
//     {
//       e.printStackTrace();
//       fail(e.toString());
//     }
//   }
}
