package ch.ige.adr.server.test;

import java.util.ArrayList;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import ch.ige.adr.server.AdresseFacade;
import ch.ige.adr.server.AdresseFacadeHome;
import ch.ige.adr.vo.BlandAVO;
import ch.ige.adr.vo.LandAVO;
import ch.ige.util.ServiceLocator;
import ch.ige.util.vo.RegadrVO;
import ch.ige.util.vo.VersadrVO;

/**
 * Dient zum Testen der AdresseFacade
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseFacadeTestJUnit.java,v 1.10 2007/07/03 15:20:54 nydegger Exp $
 */

public class AdresseFacadeTestJUnit extends TestCase
{

  private AdresseFacadeHome adresseFacadeHome = null;
  private AdresseFacade adresseFacade = null;

  /**
   * @param parName
   */
  public AdresseFacadeTestJUnit(String parName)
  {
    super (parName);
  }

  /**
   * suite überschreibt Klassenmethode. Gibt die eigene Klasse als Testsuite zurück
   * @return Test
   */

  public static Test suite()
  {
    return new TestSuite(AdresseFacadeTestJUnit.class);
  }

  /**
   * Test der Adresse
   */
  public void testAdresse()
  {
    getFacade();
    try
    {
      // Create neue Adresse
      VersadrVO versadrVO = new VersadrVO();
      versadrVO.setName("test Create Adresse");
      versadrVO.setPlz("3000");
      versadrVO.setOrt("Bern");
      versadrVO.setSuchbegr("testCreate");
      versadrVO.setAdrNr(adresseFacade.createVersadr(versadrVO));

      // Selektieren der neuen Adresse
      VersadrVO versadrResultVO = adresseFacade.getVersadr(versadrVO.getAdrNr());

      assertEquals("1. Test - adrNr: ", versadrVO.getAdrNr(), versadrResultVO.getAdrNr());
      assertEquals("2. Test - name: ", versadrVO.getName(), versadrResultVO.getName());
      assertEquals("3. Test - plz: ", versadrVO.getPlz(), versadrResultVO.getPlz());
      assertEquals("4. Test - ort: ", versadrVO.getOrt(), versadrResultVO.getOrt());
      assertEquals("5. Test - suchbegr:", versadrVO.getSuchbegr(), versadrResultVO.getSuchbegr());

      // Mutieren der Adresse
      versadrVO.setName("test Create Adresse 2");
      versadrVO.setPlz("3002");
      versadrVO.setOrt("Bern 2");
      versadrVO.setSuchbegr("testCreate");
      adresseFacade.setVersadr(versadrVO);

      // Selektieren der neuen Adresse
      versadrResultVO = adresseFacade.getVersadr(versadrVO.getAdrNr());

      assertEquals("6. Test - adrNr: ", versadrVO.getAdrNr(), versadrResultVO.getAdrNr());
      assertEquals("7. Test - name: ", versadrVO.getName(), versadrResultVO.getName());
      assertEquals("8. Test - plz: ", versadrVO.getPlz(), versadrResultVO.getPlz());
      assertEquals("9. Test - ort: ", versadrVO.getOrt(), versadrResultVO.getOrt());
      assertEquals("10. Test - suchbegr:", versadrVO.getSuchbegr(), versadrResultVO.getSuchbegr());

      // Löschen der Adresse
      adresseFacade.removeVersadr(versadrVO.getAdrNr());
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Test des Landes
   */
  public void testFindLaender()
  {
    getFacade();
    try
    {
      adresseFacade.findLaender("F", null);
      ArrayList al = adresseFacade.findLaender("F", "");
      assertTrue("1. Test - Länder mit Code F%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("2. Test - Länder mit Code F%: ", ((LandAVO)al.get(i)).landCd.startsWith("F"));
      }
      adresseFacade.findLaender(null, "F");
      al = adresseFacade.findLaender("", "F");
      assertTrue("3. Test - Länder mit Bezeichnung F%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("4. Test - Länder mit Bezeichnung F%: ", 
            ((LandAVO)al.get(i)).landBez.startsWith("F") || ((LandAVO)al.get(i)).landBez.substring( 3 ).startsWith("F") );
      }
      al = adresseFacade.findLaender("L", "L");
      assertTrue("4. Test - Lichtenstein L% L%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("5. Test - Lichtenstein L%: ", ((LandAVO)al.get(i)).landCd.startsWith("L"));
        assertTrue("6. Test - Lichtenstein L%: ", ((LandAVO)al.get(i)).landBez.startsWith("L"));
      }
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Test der Existenz des Landes
   */
  public void testExistsLand()
  {
    getFacade();
    try
    {
      adresseFacade.existsLand(null);
      fail("1. Test - Ländercode null");
    }
    catch (Exception e)
    {
      // Test ok!
    }
    try
    {
      assertTrue("2. Test - Ländercode = CH", adresseFacade.existsLand("CH"));
      assertTrue("3. Test - Ländercode = @@", !adresseFacade.existsLand("@@"));
      assertTrue("4. Test - Ländercode = ''", !adresseFacade.existsLand(""));
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Test des BLandes
   */
  public void testFindBland()
  {
    getFacade();
    try
    {
      adresseFacade.findBland("F", null);
      ArrayList al = adresseFacade.findBland("F", "");
      assertTrue("1. Test - Länder mit Code F%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("2. Test - Länder mit Code F%: ", ((BlandAVO)al.get(i)).blandCd.startsWith("F"));
      }
      adresseFacade.findBland(null, "F");
      al = adresseFacade.findBland("", "F");
      assertTrue("3. Test - Länder mit Bezeichnung F%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("4. Test - Länder mit Bezeichnung F%: ", ((BlandAVO)al.get(i)).landBezD.startsWith("F"));
      }

      al = adresseFacade.findBland("A", "AE");
      assertTrue("4. Test - Vereinigte Arabische Emirate A% AE%: ", al.size() > 0);
      for(int i = 0; i < al.size(); i++)
      {
        assertTrue("5. Test - Vereinigte Arabische Emirate A%: ", ((BlandAVO)al.get(i)).blandCd.startsWith("A"));
        assertTrue("6. Test - Vereinigte Arabische Emirate AE%", ((BlandAVO)al.get(i)).landBezD.startsWith("AE"));
      }
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Test der Existenz des BLandes
   */
  public void testExistsBland()
  {
    getFacade();
    try
    {
      adresseFacade.existsBland(null);
      fail("1. Test - Ländercode null");
    }
    catch (Exception e)
    {
      // Test ok!
    }
    try
    {
      assertTrue("2. Test - Ländercode = FR", adresseFacade.existsBland("FR"));
      assertTrue("3. Test - Ländercode = @@", !adresseFacade.existsLand("@@"));
      assertTrue("4. Test - Ländercode = ''", !adresseFacade.existsLand(""));
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Test der Registeradresse
   */
  public void testRegadr()
  {
    getFacade();
    try
    {
      // Create neue Regadr
      RegadrVO regadrVO = new RegadrVO();
      regadrVO.setBlandCd("CH");
      regadrVO.setRegName("test Create Regadr GmbH");
      regadrVO.setRegStrasse("test Strasse");
      regadrVO.setRegOrt("3000 Bern BE");
      regadrVO.setUpRegOrt(regadrVO.getRegOrtOrEmptyString().toUpperCase());
      regadrVO.setUpVerzName("TEST CREATE");
      regadrVO.setRegadrNr(adresseFacade.createRegadr(regadrVO));

      // Selektieren der neuen Regadr
      RegadrVO regadrResultVO = adresseFacade.getRegadr(regadrVO.getRegadrNr());

      assertEquals("1. Test - regadrNr: ", regadrVO.getRegadrNr(), regadrResultVO.getRegadrNr());
      assertEquals("2. Test - blandCd: ", regadrVO.getBlandCd(), regadrResultVO.getBlandCd());
      assertEquals("3. Test - regName: ", regadrVO.getRegName(), regadrResultVO.getRegName());
      assertEquals("4. Test - regStrasse: ", regadrVO.getRegStrasse(), regadrResultVO.getRegStrasse());
      assertEquals("5. Test - regOrt: ", regadrVO.getRegOrt(), regadrResultVO.getRegOrt());
      assertEquals("6. Test - upRegOrt: ", regadrVO.getUpRegOrt(), regadrResultVO.getUpRegOrt());
      assertEquals("7. Test - upVerzName: ", regadrVO.getUpVerzName(), regadrResultVO.getUpVerzName());

      // Mutieren der Regadr
      regadrVO.setBlandCd("DE");
      regadrVO.setRegName("test Create Regadr 2");
      regadrVO.setRegStrasse("test Strasse 2");
      regadrVO.setRegOrt("3002 Bern 2");
      regadrVO.setUpRegOrt(regadrVO.getRegOrtOrEmptyString().toUpperCase());
      regadrVO.setUpVerzName("TEST CREATE 2");
      adresseFacade.setRegadr(regadrVO);

      // Selektieren der neuen Regadr
      regadrResultVO = adresseFacade.getRegadr(regadrVO.getRegadrNr());

      assertEquals("9. Test - regadrNr: ", regadrVO.getRegadrNr(), regadrResultVO.getRegadrNr());
      assertEquals("10. Test - blandCd: ", regadrVO.getBlandCd().trim(), regadrResultVO.getBlandCd().trim());
      assertEquals("11. Test - regName: ", regadrVO.getRegName(), regadrResultVO.getRegName());
      assertEquals("12. Test - regStrasse: ", regadrVO.getRegStrasse(), regadrResultVO.getRegStrasse());
      assertEquals("13. Test - regOrt: ", regadrVO.getRegOrt(), regadrResultVO.getRegOrt());
      assertEquals("14. Test - upRegOrt: ", regadrVO.getUpRegOrt(), regadrResultVO.getUpRegOrt());
      assertEquals("15. Test - upVerzName: ", regadrVO.getUpVerzName(), regadrResultVO.getUpVerzName());

      // Löschen der Regadr
      adresseFacade.removeRegadr(regadrVO.getRegadrNr());
    }
    catch (Exception e)
    {
      fail(e.toString());
    }
  }

  /**
   * Holt die Facade.
   */
  private void getFacade()
  {
    try
    {
      if (adresseFacadeHome == null)
      {
        adresseFacadeHome =  (AdresseFacadeHome)
             ServiceLocator.getInstance().getHome(AdresseFacadeHome.JNDI_NAME, AdresseFacadeHome.class);
      }
      if (adresseFacade == null)
      {
        adresseFacade = adresseFacadeHome.create();
      }
    }
    catch (Exception ex)
    {
      fail(ex.toString());
    }
  }



  /**
   * Main Methode
   * @param args
   */
  public static void main (String[] args)
  {
     junit.textui.TestRunner.run(suite());
  }
}
