package ch.ige.adr.vo.test;

import junit.framework.TestCase;
import ch.ige.adr.vo.AdresseVO;

/**
 * Test des Adressen Value Objects.
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: AdresseVOTest.java,v 1.8 2006/12/12 07:55:44 nydegger Exp $
 */

public class AdresseVOTest extends TestCase
{

  /**
   * @param s
   */
  public AdresseVOTest(String s) {
    super(s);
  }


  /**
   * testet das Truncaten bei zu langen Attributen
   */
  public void testTruncateVO() {
    try
    {
      AdresseVO vo = new AdresseVO();

      vo.setSuchbegr("0123456789");
      vo.setSprachCd("de");
      vo.setName("01234567890123456789012345678901234567890123456789");
      vo.setZeile1("01234567890123456789012345678901234567890123456789");
      vo.setZeile2("01234567890123456789012345678901234567890123456789");
      vo.setZeile3("01234567890123456789012345678901234567890123456789");
      vo.setZeile4("01234567890123456789012345678901234567890123456789");
      vo.setLandCd("CH");
      vo.setPfPlz("12345678");
      vo.setPlz("12345678");
      vo.setOrt("01234567890123456789012345678901234567890123456789");

      // alle Felder sind nicht zu lang
      assertNull(vo.truncateFields());

      // suchbegr zu lang
      vo.setSuchbegr("01234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("suchbegr") > -1);
      assertEquals(vo.getSuchbegr().length(), AdresseVO.SUCHBEGR_LENGTH);

      // sprachCd zu lang
      vo.setSprachCd("DEUTSCH");
      assertTrue(vo.truncateFields().indexOf("sprachCd") > -1);
      assertEquals(vo.getSprachCd().length(), AdresseVO.SPRACH_CD_LENGTH);

      // name zu lang
      vo.setName("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("name") > -1);
      assertEquals(vo.getName().length(), AdresseVO.NAME_LENGTH);

      // zeile1 zu lang
      vo.setZeile1("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("zeile1") > -1);
      assertEquals(vo.getZeile1().length(), AdresseVO.ZEILE_1_LENGTH);

      // zeile2 zu lang
      vo.setZeile2("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("zeile2") > -1);
      assertEquals(vo.getZeile2().length(), AdresseVO.ZEILE_2_LENGTH);

      // zeile3 zu lang
      vo.setZeile3("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("zeile3") > -1);
      assertEquals(vo.getZeile3().length(), AdresseVO.ZEILE_3_LENGTH);

      // zeile4 zu lang
      vo.setZeile4("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("zeile4") > -1);
      assertEquals(vo.getZeile4().length(), AdresseVO.ZEILE_4_LENGTH);

      // landCd zu lang
      vo.setLandCd("SCHWEIZ");
      assertTrue(vo.truncateFields().indexOf("landCd") > -1);
      assertEquals(vo.getLandCd().length(), AdresseVO.LAND_CD_LENGTH);

      // pfPlz zu lang
      vo.setPfPlz("0123456789");
      assertTrue(vo.truncateFields().indexOf("pfPlz") > -1);
      assertEquals(vo.getPfPlz().length(), AdresseVO.PF_PLZ_LENGTH);

      // plz zu lang
      vo.setPlz("0123456789");
      assertTrue(vo.truncateFields().indexOf("plz") > -1);
      assertEquals(vo.getPlz().length(), AdresseVO.PLZ_LENGTH);

      // uport zu lang
      vo.setOrt("012345678901234567890123456789012345678901234567890123456789");
      assertTrue(vo.truncateFields().indexOf("ort") > -1);
      assertEquals(vo.getOrt().length(), AdresseVO.ORT_LENGTH);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.getMessage());
    }
  }
}
