package ch.ige.art.client.art;

import java.awt.Dialog;
import java.awt.Frame;
import java.util.Observer;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.UserMessage;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.SAPController;
import ch.ige.art.util.SwissregController;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.MaVO;
import ch.ige.main.client.WizardMediator;

/**
 * <p>Überschrift: Mediator für Artikel-Wizard</p>
 * <p>Beschreibung: Der ArtWizardMediator steuert die Anzeige der einzelnen Panels
 * beim Erfass-/Bearbeitungsprozess eines Artikels
 * Die Attribute der Oberklasse alPanel, maxPanel und title werden gesetzt
 * und die Methoden back(), cancel(), forward und finish() überschrieben.
 * <p>Copyright: Copyright (c) 2002 </p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: ArtWizardMediator.java,v 1.27 2008/02/05 20:51:40 ruegsegger Exp $
 */
public class ArtWizardMediator extends WizardMediator implements Observer
{
  private static final Logger LOG = Logger.getLogger( WizardMediator.class );

  protected ArtikelGuiAVO artikelGuiAVO = new ArtikelGuiAVO();

  protected String artNr = null;

  protected PnlArtGrunddaten pnlArtGrunddaten = new PnlArtGrunddaten();

  protected PnlArtBagis1 pnlArtBagis1 = new PnlArtBagis1();

  protected PnlArtBagis2 pnlArtBagis2 = new PnlArtBagis2();

  protected PnlArtTexte pnlArtTexte = new PnlArtTexte();

  protected PnlArtPreise pnlArtPreise = new PnlArtPreise();

  protected PnlArtFolge pnlArtFolge = new PnlArtFolge();

  protected UserMessage userMessage = new UserMessage();

  protected boolean finish = false;

  /**
   * Initialisiert Panels<br>
   * @param parent Frame
   */
  public ArtWizardMediator( Frame parent )
  {
    super( parent );
    initializePanel();
  }

  /**
   * Konstruktor
   * @param parent aufrufender Dialog
   */
  public ArtWizardMediator( Dialog parent )
  {
    super( parent );
    initializePanel();
  }

  /**
   * Initialisiert Panel<br>
   */
  protected void initializePanel()
  {
  }

  /**
   * Selektiert Artikel mit der gewünschten Artikelnummer
   * füllt die Werte in ein ValueObject ab
   */
  public void read( String artNr )
  {
    try
    {
      if ( artNr != null )
      {
        artikelGuiAVO = ArtikelDelegate.getInstance().getArtikel( artNr.toString() );
      }
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Fehlermeldung", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * überschreibt protected Methode der Oberklasse</br>
   * Speichert Daten des Panels in VO und in Datenbank
   * fehler == null --> keine Fehler Artikel kann gespeicher werden
   * fehler.equals("") --> Fehler aufgetreten und angezeigt, Panel nicht wechseln
   * anderenfalls den im fehler enthaltenen String anzeigen
   *
   * @return true  - nächstes Panel anzeigen<br>
   *         false - auf aktuellem Panel bleiben, z.B. wegen Validierungsfehler
   */
  public boolean finish()
  {
    //Speichern des Artikels in DB
    try
    {
      // Überprüfung auf SAP ob Artikel vorhanden
      if ( !new SAPController().checkExistenceOnSAP( artikelGuiAVO ) )
      {
        return finish = false;
      }

      // Überprüft ob PBL oder REG Text erforderlich
      if ( !checkTextarten() )
      {
        return finish = false;
      }

      // Überprüfen ob Nachführung domain_art auf Swissreg2 erforderlich
      // Dies ist der Fall, wenn sich Texte mit Textart PBL oder REG geändert haben
      // domainMessage enthält das generierte XML welches dem CH-Domain Schema entspricht
      // oder null, falls keine Nachführung erforderlich ist
      String domainMessage = new SwissregController().checkUpdateDomain( artikelGuiAVO );
      LOG.info( "domainMessage: " + domainMessage );

      //Username
      String userName = System.getProperties().getProperty( "user.name" );
      MaVO maVO = ArtikelDelegate.getInstance().getMa( userName );
      if ( maVO != null )
      {
        artikelGuiAVO.userName = maVO.maKurzz;
      }

      if ( artikelGuiAVO.status.equals( "U" ) )
      {
        ArtikelDelegate.getInstance().setArtikel( artikelGuiAVO, domainMessage );
      }
      else if ( artikelGuiAVO.status.equals( "I" ) )
      {
        ArtikelDelegate.getInstance().createArtikel( artikelGuiAVO, domainMessage );
      }
      super.finish();
    }
    catch ( Throwable ex )
    {
      LOG.error( ex.toString() );
      String message = userMessage.getUserMessage( (Exception) ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      return finish = false;
    }

    return finish = true;
  }

  /**
   * Publikationspflichtige Artikel oder Artikel mit einer Registerauszugsgruppe
   * müssen eine PBL- oder eine REG-Textart in mindestens einer Sprache enthalten
   * (Der Inhalt des Textes darf leer sein)
   * @return boolean
   */
  private boolean checkTextarten()
  {
    if ( artikelGuiAVO.regauszGrp != 0 || artikelGuiAVO.publPflicht.equals( "1" ) )
    {
      boolean hasPBLorREGText = false;

      if ( artikelGuiAVO.artextVO != null )
      {
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
        {
          if ( artikelGuiAVO.artextVO[i].isPublicationTextToUpdate() || artikelGuiAVO.artextVO[i].isRegisterTextToUpdate() )
          {
            hasPBLorREGText = true;
          }
        }
      }

      if ( !hasPBLorREGText )
      {
        String fehler = "Publikationspflichtige Artikel oder Artikel mit einer Registerauszugsgruppe\n"
            + "müssen eine PBL- oder eine REG-Textart in mindestens einer Sprache enthalten\n "
            + "Bitte erfassen Sie die entsprechenden Textarten (Der Inhalt des Textes darf leer sein). \n";

        JOptionPane.showMessageDialog( null, fehler, "Ungültige Eingabe", JOptionPane.INFORMATION_MESSAGE );
        return false;
      }
    }
    return true;
  }
}
