package ch.ige.art.client.art;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.DtmArtikelWizard;
import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.ArtpreisVO;

/**
 * <p>Überschrift: DefaultTableModel der Preisdaten</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Preistdaten verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DtmArtPreise.java,v 1.11 2008/07/02 11:00:20 heinzer Exp $
 */

public class DtmArtPreise extends DtmArtikelWizard
{

  /**
   * xxDomainWerte werden geladen und Spaltenüberschriften werden gesetzt
   * @throws Exception falls Domainwerte nicht geladen werden können
   */
  public DtmArtPreise() throws Exception
  {
    super();
    columnIdentifiers.addElement( "Preisart" );
    columnIdentifiers.addElement( "Gültig ab" );
    columnIdentifiers.addElement( "Preis" );
    columnIdentifiers.addElement( "Pro" );
    columnIdentifiers.addElement( "Status" );
    columnIdentifiers.addElement( "Version" );
  }

  /**
   * Editierbarkeit der persistenten Daten verhindern,
   * die Bestandteil des PrimaryKeys sind
   *
   * @param row selektierte Zeile
   * @param col selektierte Spalte
   * @return true falls Zellen editierbar sind
   */
  public boolean isCellEditable( int row, int col )
  {
    if ( row < getPersistentRows() )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param col Spalte
   * @return gibt die Klasse der Column zurück
   */
  public Class getColumnClass( int col )
  {
    if ( col == 1 )
    {
      return Date.class;
    }
    else
    {
      return getValueAt( 0, col ).getClass();
    }
  }

  /**
   * Wert wird ins TableModel geschreiben
   *
   * @param aValue  Wert
   * @param row     Zeile
   * @param column  Spalte
   */
  public void setValueAt( Object aValue, int row, int column )
  {
    if ( column == 1 )
    {
      try
      {
        SimpleDateFormat dateFormat = new SimpleDateFormat( "dd.MM.yyyy" );
        dateFormat.setLenient( false );
        aValue = dateFormat.parse( (String) aValue ); // Format prüfen
      }
      catch ( ParseException ex )
      {
        return;
      }
    }
    super.setValueAt( aValue, row, column );
  }

  /**
   * Tabellendaten werden gesetzt.<br>
   * Daten werden in Form eines ValueObjects übergeben.
   * Daten des VOs werden augesplittet und in den Datenhaltungsvector(dataVector)
   * des Tabellenmodels geschrieben.
   *
   * @param data Alle Preise eines Artikels
   */
  public void setData( ArtpreisVO[] data )
  {
    dataVector.clear();
    int persitentRows = 0;
    setDeleteAllowed( false );
    if ( data != null )
    {
      for ( int i = 0; i < data.length; i++ )
      {
        ArtpreisVO artpreisVO = data[i];
        Vector artPreis = new Vector();

        artPreis.add( new ComboBoxObject( artpreisVO.artpreisartCd, XxDomain.getBez( "preisart_cd", artpreisVO.artpreisartCd ) ) );
        artPreis.add( artpreisVO.gueltigAb );
        artPreis.add( new String( artpreisVO.preis.toString() ) ); // Initialisierung ""
        artPreis.add( artpreisVO.pro == null ? new Integer( 0 ) : artpreisVO.pro );
        artPreis.add( artpreisVO.status );
        //artPreis.add(new Integer(artpreisVO.version));
        artPreis.add( artpreisVO.uVersion );

        dataVector.add( artPreis );

        // Anzahl Rows ermitteln die persistent sind
        if ( artpreisVO.status.equals( "U" ) || artpreisVO.status.equals( "" ) )
        {
          persitentRows++;
        }
      }
      setPersistentRows( persitentRows );
      fireTableDataChanged();
    }
  }

  /**
   * Gibt die Tabellendaten in Form eines Arrays von ArtpreisVOs zurück
   *
   * @return Tabellendaten gekapselt in einem Array von ArtpreisVOs
   */
  public ArtpreisVO[] getData()
  {
    if ( dataVector.size() > 0 )
    {
      ArtpreisVO[] arArtpreisVO = new ArtpreisVO[dataVector.size()];

      for ( int i = 0; i < dataVector.size(); i++ )
      {
        Vector vec = new Vector();

        arArtpreisVO[i] = new ArtpreisVO();
        vec = (Vector) dataVector.get( i );

        ComboBoxObject cbo = (ComboBoxObject) vec.get( 0 );
        arArtpreisVO[i].artpreisartCd = cbo.key == null ? "" : cbo.key;
        arArtpreisVO[i].gueltigAb = (Date) vec.get( 1 );
        arArtpreisVO[i].preis = vec.get( 2 ).equals( "" ) ? null : new BigDecimal( vec.get( 2 ).toString() );
        arArtpreisVO[i].pro = vec.get( 3 ).equals( "" ) ? null : new Integer( vec.get( 3 ).toString() );
        arArtpreisVO[i].status = (String) vec.get( 4 );
        //arArtpreisVO[i].version = ((Integer)vec.get(5)).intValue();
        arArtpreisVO[i].uVersion = (String) vec.get( 5 );
      }
      return arArtpreisVO;
    }
    return null;
  }

  /**
   * Für eine neue Zeile werden die Initialwerte pro Zelle gesetzt
   * Spalte Status wird mit I=Insert initialisiert
   *
   * @return gibt eine neue Zeile mit den initialisierten Zellen zurück
   */
  public Vector getNewRow()
  {
    Vector vec = new Vector();
    vec.setSize( 6 );
    vec.add( 0, new ComboBoxObject( "NP", XxDomain.getBez( "preisart_cd", "NP" ) ) );
    vec.add( 1, new Date() );
    vec.add( 2, "" );
    vec.add( 3, new Integer( 1 ) );
    vec.add( 4, "I" );
    //vec.add(5, new Integer(0));
    vec.add( 5, "!" );
    return vec;
  }
}
