package ch.ige.art.client.art;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.PnlArtikelWizard;
import ch.ige.art.client.util.TblArtikelWizard;
import ch.ige.art.client.util.TblNavToolBar;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.arthelp.HelpUtil;
import ch.ige.util.client.LimitedDocument;

/**
 * <p>Überschrift: Artikel-Folgeereignisse</p>
 * <p>Beschreibung: Panel mit Daten der Folgeereignisse</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: PnlArtFolge.java,v 1.21 2008/07/02 11:00:20 heinzer Exp $
 */

public class PnlArtFolge extends PnlArtikelWizard
{
  private DtmArtFolge dtmFolge;

  private ArtikelGuiAVO artikelGuiAVO;

  private TblArtikelWizard tblFolge;

  private TblNavToolBar tblNavToolBar;

  // Logging
  private static final Logger LOG = Logger.getLogger( PnlArtFolge.class );

  private UserMessage userMessage = new UserMessage();

  JPanel pnlFolge = new JPanel();

  JPanel pnlNav = new JPanel();

  JScrollPane scpFolge = new JScrollPane();

  BorderLayout borderLayout1 = new BorderLayout();

  BorderLayout borderLayout2 = new BorderLayout();

  BorderLayout borderLayout3 = new BorderLayout();

  /**
   * Konstruktor mit Parameter für Mutation eines Artikels<br>
   * holt Instanz von xxDomain um Werte aus xx_domain lesen zu können<br>
   * ruft jbInit() auf
   * @param artikelGuiAVO  ValueObject mit allen Daten eines Artikels
   */
  public PnlArtFolge()
  {
    try
    {
      jbInit();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      return;
    }
  }

  /**
   * Initialisierung
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    dtmFolge = new DtmArtFolge();
    tblFolge = new TblArtikelWizard( dtmFolge );
    tblNavToolBar = new TblNavToolBar( tblFolge );

    this.setLayout( borderLayout1 );
    this.setPreferredSize( new Dimension( 400, 400 ) );
    this.add( pnlFolge, BorderLayout.CENTER );

    pnlFolge.setLayout( borderLayout2 );
    pnlFolge.setPreferredSize( new Dimension( 400, 400 ) );
    pnlNav.setLayout( borderLayout3 );
    pnlNav.setPreferredSize( new Dimension( 237, 40 ) );

    scpFolge.setBorder( new TitledBorder( "Folgeereignis" ) );
    scpFolge.setPreferredSize( new Dimension( 200, 200 ) );

    pnlFolge.add( scpFolge, BorderLayout.CENTER );
    pnlFolge.add( pnlNav, BorderLayout.NORTH );
    pnlNav.add( tblNavToolBar, BorderLayout.EAST );
    scpFolge.getViewport().add( tblFolge, null );
    scpFolge.setOpaque( false );

    tblNavToolBar.setEnabled( false );
    tblNavToolBar.setBorderPainted( false );

    // Combobox prior_cd
    JComboBox cbxPriorCd = new JComboBox();
    List<XxDomainVO> listDomain = XxDomain.getDomains( XxDomainFactory.PRIOR_CD );
    for ( XxDomainVO xxDomainVO: listDomain )
    {
      cbxPriorCd.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
    }

    cbxPriorCd.setBackground( SystemColor.controlLtHighlight );

    // Eigenschaften der Spalten setzen
    TableColumn altartColumn = tblFolge.getColumnModel().getColumn( 0 );
    TableColumn priorcdColumn = tblFolge.getColumnModel().getColumn( 1 );

    tblFolge.removeColumn( tblFolge.getColumnModel().getColumn( 2 ) );
    tblFolge.removeColumn( tblFolge.getColumnModel().getColumn( 2 ) );

    tblFolge.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    tblFolge.setCellSelectionEnabled( true );
    tblFolge.setRowHeight( 20 );

    // Validierung für proColumn
    JTextField integerField = new JTextField();
    LimitedDocument ldoInteger = new LimitedDocument();
    integerField.setDocument( ldoInteger );
    ldoInteger.setInteger( true );
    ldoInteger.setMaxLength( 9 );

    // CellRenderer & CellEditoren setzten
    priorcdColumn.setCellEditor( new DefaultCellEditor( cbxPriorCd ) );

    altartColumn.setCellEditor( new DefaultCellEditor( integerField ) );

    // Table Listener
    tblFolge.addMouseListener( new java.awt.event.MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    tblFolge.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyReleased( KeyEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    // Buttons der Toolbar enablen und disablen
    tblNavToolBar.enableButtons();
  }

  /**
   * ArtikelGuiAVO wird mit den geänderten Werten abgefüllt<br>
   * @return ArtikelGuiAVO mit aktuellen Werten
   */
  public ArtikelGuiAVO getData()
  {
    artikelGuiAVO.altArtVO = dtmFolge.getData();
    return artikelGuiAVO;
  }

  /**
   * Füllt Controls mit den Werten des artikelGuiAVO<br>
   * Setzt den Index der Comboboxen auf die Werte des artikelGuiAVO
   * @param artikelGuiAVO ValueObject mit den Werten
   */
  public void setData( ArtikelGuiAVO artikelGuiAVO )
  {
    this.artikelGuiAVO = artikelGuiAVO;

    // TableModel auffüllen
    dtmFolge.setData( artikelGuiAVO.altArtVO );
    tblNavToolBar.enableButtons();
  }

  public String validForward()
  {
    return valid();
  }

  public String validBackward()
  {
    return valid();
  }

  /**
   * Überprüft Eingaben
   * @return  fehler == null --> keine Fehler Artikel kann gespeicher werden
   *          fehler.equals("") --> Fehler aufgetreten und angezeigt, Panel nicht wechseln
   *          anderenfalls den im fehler enthaltenen String anzeigen
   */
  public String valid()
  {
    DefaultCellEditor editor = (DefaultCellEditor) tblFolge.getCellEditor();

    // Werte im Editiermodus werden übernommen
    if ( editor != null )
    {
      if ( !editor.stopCellEditing() )
      {
        return "";
      }
    }

    Vector data = dtmFolge.getDataVector();

    for ( int i = 0; i < data.size(); i++ )
    {
      Vector row = (Vector) data.get( i );
      if ( row.get( 0 ).equals( "" ) )
        return "Folgeereignis muss erfasst werden";
    }
    return null;
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this, "artikelwizard.folgeereignis" );
    HelpUtil.getInstance().enableHelp( this.pnlFolge, "artikelwizard.folgeereignis" );
    HelpUtil.getInstance().enableHelp( this.pnlNav, "artikelwizard.folgeereignis" );
    HelpUtil.getInstance().enableHelp( this.tblFolge, "artikelwizard.folgeereignis" );
  }
}
