package ch.ige.art.client.art;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.PnlArtikelWizard;
import ch.ige.art.client.util.TblArtikelWizard;
import ch.ige.art.client.util.TblNavToolBar;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.client.util.editoren.TblCellDateEditor;
import ch.ige.art.client.util.editoren.TblCellDoubleEditor;
import ch.ige.art.client.util.renderer.TblCellDateRenderer;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.arthelp.HelpUtil;
import ch.ige.util.client.LimitedDocument;

/**
 * <p>Überschrift: Artikel-Preise</p>
 * <p>Beschreibung: Panel mit Daten der Artikelpreise</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: PnlArtPreise.java,v 1.21 2008/07/02 11:00:20 heinzer Exp $
 */

public class PnlArtPreise extends PnlArtikelWizard
{

  private DtmArtPreise dtmPreise;

  private ArtikelGuiAVO artikelGuiAVO;

  private TblArtikelWizard tblPreise;

  private TblNavToolBar tblNavToolBar;

  // Logging
  private static final Logger LOG = Logger.getLogger( PnlArtPreise.class );

  private UserMessage userMessage = new UserMessage();

  JPanel pnlPreise = new JPanel();

  JPanel pnlNav = new JPanel();

  JScrollPane scpPreise = new JScrollPane();

  BorderLayout borderLayout1 = new BorderLayout();

  BorderLayout borderLayout2 = new BorderLayout();

  BorderLayout borderLayout3 = new BorderLayout();

  JTextField txfPro = new JTextField();

  LimitedDocument ldoPro = new LimitedDocument();

  /**
   * Konstruktor mit Parameter für Mutation eines Artikels<br>
   * holt Instanz von xxDomain um Werte aus xx_domain lesen zu können<br>
   * ruft jbInit() auf
   * @param artikelGuiAVO  ValueObject mit allen Daten eines Artikels
   */
  public PnlArtPreise()
  {
    try
    {
      jbInit();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      return;
    }
  }

  /**
   * Initialisierung
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    dtmPreise = new DtmArtPreise();
    tblPreise = new TblArtikelWizard( dtmPreise );
    tblNavToolBar = new TblNavToolBar( tblPreise );

    this.setLayout( borderLayout1 );
    this.setPreferredSize( new Dimension( 400, 400 ) );
    this.add( pnlPreise, BorderLayout.CENTER );

    pnlPreise.setLayout( borderLayout2 );
    pnlPreise.setPreferredSize( new Dimension( 400, 400 ) );
    pnlNav.setLayout( borderLayout3 );
    pnlNav.setPreferredSize( new Dimension( 237, 40 ) );

    scpPreise.setBorder( new TitledBorder( "Preise" ) );
    scpPreise.setPreferredSize( new Dimension( 200, 200 ) );
    scpPreise.setOpaque( false );

    pnlPreise.add( scpPreise, BorderLayout.CENTER );
    pnlPreise.add( pnlNav, BorderLayout.NORTH );
    pnlNav.add( tblNavToolBar, BorderLayout.EAST );
    scpPreise.getViewport().add( tblPreise, null );

    tblNavToolBar.setEnabled( false );
    tblNavToolBar.setBorderPainted( false );

    // Combobox preisart_cd
    JComboBox cbxPreisartCd = new JComboBox();
//		cbxPreisartCd.setKeySelectionManager(new ComboKeySelectionManager());    
    List<XxDomainVO> listDomain = XxDomain.getDomains( XxDomainFactory.PREISART_CD );
    for ( XxDomainVO xxDomainVO: listDomain )
    {
      cbxPreisartCd.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
    }

    cbxPreisartCd.setBackground( SystemColor.controlLtHighlight );

    // Eigenschaften der Spalten setzen
    TableColumn preisartColumn = tblPreise.getColumnModel().getColumn( 0 );
    TableColumn gueltigabColumn = tblPreise.getColumnModel().getColumn( 1 );
    TableColumn preisColumn = tblPreise.getColumnModel().getColumn( 2 );
    TableColumn proColumn = tblPreise.getColumnModel().getColumn( 3 );

    tblPreise.removeColumn( tblPreise.getColumnModel().getColumn( 4 ) );
    tblPreise.removeColumn( tblPreise.getColumnModel().getColumn( 4 ) );

    tblPreise.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    tblPreise.setCellSelectionEnabled( true );
    tblPreise.setRowHeight( 20 );

    preisartColumn.setPreferredWidth( 200 );
    gueltigabColumn.setPreferredWidth( 100 );
    preisColumn.setPreferredWidth( 50 );
    proColumn.setPreferredWidth( 50 );

    // Validierung für proColumn
    txfPro.setDocument( ldoPro );
    ldoPro.setInteger( true );
    ldoPro.setMaxLength( 9 );

    // CellRenderer & CellEditoren setzten
    preisartColumn.setCellEditor( new DefaultCellEditor( cbxPreisartCd ) );
    gueltigabColumn.setCellEditor( new TblCellDateEditor( new JTextField() ) );
    gueltigabColumn.setCellRenderer( new TblCellDateRenderer() );

    preisColumn.setCellEditor( new TblCellDoubleEditor( new JTextField() ) );
    DefaultTableCellRenderer preisRenderer = new DefaultTableCellRenderer();
    preisRenderer.setHorizontalAlignment( SwingConstants.RIGHT );
    preisColumn.setCellRenderer( preisRenderer );

    proColumn.setCellEditor( new DefaultCellEditor( txfPro ) );

    // Table Listener
    tblPreise.addMouseListener( new java.awt.event.MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    tblPreise.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyReleased( KeyEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    // Buttons der Toolbar enablen und disablen
    tblNavToolBar.enableButtons();
  }

  /**
   * ArtikelGuiAVO wird mit den geänderten Werten abgefüllt<br>
   * @return ArtikelGuiAVO mit aktuellen Werten
   */
  public ArtikelGuiAVO getData()
  {
    artikelGuiAVO.artpreisVO = dtmPreise.getData();
    return artikelGuiAVO;
  }

  public String validForward()
  {
    return valid();
  }

  public String validBackward()
  {
    return valid();
  }

  /**
   * Überprüft Eingaben
   * @return  fehler == null --> keine Fehler Artikel kann gespeicher werden
   *          fehler.equals("") --> Fehler aufgetreten und angezeigt, Panel nicht wechseln
   *          anderenfalls den im fehler enthaltenen String anzeigen
   */
  public String valid()
  {
    DefaultCellEditor editor = (DefaultCellEditor) tblPreise.getCellEditor();

    // Werte im Editiermodus werden übernommen
    if ( editor != null )
    {
      if ( !editor.stopCellEditing() )
      {
        return "";
      }
    }

    Vector data = dtmPreise.getDataVector();

    for ( int i = 0; i < data.size(); i++ )
    {
      Vector row = (Vector) data.get( i );
      if ( row.get( 1 ).equals( "" ) )
        return "Gültig ab  Datum muss erfasst werden";
      if ( row.get( 2 ).equals( "" ) )
        return "Preis muss erfasst werden";
      if ( row.get( 3 ).equals( "" ) )
        return "Pro muss erfasst werden";
    }
    return null;
  }

  /**
   * Füllt Controls mit den Werten des artikelGuiAVO<br>
   * Setzt den Index der Comboboxen auf die Werte des artikelGuiAVO
   * @param artikelGuiAVO ValueObject mit den Werten
   */
  public void setData( ArtikelGuiAVO artikelGuiAVO )
  {
    this.artikelGuiAVO = artikelGuiAVO;

    // TableModel auffüllen
    dtmPreise.setData( artikelGuiAVO.artpreisVO );
    tblNavToolBar.enableButtons();
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this, "artikelwizard.preise" );
    HelpUtil.getInstance().enableHelp( this.pnlPreise, "artikelwizard.preise" );
    HelpUtil.getInstance().enableHelp( this.pnlNav, "artikelwizard.preise" );
    HelpUtil.getInstance().enableHelp( this.tblPreise, "artikelwizard.preise" );
    HelpUtil.getInstance().enableHelp( this.txfPro, "artikelwizard.preise" );
  }
}
