package ch.ige.art.client.art;

import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.vo.ArtikelAVO;
import ch.ige.statistikweb.statistikartikel.util.NumberRangeList;

/**
 * <p>Überschrift: Table Model für Artikelauswahl</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Suchergebnisse im Anzeigemodus verantwortlich</p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger
 * @version $Id: TmArtAuswahl.java,v 1.1 2008/05/22 09:02:31 ruegsegger Exp $
 */
public class TmArtAuswahl extends AbstractTableModel
{
  /**
   * represents a column of the table.
   */
  public enum Column
  {
    ARTIKEL_NR( "Artikel-Nr", Integer.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return Integer.valueOf( statisticAwareArtikelAVO.artikelAVO.artNr );
      }
    },

    BEZEICHNUNG( "Artikel-Bezeichnung" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.bezArt;
      }
    },

    GRATIS( "Gratis", Boolean.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getKzGratis();
      }
    },

    STATISTIK( "Stat", Boolean.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {

        return statisticAwareArtikelAVO.isRelevant;
      }
    },

    EREIGNIS_GRUPPE( "Ereig-Grp" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getKondartCd();
      }
    },

    SPERRUNG( "Sperrung", Boolean.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getSperrCd();
      }
    },

    LISTENTYP( "Listentyp" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getStatcd1();
      }
    },

    LOESCHREGEL( "Löschregel" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getStatcd2();
      }
    },

    UPUBL( "U-Publ" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getStatcd3();
      }
    },

    DRUCK_MODUS( "Druckmod" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getDruckModus();
      }
    },

    EINZFAKTPOS( "Einzfaktpos" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getEinzFaktpos();
      }
    },

    FORMULAR( "Formular" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getFormartCd();
      }
    },

    EINSCHREIBEN( "Einschr", Boolean.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getKzEinschr();
      }
    },

    PUBL_PFLICHT( "Publpflicht" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getPublPflicht();
      }
    },

    AUSZUGS_GRUPPE( "Auszugs-Grp", Integer.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getRegauszGrp();
      }
    },

    FRIST_ART( "Fristart" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getFristArt();
      }
    },

    FREIST_EINHEIT( "Fristeinh" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getFristEinh();
      }
    },

    FRIST_DAUER( "Fristdauer" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getFristDauer();
      }
    },

    BESCHEINIGUNG( "Besch", Boolean.class )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getKzBesch();
      }
    },

    ERFASS_PROZESS( "Erfassproz" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getErfassProc();
      }
    },

    PROZESS_MODUS( "Prozmod" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getProcessModus();
      }
    },

    PROZESS_NR( "Proznr" )
    {
      @Override
      public Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO )
      {
        return statisticAwareArtikelAVO.artikelAVO.getProcessNr();
      }
    };

    private final String columnName;

    private final Class<?> columnClass;

    /**
     * calls constructor with default column class String
     * @param columnName
     */
    private Column( final String columnName )
    {
      this( columnName, String.class );
    }

    /**
     * constructor 
     * @param columnName
     * @param columnClass
     */
    private Column( final String columnName, final Class<?> columnClass )
    {
      this.columnName = columnName;
      this.columnClass = columnClass;
    }

    /**
     * Gets the columnName.
     * @return the columnName
     */
    public String getColumnName()
    {
      return this.columnName;
    }

    /**
     * Gets the columnClass.
     * @return the columnClass
     */
    public Class<?> getColumnClass()
    {
      return this.columnClass;
    }

    /**
     * Return the value for this column for the specified
     * article.
     */
    public abstract Object getValue( StatisticAwareArtikelAVO statisticAwareArtikelAVO );
  }

  private class StatisticAwareArtikelAVO
  {
    public ArtikelAVO artikelAVO;

    public boolean isRelevant = false;

    /**
     * 
     */
    public StatisticAwareArtikelAVO( ArtikelAVO avo, boolean isRelevant )
    {
      this.artikelAVO = avo;
      this.isRelevant = isRelevant;
    }
  }

  private List<StatisticAwareArtikelAVO> tableData = new ArrayList<StatisticAwareArtikelAVO>();

  private NumberRangeList rangeOfArticlesInUse = new NumberRangeList();

  public Object getValueAt( int rowIndex, int columnIndex )
  {
    StatisticAwareArtikelAVO statisticAwareArtikelAVO = tableData.get( rowIndex );
    Column column = getColumn( columnIndex );
    return column.getValue( statisticAwareArtikelAVO );
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Ergebnisse einer Suche gekapselt in einem ArtikelAVO
   */
  public void setData( List<ArtikelAVO> articleData )
  {
    this.tableData.clear();

    if ( articleData != null )
    {
      for ( ArtikelAVO artikelAVO: articleData )
      {
        StatisticAwareArtikelAVO statisticAwareArtikelAVO = new StatisticAwareArtikelAVO( artikelAVO, rangeOfArticlesInUse.contains( new Integer(
            artikelAVO.artNr ) ) );
        tableData.add( statisticAwareArtikelAVO );
      }
    }

    fireTableDataChanged();
  }

  public void setRangeOfArticlesInStatistic( NumberRangeList rangeOfArticlesInUse )
  {
    this.rangeOfArticlesInUse = rangeOfArticlesInUse;
  }

  public void clear()
  {
    this.tableData.clear();
    fireTableDataChanged();
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.size();
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return Column.values().length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  @Override
  public String getColumnName( int columnIndex )
  {
    return getColumn( columnIndex ).getColumnName();
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param column Spalte
   * @return gibt die Klasse der Column zurück
   */
  @Override
  public Class<?> getColumnClass( int columnIndex )
  {
    return getColumn( columnIndex ).getColumnClass();
  }

  protected Column getColumn( int columnIndex )
  {
    return Column.values()[columnIndex];
  }
}
