package ch.ige.art.client.resources;

import javax.swing.AbstractButton;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

/**
 * Represents a database column with all information required by the gui to display the title.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: DBColumn.java,v 1.2 2008/07/03 13:26:42 heinzer Exp $
 */
public class DBColumn
{

  private final String SEPARATOR = ":";

  private String column;

  private String label;

  /**
   * Public constructor.
   * @param column
   * @param label
   */
  public DBColumn( String column, String label )
  {
    this.column = column;
    this.label = label;
  }

  /**
   * Gets the label of this db column. Optionally suffixed by the separator.
   * @param includeSeparator When <code>true</code> the separator is included.
   * @return The label
   */
  public String getLabel( boolean includeSeparator )
  {
    if ( includeSeparator )
    {
      return label + SEPARATOR;
    }
    else
    {
      return label;
    }
  }

  /**
   * Gets the label of this db column.
   * @return The label
   */
  public String getLabel()
  {
    return label;
  }

  /**
   * Gets the column identifier of this db column.
   * @return The column as defined in the db
   */
  public String getColumn()
  {
    return column;
  }

  /**
   * Initializes a component with the information for this column.
   * @param component The component to setup
   * @param useSeparator When <code>true</code> the separator is included for the label.
   */
  public final void setupComponent( JLabel component, boolean useSeparator )
  {
    component.setText( getLabel( useSeparator ) );
    component.setToolTipText( getColumn() );
    component.setHorizontalAlignment( SwingConstants.RIGHT ); //this could be too specific for all cases - better optional?0 
  }

  /**
   * Initializes a component with the information for this column.
   * @param component The component to setup
   */
  public final void setupComponent( AbstractButton component )
  {
    component.setText( getLabel() );
    component.setToolTipText( getColumn() );
  }

}
