package ch.ige.art.client.util;

import java.util.Vector;

import javax.swing.table.DefaultTableModel;

/**
 * <p>Überschrift: Superklass für Wizard-Tabellen</p>
 * <p>Beschreibung: Superklasse für bearbeitbare Tabellen die in einem
 * Erfassungs-/BearbeitungsWizard benötigt werden. </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DtmArtikelWizard.java,v 1.9 2008/07/02 11:00:20 heinzer Exp $
 */

public class DtmArtikelWizard extends DefaultTableModel
{

  private int persistentRows = 0; // Anzahl persistenter Rows

  private boolean deleteAllowed = true; // true wenn persistente Daten gelöscht werden dürfen

  /**
   * Bereitstellen des Zugriffs auf Domainwerte
   * @throws Exception falls Domainwerte nicht geladen werden können
   */
  public DtmArtikelWizard() throws Exception
  {
    super();
  }

  /**
   * Gibt die Anzahl persistenter Rows zurück
   * @return Anzahl der persistenter Rows
   */
  protected int getPersistentRows()
  {
    return persistentRows;
  }

  /**
   * Setzt Anzahl persistenter Rows
   * @param persistentRows Anzahl persistenter Rows
   */
  protected void setPersistentRows( int persistentRows )
  {
    this.persistentRows = persistentRows;
  }

  /**
   * Sagt aus, ob persistenten Daten gelöscht werden dürfen
   * @return true wenn persistente Daten gelöscht werden dürfen
   */
  public boolean getDeleteAllowed()
  {
    return deleteAllowed;
  }

  /**
   * Definieren ob persistente Daten gelöscht werden dürfen
   * @param deleteAllowed true wenn persistente Daten gelöscht werden dürfen
   */
  public void setDeleteAllowed( boolean deleteAllowed )
  {
    this.deleteAllowed = deleteAllowed;
  }

  /**
   * Diese Methode sollte in der abgeleiteten Klasse überschreiben werden.<br>
   * In der abgeleiteten Klasse einen Vector definieren,
   * der die Initialwerte der Zellen einer Zelle beinhaltet.
   * Für jede Zelle muss ein Initialwert definiert werden
   *
   * @return gibt eine neue Zeile mit den initialisierten Zellen zurück
   */
  protected Vector getNewRow()
  {
    return null;
  }

  /**
   * Diese Methode in der abgeleiteten Klasse überschreiben, sofern die
   * persistenten Daten die aus dem TableModel gelöscht wurden auch auf der
   * Datenbank gelöscht werden sollen.
   * Die Daten müssen zwischengespeichert werden.
   *
   * @param iRow Index der Zeile die gelöscht werden soll
   */
  protected void saveDeletedRow( int iRow )
  {
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param col Spalte
   * @return gibt die Klasse der Column zurück
   */
  public Class getColumnClass( int col )
  {
    return getValueAt( 0, col ).getClass();
  }
}
