package ch.ige.art.client.util.editoren;

import java.awt.Component;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.*;

import ch.ige.art.client.util.verifier.DateVerifier;


/**
 * <p>Überschrift: Editor für Datumeingabe</p>
 * <p>Beschreibung: Editor für Datumeingabe innerhalb einer Tabelle, Validierung auf Datum-Wert</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger / Stefan Nydegger
 * @version $Id: TblCellDateEditor.java,v 1.5 2006/12/13 10:08:31 ruegsegger Exp $
 */
public class TblCellDateEditor extends DefaultCellEditor
{

  private JTextField textField;
  private SimpleDateFormat sdfLong = new SimpleDateFormat("dd.MM.yyyy");


  public TblCellDateEditor(JTextField tf)
  {
    super(tf);
    textField = tf;
    sdfLong.setLenient(false);
  }

  /**
   * Überschreibt Methode im DefaultCellEditor<br>
   * Setzt Darstellungsformat dd.MM.yyyy im Editiermodus
   *
   * @param table       JTable
   * @param value       Object
   * @param isSelected  boolean, ob Zelle selektiert ist
   * @param row         Reihe
   * @param column      Spalte
   * @return            JTextfield
   */
  public Component getTableCellEditorComponent ( JTable table,
      Object value, boolean isSelected, int row, int column ) {

    try
    {
      textField.setText(sdfLong.format(value));
    }
    catch (Exception ex)
    {
      textField.setText(value.toString());
    }
    return textField;
  }

  /**
   * Beendet Editiermodus der Zelle
   * @return boolean true  - Zelle beinhaltet gültiges Datum
   *                 false - Zelle beinhaltet ungültiges Datum
   */
  public boolean stopCellEditing()
  {
    DateVerifier verifier = new DateVerifier();

    if (textField.getText().length()==0) //Tagesdatum setzen falls Datum gelöscht
    {
      textField.setText(sdfLong.format(new Date()));
    }
    else if(!verifier.verify(editorComponent))
    {
      String message  = "Datum ist ungültig oder entspricht nicht dem erlaubten Eingabeformat" +
                        "\nerlaubte Eingabeformate:  DDMMYY,  DD.MM.YY,  DD.MM.YYYY";
      String title    = "Ungültiges Datum";
      JOptionPane.showMessageDialog
      (null, message, title, JOptionPane.INFORMATION_MESSAGE);

      return false;
    }

    return super.stopCellEditing();
  }
}

