package ch.ige.art.client.util.verifier;

import java.util.ArrayList;

import ch.ige.art.util.XxDomain;

/**
 * <p>Überschrift: Textüberprüfung</p>
 * <p>Beschreibung: Überprüft den Inhalt eines Textes auf bestimmte Characters </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: TextVerifier.java,v 1.13 2008/07/02 11:00:20 heinzer Exp $
 */

public class TextVerifier
{

  private String message = null; // Fehlermeldung

  public TextVerifier()
  {
  }

  /**
   * Überprüft den übergebenen Text auf ungültige Platzhalter.<br>
   * Die gültigen Platzhalter sind auf xx_domain erfasst
   * @param text Text der auf ungültige Platzhalter überprüft werden soll
   * @return Position eines ungültigen Platzhalters, -1 Ende des Textes
   * @throws Exception falls Probleme beim Laden der Domainwerte
   */
  public int verifyPlatzhalter( String text ) throws Exception
  {
    int pos = 0;
    boolean verify = false;
    ArrayList alPlatzhalter = XxDomain.getDomainKbez( "platzhalter" );

    // Dokument durchsuchen nach platzhalter
    for ( int j = 0; pos != -1; j++ )
    {
      pos = text.indexOf( "%%$$", j );

      if ( pos != -1 )
      {
        for ( int i = 0; i < alPlatzhalter.size(); i++ ) // überprüfen ob gültger platzhalter
        {
          String s = (String) alPlatzhalter.get( i ); //gültiger platzhalter
          String p = "";

          try
          {
            p = text.substring( pos, pos + s.length() );
            if ( p.equals( s ) )
            {
              verify = true;
              break;
            }
          }
          catch ( java.lang.IndexOutOfBoundsException ex )
          {
            ;
          }
        }
        if ( verify )
        {
          verify = false;
        }
        else
        {
          return pos;
        }
      }
      j = pos;
    }
    return pos;
  }

  /**
   * Überprüft den übergebenen Text auf Escape-Sequenzen.
   * @param text Text der auf Escape-Sequenzen überprüft werden soll
   * @return Position >-1 Position der Escape-Sequenze / fehlerhaften position
   *                  -1 Ende des Textes , keine Fehler gefunden
   */
  public int verifyEscapeSequenzen( String text )
  {
    StringBuffer sb = new StringBuffer( text );
    for ( int i = 0; i < sb.length(); i++ )
    {
      Character c = new Character( sb.charAt( i ) );

      if ( c.equals( new Character( '\n' ) ) || c.equals( new Character( '\r' ) ) || c.equals( new Character( '\b' ) ) || c.equals( new Character( '\f' ) )
          || c.equals( new Character( '\t' ) ) )
      {
        return i;
      }
    }
    return -1; // end of document
  }

  /**
   * Überprüft den übergebenen Text auf die Vollständigkeit der Kombination <b>..<\b>.
   * @param text Text der überprüft werden soll
   * @return Position -1  Ende des Textes, keine Fehler gefunden
   *                  >-1 Position der fehlerhaften Kombination
   */
  public int verifyTextMarks( String text )
  {
    int pos = -1; //return-value
    int posStart = -1;
    int posEnd = -1;
    message = null;

    // Text auf Vollständigkeit der Kombination <b>..<\b> überprüfen.
    // Überprüfung durchführen bis Fehler aufgetreten oder Ende des Textes erreicht
    do
    {
      posEnd++;
      posStart++;

      posStart = text.indexOf( "<b>", posStart ); // Begin-Tag <b> suchen

      if ( posStart == -1 || posStart >= posEnd ) // Begin-Tag(wenn vorh.) muss grösser als letztes End-Tag sein
      {
        posEnd = text.indexOf( "</b>", posEnd );

        if ( posStart == -1 && posEnd > -1 )
        {
          message = "Fehlendes Begin-Tag";
          pos = posEnd;
        }
        else if ( posStart > -1 && posEnd == -1 )
        {
          message = "Fehlendes End-Tag";
          pos = posStart;
        }
        else if ( posEnd < posStart )
        {
          message = "Begin-Tag und End-Tag vertauscht";
          pos = posEnd;
        }
      }
      else
      //Fehler <b><b></b>
      {
        message = "Aufeinanderfolgende Begin-Tags sind nicht erlaubt";
        pos = posStart;
      }
    } while ( posStart != -1 && posEnd != -1 && message == null );

    if ( message != null )
    {
      message = message + "\n\nUm einen Text Fett zu markieren muss sich dieser " + "\nzwischen dem Begin-Tag <b> und dem End-Tag </b> befinden"
          + "\nBeispiel: <b>Text</b>";
    }

    return pos;
  }

  /**
   * Gibt die Fehlermeldung zurück
   * @return Fehlermeldung
   */
  public String getMessage()
  {
    return message;
  }
}
