package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ch.ige.art.vo.BArtikelVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für bartikel
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BArtikelDAO.java,v 1.13 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class BArtikelDAO
{
  /**
   * Selektiert einen Datensatz aus der Tabelle bartikel
   *
   * @param  artNr         ArtikelNr
   * @return BArtikelVO    Value Object
   * @throws SQLException  Unerwarteter SQL Fehler
   */
  public BArtikelVO select( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    BArtikelVO vo = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "anz_kopie, " + "trim(druck_modus) as druck_modus, " + "trim(einz_faktpos) as einz_faktpos, "
          + "trim(formart_cd) as formart_cd, " + "trim(kz_einschr) as kz_einschr, " + "trim(kz_untschr) as kz_untschr, "
          + "trim(publ_pflicht) as publ_pflicht, " + "regausz_grp, " + "trim(frist_art) as frist_art, " + "trim(frist_einh) as frist_einh, " + "frist_dauer, "
          + "trim(kz_besch) as kz_besch, " + "ereig_plaus, " + "erfass_proc, " + "trim(process_modus) as process_modus, " + "process_nr, "
          + "trim(erst_ben) as erst_ben, " + "erst_dat, " + "trim(geaend_ben) as geaend_ben, " + "geaend_dat, " + "u_version "
          + "FROM bartikel WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        vo = new BArtikelVO();
        vo.artNr = rs.getString( "art_nr" );

        vo.anzKopie = new Integer( rs.getInt( "anz_kopie" ) );
        if ( rs.wasNull() )
        {
          vo.anzKopie = null;
        }

        vo.druckModus = rs.getString( "druck_modus" );
        vo.einzFaktpos = rs.getString( "einz_faktpos" );
        vo.formartCd = rs.getString( "formart_cd" );
        vo.kzEinschr = rs.getString( "kz_einschr" );
        vo.kzUntschr = rs.getString( "kz_untschr" );
        vo.publPflicht = rs.getString( "publ_pflicht" );
        vo.regauszGrp = rs.getInt( "regausz_grp" );
        vo.fristArt = rs.getString( "frist_art" );
        vo.fristEinh = rs.getString( "frist_einh" );

        vo.fristDauer = new Integer( rs.getInt( "frist_dauer" ) );
        if ( rs.wasNull() )
        {
          vo.fristDauer = null;
        }

        vo.kzBesch = rs.getString( "kz_besch" );

        vo.ereigPlaus = new Integer( rs.getInt( "ereig_plaus" ) );
        if ( rs.wasNull() )
        {
          vo.ereigPlaus = null;
        }

        vo.erfassProc = new Integer( rs.getInt( "erfass_proc" ) );
        if ( rs.wasNull() )
        {
          vo.erfassProc = null;
        }

        vo.processModus = rs.getString( "process_modus" );

        vo.processNr = new Integer( rs.getInt( "process_nr" ) );
        if ( rs.wasNull() )
        {
          vo.processNr = null;
        }

        vo.erstDat = rs.getDate( "erst_dat" );
        vo.erstBen = rs.getString( "erst_ben" );
        vo.geaendDat = rs.getDate( "geaend_dat" );
        vo.geaendBen = rs.getString( "geaend_ben" );
        vo.uVersion = rs.getString( "u_version" );
      }
      return vo;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BArtikelDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle bartikel ein
   *
   * @param     bArtikelVO       BArtikelVO Value Object
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( BArtikelVO bArtikelVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO bartikel(" + "art_nr, " + "anz_kopie, " + "druck_modus, " + "einz_faktpos, " + "formart_cd, " + "kz_einschr, " + "kz_untschr, "
          + "publ_pflicht, " + "regausz_grp, " + "frist_art, " + "frist_einh, " + "frist_dauer, " + "kz_besch, " + "ereig_plaus, " + "erfass_proc, "
          + "process_modus, " + "process_nr, " + "erst_dat, " + "erst_ben, " + "geaend_dat, " + "geaend_ben, " + "u_version ) "
          + "VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?, ?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, bArtikelVO.artNr );
      if ( bArtikelVO.anzKopie == null )
      {
        ps.setNull( 2, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 2, bArtikelVO.anzKopie.intValue() );
      }
      ps.setString( 3, bArtikelVO.druckModus );
      ps.setString( 4, bArtikelVO.einzFaktpos );
      ps.setString( 5, bArtikelVO.formartCd );
      ps.setString( 6, bArtikelVO.kzEinschr );
      ps.setString( 7, bArtikelVO.kzUntschr );
      ps.setString( 8, bArtikelVO.publPflicht );
      ps.setInt( 9, bArtikelVO.regauszGrp );
      ps.setString( 10, bArtikelVO.fristArt );
      ps.setString( 11, bArtikelVO.fristEinh );
      if ( bArtikelVO.fristDauer == null )
      {
        ps.setNull( 12, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 12, bArtikelVO.fristDauer.intValue() );
      }
      ps.setString( 13, bArtikelVO.kzBesch );
      if ( bArtikelVO.ereigPlaus == null )
      {
        ps.setNull( 14, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 14, bArtikelVO.ereigPlaus.intValue() );
      }
      if ( bArtikelVO.erfassProc == null )
      {
        ps.setNull( 15, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 15, bArtikelVO.erfassProc.intValue() );
      }
      ps.setString( 16, bArtikelVO.processModus );
      if ( bArtikelVO.processNr == null )
      {
        ps.setNull( 17, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 17, bArtikelVO.processNr.intValue() );
      }
      if ( bArtikelVO.erstDat == null )
      {
        ps.setNull( 18, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 18, new java.sql.Date( bArtikelVO.erstDat.getTime() ) );
      }
      ps.setString( 19, bArtikelVO.erstBen );
      if ( bArtikelVO.geaendDat == null )
      {
        ps.setNull( 20, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 20, new java.sql.Date( bArtikelVO.geaendDat.getTime() ) );
      }
      ps.setString( 21, bArtikelVO.geaendBen );
      ps.setString( 22, bArtikelVO.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BArtikelDAO.class, query + bArtikelVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle bartikel
   *
   * @param     bArtikelVO    BArtikelVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( BArtikelVO bArtikelVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE bartikel SET " + "anz_kopie = ?, " + "druck_modus = ?, " + "einz_faktpos = ?, " + "formart_cd = ?, " + "kz_einschr = ?, "
          + "kz_untschr = ?, " + "publ_pflicht = ?, " + "regausz_grp = ?, " + "frist_art = ?, " + "frist_einh = ?, " + "frist_dauer = ?, " + "kz_besch = ?, "
          + "erfass_proc = ?, " + "process_modus = ?, " + "process_nr = ?, " + "geaend_dat = ?, " + "geaend_ben = ?, " + "u_version = ? "
          + "WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      if ( bArtikelVO.anzKopie == null )
      {
        ps.setNull( 1, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 1, bArtikelVO.anzKopie.intValue() );
      }
      ps.setString( 2, bArtikelVO.druckModus );
      ps.setString( 3, bArtikelVO.einzFaktpos );
      ps.setString( 4, bArtikelVO.formartCd );
      ps.setString( 5, bArtikelVO.kzEinschr );
      ps.setString( 6, bArtikelVO.kzUntschr );
      ps.setString( 7, bArtikelVO.publPflicht );
      ps.setInt( 8, bArtikelVO.regauszGrp );
      ps.setString( 9, bArtikelVO.fristArt );
      ps.setString( 10, bArtikelVO.fristEinh );
      if ( bArtikelVO.fristDauer == null )
      {
        ps.setNull( 11, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 11, bArtikelVO.fristDauer.intValue() );
      }
      ps.setString( 12, bArtikelVO.kzBesch );
      if ( bArtikelVO.erfassProc == null )
      {
        ps.setNull( 13, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 13, bArtikelVO.erfassProc.intValue() );
      }
      ps.setString( 14, bArtikelVO.processModus );
      if ( bArtikelVO.processNr == null )
      {
        ps.setNull( 15, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 15, bArtikelVO.processNr.intValue() );
      }
      if ( bArtikelVO.geaendDat == null )
      {
        ps.setNull( 16, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 16, new java.sql.Date( bArtikelVO.geaendDat.getTime() ) );
      }
      ps.setString( 17, bArtikelVO.geaendBen );
      ps.setString( 18, bArtikelVO.uVersion );
      ps.setString( 19, bArtikelVO.artNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BArtikelDAO.class, query + bArtikelVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle bartikel
   *
   * @param     artNr          ArtikelNr      Primary Key
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "DELETE FROM bartikel WHERE art_nr = RPAD(?, 18,' ') ";
      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BArtikelDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Artikel-Nummern die zu einem bestimmten Formular gehören
   * @param formartCd String Formular
   * @throws SQLException
   * @return List ArrayList von Artikel-Nummern
   */
  public List<String> selectListArtNr( String formartCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    List<String> listArtNr = new ArrayList<String>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr " + "FROM bartikel WHERE formart_cd = RPAD(?, 3,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );

      rs = ps.executeQuery();
      while ( rs.next() )
      {
        listArtNr.add( rs.getString( "art_nr" ) );
      }

      return listArtNr;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BArtikelDAO.class, query + formartCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

}
