package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import ch.ige.art.vo.BformtxtDruckVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für bformtxtDruck
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformtxtDruckDAO.java,v 1.11 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class BformtxtDruckDAO
{
  /**
   * Selektiert einen Datensatz aus der Tabelle bformtxt_druck
   * @param formartCd     Formart Code
   * @param textart       Textart
   * @param sprachCd      Sprachcode
   * @return              bformtxtDruckVO  Value Object
   * @throws SQLException Unerwarteter SQL Fehler
   */
  public BformtxtDruckVO select( String formartCd, String textart, String sprachCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    BformtxtDruckVO bformtxtDruckVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(formart_cd) as formart_cd, trim(textart) as textart, " + "trim(sprach_cd) as sprach_cd, trim(text) as text, version "
          + "FROM bformtxt_druck " + "WHERE formart_cd = RPAD(?, 3, ' ') " + "AND textart = ? " + "AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );
      ps.setString( 2, textart );
      ps.setString( 3, sprachCd );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        bformtxtDruckVO = new BformtxtDruckVO();
        bformtxtDruckVO.formartCd = rs.getString( "formart_cd" );
        bformtxtDruckVO.textart = rs.getString( "textart" );
        bformtxtDruckVO.sprachCd = rs.getString( "sprach_cd" );
        bformtxtDruckVO.text = rs.getString( "text" );
        bformtxtDruckVO.version = rs.getInt( "version" );
      }
      return bformtxtDruckVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + "formartCd: " + formartCd + "textart: " + textart + "sprachCd: " + sprachCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle bformtxt_druck für das Schreiben.
   * @param formartCd         Primary Key
   * @param sprachCd          Sprachcode
   * @return returnArrLi      ArrayList mit den ValueObjects
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public ArrayList<BformtxtDruckVO> select( String formartCd, String sprachCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    BformtxtDruckVO vo = new BformtxtDruckVO();
    ArrayList<BformtxtDruckVO> returnArrLi = new ArrayList<BformtxtDruckVO>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(formart_cd) as formart_cd, trim(textart) as textart, " + "trim(sprach_cd) as sprach_cd, text, version " + "FROM bformtxt_druck "
          + "WHERE formart_cd = RPAD(?, 3, ' ') " + "AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );
      ps.setString( 2, sprachCd );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        vo = new BformtxtDruckVO();
        vo.formartCd = rs.getString( "formart_cd" );
        vo.sprachCd = rs.getString( "sprach_cd" );
        vo.textart = rs.getString( "textart" );
        vo.text = rs.getString( "text" );
        vo.version = rs.getInt( "version" );
        returnArrLi.add( vo );
      }
      return returnArrLi;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + "formartCd: " + formartCd + "sprachCd: " + sprachCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Datensätze eines Schreibens aus der Tabelle bformtxt_druck.
   * @param formartCd         Primary Key
   * @return returnArrLi      ArrayList mit den ValueObjects
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public ArrayList<BformtxtDruckVO> select( String formartCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    BformtxtDruckVO bformtxtDruckVO = new BformtxtDruckVO();
    ArrayList<BformtxtDruckVO> returnArrLi = new ArrayList<BformtxtDruckVO>();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(formart_cd) as formart_cd, trim(textart) as textart, " + "trim(sprach_cd) as sprach_cd, text, version " + "FROM bformtxt_druck "
          + "WHERE formart_cd = RPAD(?, 3, ' ') " + "ORDER BY textart, sprach_cd ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        bformtxtDruckVO = new BformtxtDruckVO();
        bformtxtDruckVO.formartCd = rs.getString( "formart_cd" );
        bformtxtDruckVO.sprachCd = rs.getString( "sprach_cd" );
        bformtxtDruckVO.textart = rs.getString( "textart" );
        bformtxtDruckVO.text = rs.getString( "text" );
        bformtxtDruckVO.version = rs.getInt( "version" );

        returnArrLi.add( bformtxtDruckVO );
      }
      return returnArrLi;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + formartCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle bformtxt_druck ein
   *
   * @param     bformtxtDruckVO  ValueObject BformtxtDruckVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( BformtxtDruckVO bformtxtDruckVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO bformtxt_druck(" + "formart_cd, " + "textart, " + "sprach_cd, " + "text) " + "VALUES( ?,?,?,?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, bformtxtDruckVO.formartCd );
      ps.setString( 2, bformtxtDruckVO.textart );
      ps.setString( 3, bformtxtDruckVO.sprachCd );
      ps.setString( 4, bformtxtDruckVO.text );
      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + bformtxtDruckVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle bformtxt_druck
   *
   * @param     bformtxtDruckVO  BformtxtDruckVO ValueObject
   * @return    int              Anzahl aktualisierter Datensätze
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public int update( BformtxtDruckVO bformtxtDruckVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE bformtxt_druck SET " + "text = ?, " + "version = ? "
          + "WHERE formart_cd = RPAD(?, 3, ' ') AND textart = ? AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, bformtxtDruckVO.text );
      ps.setInt( 2, bformtxtDruckVO.version );
      ps.setString( 3, bformtxtDruckVO.formartCd );
      ps.setString( 4, bformtxtDruckVO.textart );
      ps.setString( 5, bformtxtDruckVO.sprachCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + bformtxtDruckVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle bformtxt_druck
   *
   * @param formartCd        Primary Key
   * @param textart          Primary Key
   * @param sprachCd         Primary Key
   * @return  int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */

  public int delete( String formartCd, String textart, String sprachCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "DELETE FROM bformtxt_druck " + "WHERE formart_cd = RPAD(?, 3, ' ') AND textart = ? AND sprach_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );
      ps.setString( 2, textart );
      ps.setString( 3, sprachCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformtxtDruckDAO.class, query + "formartCd: " + formartCd + "textart: " + textart + "sprachCd: " + sprachCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
