package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.XxDomainSelKritVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle xx_domain; es werden nur die nötigen Methoden
 * implementiert.
 * <p>
 * Copyright (c) 2002 - 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger, Marc Bouquet
 * @version $Id: XxDomainDAO.java,v 1.10 2008/06/25 13:44:29 ruegsegger Exp $
 */
public class XxDomainDAO
{
  /**
   * Selektiere alle Domains, welche die Suchkriterien erfüllen
   *
   * @param  xxDomainSelKritVO  Value Object mit Selektionskriterien
   * @return XxDomainVO[]       Value Object mit gefundenen Domains
   * @throws SQLException       Unerwarteter SQL-Fehler
   */
  public ArrayList<XxDomainVO> select( XxDomainSelKritVO xxDomainSelKritVO ) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    ArrayList<XxDomainVO> al = new ArrayList<XxDomainVO>();
    String query = null;

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer where = new StringBuffer();
    StringBuffer order = new StringBuffer();
    String and = " AND ";

    select.append( "SELECT domain, domain_lnr, sprache, kbez, bez, order_nr, version " );
    from.append( "FROM xx_domain " );
    where.append( "WHERE domain IN (" );
    order.append( " ORDER BY domain, order_nr, sprache, domain_lnr" );

    for ( int i = 0; i < xxDomainSelKritVO.domain.size(); i++ )
    {
      where.append( "'" + xxDomainSelKritVO.domain.get( i ).toString() + "'," );
    }
    where.deleteCharAt( where.length() - 1 );
    where.append( ")" );
    where.append( and );
    where.append( "sprache = " );
    where.append( xxDomainSelKritVO.sprache );

    query = select.toString() + from.toString() + where.toString() + order.toString();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      stmt = con.createStatement();
      rs = stmt.executeQuery( query );

      while ( rs.next() )
      {
        XxDomainVO xxDomainVO = XxDomainFactory.create( rs.getString( "domain" ) );

        xxDomainVO.domain = rs.getString( "domain" );
        xxDomainVO.domainLnr = rs.getInt( "domain_lnr" );
        xxDomainVO.sprache = rs.getInt( "sprache" );
        xxDomainVO.setKbez( rs.getString( "kbez" ) );
        xxDomainVO.setBez( rs.getString( "bez" ) );
        xxDomainVO.orderNr = rs.getInt( "order_nr" );
        xxDomainVO.version = rs.getInt( "version" );
        al.add( xxDomainVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, XxDomainDAO.class, query );
      throw ex;
    }
    finally
    {
      DBHelper.close( stmt, con );
    }
  }

  /**
   * select all domains in german language
   * @return list of xxDomainVOs
   * @throws SQLException
   */
  public ArrayList<XxDomainVO> select() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<XxDomainVO> al = new ArrayList<XxDomainVO>();

    try
    {
      query = "SELECT domain, domain_lnr, sprache, kbez, bez, order_nr, version " + "FROM xx_domain WHERE sprache = 1 "
          + "ORDER BY domain, order_nr, domain_lnr";

      con = DBHelper.getConnection( DBHelper.BAGIS );
      ps = con.prepareStatement( query );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        XxDomainVO xxDomainVO = XxDomainFactory.create( rs.getString( "domain" ) );

        xxDomainVO.domain = rs.getString( "domain" );
        xxDomainVO.domainLnr = rs.getInt( "domain_lnr" );
        xxDomainVO.sprache = rs.getInt( "sprache" );
        xxDomainVO.setKbez( rs.getString( "kbez" ) );
        xxDomainVO.setBez( rs.getString( "bez" ) );
        xxDomainVO.orderNr = rs.getInt( "order_nr" );
        xxDomainVO.version = rs.getInt( "version" );
        al.add( xxDomainVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, XxDomainDAO.class, query );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * select all values of one specific domain in german 
   * @param domain gewünschte Domain
   * @return ArrayList mit XXDomainVO's
   * @throws SQLException
   */
  public ArrayList<XxDomainVO> select( String domain ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<XxDomainVO> al = new ArrayList<XxDomainVO>();

    try
    {
      query = "SELECT domain, domain_lnr, sprache, kbez, bez, order_nr, version " + "FROM xx_domain WHERE domain = ? and sprache = 1 "
          + "ORDER BY domain, order_nr, domain_lnr";

      con = DBHelper.getConnection( DBHelper.BAGIS );

      ps = con.prepareStatement( query );
      ps.setString( 1, domain );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        XxDomainVO xxDomainVO = new XxDomainVO();

        xxDomainVO.domain = rs.getString( "domain" );
        xxDomainVO.domainLnr = rs.getInt( "domain_lnr" );
        xxDomainVO.sprache = rs.getInt( "sprache" );
        xxDomainVO.setKbez( rs.getString( "kbez" ) );
        xxDomainVO.setBez( rs.getString( "bez" ) );
        xxDomainVO.orderNr = rs.getInt( "order_nr" );
        xxDomainVO.version = rs.getInt( "version" );
        al.add( xxDomainVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, XxDomainDAO.class, query );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
