package ch.ige.art.util;

import ch.ige.util.jms.JMSSender;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.naming.NamingException;

import ch.ige.util.jms.IGEJMSFactory;
import ch.ige.util.jms.JMSAccessor;
import ch.ige.util.jms.JMSFactory;
import ch.ige.util.jms.JMSSender;
import ch.ige.util.jms.JMSMessageConverter;
import javax.jms.Message;
import javax.jms.TextMessage;

/**
 * Sends TextMessage to Queue QUEUE_DOMAIN_INFORMATION
 * origin property ist set om Textmessage for
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * @author A. Rueegsegger
 * @version $Id: DomainJMSSender.java,v 1.4 2007/04/04 06:25:36 locher Exp $
 */
public class DomainJMSSender extends JMSSender
{
  private static final String APPLICATION_NAME = "artikelgui";
  private static final String ORIGIN_PROPERTY_NAME = "ORIGIN";
  public final static String ORIGIN_ART = "ART";

  /**
   * submits the message in form of a TextMessage to Queue QUEUE_DOMAIN_INFORMATION
   * property ORIGIN is set on the message
   * @param message String
   * @throws NamingException
   * @throws JMSException
   */
  public final static void submit( String message ) throws NamingException, JMSException
  {
    DomainJMSSender delivery = null;
    try
    {
      delivery = DomainJMSSender.createInstance( APPLICATION_NAME );
      delivery.processMessage(message);
    }
    finally
    {
      if ( delivery != null )
      {
        delivery.disposeWithAccessor();
      }
    }
  }

  public final static DomainJMSSender createInstance( String jmsClientID ) throws NamingException
  {
    JMSFactory jmsFactory = IGEJMSFactory.getInstance();
    DomainJMSSender sender = new DomainJMSSender( jmsFactory.getAccessor(
        jmsClientID ) );
    sender.setDestinationQueue( jmsFactory.getQueue( IGEJMSFactory.QUEUE_DOMAIN_INFORMATION ) );
    return sender;
  }

  private Queue queue;

  private DomainJMSSender( JMSAccessor accessor )
  {
    super( accessor, JMSMessageConverter.DEFAULT );
  }

  private void setDestinationQueue( Queue queue )
  {
    this.queue = queue;
  }

  private void processMessage( String content ) throws JMSException
  {
    TextMessage message = this.getSession().createTextMessage( content );
    message.setStringProperty( ORIGIN_PROPERTY_NAME, ORIGIN_ART );

    this.submitToJMSQueue( message );
  }


  private void submitToJMSQueue( Message message ) throws JMSException
  {
    this.send( this.queue, message );
  }

}
