package ch.ige.art.xml;

import java.io.File;
import java.io.InputStream;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.xml.jaxb.listen.Spalte;
import ch.ige.art.xml.jaxb.listen.Spalten;
import ch.ige.art.xml.jaxb.listen.Zeile;
import ch.ige.art.xml.jaxb.listen.Zeilen;
import ch.ige.art.xml.jaxb.listen.Zelle;
import ch.ige.art.xml.jaxb.listen.impl.SpalteImpl;
import ch.ige.art.xml.jaxb.listen.impl.SpaltenImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZeileImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZeilenImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZelleImpl;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-File zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: ArtExportListe.java,v 1.6 2008/02/26 16:59:15 ruegsegger Exp $
 */

public class ArtExportListe extends Liste
{

  public ArtExportListe() throws Exception
  {
  }

  /**
   * Druckt Trefferliste
   * @param listHits List zu druckende Daten
   * @param mapKriterium Map zu druckende Kriterien
   * @throws Exception
   */
  public void save( List listHits, Map mapKriterium, File csvFile ) throws Exception
  {
    List listArtikel = read( listHits );
    String xmlString = super.getXMLString( listArtikel, mapKriterium );

    this.transform( xmlString, csvFile );
  }

  public void transform( String xml, File csvFile ) throws Exception
  {
    TransformerFactory tFactory = TransformerFactory.newInstance();

    InputStream xsl = ArtExportListe.class.getResourceAsStream( "/res/artexportliste.xsl" );

    Source xslSource = new StreamSource( xsl );
    Source xmlSource = new StreamSource( new StringReader( xml ) );
    Result xmlResult = new StreamResult( csvFile.toURI().getPath() );

    Transformer trans = tFactory.newTransformer( xslSource );

    trans.setOutputProperty( OutputKeys.METHOD, "text" );
    trans.setOutputProperty( OutputKeys.OMIT_XML_DECLARATION, "yes" );
    trans.setOutputProperty( OutputKeys.ENCODING, "windows-1252" );

    trans.transform( xmlSource, xmlResult );
  }

  /**
   * Liest die Daten von der Datenbank
   */
  public List read( List listHits ) throws Exception
  {
    ArtikelGuiAVO artikelGuiAVO;
    List listArtikel = new ArrayList();

    for ( int i = 0; i < listHits.size(); i++ )
    {
      ArtikelAVO artikel = (ArtikelAVO) listHits.get( i );

      artikelGuiAVO = ArtikelDelegate.getInstance().getArtikel( artikel.artNr );
      listArtikel.add( artikelGuiAVO );
    }

    return listArtikel;
  }

  /**
   * Gibt den Titel der Liste zurück
   * @return String Titel der Liste
   */
  private String getTitel()
  {
    return "Export Trefferliste Artikel vom " + currentDate;
  }

  /**
   * Gibt eine Map mit den Kriterien und ihrer Bezeichnung zurück
   * @param selKritVO ArtikelSelKritVO
   * @return Map mit Kriterien und Bezeichnungen
   */
  public Map getKriteriumList( ArtikelSelKritVO selKritVO )
  {
    Map map = new LinkedHashMap();

    map.put( "Artikel-Nr", selKritVO.artNr );
    map.put( "Artikel-Bezeichnung", selKritVO.bezArt );
    map.put( "Formular", selKritVO.formartCd );
    map.put( "Übergruppe Ereignis", getLegend( selKritVO.kondartCd, XxDomainFactory.KONDART_CD ) );
    map.put( "Erfassungsprozess", getLegend( selKritVO.erfassProc, XxDomainFactory.ERFASS_PROC ) );
    map.put( "Registerauszugsgruppe", getLegend( selKritVO.regauszGrp, XxDomainFactory.REGAUSZ_GRP ) );
    map.put( "Publikationspflicht", getLegend( selKritVO.publPflicht, XxDomainFactory.PUBL_PFLICHT ) );
    map.put( "Unterabschnitt Publikation", getLegend( selKritVO.statcd3, XxDomainFactory.PUBL_KAP ) );
    map.put( "Sperr-Code", selKritVO.getSperrCdText() );

    return map;
  }

  /**
   * Bereitet die Spalte auf für den Eintrag ins XML-File
   * @return List aufbereitete Spalten
   */
  protected Spalten prepareSpalte()
  {
    Spalten spalten = new SpaltenImpl();
    List listSpalte = spalten.getSpalte();
    Spalte spalte = null;

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Artikel-Nr." );
    spalte.setGroesse( "5" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bezeichnung" );
    spalte.setGroesse( "25" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Ereignis Gruppe" );
    spalte.setGroesse( "8" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Sperrcode (9=gesperrt)" );
    spalte.setGroesse( "3" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Gratisartikel (1=gratis)" );
    spalte.setGroesse( "3" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Preis" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Textart" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Suchbegriff" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Anzahl Kopien" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Druckmodus" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Einzelfakturaposition" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Formular" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Einschreiben" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bescheinigung" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Kennzeichen Unterschrift" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Erfassprozess" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Prozessmodus" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Proszessnummer" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Listentyp EP-Jahresgebühr" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Löschregel Ereignis" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fristeinheit" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fristdauer" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fristart" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Lagereinheit" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Unterabschnitt Publikation" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Publikationspflicht" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Registerauszugsgruppe" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bonus-Code" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Marna3-Auftrag" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    return spalten;
  }

  /**
   * Bereitet die Zeilen für den Eintrag ins XML-File auf
   * @return List aufbereitete Spalten
   */
  protected Zeilen prepareZeile( List listArtikel )
  {
    Zeilen zeilen = new ZeilenImpl();
    List listZeile = zeilen.getZeile();
    Zeile zeile = null;
    Zelle zelle = null;

    for ( int i = 0; i < listArtikel.size(); i++ )
    {
      zeile = new ZeileImpl();
      List listZelle = zeile.getZelle();

      ArtikelGuiAVO artikel = (ArtikelGuiAVO) listArtikel.get( i );

      zelle = new ZelleImpl();
      zelle.setText( String.valueOf( artikel.artNr ) );
      zelle.setGroesse( "5" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.bezArt == null ? "" : artikel.bezArt );
      zelle.setGroesse( "25" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kondartCd == null ? "" : artikel.kondartCd );
      zelle.setGroesse( "8" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.sperrCd == null ? "" : artikel.sperrCd );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzGratis == null ? "" : artikel.kzGratis );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( getArtPreisContent( artikel ) );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( this.getArtTextContent( artikel ) );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.suchbegrArt == null ? "" : artikel.suchbegrArt );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.anzKopie == null ? "" : String.valueOf( artikel.anzKopie ) );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.druckModus == null ? "" : artikel.druckModus );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.einzFaktpos == null ? "" : artikel.einzFaktpos );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.formartCd == null ? "" : artikel.formartCd );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzEinschr == null ? "" : artikel.kzEinschr );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzBesch == null ? "" : artikel.kzBesch );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzUntschr == null ? "" : artikel.kzUntschr );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.erfassProc == null ? "" : String.valueOf( artikel.erfassProc ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.processModus == null ? "" : artikel.processModus );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.processNr == null ? "" : String.valueOf( artikel.processNr ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd1 == null ? "" : artikel.statcd1 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd2 == null ? "" : artikel.statcd2 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristEinh == null ? "" : artikel.fristEinh );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristArt == null ? "" : artikel.fristArt );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristDauer == null ? "" : String.valueOf( artikel.fristDauer ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.lageinh == null ? "" : artikel.lageinh );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd3 == null ? "" : artikel.statcd3 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.publPflicht == null ? "" : artikel.publPflicht );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( String.valueOf( artikel.regauszGrp ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.bonusCd == null ? "" : artikel.bonusCd );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzMetnot == null ? "" : artikel.kzMetnot );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      listZeile.add( zeile );
    }

    return zeilen;
  }

  /**
   * Schreibt den Abschnitt Artikelpreise ins XMLFile.
   * @param artikel ArtikelGuiAVO Detaildaten eines Artikels
   * @return String formatierte Darstellung des jüngsten Preises Bsp.: 100.00(01.01.2006)
   */
  public String getArtPreisContent( ArtikelGuiAVO artikel )
  {
    SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );
    DecimalFormat df = new DecimalFormat( "0.00", new DecimalFormatSymbols( Locale.ENGLISH ) );

    StringBuffer buf = new StringBuffer();

    if ( artikel.artpreisVO != null && artikel.artpreisVO.length > 0 )
    {
      if ( artikel.artpreisVO[0].preis != null )
      {
        buf.append( df.format( artikel.artpreisVO[0].preis ) );
      }
      if ( artikel.artpreisVO[0].gueltigAb != null )
      {
        buf.append( "(" );
        buf.append( sdf.format( artikel.artpreisVO[0].gueltigAb ) );
        buf.append( ")" );
      }
    }
    return buf.toString();
  }

  /**
   * Schreibt Textarten ins XMLFile.
   * @param artikel ArtikelGuiAVO Detaildaten eines Artikels
   * @return String formatierte sprachunabhängige Darstellung der Textarten Bsp.: NP,MRP,BSC...
   */
  public String getArtTextContent( ArtikelGuiAVO artikel )
  {
    Map<String, String> map = new TreeMap<String, String>();
    StringBuffer buf = new StringBuffer();

    for ( int i = 0; i < artikel.artextVO.length; i++ )
    {
      map.put( artikel.artextVO[i].artTextArtCd, artikel.artextVO[i].artTextArtCd );
    }

    for ( String artCd: map.values() )
    {
      buf.append( artCd );
      buf.append( "," );
    }

    return buf.toString();
  }
}
