package ch.ige.art.xml;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelSelKritVO;
import ch.ige.art.xml.jaxb.listen.Spalte;
import ch.ige.art.xml.jaxb.listen.Spalten;
import ch.ige.art.xml.jaxb.listen.Zeile;
import ch.ige.art.xml.jaxb.listen.Zeilen;
import ch.ige.art.xml.jaxb.listen.Zelle;
import ch.ige.art.xml.jaxb.listen.impl.SpalteImpl;
import ch.ige.art.xml.jaxb.listen.impl.SpaltenImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZeileImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZeilenImpl;
import ch.ige.art.xml.jaxb.listen.impl.ZelleImpl;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-File zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: ArtListe.java,v 1.9 2008/01/14 09:49:15 ruegsegger Exp $
 */

public class ArtListe extends Liste
{

  public ArtListe() throws Exception
  {
  }


  /**
   * Gibt den Titel der Liste zurück
   * @return String Titel der Liste
   */
  private String getTitel()
  {
    return "Trefferliste Artikel vom " + currentDate;
  }

  /**
   * Gibt eine Map mit den Kriterien und ihrer Bezeichnung zurück
   * @param selKritVO ArtikelSelKritVO
   * @return Map mit Kriterien und Bezeichnungen
   */
  public Map getKriteriumList(ArtikelSelKritVO selKritVO)
  {
    Map map = new LinkedHashMap ();

    map.put("Artikel-Nr", selKritVO.artNr );
    map.put("Artikel-Bezeichnung", selKritVO.bezArt);
    map.put("Formular", selKritVO.formartCd );
    map.put("Übergruppe Ereignis", getLegend( selKritVO.kondartCd, XxDomainFactory.KONDART_CD ) );
    map.put("Erfassungsprozess", getLegend( selKritVO.erfassProc, XxDomainFactory.ERFASS_PROC ));
    map.put("Registerauszugsgruppe", getLegend( selKritVO.regauszGrp, XxDomainFactory.REGAUSZ_GRP ));
    map.put("Publikationspflicht", getLegend( selKritVO.publPflicht, XxDomainFactory.PUBL_PFLICHT ));
    map.put("Unterabschnitt Publikation", getLegend( selKritVO.statcd3, XxDomainFactory.PUBL_KAP )); 
    map.put("Sperr-Code", selKritVO.getSperrCdText());

    return map;
  }


  /**
   * Bereitet die Spalte auf für den Eintrag ins XML-File
   * @return List aufbereitete Spalten
   */
  protected Spalten prepareSpalte()
  {
    Spalten spalten = new SpaltenImpl();
    List listSpalte = spalten.getSpalte();
    Spalte spalte = null;

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Nr" );
    spalte.setGroesse( "5" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bezeichnung" );
    spalte.setGroesse( "25" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Ereignis Gruppe" );
    spalte.setGroesse( "8" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Gr" );
    spalte.setGroesse( "3" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Sp" );
    spalte.setGroesse( "3" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Cd1" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Cd2" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Cd3" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Dru Mod" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Ezf Pos" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "For" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Ein  sch" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Pub Pfl" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Reg Grp" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fri Art" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fri Ein" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Fri Dau" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bes" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Erf Prz" );
    spalte.setGroesse( "4" );
    listSpalte.add( spalte );

    return spalten;
  }

  /**
   * Bereitet die Zeilen für den Eintrag ins XML-File auf
   * @return List aufbereitete Spalten
   */
  protected Zeilen prepareZeile( List artikelAVO )
  {
    Zeilen zeilen = new ZeilenImpl();
    List listZeile = zeilen.getZeile();
    Zeile zeile = null;
    Zelle zelle = null;

    for ( int i = 0; i < artikelAVO.size(); i++ )
    {
      zeile = new ZeileImpl();
      List listZelle = zeile.getZelle();

      ArtikelAVO artikel = (ArtikelAVO)artikelAVO.get(i);

      zelle = new ZelleImpl();
      zelle.setText( String.valueOf( artikel.artNr ) );
      zelle.setGroesse( "5" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.bezArt == null ? "" : artikel.bezArt );
      zelle.setGroesse( "25" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kondartCd == null ? "" : artikel.kondartCd );
      zelle.setGroesse( "8" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzGratis == null ? "" : artikel.kzGratis );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.sperrCd == null ? "" : artikel.sperrCd );
      zelle.setGroesse( "3" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd1 == null ? "" : artikel.statcd1 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd2 == null ? "" : artikel.statcd2 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.statcd3 == null ? "" : artikel.statcd3 );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.druckModus == null ? "" : artikel.druckModus );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.einzFaktpos == null ? "" : artikel.einzFaktpos );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.formartCd == null ? "" : artikel.formartCd );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzEinschr == null ? "" : artikel.kzEinschr );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.publPflicht == null ? "" : artikel.publPflicht );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( String.valueOf( artikel.regauszGrp ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristArt == null ? "" : artikel.fristArt );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristEinh == null ? "" : artikel.fristEinh );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.fristDauer == null ? "" : ( artikel.fristDauer ).toString() );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.kzBesch == null ? "" : artikel.kzBesch );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( artikel.erfassProc == null ? "" :
                     String.valueOf( artikel.erfassProc ) );
      zelle.setGroesse( "4" );
      listZelle.add( zelle );

      listZeile.add( zeile );
    }

    return zeilen;
  }
}


