package ch.ige.art.xml;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import javax.xml.bind.JAXBContext;

import org.apache.log4j.Logger;

import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.PublicationChapter;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.AltArtVO;
import ch.ige.art.vo.ArtextVO;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.ArtpreisVO;
import ch.ige.art.xml.jaxb.stammblatt.Art;
import ch.ige.art.xml.jaxb.stammblatt.ArtData;
import ch.ige.art.xml.jaxb.stammblatt.Artfollow;
import ch.ige.art.xml.jaxb.stammblatt.ArtfollowData;
import ch.ige.art.xml.jaxb.stammblatt.Artprice;
import ch.ige.art.xml.jaxb.stammblatt.ArtpriceData;
import ch.ige.art.xml.jaxb.stammblatt.Arttext;
import ch.ige.art.xml.jaxb.stammblatt.ArttextData;
import ch.ige.art.xml.jaxb.stammblatt.Header;
import ch.ige.art.xml.jaxb.stammblatt.PrintOrder;
import ch.ige.art.xml.jaxb.stammblatt.StammblattEl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtDataImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtfollowDataImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtfollowImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtpriceDataImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArtpriceImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArttextDataImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.ArttextImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.HeaderImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.PrintOrderImpl;
import ch.ige.art.xml.jaxb.stammblatt.impl.StammblattElImpl;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-Stream zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: ArtStammblatt.java,v 1.17 2008/07/02 14:30:39 heinzer Exp $
 */

public class ArtStammblatt extends Stammblatt
{
  private static final Logger LOG = Logger.getLogger( ArtStammblatt.class );

  private JAXBContext jc = JAXBContext.newInstance( "ch.ige.art.xml.jaxb.stammblatt" );

  public ArtStammblatt() throws Exception
  {
  }

  /**
   * Liest die Daten von der Datenbank
   */
  public List read( List listArtNr ) throws Exception
  {
    ArtikelGuiAVO artikelGuiAVO;
    List listArtikel = new ArrayList();

    for ( int i = 0; i < listArtNr.size(); i++ )
    {
      String artNr = (String) listArtNr.get( i );
      artikelGuiAVO = ArtikelDelegate.getInstance().getArtikel( artNr );
      listArtikel.add( artikelGuiAVO );
    }

    return listArtikel;
  }

  /**
   * Erstellt den String mit den XMLDaten
   * @param auszugVO ValueObject mit den Daten für eines Auszuges
   * @return String mit den XMLDaten
   * @throws XMLException
   */
  public String getXMLString( List listArtikel ) throws Exception
  {
    ArtikelGuiAVO artikelGuiAVO;
    StammblattEl stammblattEl;
    PrintOrder printOrder = new PrintOrderImpl();

    printOrder.setDoktyp( String.valueOf( PrintDoc.DOKTYP_STAMMBLATT ) );

    printOrder.setMaNr( getMaNr() );
    printOrder.setNtusername( getNtusername() );

    LOG.debug( "PrintOrder" );

    for ( int i = 0; i < listArtikel.size(); i++ )
    {
      artikelGuiAVO = (ArtikelGuiAVO) listArtikel.get( i );
      stammblattEl = writeStammblatt( artikelGuiAVO );
      printOrder.getStammblattEl().add( stammblattEl );
    }

    String xmlStream = marshalToString( jc, printOrder );

    return replaceTags( xmlStream );
  }

  private StammblattEl writeStammblatt( ArtikelGuiAVO artikelGuiAVO )
  {
    StammblattEl stammblattEl = new StammblattElImpl();

    //Header füllen
    Header header = writeHeader( artikelGuiAVO );
    stammblattEl.setHeader( header );
    LOG.debug( "Header" );

    // ArtData Artikelstamm füllen
    List listArt = stammblattEl.getArt();
    Art artStamm = writeDataArtikelstamm( artikelGuiAVO );
    listArt.add( artStamm );
    LOG.debug( "Artikelstamm" );

    // ArtData Artikeldaten füllen
    listArt = stammblattEl.getArt();
    Art artDaten = writeDataArtikeldaten1( artikelGuiAVO );
    listArt.add( artDaten );
    LOG.debug( "Artikeldaten 1" );

    // ArtData Artikeldaten füllen
    listArt = stammblattEl.getArt();
    artDaten = writeDataArtikeldaten2( artikelGuiAVO );
    listArt.add( artDaten );
    LOG.debug( "Artikeldaten 2" );

    // Artprice Artikelpreise füllen
    List listArtprice = stammblattEl.getArtprice();
    Artprice artprice = writeArtikelpreise( artikelGuiAVO.artpreisVO );
    listArtprice.add( artprice );
    LOG.debug( "Artikelpreise" );

    // Artfollow Folgeartikel füllen
    List listArtfollow = stammblattEl.getArtfollow();
    Artfollow artFollow = writeFolgeartikel( artikelGuiAVO.altArtVO );
    listArtfollow.add( artFollow );
    LOG.debug( "Folgeartikel" );

    // Arttext Artikeltexte füllen
    writeArtikeltexe( artikelGuiAVO.artextVO, stammblattEl );
    LOG.debug( "Artikeltexte" );

    return stammblattEl;
  }

  /**
   * Schreibt die Headerzeile ins XMLFile
   * @param ArtikelGuiAVO enthält alle für ein Stammblatt benötigten Werte
   * @return header Die geschriebene headerZeile
   */
  private Header writeHeader( ArtikelGuiAVO artikelGuiAVO )
  {
    Header header = new HeaderImpl();
    header.setHeaderTitle( "Artikel-Stammblatt" );
    header.setHeaderDate( currentDate );
    header.setHeaderArtNr( artikelGuiAVO.artNr );
    header.setHeaderSearchitem( artikelGuiAVO.suchbegrArt == null ? "" : artikelGuiAVO.suchbegrArt );
    header.setHeaderDescription( artikelGuiAVO.bezArt == null ? "" : artikelGuiAVO.bezArt );

    return header;
  }

  /**
   * Schreibt den Abschnitt Artikelstamm ins XMLFile.
   * @param ArtikelGuiAVO enthält alle für ein Stammblatt benötigten Werte
   * @return Art der geschriebene Abschnitt Artikelstamm
   */
  private Art writeDataArtikelstamm( ArtikelGuiAVO artikelGuiAVO )
  {
    Art art = new ArtImpl();
    List listArtData = art.getArtData();
    ArtData artData = null;

    art.setArtTitle( "Artikelstamm" );

    artData = new ArtDataImpl();
    artData.setArtKey( "Übergruppe Ereignis" );
    String artValue = getLegend( artikelGuiAVO.kondartCd, XxDomainFactory.KONDART_CD );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Minimale Menge" );
    artData.setArtValue( artikelGuiAVO.minverkMenge == null ? "" : String.valueOf( artikelGuiAVO.minverkMenge ) );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Maximale Menge" );
    artData.setArtValue( artikelGuiAVO.optBestMenge == null ? "" : String.valueOf( artikelGuiAVO.optBestMenge ) );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Sperrcode" );
    if ( artikelGuiAVO.sperrCd == null )
    {
      artData.setArtValue( "" );
    }
    else
    {
      if ( artikelGuiAVO.sperrCd.equals( "9" ) )
      {
        artData.setArtValue( artikelGuiAVO.sperrCd + ", gesperrt" );
      }
      else
      {
        artData.setArtValue( artikelGuiAVO.sperrCd + ", nicht gesperrt" );
      }
    }
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Gratisartikel" );

    if ( artikelGuiAVO.kzGratis == null )
    {
      artData.setArtValue( "" );
    }
    else
    {
      if ( artikelGuiAVO.kzGratis.equals( "1" ) )
      {
        artData.setArtValue( artikelGuiAVO.kzGratis + ", gratis" );
      }
      else
      {
        artData.setArtValue( artikelGuiAVO.kzGratis + ", kostenpflichtig" );
      }
    }
    listArtData.add( artData );

    //only for design
    if ( XxDomainFactory.PUBL_KAP_DES.equals( PublicationChapter.getCorrespondingDomainValue( artikelGuiAVO.artNr ) ) )
    {
      artData = new ArtDataImpl();
      artData.setArtKey( "Aufschub: zeige in Historie" );

      if ( "1".equals( artikelGuiAVO.kzHistDesign ) )
      {
        artData.setArtValue( artikelGuiAVO.kzHistDesign + ", Anzeigen erlaubt" );
      }
      else
      {
        artData.setArtValue( artikelGuiAVO.kzHistDesign + ", Anzeigen untersagt" ); //note: can be NULL or 0
      }
      listArtData.add( artData );
    }

    return art;
  }

  /**
   * Schreibt den Abschnitt Artikeldaten ins XMLFile.
   * @param ArtikelGuiAVO enthält alle für ein Stammblatt benötigten Werte
   * @return Art der geschriebene Abschnitt Artikeldaten
   */
  private Art writeDataArtikeldaten1( ArtikelGuiAVO artikelGuiAVO )
  {
    Art art = new ArtImpl();
    List listArtData = art.getArtData();
    ArtData artData = null;
    String artValue = null;

    art.setArtTitle( "Artikeldaten" );

    artData = new ArtDataImpl();
    artData.setArtKey( "Anzahl Kopien" );
    artData.setArtValue( artikelGuiAVO.anzKopie == null ? "" : artikelGuiAVO.anzKopie.toString() );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Druckmodus" );
    artValue = getLegend( artikelGuiAVO.druckModus, XxDomainFactory.DRUCK_MODUS );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Einzelfakturaposition" );
    artValue = getLegend( artikelGuiAVO.einzFaktpos, XxDomainFactory.EINZ_FAKTPOS );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Formular" );
    artData.setArtValue( artikelGuiAVO.formartCd == null ? "" : artikelGuiAVO.formartCd );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Einschreiben" );
    if ( artikelGuiAVO.kzEinschr == null )
    {
      artData.setArtValue( "" );
    }
    else
    {
      if ( artikelGuiAVO.kzEinschr.equals( "1" ) )
      {
        artData.setArtValue( artikelGuiAVO.kzEinschr + ", erforderlich" );
      }
      else
      {
        artData.setArtValue( artikelGuiAVO.kzEinschr + ", nicht erforderlich" );
      }
    }
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Bescheinigung" );
    if ( artikelGuiAVO.kzBesch == null )
    {
      artData.setArtValue( "" );
    }
    else
    {
      if ( artikelGuiAVO.kzBesch.equals( "1" ) )
      {
        artData.setArtValue( artikelGuiAVO.kzBesch + ", erforderlich" );
      }
      else
      {
        artData.setArtValue( artikelGuiAVO.kzBesch + ", nicht erforderlich" );
      }
    }
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Kennzeichen Unterschrift" );
    artValue = getLegend( artikelGuiAVO.kzUntschr, XxDomainFactory.KZ_UNTSCHR );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Erfassprozess" );
    artValue = getLegend( artikelGuiAVO.erfassProc, XxDomainFactory.ERFASS_PROC );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Prozessmodus" );
    artValue = getLegend( artikelGuiAVO.processModus, XxDomainFactory.PROCESS_MODUS );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Prozessnummer" );
    artData.setArtValue( artikelGuiAVO.processNr == null ? "" : artikelGuiAVO.processNr.toString() );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Listentyp EP-Jahresgebühr" );
    artValue = getLegend( artikelGuiAVO.statcd1, XxDomainFactory.STATCD_1 );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Löschregel Ereignis" );
    artValue = getLegend( artikelGuiAVO.statcd2, XxDomainFactory.STATCD_2 );
    artData.setArtValue( ( artValue ) );
    listArtData.add( artData );

    return art;
  }

  /**
   * Schreibt den Abschnitt Artikeldaten ins XMLFile.
   * @param ArtikelGuiAVO enthält alle für ein Stammblatt benötigten Werte
   * @return Art der geschriebene Abschnitt Artikeldaten
   */
  private Art writeDataArtikeldaten2( ArtikelGuiAVO artikelGuiAVO )
  {
    Art art = new ArtImpl();
    List listArtData = art.getArtData();
    ArtData artData = null;
    String artValue = null;

    art.setArtTitle( "Artikeldaten" );

    artData = new ArtDataImpl();
    artData.setArtKey( "Fristeinheit" );
    artValue = getLegend( artikelGuiAVO.fristEinh, XxDomainFactory.FRIST_EINH );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Fristdauer" );
    artData.setArtValue( artikelGuiAVO.fristDauer == null ? "" : artikelGuiAVO.fristDauer.toString() );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Fristart" );
    artValue = getLegend( artikelGuiAVO.fristArt, XxDomainFactory.FRIST_ART );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Lagereinheit" );
    artValue = getLegend( artikelGuiAVO.lageinh, XxDomainFactory.LAGEINH );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Unterabschnitt Publikation" );
    String domain = PublicationChapter.getCorrespondingDomainValue( artikelGuiAVO.artNr );
    artValue = getLegend( artikelGuiAVO.statcd3, domain );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Publikationspflicht" );
    artValue = getLegend( artikelGuiAVO.publPflicht, XxDomainFactory.PUBL_PFLICHT );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Registerauszugsgruppe" );
    artValue = getLegend( String.valueOf( artikelGuiAVO.regauszGrp ), XxDomainFactory.REGAUSZ_GRP );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Bonus-Code" );
    artValue = getLegend( artikelGuiAVO.bonusCd, XxDomainFactory.BONUS_CD );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    artData = new ArtDataImpl();
    artData.setArtKey( "Marna3 - Auftrag" );
    artValue = getLegend( artikelGuiAVO.kzMetnot, XxDomainFactory.KZ_METNOT );
    artData.setArtValue( artValue );
    listArtData.add( artData );

    return art;
  }

  /**
   * Schreibt den Abschnitt Artikelpreise ins XMLFile.
   * @param ArtpreisVO enthält alle für die Artikelpreise benötigten Werte
   * @return Artprice der geschriebene Abschnitt Artikelpreise
   */
  public Artprice writeArtikelpreise( ArtpreisVO[] artpreisVO )
  {
    Artprice artprice = new ArtpriceImpl();
    SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );
    DecimalFormat df = new DecimalFormat( "0.00", new DecimalFormatSymbols( Locale.ENGLISH ) );

    List listArtpriceData = artprice.getArtpriceData();
    artprice.setArtpriceTitle( "Artikel-Preise" );

    for ( int i = 0; i < artpreisVO.length; i++ )
    {
      ArtpriceData artpriceData = new ArtpriceDataImpl();
      artpriceData.setArtpriceDescription( artpreisVO[i].artpreisartCd );
      artpriceData.setArtpriceDate( sdf.format( artpreisVO[i].gueltigAb ) );
      if ( artpreisVO[i].preis != null )
      {
        artpriceData.setArtpricePrice( "CHF " + df.format( artpreisVO[i].preis ) );
      }
      else
      {
        artpriceData.setArtpricePrice( "" );
      }
      listArtpriceData.add( artpriceData );
    }

    return artprice;
  }

  /**
   * Schreibt den Abschnitt Folgeartikel ins XMLFile.
   * @param AltArtVO enthält alle für die Folgeartikel benötigten Werte
   * @return Artfollow der geschriebene Abschnitt Folgeartikel
   */
  public Artfollow writeFolgeartikel( AltArtVO[] altArtVO )
  {
    Artfollow artfollow = new ArtfollowImpl();
    List listArtfollowData = artfollow.getArtfollowData();

    artfollow.setArtfollowTitle( "Folge-Artikel" );

    for ( int i = 0; i < altArtVO.length; i++ )
    {
      ArtfollowData artfollowData = new ArtfollowDataImpl();
      artfollowData.setArtfollowKey( altArtVO[i].altArtNr );
      artfollowData.setArtfollowValue( altArtVO[i].priorCd );

      listArtfollowData.add( artfollowData );
    }

    return artfollow;
  }

  /**
   * Schreibt den Abschnitt Artikeltexte ins XMLFile.
   * Pro Textart wird eine Gruppe ausgedruckt
   * @param ArtextVO enthält alle für die Artikeltexte benötigten Werte
   * @return Arttext der geschriebene Abschnitt Artikeltexte
   */
  public void writeArtikeltexe( ArtextVO[] artextVO, StammblattEl stammblattEl )
  {
    List listArttext = stammblattEl.getArttext();

    Arttext arttext = null;
    List listArttextData = null;
    String artTextArtCd = null;

    int i = 0;
    while ( i < artextVO.length )
    {
      if ( artextVO[i].isValidTextart() )
      {
        arttext = new ArttextImpl();
        listArttextData = arttext.getArttextData();

        artTextArtCd = artextVO[i].artTextArtCd;
        arttext.setArttextTitle( "Artikel-Texte " + artTextArtCd );

        do
        {
          ArttextData arttextData = new ArttextDataImpl();
          arttextData.setArttextKey( artextVO[i].artTextArtCd );
          arttextData.setArttextLanguage( artextVO[i].sprachCd );
          arttextData.setArttextValue( artextVO[i].text == null ? "" : artextVO[i].text );

          listArttextData.add( arttextData );
          i++;
        } while ( i < artextVO.length && artextVO[i].artTextArtCd.equals( artTextArtCd ) );

        listArttext.add( arttext );
        LOG.debug( "Artikel-Texte " + artTextArtCd );
      }
      else
      {
        i++;
      }
    }
  }
}
