package ch.ige.art.xml;

import java.util.*;

import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.BformartVO;
import ch.ige.art.xml.jaxb.listen.*;
import ch.ige.art.xml.jaxb.listen.impl.*;

/**
 * <p>Überschrift: Mit dieser Klasse werden die Daten für die Erstellung des XML-File zusammengetragen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author  Anita Rueegsegger
 * @version $Id: FormListe.java,v 1.7 2008/01/14 09:49:15 ruegsegger Exp $
 */

public class FormListe extends Liste
{

  public FormListe() throws Exception
  {
  }


  /**
   * Gibt den Titel der Liste zurück
   * @return String Titel der Liste
   */
  private String getTitel()
  {
    return "Trefferliste Formular vom " + currentDate;
  }

  /**
   * Gibt eine Liste mit den Kriterien zurück
   * @param selKritVO BformartVO
   * @return Map Kriterien und Werte
   */
  public Map getKriteriumList(BformartVO selKritVO )
  {
    Map map = new LinkedHashMap ();

    map.put("Formular-Nr", selKritVO.formartCd );
    map.put("Formular-Bezeichnung", selKritVO.bez);
    map.put("Drucksystem", getLegend( selKritVO.druckSystem, XxDomainFactory.DRUCK_SYSTEM ) );

    return map;
  }

  /**
   * Bereitet die Spalte auf für den Eintrag ins XML-File
   * @return List aufbereitete Spalten
   */
  protected Spalten prepareSpalte()
  {
    Spalten spalten = new SpaltenImpl();
    List listSpalte = spalten.getSpalte();
    Spalte spalte = null;

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Formular-Nr" );
    spalte.setGroesse( "15" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Bezeichnung" );
    spalte.setGroesse( "70" );
    listSpalte.add( spalte );

    spalte = new SpalteImpl();
    spalte.setSpaltentitel( "Drucksystem" );
    spalte.setGroesse( "15" );
    listSpalte.add( spalte );


    return spalten;
  }

  /**
   * Bereitet die Zeilen für den Eintrag ins XML-File auf
   * @return List aufbereitete Spalten
   */
  protected Zeilen prepareZeile( List bformartAVO )
  {
    Zeilen zeilen = new ZeilenImpl();
    List listZeile = zeilen.getZeile();
    Zeile zeile = null;
    Zelle zelle = null;

    for ( int i = 0; i < bformartAVO.size(); i++ )
    {
      zeile = new ZeileImpl();
      List listZelle = zeile.getZelle();

      BformartVO bformart = (BformartVO) bformartAVO.get(i);

      zelle = new ZelleImpl();
      zelle.setText( String.valueOf( bformart.formartCd ) );
      zelle.setGroesse( "15" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( bformart.bez == null ? "" : bformart.bez );
      zelle.setGroesse( "70" );
      listZelle.add( zelle );

      zelle = new ZelleImpl();
      zelle.setText( bformart.druckSystem == null ? "" : bformart.druckSystem );
      zelle.setGroesse( "15" );
      listZelle.add( zelle );

      listZeile.add( zeile );
    }

    return zeilen;
  }
}


