package ch.ige.epband.clientneu;

import java.util.Date;
import java.util.Map;

import ch.ige.epband.util.MappingUnicode;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreigTxtVO;
import ch.ige.epband.vo.EreignisVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: Ereignis.java,v 1.28 2009/01/12 14:21:06 blank_blt Exp $
 */

public class Ereignis
{
  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  private XMLDoc xml;

  private String tagdata = "";

  /**
   * Aufbereiten von Entität ereignis (public wegen Junit)
   * 
   * @param xml
   * @param epbandVO
   * @throws Exception
   */
  public void parseSGML( XMLDoc xmlDoc, EpbandVO epbandVO ) throws Exception
  {
    this.xml = xmlDoc;

    EreignisVO ereignisVO = new EreignisVO();
    ereignisVO.artNr = EreignisVO.ERSTERFASSUNG;
    prepEreig( ereignisVO, epbandVO );
    epbandVO.ereigAL.add( ereignisVO );

    // Erteilung von EP
    ereignisVO = new EreignisVO();
    if ( epbandVO.epArt.equals( "B1" ) && epbandVO.epStatus.equalsIgnoreCase( "N" ) )
    {
      // Achtung, tagdata (Erteilungsdatum wird auch noch beim Ereignis 20140
      // nachfolgend gebraucht
      tagdata = xml.getValueOf( "B450", "date" );
      if ( tagdata.equals( "" ) )
      {
        ereignisVO.ereigIdat = epbandVO.anmeldeIdat;
        tagdata = epbandVO.anmeldedateString;
        String line = "ERROR \t Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr
            + " kein Erteilungsdatum vorhanden! Anmeldetatum eingesetzt!";
        logFile.write( line );
      }
      else
      {
        ereignisVO.ereigIdat = Util.calculateIdat( tagdata );
      }
      ereignisVO.artNr = EreignisVO.ERTEILUNG;
      prepEreig( ereignisVO, epbandVO );

      Date ereigDat = Util.createDate( ereignisVO.ereigIdat );

      // Datum der Erteilung liegt in der Zukunft
      if ( ereigDat.compareTo( new Date( new java.util.Date().getTime() ) ) > 0 )
      {
        ereignisVO.publStat = EreignisVO.PUBLSTAT_WAITLIST;
        ereignisVO.publDat = ereigDat;
      }
      // Datum der Erteilung liegt in der Vergangenheit
      else
      {
        ereignisVO.publStat = EreignisVO.PUBLSTAT_READY_PUBLICATE;
      }

      epbandVO.ereigAL.add( ereignisVO );

      // Bei eng. Veröffentlichung das Ererignis
      // 20140 ab März 08 keine Frist mehr für dieses Ereignis

      if ( epbandVO.patVO.epApublSpra.equals( "en" ) )
      {
        int idat = ereignisVO.ereigIdat;
        ereignisVO = new EreignisVO();
        ereignisVO.artNr = EreignisVO.VER_ENGL;
        // Eintragung aus "20010"
        ereignisVO.ereigIdat = idat + 5;
        prepEreig( ereignisVO, epbandVO );
        epbandVO.ereigAL.add( ereignisVO );
      }
    }

    // hat es Zurückweisung, Zurücknahme oder Zurücknahme Fiktion gegeben
    // muss das jeweilige Ereignis erstellt, und der Schutztitel (Gesuch)
    // gelöscht werden
    if ( epbandVO.isGesuch() )
    {
      tagdata = xml.getValueOf( "B235", "date" ); // 20230 Zurückweisung
      if ( !tagdata.equals( "" ) )
      {
        ereignisVO = new EreignisVO();
        ereignisVO.ereigIdat = Util.calculateIdat( tagdata );
        ereignisVO.artNr = EreignisVO.ZURUECKWEISUNG;
        prepEreig( ereignisVO, epbandVO );
        epbandVO.ereigAL.add( ereignisVO );
        epbandVO.schutitVO.loeschDat = Util.createDate( tagdata );
      }
      tagdata = xml.getValueOf( "B236", "date" ); // 20235 Zurücknahme
      if ( !tagdata.equals( "" ) )
      {
        ereignisVO = new EreignisVO();
        ereignisVO.ereigIdat = Util.calculateIdat( tagdata );
        ereignisVO.artNr = EreignisVO.ZURUECKNAHME;
        prepEreig( ereignisVO, epbandVO );
        epbandVO.ereigAL.add( ereignisVO );
        epbandVO.schutitVO.loeschDat = Util.createDate( tagdata );
      }
      tagdata = xml.getValueOf( "B237", "date" ); // 20240 Zurücknahme Fiktion
      if ( !tagdata.equals( "" ) )
      {
        ereignisVO = new EreignisVO();
        ereignisVO.ereigIdat = Util.calculateIdat( tagdata );
        ereignisVO.artNr = EreignisVO.FIK_ZURUECKNAHME;
        prepEreig( ereignisVO, epbandVO );
        epbandVO.ereigAL.add( ereignisVO );
        epbandVO.schutitVO.loeschDat = Util.createDate( tagdata );
      }
    }

    // Ereignistext aufbereiten
    this.genEreigTxt( epbandVO );

  }

  /**
   * Ereignis aufbereiten
   * 
   * @param ereignisVO
   * @param epbndVO
   * @throws Exception 
   */
  public void prepEreig( EreignisVO ereignisVO, EpbandVO epbndVO ) throws Exception
  {
    if ( ereignisVO.ereigIdat == 0 )
    {
      ereignisVO.ereigIdat = epbndVO.anmeldeIdat;
    }

    ereignisVO.erfassDat = new Date();

    ereignisVO.publZiel = 1;

    if ( ereignisVO.fristablDat == null )
    {
      ereignisVO.ereigStatNr = 20;
    }
    // EP erteilt
    if ( epbndVO.schutitVO.registerKz == 1 )
    {
      ereignisVO.regStat = 9;
    }
    else
    {
      ereignisVO.regStat = 8;
    }
  }

  /**
   * Aufbereiten von Entität ereigTxt Falls die Publikations Sprache EN wird in schutit.beschreibung die deutsche
   * Beschreibung und im ereigTxtVO.ereigTxt die französiche Beschreibung gespeichert Falls die Publikations Sprache
   * nicht EN ist oder die französische Übersetzung fehlt, wird kein Ereignistext gespeichert
   * 
   */
  private void genEreigTxt( EpbandVO epbandVO ) throws Exception
  {
    String publSprache = xml.getValueOf( "B260" ); // "" falls nicht vorhanden
    if ( publSprache.equals( "en" ) )
    {
      Map<String, String> map = new Schutit().getBeschreibungSprachMapping( xml );
      String beschreibung = map.get( "fr" ); // null falls nicht enthalten

      if ( beschreibung != null && !beschreibung.equals( "" ) )
      {
        EreigTxtVO ereigTxtVO = new EreigTxtVO();
        ereigTxtVO.ereigIdat = epbandVO.anmeldeIdat;
        ereigTxtVO.ereigTxt = MappingUnicode.getInstance().replaceUnicodeValues( beschreibung );
        ereigTxtVO.ereigTxtCd = 1;
        epbandVO.ereigTxtAL.add( ereigTxtVO );
      }
    }
  }

  /**
   * Generiert Ereignis für WIPO 30060
   * 
   * @return Ereignis WIPO
   */
  public EreignisVO getEreignisWipo()
  {
    EreignisVO ereignisVO = new EreignisVO();

    ereignisVO.artNr = EreignisVO.WIPO;
    ereignisVO.erfassDat = new Date();
    ereignisVO.regStat = 8;
    ereignisVO.ereigStatNr = 20;
    ereignisVO.publZiel = 1;

    return ereignisVO;
  }

  /**
   * Generiert Ereignis für Zurücknahme 20250
   * 
   * @return Ereignis Zurücknahme
   */
  public EreignisVO getEreignisZuruecknahme()
  {
    EreignisVO ereignisVO = new EreignisVO();

    ereignisVO.artNr = EreignisVO.CH_ZURUECKNAHME;
    ereignisVO.erfassDat = new Date();
    ereignisVO.regStat = 8;
    ereignisVO.ereigStatNr = 20;
    ereignisVO.publZiel = 1;

    return ereignisVO;
  }

  /**
   * Generiert Ereignis für KeinEinspruch 20450
   * 
   * @return Ereignis KeinEinspruch
   */
  public EreignisVO getEreignisKeinEinspruch()
  {
    EreignisVO ereignisVO = new EreignisVO();

    ereignisVO.artNr = EreignisVO.KEIN_EINSP;
    ereignisVO.erfassDat = new Date();
    ereignisVO.regStat = 9;
    ereignisVO.ereigStatNr = 20;
    ereignisVO.publZiel = 1;

    return ereignisVO;
  }
}
