package ch.ige.epband.clientneu;

import java.text.DecimalFormat;

/**
 * Darstellung eines IPC-Symbols, wie es in Bagis gespeichert wird Copyright (c) 2005, Eidgenössisches Institut für
 * Geistiges Eigentum
 * 
 * @author A. Rueegsegger
 * @version $id: $
 */

public class IpcSymbol
{
  private String section = "";

  private String ipcclass = "";

  private String subclass = "";

  private String maingroup = "";

  private String subgroup = "";

  private DecimalFormat dfMain = new DecimalFormat( "0000" );

  /**
   * Konvertierung Ipc-Symbol von Epband-File-Format nach Bagis-Format
   * 
   * @param section
   * @param ipcclass
   * @param subclass
   */
  public IpcSymbol( String section, String ipcclass, String subclass )
  {
    this.section = section;
    this.ipcclass = ipcclass;
    this.subclass = subclass;
  }

  /**
   * Konvertierung Ipc-Symbol von Epband-File-Format nach Bagis-Format
   * 
   * @param ipcSymb String Ipc-Symbol wie es im Epband-File gepseichert wird
   * @throws Exception
   */
  public IpcSymbol( String ipcSymb ) throws Exception
  {
    section = ipcSymb.substring( 2, 3 );
    ipcclass = ipcSymb.substring( 4, 6 );
    subclass = ipcSymb.substring( 6, 7 );

    maingroup = ipcSymb.substring( 7, 11 ).replace( ' ', '0' );
    subgroup = ipcSymb.substring( 12, 17 ).trim();
  }

  /**
   * Ein IPC-Symbol kann einen group part beinhalten. Group part ist optional und beinhaltet einen maingroup sowie eine
   * subgroub Maingroup wird rechtsbündig mit führenden Nullen abgespeichert Subgroup wird linksbündig abgespeichert
   * 
   * @param maingroup String gültige Werte sind 1-9999
   * @param subgroup String gültige Werte sind 00-999999
   * @throws Exception
   */
  public void setGroupPart( String maingroup, String subgroup ) throws Exception
  {
    try
    {
      if ( maingroup != null && !maingroup.trim().equals( "" ) )
      {
        this.maingroup = dfMain.format( dfMain.parse( maingroup ) );
      }
      // TODO abklären wie subgroup behandelt werden soll, falls maingroup leer
      if ( subgroup != null && !subgroup.trim().equals( "" ) )
      {
        this.subgroup = subgroup;
      }
    }
    catch ( Exception ex )
    {
      throw new Exception( "invalid group part of symbol" );
    }
  }

  /**
   * Gibt das Ipc-Symbol zurück, wie es in Bagis gespeichert wird
   * 
   * @return String Ipc-Symbol
   */
  public String getIpcSymbol()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( section );
    buf.append( ipcclass );
    buf.append( subclass );
    buf.append( maingroup );
    buf.append( subgroup );

    return buf.toString();
  }

  /**
   * Gibt das Ipc-Symbol als String zurück
   * 
   * @return String Ipc-Symbol als String
   */
  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "  section          = <" + this.section + ">\n" );
    buf.append( "  ipcclass         = <" + this.ipcclass + ">\n" );
    buf.append( "  subclass         = <" + this.subclass + ">\n" );
    buf.append( "  maingroup        = <" + this.maingroup + ">\n" );
    buf.append( "  subgroup         = <" + this.subgroup + ">\n" );

    return buf.toString();
  }

}
