package ch.ige.epband.clientneu;

import java.util.ArrayList;
import java.util.List;

import ch.ige.epband.util.XMLDoc;

/**
 * XML auf die Beschränkungsinformationenen abfragen.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: ListeMutationLimitation.java,v 1.4 2008/05/26 09:02:34 ruegsegger Exp $
 */
public class ListeMutationLimitation
{
  private static final String IH = "<inhalt>";

  private static final String IHE = "</inhalt>";

  /**
   * XML auf die Beschränkungsinformationenen abfragen
   * 
          <B0900EP limitation-sequence="1">

            <B091EP> // Beschränkung (Antrag)
              <B0911EP>
                <date>20081010</date>
              </B0911EP>
              <B0912EP>02</B0912EP>
              <B0913EP>04</B0913EP>
            </B091EP>
            
            <B092EP id="ep-chg0023"> //Beschränkung Zurückweisung
              <B0923EP>
                 <date>20080228</date> // Datum
              </B0923EP>
            </B092EP>
            
            <B093EP> //Beschränkung Gutheissung
              <B0931EP>
                <date>20080228</date> // Absendedatum
              </B0931EP>
            </B093EP>
          </B0900EP>
          
          <B400>
            <B453EP> // Entscheid Beschränkung
              <B4530EP sequence="1" limitation-sequence="1">  //limitation-sequence relates to the number in B090EP
                <kind>B3</kind>
                <date>20080424</date> // Entscheiddatum
                <bnum>200817</bnum>
              </B4530EP>         
            </B453EP>
          </B400>          
          
   *  
   * @param sbInhalt adds content to sbInhalt
   * @param xml for extracting data
   */
  public void genLimitationText( StringBuffer sbInhalt, XMLDoc xml )
  {
    List<String> list = xml.getAttributes( "B0900EP", "limitation-sequence" );

    for ( int i = 0; i < list.size(); i++ )
    {
      // Beschränkungsverfahren eingeleitet
      sbInhalt.append( IH );
      sbInhalt.append( "<ktext>Beschränkung</ktext>" );

      String basePath = "B0900EP[@limitation-sequence=\"" + list.get( i ) + "\"]/B091EP";

      StringBuffer buf = genLimitationFiling( xml, basePath );
      sbInhalt.append( buf );

      // Überprüfen ob Zurückweisung Beschränkung vorhanden
      basePath = "B0900EP[@limitation-sequence=\"" + list.get( i ) + "\"]/B092EP";
      if ( xml.exists( basePath ) )
      {
        sbInhalt.append( IH );
        sbInhalt.append( "<ktext>Beschränkung Zurückweisung</ktext>" );

        buf = genLimitationRejection( xml, basePath );
        sbInhalt.append( buf );
      }

      // Überprüfen ob Gutheissung Beschränkung vorhanden 
      basePath = "B0900EP[@limitation-sequence=\"" + list.get( i ) + "\"]/B093EP";
      if ( xml.exists( basePath ) )
      {
        sbInhalt.append( IH );
        sbInhalt.append( "<ktext>Beschränkung Gutheissung</ktext>" );

        buf = genLimitationAllowed( xml, basePath );
        sbInhalt.append( buf );
      }

      // Überprüfen ob Entscheid Beschränkung vorhanden
      basePath = "B4530EP[@limitation-sequence=\"" + list.get( i ) + "\"]";
      if ( xml.exists( basePath ) )
      {
        sbInhalt.append( IH );

        buf = genLimitationDecision( xml, basePath );
        sbInhalt.append( buf );
      }
    }
  }

  /**
   * Inhalt für Antrag Beschränkung B091EP 
   */
  private StringBuffer genLimitationFiling( XMLDoc xml, String basePath )
  {
    List<String> tags = new ArrayList<String>();
    String value;
    StringBuffer sbInhalt = new StringBuffer();

    tags.add( basePath );
    tags.add( "B0911EP" );
    tags.add( "date" );
    value = xml.getValueOf( tags );
    sbInhalt.append( "<text>Datum: " + Util.stringToPrintDat( value ) + "</text>" );
    sbInhalt.append( IHE );

    //Limitations_kind
    tags.clear();
    tags.add( basePath );
    tags.add( "B0912EP" );
    value = xml.getValueOf( tags );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      if ( value.equals( "01" ) )
      {
        value = value + "; eingereicht innerhalb Einspruchsfrist";
      }
      else if ( value.equals( "02" ) )
      {
        value = value + "; eingereicht nach Einspruchsfrist";
      }
      sbInhalt.append( "<text>Limitations kind: " + value + "</text>" );
      sbInhalt.append( IHE );
    }

    //Decision code
    tags.clear();
    tags.add( basePath );
    tags.add( "B0913EP" );
    value = xml.getValueOf( tags );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      if ( value.equals( "01" ) )
      {
        value = value + "; zulässig";
      }
      else if ( value.equals( "02" ) )
      {
        value = value + "; nicht zulässig";
      }
      else if ( value.equals( "03" ) )
      {
        value = value + "; gilt als nicht eingereicht";
      }
      else if ( value.equals( "04" ) )
      {
        value = value + "; gilt als nicht eingereicht; Einspruch hängig";
      }
      else if ( value.equals( "05" ) )
      {
        value = value + "; zurückgezogen";
      }
      sbInhalt.append( "<text>Decision code: " + value + "</text>" );
      sbInhalt.append( IHE );
    }
    return sbInhalt;
  }

  /**
   * Inhalt für Beschränkung Zurückweisung B092EP 
   */
  private StringBuffer genLimitationRejection( XMLDoc xml, String basePath )
  {
    List<String> tags = new ArrayList<String>();

    String value;
    StringBuffer sbInhalt = new StringBuffer();

    tags.add( basePath );
    tags.add( "B0923EP" );
    tags.add( "date" );
    value = xml.getValueOf( tags );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( "<text>Datum: " + Util.stringToPrintDat( value ) + "</text>" );
    }

    sbInhalt.append( IHE );

    return sbInhalt;
  }

  /**
   * Inhalt für Beschränkung Gutheissung B093EP 
   */
  private StringBuffer genLimitationAllowed( XMLDoc xml, String basePath )
  {
    List<String> tags = new ArrayList<String>();

    String value;
    StringBuffer sbInhalt = new StringBuffer();

    tags.add( basePath );
    tags.add( "B0931EP" );
    tags.add( "date" );
    value = xml.getValueOf( tags );
    sbInhalt.append( "<text>Absendedatum: " + Util.stringToPrintDat( value ) + "</text>" );
    sbInhalt.append( IHE );

    return sbInhalt;
  }

  /**
   * Inhalt für Beschränkung Entscheid B4530EP 
   */
  private StringBuffer genLimitationDecision( XMLDoc xml, String basePath )
  {
    List<String> tags = new ArrayList<String>();

    String value;
    StringBuffer sbInhalt = new StringBuffer();

    tags.add( basePath );
    tags.add( "date" );
    value = xml.getValueOf( tags );
    sbInhalt.append( "<text>Entscheiddatum: " + Util.stringToPrintDat( value ) + "</text>" );
    sbInhalt.append( IHE );

    return sbInhalt;
  }
}
