package ch.ige.epband.clientneu;

import java.util.ArrayList;
import java.util.List;

import ch.ige.epband.util.XMLDoc;

/**
 * Funktionen zur Erstellung der Einträge zum Widerruf-Verfahren 
 * Wird auf Mutations-Liste ausgegeben (ep_003, ep_pat_mut)   
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: ListeMutationRevocation.java,v 1.1 2008/04/08 19:22:32 ruegsegger Exp $
 */
public class ListeMutationRevocation
{
  private static final String IH = "<inhalt>";

  private static final String IHE = "</inhalt>";

  /**
   * XML auf die Widerrufsinformationen abfragen 
        <B012EP>
          <B0120EP> // Widerruf Inhaber beantragt
            <B0121EP>
              <date>20080228</date> // Datum Antrag
            </B0121EP>
            <B0123EP id="ep-chg0001">01</B0123EP> //Entscheid Widerruf 01=zulässig, 02=unzulässig, 03=gilt als nicht eingelegt
            <B0127EP id="ep-chg0002">
              <date>20080313</date> // Datum Entscheid
            </B0127EP>
          </B0120EP>
   * 
   * @param sbInhalt 
   * @param xml
   */
  public void genRevocationText( StringBuffer sbInhalt, XMLDoc xml )
  {
    List<String> tags = new ArrayList<String>();
    String value;

    if ( xml.exists( "B012EP" ) )
    {
      int anz = xml.getNumberOf( "B0120EP" );

      for ( int i = 1; i <= anz; i++ )
      {
        // Widerspruchsverfahren eingeleitet
        sbInhalt.append( IH );
        sbInhalt.append( "<ktext>Widerruf Inhaber beantragt</ktext>" );

        String basePath = "B0120EP[" + i + "]";

        // Datum Antrag aus B0121EP
        tags.clear();
        tags.add( basePath );
        tags.add( "B0121EP" );
        tags.add( "date" );
        value = xml.getValueOf( tags );
        if ( value != null && !value.equals( "" ) )
        {
          sbInhalt.append( "<text>Datum Antrag: " + Util.stringToPrintDat( value ) + "</text>" );
        }
        sbInhalt.append( IHE );

        // Entscheid Widerruf aus B0123EP
        tags.clear();
        tags.add( basePath );
        tags.add( "B0123EP" );
        value = xml.getValueOf( tags );
        if ( value != null && !value.equals( "" ) )
        {
          sbInhalt.append( IH );

          if ( value.equals( "01" ) )
          {
            value = value + "; zulässig";
          }
          else if ( value.equals( "02" ) )
          {
            value = value + "; unzulässig";
          }
          else if ( value.equals( "03" ) )
          {
            value = value + "; gilt als nicht eingelegt";
          }

          sbInhalt.append( "<text>Entscheid Widerruf: " + value + "</text>" );
          sbInhalt.append( IHE );
        }

        // Datum Entscheid aus B0127EP
        tags.clear();
        tags.add( basePath );
        tags.add( "B0127EP" );
        tags.add( "date" );
        value = xml.getValueOf( tags );
        if ( value != null && !value.equals( "" ) )
        {
          sbInhalt.append( IH );
          sbInhalt.append( "<text>Datum Entscheid: " + Util.stringToPrintDat( value ) + "</text>" );
          sbInhalt.append( IHE );
        }
      }
    }
  }
}
