package ch.ige.epband.clientneu.test;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.epband.clientneu.ParseSGML;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreignisVO;
import ch.ige.epband.vo.PrioritaetVO;
import ch.ige.epband.vo.SchutitRegadrVO;
import ch.ige.util.IGEDate;
import ch.ige.util.vo.RegadrVO;

/**
 * Testfälle für EP
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author wild
 * @version $Id: TestEP.java,v 1.26 2008/07/10 10:16:48 wild Exp $
 */
public class TestEP extends TestCase
{
  /**
   * Lokale Variable EpbandVO
   * Braucht es meiner Meinung nicht (wlm)
   */
//  public static EpbandVO epbandVO;
  /**
   * Lokale Liste für Länder
   */
  public static List<String> blandWork;

  /**
   * Lokale Liste für IPC
   */
  public static List<String> ipcWork;

  private SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );

  private String currentDate = null;

  /**
   * Initialisieren der Klasse
   * 
   * @param s
   */
  public TestEP( String s )
  {
    super( s );
    sdf.setLenient( false );
    BasicConfigurator.configure();

  }

  @Override
  protected void setUp()
  {
    blandWork = CreateTestpatent.createblandWork();
    ipcWork = CreateTestpatent.createipcWork();
    currentDate = sdf.format( new Date() );
  }

  /**
   * Testmethode Parsen von Gesuch
   * 
   * @throws ParseException
   */
  public void test00DoParseSGMLGesuch() throws ParseException
  {
    EpbandVO epbandVO = new EpbandVO();
    String gesuch = CreateTestpatent.createtestgesuch();

    try
    {
      XMLDoc xml = new XMLDoc( gesuch );
      System.out.println( gesuch );

      ParseSGML parsesgml = new ParseSGML( blandWork );
      parsesgml.setData( epbandVO, xml );
      parsesgml.doParseSGML();
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }

    assertTrue( "Wipo falsch erkannt", !epbandVO.wipo );
    assertTrue( "Kein Einspruch falsch erkannt", !epbandVO.keinEinspruch );
    assertTrue( "CH Zurücknahme falsch erkannt", !epbandVO.chzuruecknahme );

    assertEquals( "SchutitKey falsch", 0, epbandVO.schutitVO.schutitKey );
    assertEquals( "MaNr falsch", 199, epbandVO.schutitVO.maNr.intValue() );
    assertEquals( "SchutitTyp falsch", 2, epbandVO.schutitVO.schutitTyp );
    assertEquals( "Sprache falsch", 1, epbandVO.schutitVO.sprache );
    assertEquals( "GesuchNr falsch", "968107847", epbandVO.schutitVO.gesuchNr );
    assertEquals( "SchutitNr falsch", "008426405", epbandVO.schutitVO.schutitNr );
    assertEquals( "SchutzBegin", IGEDate.stringToDate( "13.11.1996" ), epbandVO.schutitVO.schutzBegin );
    assertEquals( "SchutzEnde", IGEDate.stringToDate( "12.11.2016" ), epbandVO.schutitVO.schutzEnde );
    assertEquals( "LöschDat", IGEDate.stringToDate( "23.05.2002" ), epbandVO.schutitVO.loeschDat );
    assertEquals( "Beschreibung", "Herzkatheter mit Elektrode auf aufspannbarer Einrichtung", epbandVO.schutitVO.beschreibung );
    assertEquals( "ErfassMa falsch", 199, epbandVO.schutitVO.erfassMa );
    assertEquals( "Erfassdat", currentDate, sdf.format( epbandVO.schutitVO.erfassDat ) );
    assertNull( "MutMa falsch", epbandVO.schutitVO.mutMa );
    assertNull( "MutDat falsch", epbandVO.schutitVO.mutDat );
    assertEquals( "ErsEregIdat falsch", IGEDate.stringToDate( "13.11.1996" ), epbandVO.schutitVO.erstEreigDat );
    assertEquals( "FälligJahr falsch", 2300, epbandVO.schutitVO.faelligJahr );

    assertEquals( "PatVerfahr falsch", 6, epbandVO.patVO.patVerfahr );
    assertNull( "PctanmNr falsch", epbandVO.patVO.pctAnmNr );
    assertEquals( "Pctanmsprache falsch", "", epbandVO.patVO.pctAnmSpra );
    assertNull( "PctpublNr falsch", epbandVO.patVO.pctPublNr );
    assertNull( "PctpublSpra falsch", epbandVO.patVO.pctPublSpra );
    assertNull( "PctPublDat falsch", epbandVO.patVO.pctPublDat );
    assertEquals( "EpanmSpra falsch", "de", epbandVO.patVO.epAnmSpra );
    assertEquals( "EpapublSpra falsch", "de", epbandVO.patVO.epApublSpra );
    assertEquals( "EpaPublDat falsch", IGEDate.stringToDate( "20.05.1998" ), epbandVO.patVO.epApublDat );
    assertNull( "EpbpublSpra falsch", epbandVO.patVO.epBpublSpra );
    assertNull( "EpBpublDat", epbandVO.patVO.epBpublDat );
    assertEquals( "EpStammPatNr falsch", "006296719", epbandVO.patVO.epStammpatNr );

    assertEquals( "Anzahl Ereignisse falsch", 3, epbandVO.ereigAL.size() );
    EreignisVO ereignisVO = new EreignisVO();
    ereignisVO = epbandVO.ereigAL.get( 0 );
    // Anmeldung
    assertEquals( "EreiIdat bei 20000", 1996111305, ereignisVO.ereigIdat );
    assertEquals( "MaNr bei Ereignis falsch", 199, ereignisVO.maNr );
    assertNull( "PublKey not Null", ereignisVO.publKey );
    assertEquals( "EreigSatNr bei 20000 falsch", 20, ereignisVO.ereigStatNr );
    assertNull( "ImaNr falsch", ereignisVO.iMaNr );
    assertEquals( "Ereignis 20000 falsch", "20000", ereignisVO.artNr );
    assertNull( "AdrNr falsch", ereignisVO.adrNr );

    assertNull( "FristablDat bei 20000 falsch", ereignisVO.fristablDat );
    assertNull( "Abschlussdatum falsch", ereignisVO.abschlDat );
    assertNull( "Abschlussmutdatum falsch", ereignisVO.abschlmutDat );
    assertEquals( "Erfassdat bei 20000 falsch", currentDate, sdf.format( ereignisVO.erfassDat ) );

    assertNull( "PmmblSeite falsch", ereignisVO.pmmblSeite );
    assertNull( "PmmblJahr falsch", ereignisVO.pmmblJahr );
    assertEquals( "KzKorekt falsch", 0, ereignisVO.kzKorrekt );
    assertEquals( "RegStat 20000 falsch", 8, ereignisVO.regStat );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );

    // Zurücknahme
    ereignisVO = epbandVO.ereigAL.get( 1 );
    assertEquals( "Ereignis 20235 falsch", "20235", ereignisVO.artNr );
    assertEquals( "ereig_idat 20235 falsch", 2002052305, ereignisVO.ereigIdat );
    assertEquals( "EreigSatNr bei 20235 falsch", 20, ereignisVO.ereigStatNr );
    assertNull( "FristablDat bei 20235 falsch", ereignisVO.fristablDat );
    assertEquals( "Erfassdat bei 20235 falsch", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertEquals( "RegStat 20235 falsch", 8, ereignisVO.regStat );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );

    // Lizenznehmer
    ereignisVO = epbandVO.ereigAL.get( 2 );
    assertEquals( "Ereignis 27070 falsch", "27070", ereignisVO.artNr );
    assertEquals( "ereig_idat 27070 falsch", 2002042705, ereignisVO.ereigIdat );
    assertEquals( "EreigSatNr bei 27070 falsch", 20, ereignisVO.ereigStatNr );
    assertNull( "FristablDat bei 27070 falsch", ereignisVO.fristablDat );
    assertEquals( "Erfassdat bei 27070 falsch", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertEquals( "RegStat 27070 falsch", 8, ereignisVO.regStat );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );

    // EreigTxT
    assertEquals( "EreigTxT falsch", 1, epbandVO.ereigTxtAL.size() );
    assertEquals( "EreigTxT falsch", "Ausschliessliche Lizenz", epbandVO.ereigTxtAL.get( 0 ).ereigTxt );

    // Priorität
    assertEquals( "Anzahl Priorität falsch", 2, epbandVO.prioAL.size() );
    PrioritaetVO prioVO = new PrioritaetVO();
    prioVO = epbandVO.prioAL.get( 0 );
    assertEquals( "PrioLaufNr falsch", 1, prioVO.prioLnr );
    assertEquals( "RrioBland falsch", "AU", prioVO.blandCd );
    assertEquals( "PrioNr falsch", "PP904799", prioVO.prioNr );
    assertEquals( "PrioDat falsch", IGEDate.stringToDate( "04.03.1999" ), prioVO.prioDat );
    assertEquals( "RegStat Prio", 8, prioVO.regStat );

    prioVO = epbandVO.prioAL.get( 1 );
    assertEquals( "PrioLaufNr falsch", 2, prioVO.prioLnr );
    assertEquals( "RrioBland falsch", "AU", prioVO.blandCd );
    assertEquals( "PrioNr falsch", "PQ329499", prioVO.prioNr );
    assertEquals( "PrioDat falsch", IGEDate.stringToDate( "06.10.1999" ), prioVO.prioDat );
    assertEquals( "RegStat Prio", 8, prioVO.regStat );

    // PatIpc
    assertEquals( "Anzahl IPC falsch", 0, epbandVO.patIpcAL.size() );

    // RegAdr
    assertEquals( "Anzahl Regadr falsch", 5, epbandVO.regAdrAL.size() );

    RegadrVO regadrVO = epbandVO.regAdrAL.get( 0 );
    assertEquals( "HerkAdrNr 1 falsch", 2253470, regadrVO.getHerkAdrNr() );
    assertEquals( "RegName 1 falsch", "Sulzer Osypka GmbH", regadrVO.getRegNameOrEmptyString() );
    assertEquals( "RegStr 1 falsch", "Basler Strasse 109", regadrVO.getRegStrasseOrEmptyString() );
    assertEquals( "RegOrt 1 falsch", "79639 Grenzach-Wyhlen", regadrVO.getRegOrtOrEmptyString() );
    assertEquals( "BlandCd 1 falsch", "DE", regadrVO.getBlandCdOrEmptyString() );
    assertEquals( "Herkunft", 1, regadrVO.getHerkunft() );
    assertEquals( "AdrNr 1", 0, regadrVO.getAdrNr() );
    assertEquals( "Gesperrt 1", 0, regadrVO.getGesperrt() );

    // SchutitRegadr
    assertEquals( "Anzahl SchutitRegadrVO falsch", 6, epbandVO.schutitregAdrAL.size() );

    SchutitRegadrVO srVO = epbandVO.schutitregAdrAL.get( 0 );
    assertEquals( "SchutAdrLnr 1", 0, srVO.schutadrLnr );
    assertEquals( "SchutitTyp 1", 2, srVO.schutitTyp );
    assertEquals( "AdrRolleCd 1", 1, srVO.adrRolleCd );
    assertEquals( "VonEreigIdat 1", 1996111305, srVO.vonEreigIdat );
    assertEquals( "Regstat 1", 8, srVO.regStat );
    assertEquals( "RegadrNr 1", 0, srVO.regadrNr );

  }

  /**
   * Testmethode parsing von EP
   * @throws ParseException
   */
  public void test01DoParseSGMLEP() throws ParseException
  {

    EpbandVO epbandVO = new EpbandVO();

    String ep = CreateTestpatent.createEPEintragung();

    try
    {
      XMLDoc xml = new XMLDoc( ep );
      ParseSGML parsesgml = new ParseSGML( blandWork );
      parsesgml.setData( epbandVO, xml );
      parsesgml.doParseSGML();
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
      ex.printStackTrace();
    }

    // Es werden nur noch die unterschiede zu Gesuch gestestet,
    // da das Gesuch doch sehr ausführlich getestet wurde

    // Schutit
    assertEquals( "RegisterKZ", 1, epbandVO.schutitVO.registerKz );
    assertEquals( "Faelligjahr", 2003, epbandVO.schutitVO.faelligJahr );

    // Pat
    assertEquals( "EpbpublDat", IGEDate.stringToDate( "17.07.2002" ), epbandVO.patVO.epBpublDat );
    assertEquals( "EpbpublSpra", epbandVO.patVO.epBpublSpra, epbandVO.patVO.epApublSpra );
    assertEquals( "EpbpublSpra 2", "en", epbandVO.patVO.epBpublSpra );

    // Ereignis
    EreignisVO ereignisVO = new EreignisVO();
    ereignisVO = epbandVO.ereigAL.get( 0 );
    assertEquals( "Pat 20000", "20000", ereignisVO.artNr );

    // Eintragung testen
    ereignisVO = epbandVO.ereigAL.get( 1 );
    assertEquals( "Anzahl Ereignisse falsch", 3, epbandVO.ereigAL.size() );

    assertEquals( "EreiIdat bei 20010", 2002071705, ereignisVO.ereigIdat );
    assertEquals( "MaNr bei 20010", 199, ereignisVO.maNr );
    assertNull( "PublKey not Null bei 20010", ereignisVO.publKey );
    assertEquals( "EreigSatNr bei 20010", 20, ereignisVO.ereigStatNr );
    assertNull( "ImaNr bei 20010", ereignisVO.iMaNr );
    assertEquals( "Ereignis 20010", "20010", ereignisVO.artNr );
    assertNull( "AdrNr bei 20010", ereignisVO.adrNr );
    assertNull( "FristablDat bei 20010", ereignisVO.fristablDat );
    assertNull( "Abschlussdatum bei 20010", ereignisVO.abschlDat );
    assertNull( "Abschlussmutdatum bei 20010", ereignisVO.abschlmutDat );
    assertEquals( "Erfassdat bei 20010", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertNull( "PmmblSeite bei 20010", ereignisVO.pmmblSeite );
    assertNull( "PmmblJahr bei 20010", ereignisVO.pmmblJahr );
    assertEquals( "KzKorekt bei 20010", 0, ereignisVO.kzKorrekt );
    assertEquals( "RegStat bei 20010", 9, ereignisVO.regStat );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", EreignisVO.PUBLSTAT_READY_PUBLICATE, ereignisVO.publStat );

    // Uebersetzung
    ereignisVO = epbandVO.ereigAL.get( 2 );
    assertEquals( "Ereignis 20140", "20140", ereignisVO.artNr );
    assertEquals( "ereig_idat 20140", 2002071710, ereignisVO.ereigIdat );
    assertEquals( "EreigSatNr bei 20140", 20, ereignisVO.ereigStatNr );
    assertEquals( "FristablDat bei 20140", null, ereignisVO.fristablDat );
    assertEquals( "Erfassdat bei 20140", currentDate, sdf.format( ereignisVO.erfassDat ) );
    assertEquals( "RegStat 20140", 9, ereignisVO.regStat );
    assertNull( "publDat", ereignisVO.publDat );
    assertEquals( "publStat", 0, ereignisVO.publStat );

  }
}
