package ch.ige.epband.clientneu.test;

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.epband.clientneu.IpcrClassification;
import ch.ige.epband.clientneu.ParseSGML;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.PatIpcVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author unbekannt
 * @version $id: $
 */

public class TestIpcrClassifiation extends TestCase
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  /**
   * Construktor
   * 
   * @param s
   */
  public TestIpcrClassifiation( String s )
  {
    super( s );
    BasicConfigurator.configure();
  }

  /**
   * Testmethode
   */
  public void test00()
  {

    String ipcr = CreateTestpatent.createIpcrClassificationText();

    EpbandVO epbandVO = new EpbandVO();

    try
    {
      XMLDoc xml = new XMLDoc( ipcr );
      ParseSGML parsesgml = new ParseSGML( null );
      parsesgml.setData( epbandVO, xml );

      // Patentart und Änderungsstatus ermitteln
      epbandVO.epArt = xml.getAttribute( "ep-bulletin", "kind" );
      epbandVO.epStatus = xml.getAttribute( "ep-bulletin", "status" );

      IpcrClassification cl = new IpcrClassification( xml, epbandVO );

      ArrayList<PatIpcVO> al = cl.getPatIpcVOs();

      // check default values
      PatIpcVO vo = al.get( 0 );
      assertEquals( "A01B", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "A", vo.classLevel );
      assertEquals( "I", vo.classValue );
      assertEquals( "B", vo.geschichte );
      assertEquals( "H", vo.source );
      assertEquals( "CH", vo.generatingOffice );
      assertEquals( sdf.parse( "19961113" ), vo.erfassDat );
      assertEquals( 199, vo.erfassMa );

      vo = al.get( 1 );
      assertEquals( "A01N002500", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );

      // check ungültig werte
      vo = al.get( 2 );
      assertEquals( "A01B", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 3, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );
      assertEquals( sdf.parse( "19961113" ), vo.erfassDat );

      // check H, non-inventive
      vo = al.get( 3 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 2, vo.patIpcCd );
      assertEquals( 4, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      // check N, i.O
      vo = al.get( 4 );
      assertEquals( "A01B000102", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );

      // check N, indexed
      vo = al.get( 5 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 3, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "I", vo.classValue );

      // check N, non-inventive, indexed
      vo = al.get( 6 );
      assertEquals( "A01D001100", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 4, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      // check N, non-inventive, indexed
      vo = al.get( 7 );
      assertEquals( "A01C", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 4, vo.patIpcCd );
      assertEquals( 2, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      // check N, non-inventive, not-indexed
      vo = al.get( 8 );
      assertEquals( "C12R0001125", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 5, vo.patIpcCd );
      assertEquals( 1, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      // check N, non-inventive, not-indexed
      vo = al.get( 9 );
      assertEquals( "A01N002500", vo.ipcSymb );
      assertEquals( sdf.parse( "20060101" ), vo.ipcGueltigVon );
      assertEquals( 4, vo.patIpcCd );
      assertEquals( 3, vo.patIpcLnr );
      assertEquals( "N", vo.classValue );

      assertTrue( al.size() == 10 );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }

  }
}
