package ch.ige.epband.exception;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Bei der Verarbeitung eines Patents ist ein Fehler aufgetreten
 * </p>
 * Kein Abbruch des Programms nötig, Patent wird übersprungen und Verarbeitung fortgesetzt. 
 * Falls log = true wird der Fehler geloggt Betreffend Verarbeitungsfehler können 
 * beim EPBand folgende drei Fälle auftreten:
 * Fall 1: Fehlermeldung loggen und Patent überspringen -> throw new ProcessingException(message); 
 * Fall 2: Datensatz überspringen (da z.B.  * nicht relevant für CH) -> throw new ProcessingException();
 * Fall 3: Meldung loggen jedoch Verarbeitung forsetzen ->  * LOG.warn(message)
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: ProcessingException.java,v 1.10 2008/03/10 15:57:10 wild Exp $
 */

public class ProcessingException extends Exception
{
  /**
   *  Defaultwert für die serialVersionUID
   */
  private static final long serialVersionUID = 1L;

  /**
   * Default: true -> Fehlermeldung loggen
   */
  public boolean log = false;

  /**
   * Erstellt eine Exception ohne Detail-Meldung
   */
  public ProcessingException()
  {
    super();
  }

  /**
   * Erstellt eine ProcessingException mit Detail-Meldung
   * 
   * @param message String für Fehlermeldung
   */
  public ProcessingException( String message )
  {
    this( message, true );
  }

  /**
   * Erstellt eine ProcessingException mit Detail-Meldung
   * 
   * @param message String für Fehlermeldung
   * @param log
   */
  public ProcessingException( String message, boolean log )
  {
    super( message );
    this.log = log;
  }
}
