package ch.ige.epband.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import ch.ige.epband.vo.EreignisVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für EREIGNIS
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Michael Zaugg
 * @version $Id: EreignisDAO.java,v 1.16 2008/02/28 14:58:03 wild Exp $
 */

public class EreignisDAO
{

  /**
   * Fügt einen Datensatz in die Tabelle EREIGNIS ein.
   * @param ereignisVO mit den Daten des Ereignisses
   * @throws SQLException
   * @return int
   */
  public int insert( EreignisVO ereignisVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO ereignis(" + "schutit_key, " + "ereig_idat, " + "ma_nr, " + "publ_key, " + "ereig_stat_nr, " + "i_ma_nr, " + "art_nr, " + "adr_nr, "
          + "fristabl_dat, " + "abschl_dat, " + "abschlmut_dat, " + "erfass_dat, " + "pmmbl_seite, " + "pmmbl_jahr, " + "kz_korrekt, " + "reg_stat, "
          + "publ_dat, " + "publ_stat, " + "publ_ziel, " + "version) " + "VALUES (" + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, "
          + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?, " + "?) ";

      ps = con.prepareStatement( query );
      ps.setInt( 1, ereignisVO.schutitKey );
      ps.setInt( 2, ereignisVO.ereigIdat );
      ps.setInt( 3, ereignisVO.maNr );
      if ( ereignisVO.publKey == null )
      {
        ps.setNull( 4, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 4, ereignisVO.publKey.intValue() );
      }
      ps.setInt( 5, ereignisVO.ereigStatNr );
      if ( ereignisVO.iMaNr == null )
      {
        ps.setNull( 6, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 6, ereignisVO.iMaNr.intValue() );
      }
      ps.setString( 7, ereignisVO.artNr );
      if ( ereignisVO.adrNr == null )
      {
        ps.setNull( 8, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 8, ereignisVO.adrNr.intValue() );
      }
      if ( ereignisVO.fristablDat == null )
      {
        ps.setDate( 9, null );
      }
      else
      {
        ps.setDate( 9, new java.sql.Date( ereignisVO.fristablDat.getTime() ) );
      }
      if ( ereignisVO.abschlDat == null )
      {
        ps.setDate( 10, null );
      }
      else
      {
        ps.setDate( 10, new java.sql.Date( ereignisVO.abschlDat.getTime() ) );
      }
      if ( ereignisVO.abschlmutDat == null )
      {
        ps.setDate( 11, null );
      }
      else
      {
        ps.setDate( 11, new java.sql.Date( ereignisVO.abschlmutDat.getTime() ) );
      }
      if ( ereignisVO.erfassDat == null )
      {
        ps.setDate( 12, null );
      }
      else
      {
        ps.setDate( 12, new java.sql.Date( ereignisVO.erfassDat.getTime() ) );
      }
      if ( ereignisVO.pmmblSeite == null )
      {
        ps.setNull( 13, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 13, ereignisVO.pmmblSeite.intValue() );
      }

      if ( ereignisVO.pmmblJahr == null )
      {
        ps.setNull( 14, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 14, ereignisVO.pmmblJahr.intValue() );
      }
      ps.setInt( 15, ereignisVO.kzKorrekt );
      ps.setInt( 16, ereignisVO.regStat );
      if ( ereignisVO.publDat == null )
      {
        ps.setDate( 17, null );
      }
      else
      {
        ps.setDate( 17, new java.sql.Date( ereignisVO.publDat.getTime() ) );
      }
      ps.setInt( 18, ereignisVO.publStat );
      ps.setInt( 19, ereignisVO.publZiel );
      ps.setInt( 20, ereignisVO.version );
      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, EreignisDAO.class, query + ereignisVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Überprüft ob die übergebene Artikelnummer vorhanden ist
   * @param schutitKey  PrimaryKey
   * @param artNr       gesuchte ArtikelNr
   * @return true falls ArtikelNr vorhanden ist
   * @throws SQLException
   */
  public boolean existArtNr( int schutitKey, String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT schutit_key " + "FROM   ereignis " + "WHERE schutit_key = ? " + "AND art_nr = RPAD( ?, 18, ' ' ) ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.setString( 2, artNr );

      rs = ps.executeQuery();

      return rs.next();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, EreignisDAO.class, query + " schutitKey " + schutitKey + " artNr " + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * das nächste gültige Ereignisdatum berechnen
   * @param schutitKey
   * @param ereigIdat
   * @return int
   * @throws SQLException
   */
  public int getNextereigIdat( int schutitKey, int ereigIdat ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT max(ereig_idat) maxidat " + "FROM ereignis " + "WHERE schutit_key = ? " + "AND ereig_idat >= ? " + "AND ereig_idat <= ? ";
      // TODO evtl between 
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.setInt( 2, ( ereigIdat / 100 ) * 100 );
      ps.setInt( 3, ( ( ereigIdat / 100 ) * 100 ) + 99 );

      rs = ps.executeQuery();

      int nextereigIdat = 0;
      if ( rs.next() )
      {
        nextereigIdat = rs.getInt( "maxidat" );
      }

      if ( nextereigIdat == 0 )
      {
        nextereigIdat = ( ( ereigIdat / 100 ) * 100 ) + 5;
      }
      else if ( ( nextereigIdat % 100 ) < 80 )
      {
        nextereigIdat = nextereigIdat + 5;
      }
      else
      {
        nextereigIdat = nextereigIdat + 1;
      }

      return nextereigIdat;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, EreignisDAO.class, query + " schutitKey: " + schutitKey + " ereigIdat: " + ereigIdat );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Zwischenspeichern bestimmter Ereignisse
   * statcd = 2
   * @param schutitKey  des betr. EP
   * @return ArrayList mit den gefundenen Ereignisse
   * @throws SQLException
   */
  public ArrayList<EreignisVO> getoldEreignis( int schutitKey ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<EreignisVO> oldEreigAL = new ArrayList<EreignisVO>();
    EreignisVO ereignisVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT e.schutit_key, " + "e.ereig_idat, " + "e.ma_nr, " + "e.publ_key, " + "e.ereig_stat_nr, " + "e.i_ma_nr, " + "e.art_nr, " + "e.adr_nr, "
          + "e.fristabl_dat, " + "e.abschl_dat, " + "e.abschlmut_dat, " + "e.erfass_dat, " + "e.pmmbl_seite, " + "e.pmmbl_jahr, " + "e.kz_korrekt, "
          + "e.reg_stat, " + "e.version " + "FROM   ereignis e, artikel a " + "WHERE  e.schutit_key = ? " + "AND e.art_nr = a.art_nr "
          + "AND a.statcd_2 = '2' ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      rs = ps.executeQuery();

      while ( rs.next() )
      {
        ereignisVO = new EreignisVO();
        ereignisVO.schutitKey = rs.getInt( "schutit_key" );
        ereignisVO.ereigIdat = rs.getInt( "ereig_idat" );
        ereignisVO.maNr = rs.getInt( "ma_nr" );
        ereignisVO.publKey = new Integer( rs.getInt( "publ_key" ) );
        if ( rs.wasNull() )
        {
          ereignisVO.publKey = null;
        }
        ereignisVO.ereigStatNr = rs.getInt( "ereig_stat_nr" );
        ereignisVO.iMaNr = new Integer( rs.getInt( "i_ma_nr" ) );
        if ( rs.wasNull() )
        {
          ereignisVO.iMaNr = null;
        }
        ereignisVO.artNr = rs.getString( "art_nr" );
        if ( rs.wasNull() )
        {
          ereignisVO.artNr = null;
        }
        ereignisVO.adrNr = new Integer( rs.getInt( "adr_nr" ) );
        if ( rs.wasNull() )
        {
          ereignisVO.adrNr = null;
        }
        ereignisVO.fristablDat = rs.getDate( "fristabl_dat" );
        ereignisVO.abschlDat = rs.getDate( "abschl_dat" );
        ereignisVO.abschlmutDat = rs.getDate( "abschlmut_dat" );
        ereignisVO.erfassDat = rs.getDate( "erfass_dat" );
        ereignisVO.pmmblSeite = new Integer( rs.getInt( "pmmbl_seite" ) );
        if ( rs.wasNull() )
        {
          ereignisVO.pmmblSeite = null;
        }
        ereignisVO.pmmblJahr = new Integer( rs.getInt( "pmmbl_jahr" ) );
        if ( rs.wasNull() )
        {
          ereignisVO.pmmblJahr = null;
        }
        ereignisVO.kzKorrekt = rs.getInt( "kz_korrekt" );
        ereignisVO.regStat = rs.getInt( "reg_stat" );
        ereignisVO.version = rs.getInt( "version" );
        oldEreigAL.add( ereignisVO );
      }
      return oldEreigAL;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, EreignisDAO.class, query + ereignisVO == null ? "" : ereignisVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

}
