package ch.ige.epband.util.test;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;
import ch.ige.epband.util.XMLDoc;

/**
 * Test der Klasse XMLAccess
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class TestXMLDoc extends TestCase
{
  private XMLDoc xml;

  private XMLDoc xml2;

  private XMLDoc xml3;

  private String filename = "C:\\jp\\projects\\epband\\testfiles\\test-gesuch.xml";

  private String filename2 = "C:\\jp\\projects\\epband\\testfiles\\test-liste-limitation.xml";

  private String filename3 = "C:\\jp\\projects\\epband\\testfiles\\test-liste-revocation.xml";

  public TestXMLDoc( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    try
    {
      FileInputStream fi = new FileInputStream( filename );
      xml = new XMLDoc( fi );
      fi = new FileInputStream( filename2 );
      xml2 = new XMLDoc( fi );
      fi = new FileInputStream( filename3 );
      xml3 = new XMLDoc( fi );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
  }

  public void test00()
  {
    xml.getText( "ep-chg0001" );
  }

  public void test01()
  {
    StringBuffer xmlString = new StringBuffer();
    String line;

    try
    {
      BufferedReader in = new BufferedReader( new FileReader( filename ) );
      while ( ( line = in.readLine() ) != null )
      {
        xmlString.append( line );
      }
      in.close();

      xml = new XMLDoc( xmlString.toString() );
    }
    catch ( Exception ex )
    {
      fail( ex.getMessage() );
    }
  }

  public void test03getValue()
  {
    String value = xml.getValueOf( "B210" );
    assertEquals( "96810784.7", value );
  }

  public void test04getValue()
  {
    String value = xml.getValueOf( "B840", "ctry" );
    assertEquals( "CH", value );
  }

  public void test05getValues()
  {
    List<String> list = xml.getValuesOf( "B840", "ctry" );
    assertTrue( list.contains( "LI" ) );
  }

  public void test06getAttribute()
  {
    String value = xml.getAttribute( "ep-bulletin", "file" );
    assertEquals( "96810784.7", value );
  }

  public void test06getAttribute2()
  {
    String value = xml.getAttribute( "ep-chg", "btag" );
    assertEquals( "B620EP", value );
  }

  public void test07getAttribute()
  {
    Map<String, String> map = xml.getAttributesOfElement( "ep-bulletin" );
    String value = map.get( "date-publ" );
    assertEquals( "20020717", value );
  }

  public void test08getValues()
  {
    String value = xml.getValueOf( "B542" );
    assertEquals( "Herzkatheter mit Elektrode auf aufspannbarer Einrichtung", value );
  }

  public void test09getValues()
  {
    String value = xml.getValueOf( "B999", "dkdk" );
    assertTrue( value.equals( "" ) );
  }

  public void test10Limitations()
  {
    if ( xml2.exists( "B090EP", "B0900EP" ) )
    {
      List<String> list = xml2.getValuesOf( "B090EP", "B0900EP" );
      assertEquals( list.size(), 4 );
    }
    else
    {
      fail();
    }
  }

  public void test11Limitations()
  {
    String result = xml2.getValueOf( "B0911EP", "date" );
    assertEquals( result, "20080914" );
  }

  public void test12Limittations()
  {
    List<String> list = new ArrayList<String>();
    list.add( "B0900EP[@limitation-sequence=\"2\"]" );
    list.add( "B091EP" );
    list.add( "B0911EP" );
    list.add( "date" );

    String result = xml2.getValueOf( list );
    assertEquals( "20081009", result );
  }

  public void test13Limittations()
  {
    List<String> list = xml2.getAttributes( "B0900EP", "limitation-sequence" );
    assertEquals( "2", list.get( 1 ) );
  }

  public void test14Revocations()
  {
    List<String> list = new ArrayList<String>();
    list.add( "B0120EP[1]" );
    list.add( "B0121EP" );
    list.add( "date" );

    String result = xml3.getValueOf( list );
    assertEquals( "20080228", result );
  }

  public void test15Revocations()
  {
    List<String> list = new ArrayList<String>();
    list.add( "B0120EP[2]" );
    list.add( "B0127EP" );
    list.add( "date" );

    String result = xml3.getValueOf( list );
    assertEquals( "20070313", result );
  }
}
