package ch.ige.image.format;

import javax.imageio.metadata.IIOMetadata;

import org.apache.log4j.Logger;

import ch.ige.image.Image;

/**
 * GIF Format.
 * <p>
 * Note: GIF does not support resolution metadata.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: GIFFormat.java,v 1.13 2009/03/19 13:26:20 locher Exp $
 */
public class GIFFormat extends Format
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.13 $".hashCode();

  public static final String IDENTIFIER = "gif";

  public static final String MIME = "image/gif";

  private static final Logger LOG = Logger.getLogger( GIFFormat.class );

  public GIFFormat()
  {
    super( IDENTIFIER );
  }

  @Override
  public String getMIMEType()
  {
    return MIME;
  }

  @Override
  public EncodingAdapter getEncodingAdapter( final Image image )
  {
    return new Format.EncodingAdapter()
    {

      @Override
      public void adaptMetadata( ConversionContext conversionContext, IIOMetadata streamMetadata, IIOMetadata imageMetadata )
      {
        if ( conversionContext != null )
        {
          try
          {
            this.mergeMetadata( imageMetadata, "javax_imageio_gif_image_1.0", createConversionContext( conversionContext ) );
          }
          catch ( Exception e )
          {
            LOG.info( "merge of conversion metadata failed", e );
            throw new RuntimeException( e );
          }
        }
      }

      private String createConversionContext( ConversionContext conversionContext )
      {
        StringBuilder mergeDoc = new StringBuilder( "<javax_imageio_gif_image_1.0><CommentExtensions><CommentExtension value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_METADATA );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getOriginalMetadata() );
        mergeDoc.append( "'/><CommentExtension value='" );
        mergeDoc.append( Format.KEYWORD_MAC );
        mergeDoc.append( ":" );
        mergeDoc.append( EmbeddedHashes.getTemplate( null ) );
        mergeDoc.append( "'/><CommentExtension value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_RULE_NAME );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getRulesName() );
        mergeDoc.append( "'/></CommentExtensions></javax_imageio_gif_image_1.0>" );
        return mergeDoc.toString();
      }
    };
  }

}
