package ch.ige.image.format.png.test;

import java.io.IOException;
import java.net.URL;
import java.nio.channels.WritableByteChannel;

import junit.framework.TestCase;
import ch.ige.image.EncodedImage;
import ch.ige.image.ImageFactoryImpl;
import ch.ige.image.format.png.PNGChunkFilter;

/**
 * Test for {@link PNGChunkFilter}.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: PNGChunkFilterTest.java,v 1.4 2009/03/19 13:26:23 locher Exp $
 */
public class PNGChunkFilterTest extends TestCase
{

  private PNGChunkFilter passthrough;

  private PNGChunkFilter updateTimeAndAddComment;

  public PNGChunkFilterTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    this.passthrough = new PNGChunkFilter()
    {

      // @see ch.ige.image.format.png.PNGFilter#receiveChunk(ch.ige.image.format.png.PNGFilter.EmitableChunk)
      @Override
      protected void handleChunk( EmitableChunk in ) throws IOException
      {
        super.handleChunk( in );
      }
    };

    this.updateTimeAndAddComment = new PNGChunkFilter()
    {
      @Override
      protected void handleChunk( EmitableChunk in ) throws IOException
      {
        if ( !in.isType( Chunk.TIME_TYPE ) )
        {
          in.emit();
        }
      }

      @Override
      protected void beforeLast( WritableByteChannel out ) throws IOException
      {
        EmitableChunk.createTimeChunkNow( out ).emit();
        EmitableChunk.createTextChunk( "IPI_Signature", "foo", out ).emit();
      }

    };

  }

  public void testPassthrough() throws Exception
  {
    EncodedImage orig = getCircle();
    EncodedImage result = this.passthrough.filter( orig );
    //result.write( new File( "c:/test.png" ) );
    assertEquals( orig.getDataBuffer(), result.getDataBuffer() );
  }

  public void testUpdateTimeAndAddComment() throws Exception
  {
    EncodedImage orig = getCircle();
    EncodedImage result = this.updateTimeAndAddComment.filter( orig );
    assertNotNull( result );
    //result.write( new File( "c:/test2.png" ) );
    //TODO assert something
  }

  //-------------------------

  private EncodedImage getCircle() throws Exception
  {
    return load( "/ch/ige/image/test/circle.png" );
  }

  private static EncodedImage load( String name ) throws Exception
  {
    URL resource = PNGChunkFilterTest.class.getResource( name );
    assertNotNull( resource );
    return new ImageFactoryImpl().loadURL( resource );
  }

}
