package ch.ige.image.test;

import java.awt.Dimension;
import java.io.IOException;

import junit.framework.TestCase;
import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.TransparentGrayscaleProblem;
import ch.ige.image.format.Format;
import ch.ige.image.format.PNGFormat;

/**
 *  Tests the scaling operations. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: ScaleTest.java,v 1.3 2008/08/07 17:16:17 locher Exp $
 */
public class ScaleTest extends TestCase
{
  public ScaleTest( String name )
  {
    super( name );
  }

  public void testCYMK() throws Exception
  {
    check( false, SampleImages.getCYMKImageWithICCProfile() );
  }

  public void testCirclePNG() throws Exception
  {
    check( false, SampleImages.getCirclePNG() );
  }

  /**
   * Inspired by 410001_1472.
   * A transparent GRAY TIFF throws NPE when scaled.
   */
  public void testGrayDesignTIFF() throws Exception
  {
    check( true, SampleImages.getGrayDesignTIFF() );
  }

  private void check( boolean expectProblem, final Image normalized ) throws IOException, ImageException
  {
    assertEquals( expectProblem, TransparentGrayscaleProblem.hasProblem( normalized.asRenderedImage() ) );
    final Dimension targetSize = new Dimension( 100, 100 );

    Image resized = normalized.resize( targetSize );
    EncodedImage resizedPNG = Format.fromTypeString( PNGFormat.IDENTIFIER ).encode( resized, true );
    assertNotNull( resizedPNG );

    Image resizedKeepAspect = normalized.resizeKeepAspect( targetSize );
    EncodedImage resizedKeepAspectdPNG = Format.fromTypeString( PNGFormat.IDENTIFIER ).encode( resizedKeepAspect, true );
    assertNotNull( resizedKeepAspectdPNG );
  }

}
