package ch.ige.image.test;

import java.awt.Dimension;

import ch.ige.image.Image;

/**
 * Test Thumbnail resizing.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: ThumbnailTest.java,v 1.7 2009/03/19 13:26:20 locher Exp $
 */
public class ThumbnailTest extends AbstractImageTester
{

  public ThumbnailTest(String name)
  {
    super(name);
  }

  public void testAspect()
  {
    int[][] thumbSizes = { { 48, 240 }, { 240, 48 }, { 64, 64 } };
    int[] testImageWidths = { 20, 60, 100, 160, 300 };
    float[] testImageRatios = { 0.25f, 0.75f, 1.0f, 1.25f, 2.0f };

    for ( int[] thumbSize: thumbSizes )
    {
      for ( int imageWidth: testImageWidths )
      {
        for ( float ratio: testImageRatios )
        {
          this.checkAspectPreserved(imageWidth, (int) (imageWidth * ratio), thumbSize[0], thumbSize[1]);
        }
      }
    }
  }

  private void checkAspectPreserved(int imageWidth, int imageHeight, int thumbWidth, int thumbHeight)
  {
    double ratioOrig = (double) imageWidth / (double) imageHeight;
    final Image dummy = this.createDummyImage(imageWidth, imageHeight);
    Image thumb = dummy.resizeKeepAspect(new Dimension(thumbWidth, thumbHeight));
    Dimension thumbSize = thumb.getMetadata().getSize();
    double ratioThumb = thumbSize.getWidth() / thumbSize.getHeight();
    final String task = this.describeTask(imageWidth, imageHeight, thumbWidth, thumbHeight, thumb);
    assertTrue("Thumb too wide: " + task, thumbSize.getWidth() <= thumbWidth);
    assertTrue("Thumb too high: " + task, thumbSize.getHeight() <= thumbHeight);
    assertTrue("Thumb too small: " + task, thumbSize.getHeight() == thumbHeight || thumbSize.getWidth() == thumbWidth);
    assertEquals("Aspect not preserved: " + task, ratioOrig, ratioThumb, 0.05);
  }

  private String describeTask(int imageWidth, int imageHeight, int thumbWidth, int thumbHeight, Image thumb)
  {
    StringBuffer sb = new StringBuffer("[");
    sb.append(imageWidth);
    sb.append('x');
    sb.append(imageHeight);
    sb.append("] -> [");
    sb.append(thumbWidth);
    sb.append('x');
    sb.append(thumbHeight);
    sb.append("] resulted in [");
    Dimension thumbSize = thumb.getMetadata().getSize();
    sb.append(thumbSize.width);
    sb.append('x');
    sb.append(thumbSize.height);
    sb.append(']');
    return sb.toString();
  }

}
