package ch.ige.imagehandler;

import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.imagehandler.proxy.ImageHandlerFacade;
import ch.ige.imagehandler.proxy.ImageHandlerFacadeHome;

/**
 * Delegate to the ImageHandlerFacade
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lang
 * @version  $Id: ImageHandlerDelegate.java,v 1.7 2009/04/22 12:49:00 locher Exp $
 */
public class ImageHandlerDelegate
{
  private static final Logger LOG = Logger.getLogger( ImageHandlerDelegate.class );

  private static ImageHandlerDelegate me;

  private ImageHandlerFacade imageHandlerFacade;

  private ImageHandlerDelegate()
  {
    super();
  }

  public synchronized static ImageHandlerDelegate getInstance()
  {
    if ( me == null )
    {
      me = new ImageHandlerDelegate();
    }
    return me;
  }

  private ImageHandlerFacade getFacade() throws NamingException, RemoteException, CreateException
  {
    if ( imageHandlerFacade == null )
    {
      Context context = new InitialContext();
      ImageHandlerFacadeHome imageHandlerFacadeHome = (ImageHandlerFacadeHome) context.lookup( ImageHandlerFacadeHome.JNDI_NAME );
      this.imageHandlerFacade = imageHandlerFacadeHome.create();
    }
    return this.imageHandlerFacade;
  }

  public boolean check( Image image, String rulesName ) throws RemoteException, NamingException, CreateException
  {
    return getFacade().check( image, rulesName );
  }

  public boolean check( byte[] image, String rulesName ) throws RemoteException, NamingException, CreateException
  {
    return getFacade().check( image, rulesName );
  }

  public byte[] convert( byte[] image, String rulesName ) throws RemoteException, NamingException, CreateException
  {
    LOG.debug( "Sending the image to the server to transform it according to the defined rules." );
    return this.getFacade().convert( image, rulesName );
  }

  public Image convert( Image image, String rulesName ) throws RemoteException, NamingException, CreateException
  {
    return this.getFacade().convert( image, rulesName );
  }

}
