package ch.ige.imagehandler.batch;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import ch.ige.image.EncodedImage;
import ch.ige.image.Identify;
import ch.ige.image.Image;

public class ImageWithContext<T extends Image>
{
  private static final boolean IDENTIFY_AVAILABLE = Identify.available();

  public static final String METADATA_CONTEXT_KEY = "METADATA";

  private final T image;

  private final Map<String, Object> context;

  public ImageWithContext( T image )
  {
    super();
    if ( image == null )
    {
      throw new IllegalArgumentException();
    }
    this.image = image;
    this.context = Collections.synchronizedMap( new HashMap<String, Object>() );
  }

  /**
   * Gets the image.
   * @return the image
   */
  public T getImage()
  {
    return this.image;
  }

  public boolean isEncoded()
  {
    return this.image instanceof EncodedImage;
  }

  @SuppressWarnings( "unchecked" )
  public ImageWithContext<EncodedImage> asEncoded()
  {
    assert this.isEncoded();
    return (ImageWithContext<EncodedImage>) this;
  }

  public Set<Entry<String, Object>> getContextEntries()
  {
    return this.context.entrySet();
  }

  public Object getContextValue( String key )
  {
    return this.context.get( key );
  }

  public void putContext( String key, Object value )
  {
    this.context.put( key, value );
  }

  public boolean hasMetadata()
  {
    return this.context.containsKey( METADATA_CONTEXT_KEY );
  }

  public CharSequence getMetadata()
  {
    return (CharSequence) this.context.get( METADATA_CONTEXT_KEY );
  }

  public void putMetadataContext()
  {
    if ( IDENTIFY_AVAILABLE && this.image != null && this.isEncoded() )
    {
      try
      {
        this.putContext( METADATA_CONTEXT_KEY, Identify.performIdentification( (EncodedImage) this.image ) );
      }
      catch ( IOException e )
      {
        // ignore
      }
    }
  }

  public void dispose()
  {
    if ( this.image != null )
    {
      this.image.dispose();
    }
  }

}