package ch.ige.imagehandler.rule;

import java.util.Collections;
import java.util.List;

import ch.ige.image.Image;
import ch.ige.imagehandler.profile.Profile;
import ch.ige.imagehandler.rule.parameter.Parameter;

/**
 * A Rule.
 * <p>
 * A rule belongs to a profile and owns a parameter environment. It can accept
 * RuleVisitors and check itself for proper configuration.
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: Rule.java,v 1.14 2009/04/23 13:57:10 galli Exp $
 */
public interface Rule<I extends Image>
{
  /** The value of undefined parameters. */
  public static final String UNDEFINED_PARAM = "UNDEFINED";

  /** The value of undefined parameter lists. */
  public static final List<String> UNDEFINED_PARAMLIST = Collections.emptyList();

  public static final String CONFIG_WIDTH = "width";

  public static final String CONFIG_HEIGHT = "height";

  public static final String CONFIG_MAX_WIDTH = "maxWidth";

  public static final String CONFIG_MAX_HEIGHT = "maxHeight";

  public static final String CONFIG_UNIT = "unit";

  public static final String CONFIG_KEEP_ASPECT_RATIO = "keepAspectRatio";

  public static final String CONFIG_RESOLUTION = "resolution";

  /**
   * Gives the profile which owns this rule.
   * @return the owning profile
   */
  public Profile getProfile();

  /**
   * Sets the profile which owns this rule.
   * @param profile the owning profile
   */
  public void setProfile( Profile profile );

  /**
   * Adds a parameter to the parameter environment.
   * @param binding the parameter
   */
  public void addParameter( Parameter binding );

  /**
   * Invokes the handleRule callback on the visitor if appropriate.
   * 
   * @see RuleVisitor#handleRule(Rule)
   * @see "GOF Visitor Pattern"
   * @param visitor the visitor
   */
  public void accept( RuleVisitor visitor ) throws RuleException;

  /**
   * Checks the consistency of the rule.
   * 
   * @throws RuleException
   *           if invalid or missing parameters are found.
   */
  public void checkParameters() throws RuleException;
}
