package ch.ige.imagehandler.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Properties;

import junit.framework.TestCase;

import org.apache.commons.io.IOUtils;
import org.apache.log4j.BasicConfigurator;

import ch.ige.imagehandler.ImageHandlerFactory;

/**
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lang
 * @version  $Id: DelegateTest.java,v 1.8 2009/03/19 13:26:21 locher Exp $
 */
public class DelegateTest extends TestCase
{

  private static final Properties ORIG_PROPS = System.getProperties();

  static
  {
    BasicConfigurator.configure();
  }

  /**
   * path to the test-images
   */
  private static final File IMAGEHANDLER_TEST_IMG_PATH = new File( System.getProperty( "testimages", "c:/jp/projects/imagehandler/test_img/" ) );

  private static final File DESTINATION = new File( System.getProperty( "java.io.tmpdir", "c:/jp/projects/imagehandler/test_img/" ) );

  public DelegateTest( String name )
  {
    super( name );
  }

  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    System.setProperty( "java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory" );
    System.setProperty( "java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces" );
    System.setProperty( "java.naming.provider.url", "jnp://ipiebgs010.ipie.ch:11099" );
  }

  // @see junit.framework.TestCase#tearDown()
  @Override
  protected void tearDown() throws Exception
  {
    System.setProperties( ORIG_PROPS );
    super.tearDown();
  }

  public void testConvertImage1() throws Exception
  {
    processConversion( "col.jpeg" );
  }

  public void testConvertImage2() throws Exception
  {
    processConversion( "col.gif" );
  }

  public void testConvertImage3() throws Exception
  {
    processConversion( "alienregal.tif" );
  }

  public void testConvertImage4() throws Exception
  {
    processConversion( "handrad.png" );
  }

  /**
   * @param filename the name of the imagefile to convert
   */
  private void processConversion( String filename ) throws Exception
  {
    OutputStream os = null;
    try
    {
      byte[] source = IOUtils.toByteArray( new FileInputStream( new File( IMAGEHANDLER_TEST_IMG_PATH, filename ) ) );
      byte[] converted = ImageHandlerFactory.getDelegateInstance().convert( source, "convert_highres_trademark" );
      os = new FileOutputStream( new File( DESTINATION, "converted_" + filename ) );
      IOUtils.write( converted, os );
    }
    finally
    {
      IOUtils.closeQuietly( os );
    }
  }
}
