package ch.ige.ipcloadscheme.client.file;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Date;
import java.util.Enumeration;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.client.SchemeProperties;

/**
 * Zipfile welches Bilddateien beinhaltet
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class ImageZipFile extends HttpFile
{
  private static final Logger LOG = Logger.getLogger( ImageZipFile.class );

  private File unzipTarget = null;

  /**
   * Filename des Zipfiles wird aus Properties gelesen.
   * @param processDate Processing date
   */
  public ImageZipFile( Date processDate )
  {
    super( SchemeProperties.getInstance().getProperty( SchemeProperties.IMAGE_ZIPFILE_TARGET ), SchemeProperties.getInstance().getProperty(
        SchemeProperties.IMAGE_ZIPFILE_NAME, processDate ), processDate );

    this.url = SchemeProperties.getInstance().getProperty( SchemeProperties.IMAGE_URL, processDate );
    this.unzipTarget = new File( SchemeProperties.getInstance().getProperty( SchemeProperties.IMAGE_ZIPFILE_UNZIP ) );

    this.contentType = "application/zip";
  }

  public String unzip() throws Exception
  {
    ZipFile zipFile = new ZipFile( this );
    ZipEntry entry;
    File outFile;
    InputStream is = null;

    try
    {
      unzipTarget.mkdirs();

      Enumeration enumeration = zipFile.entries();

      while ( enumeration.hasMoreElements() )
      {
        entry = (ZipEntry) enumeration.nextElement();
        is = zipFile.getInputStream( entry );

        outFile = new File( unzipTarget, entry.getName() );
        copyToFile( is, outFile );

        LOG.debug( "Extracting file: " + entry.getName() );
      }
      zipFile.close();
    }
    catch ( Exception ex )
    {
      LOG.error( "Exception in unzip: " + ex );
      throw ex;
    }
    return unzipTarget.getPath();
  }

  public void copyToFile( InputStream in, File file ) throws Exception
  {
    BufferedOutputStream out = new BufferedOutputStream( new FileOutputStream( file ) );

    int size = 0;
    byte[] buf = new byte[1024];

    while ( ( size = in.read( buf, 0, buf.length ) ) != -1 )
    {
      out.write( buf, 0, size );
    }

    in.close();
    out.close();
  }

  /**
   * Löscht das unzip-Verzeichnis und die darin enthaltenen Files
   */
  public void deleteUnzippedFiles()
  {
    File[] files = unzipTarget.listFiles();
    for ( int i = 0; i < files.length; i++ )
    {
      files[i].delete();
    }
    unzipTarget.delete();
  }
}
