package ch.ige.ipcloadscheme.client.test;

import ch.ige.ipcloadscheme.client.IPCLoad;

/**
 * Tests {@link IPCLoad}. Note that this test requires a running appliation
 * server with the server part of this deployed.
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: TestIPCLoad.java,v 1.13 2008/07/01 06:27:02 lewis_lee Exp $
 */

public class TestIPCLoad extends ClientTestCase
{
  /**
   * The process date to use.
   */
  private String processDate;

  /**
   * Creates this.
   * @param name The test's name
   */
  public TestIPCLoad( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp()
  {
    try
    {
      this.processDate = "01.04.2008";
      System.setProperty( "log4j.configuration", "file:///C:/jp/projects/ipcloadscheme/install/programme/ige/ipcloadscheme/config/ipcloadscheme_config.xml" );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Transfers the English texts and saves them.
   */
  public void _testTransferText()
  {
    run( "-mode", "text", "-date", this.processDate, "-language", "en", "-save", "update" );
  }

  /**
   * Transfers the images and tests them.
   */
  public void _testTransferImage()
  {
    run( "-mode", "transfer", "-date", this.processDate, "-file", "img" );
  }

  /**
   * Tests only the transfer of English texts.
   */
  public void _testTransfer()
  {
    run( "-mode", "transfer", "-date", this.processDate, "-language", "en", "-file", "text" );
  }

  /**
   * Tests only English texts.
   */
  public void _testTextEnglish()
  {
    run( "-mode", "text", "-date", this.processDate, "-language", "en" );
  }

  /**
   * Tests only English texts and alters them.
   */
  public void _testTextEnglishAlter()
  {
    run( "-mode", "text", "-date", this.processDate, "-language", "en", "-file", "text", "-save", "update" );
  }

  /**
   * Tests the text mode.
   */
  public void _testText()
  {
    run( "-mode", "text", "-date", this.processDate );
  }

  /**
   * Tests the image mode.
   */
  public void _testImage()
  {
    run( "-mode", "image", "-date", this.processDate );
  }

  /**
   * Tests all the modes.
   */
  public void _testAll()
  {
    run( "-mode", "all", "-date", this.processDate );
  }

  /**
   * Runs the IPC load batch run.
   * @param args The command-line arguments
   */
  private void run( String... args )
  {
    IPCLoad.main( args );
  }
}
