package ch.ige.ipcloadscheme.client.text;

import java.io.File;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TreeMap;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.client.SchemeProperties;
import ch.ige.ipcloadscheme.client.file.LanguageFile;
import ch.ige.ipcloadscheme.vo.IpcBildKlasseVO;
import ch.ige.ipcloadscheme.vo.IpcKlasseVO;
import ch.ige.ipcloadscheme.xml.generate.Img;
import ch.ige.ipcloadscheme.xml.generate.IpcEdition;
import ch.ige.ipcloadscheme.xml.generate.IpcEntry;
import ch.ige.ipcloadscheme.xml.generate.Text;

/**
 * XML-Datei mit Texten wird mittels XSL-Transformation in ein neues XML transformiert,
 * reduziert auf die benötigten Elmente pro Sprache.
 * Das generierte XML wird mittels JAXB in Objekte abgefüllt (unmarshalling).
 * Anhand der Objekte werden die Daten zusammengesetzt und gekapselt in einem IpcKlassenVO in einer Map abgelegt
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcKlassenParser.java,v 1.22 2008/10/13 11:12:48 wild Exp $
 */

public class IpcKlassenParser
{
  private static final Logger LOG = Logger.getLogger( IpcKlassenParser.class );

  /** TODO grösse eventuell 130'000 */
  private Map mapKlassen = new TreeMap();

  private Map mapBilder = new TreeMap();

  private Locale language = null;

  private DecimalFormat dfGroup = new DecimalFormat( "###0.00####" );

  public IpcKlassenParser()
  {
    /** TODO verarbeitung eventuell aufteilen in mehrere files abschnitte
     * memory muss auf > 256 damit unmarshalling keinen fehler gibt
     * verarbeitung pro section wäre sinnvoll */
  }

  public Map getMap()
  {
    return mapKlassen;
  }

  public Map getMapBilder()
  {
    return mapBilder;
  }

  public void parse( InputStream is, Locale language ) throws Exception
  {
    this.language = language;

    try
    {
      TransformerFactory tFactory = TransformerFactory.newInstance();

      String systemId = SchemeProperties.getInstance().getProperty( SchemeProperties.CONFIG_PATH );
      String xsltFileName = SchemeProperties.getInstance().getProperty( SchemeProperties.XSL_FILE, language );

      LanguageFile langFile = new LanguageFile( language );
      langFile.createNewFile();

      File xslFile = new File( xsltFileName );

      Source xslSource = new StreamSource( xslFile );
      Source xmlSource = new StreamSource( is, systemId );
      Result xmlResult = new StreamResult( langFile );

      try
      {
        Transformer xmlTransformer = tFactory.newTransformer( xslSource );

        LOG.debug( "start transformation with xsl: " + xslFile.getPath() );
        xmlTransformer.transform( xmlSource, xmlResult );
      }
      catch ( TransformerException tex )
      {
        LOG.error( "transformation failed with Exception: " + tex );
        throw tex;
      }

      LOG.debug( "start unmarshalling file: " + langFile.getPath() );
      unmarshal( langFile );

    }
    finally
    {
      try
      {
        if ( is != null )
          is.close();
      }
      catch ( Exception e )
      {
      }
    }
  }

  private void unmarshal( File langFile ) throws JAXBException
  {
    IpcEdition ipcEdition = null;

    try
    {
      JAXBContext ctx = JAXBContext.newInstance( "ch.ige.ipcloadscheme.xml.generate" );
      Unmarshaller u = ctx.createUnmarshaller();

      ipcEdition = (IpcEdition) u.unmarshal( langFile );
      LOG.debug( "unmarshalling done" );
    }
    catch ( JAXBException ex )
    {
      LOG.error( "unmarshalling failed with Exception: " + ex );
      throw ex;
    }

    List listIpcEntry = ipcEdition.getIpcEntry();
    extractIpcEntries( listIpcEntry );
  }

  /**
   * Rekursive Funktion wo IPC-Einträge in Value Objects abgefüllt werden
   * @param listIpcEntry List
   */
  private void extractIpcEntries( List listIpcEntry )
  {
    for ( int i = 0; i < listIpcEntry.size(); i++ )
    {
      IpcEntry entry = (IpcEntry) listIpcEntry.get( i );

      try
      {
        fillUpIpcEntry( entry );
      }
      catch ( Exception ex )
      {
        LOG.error( "fill up failed for " + entry.getSymbol() + " Exception: " + ex );
      }

      extractIpcEntries( entry.getIpcEntry() );
    }
  }

  private void fillUpIpcEntry( IpcEntry entry ) throws Exception
  {
    IpcKlasseVO ipcKlasseVO;

    String key = getFormattedId( entry.getSymbol().toUpperCase() );

    if ( mapKlassen.containsKey( key ) )
    {
      ipcKlasseVO = (IpcKlasseVO) mapKlassen.get( key );
    }
    else
    {
      ipcKlasseVO = new IpcKlasseVO();
      ipcKlasseVO.ipcKlasse = key;
      ipcKlasseVO.elternklasse = getFormattedId( entry.getParent() );

      if ( !entry.getKind().equals( "s" ) && !entry.getKind().equals( "c" ) && !entry.getKind().equals( "u" ) && !entry.getKind().equals( "m" ) )
      {
        try
        {
          if ( entry.getKind().equals( "A" ) )
          {
            ipcKlasseVO.dot = 10;
          }
          else if ( entry.getKind().equals( "B" ) )
          {
            ipcKlasseVO.dot = 11;
          }
          else
          {
            ipcKlasseVO.dot = Integer.parseInt( entry.getKind() );
          }
        }
        catch ( NumberFormatException ex )
        {
          LOG.warn( "invalid dot: " + entry.getKind() + " for " + ipcKlasseVO.ipcKlasse + " inserted 0" );
          ipcKlasseVO.dot = 0;
        }
      }
    }

    Text text = (Text) entry.getText();
    fillUpTextAndBild( ipcKlasseVO, text );

    mapKlassen.put( key, ipcKlasseVO );
  }

  private void fillUpTextAndBild( IpcKlasseVO ipcKlasseVO, Text text )
  {
    StringBuffer bufText = new StringBuffer();
    List content = text.getContent();

    for ( int i = 0; i < content.size(); i++ )
    {
      Object obj = content.get( i );
      if ( obj instanceof Img )
      {
        Img image = (Img) obj;
        String bildName = fillUpBild( ipcKlasseVO, image );
        bufText.append( bildName );
      }
      else
      {
        bufText.append( obj.toString() );
      }
    }

    if ( language.equals( Locale.GERMAN ) )
    {
      ipcKlasseVO.textD = bufText.toString();
    }
    else if ( language.equals( Locale.ENGLISH ) )
    {
      ipcKlasseVO.textE = bufText.toString();
    }
    else if ( language.equals( Locale.FRENCH ) )
    {
      ipcKlasseVO.textF = bufText.toString();
    }
  }

  private String fillUpBild( IpcKlasseVO ipcKlasseVO, Img image )
  {
    IpcBildKlasseVO bildKlasseVO = new IpcBildKlasseVO();
    bildKlasseVO.ipcKlasse = ipcKlasseVO.ipcKlasse;

    String bildName = image.getSrc().toLowerCase();

    ipcKlasseVO.bild = true;
    if ( bildName.indexOf( "." ) > 0 )
    {
      bildKlasseVO.bildname = bildName.substring( 0, bildName.indexOf( "." ) );
    }
    else
    {
      bildKlasseVO.bildname = bildName;
    }
    mapBilder.put( bildKlasseVO.getContent(), bildKlasseVO );

    return bildName;
  }

  /**
   * Formatiert die ID Bsp: C07C005093 nach C07C5/093
   * K01B0001360000 K01B1/36
   * @param sId IPC-Klasse
   * @return formatierte IPC-Klasse
   */
  private String getFormattedId( String ipcKlasse ) throws Exception
  {
    if ( ipcKlasse.length() > 4 )
    {
      try
      {
        String group = ipcKlasse.substring( 4, 8 ) + "." + ipcKlasse.substring( 8, ipcKlasse.length() );
        double dGroup = new Double( group ).doubleValue();
        group = dfGroup.format( dGroup );

        StringBuffer buffer = new StringBuffer();

        buffer.append( ipcKlasse.substring( 0, 4 ) );

        if ( dGroup > 0.000000 )
        {
          buffer.append( group.replace( '.', '/' ) );
        }
        return buffer.toString();
      }
      catch ( Exception ex )
      {
        throw new Exception( ipcKlasse + " is not a valid ipc symbol Message: " + ex );
      }
    }
    else
      return ipcKlasse;
  }
}
