package ch.ige.ipcloadscheme.server.dao;

import java.sql.*;
import ch.ige.util.server.DBHelper;
import ch.ige.util.log.LogLevelHelper;

/**
 * DataAccessObject für die Tabelle IPC
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rüegsegger
 * @version $Id: IpcDAO.java,v 1.5 2006/12/19 16:19:52 ruegsegger Exp $
 */

public class IpcDAO
{


  /**
   * Selektiert Attribut gueltigVon Datum eines Datensatzes aus der Tabelle IPC
   * Das grösste gueltig_von datum wird selektiert
   * Oracle behandelt NULL-Werte als grössere Werte, dies hängt jedoch von der Datenbank ab
   *
   * @param  ipcSymb
   * @return Date gueltig von date
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public Date selectGueltigVon( String ipcSymb ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    String query = null;
    Date gueltigVon = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT gueltig_von FROM ipc WHERE ipc_symb = RPAD(?, 14, ' ') ORDER BY gueltig_von DESC";
      ps = con.prepareStatement(query);
      ps.setString( 1, ipcSymb );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        gueltigVon = rs.getDate( "gueltig_von" );
      }

      return gueltigVon;
    }
    catch (SQLException ex)
     {
       LogLevelHelper.log(ex, IpcDAO.class, query + ipcSymb);
       throw ex;
     }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
