package ch.ige.ipcloadvalidity.client.batch.test;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.commons.cli.CommandLine;

import ch.ige.ipcloadvalidity.client.batch.IpcLoadValidity;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;

/**
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author M. Kuenzli
 * @version $Id: IpcLoadValidityParamTest.java,v 1.1 2008/07/01 06:33:59 kuenzli Exp $
 */
public class IpcLoadValidityParamTest extends TestCase
{

  /**
   * The unit under test.
   */
  private IPCLoadForTests testIPCLoad;
  private String filename;

  /**
   * Creates this.
   * @param name The test name
   */
  public IpcLoadValidityParamTest( String name )
  {
    super( name );
  }

  @Override
  protected void setUp() throws Exception
  {
    this.filename = "test";
    super.setUp();
  }

  /**
   * Tests the command line options.
   * @throws Exception if the test fails 
   */
  public void testCheckCommandLine() throws Exception
  {
    this.testIPCLoad = new IPCLoadForTests( null );
    NewAbstractBatch.start( this.testIPCLoad, new String[] {} );
    assertEquals( "Checking return code", BatchReturnCode.WRONG_CLI_OPTIONS, this.testIPCLoad.getCurrentReturnCode() );

    // test proper command line with parameters -m and -f passed
    Map<String, Object> expectedCommandLine = new HashMap<String, Object>();
    expectedCommandLine.put( "mode", IpcLoadValidity.ProgramMode.DOWNLOAD );
    expectedCommandLine.put( "file", this.filename );

    this.testIPCLoad = new IPCLoadForTests( expectedCommandLine );
    NewAbstractBatch.start( this.testIPCLoad, new String[] { "-m", "download", "-f", this.filename } );
    assertEquals( "Checking return code", BatchReturnCode.OK, this.testIPCLoad.getCurrentReturnCode() );

    // test proper command line with only -f parameter passed --> -m should be defaulted to ALL
    expectedCommandLine = new HashMap<String, Object>();
    expectedCommandLine.put( "mode", IpcLoadValidity.ProgramMode.ALL );
    expectedCommandLine.put( "file", this.filename );
    this.testIPCLoad = new IPCLoadForTests( expectedCommandLine );
    NewAbstractBatch.start( this.testIPCLoad, new String[] {"-f", this.filename } );
    assertEquals( "Checking return code", BatchReturnCode.OK, this.testIPCLoad.getCurrentReturnCode() ); 
  }

  /**
   * Used for getting access to internals.
   */
  private static class IPCLoadForTests extends IpcLoadValidity
  {
    /**
     * Contains the expected command line options.
     */
    private Map<String, Object> expectedCommandLine;

    /**
     * Creates this.
     * @param expectedCommandLine The command line as it is expected
     */
    public IPCLoadForTests( Map<String, Object> expectedCommandLine )
    {
      super( "IPC Load test", OPTIONS, false );
      this.expectedCommandLine = expectedCommandLine;
    }

    @Override
    public BatchReport[] getBatchReports() throws BatchException
    {
      return null;
    }

    @Override
    @SuppressWarnings( "unused" )
    protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
    {
      assertEquals( "Checking number of options", this.expectedCommandLine.size() * 2, parsedCommandLine.size() );
      for ( String optionKey: this.expectedCommandLine.keySet() )
      {
        Object expectedValue = this.expectedCommandLine.get( optionKey );
        Object currentValue = parsedCommandLine.get( optionKey );
        if ( expectedValue instanceof List<?> )
        {
          List<?> expectedValues = (List<?>) expectedValue;
          List<?> currentValues = (List<?>) currentValue;
          assertEquals( "Checking number of values", expectedValues.size(), currentValues.size() );
          for ( int i = 0; i < expectedValues.size(); i++ )
          {
            assertEquals( "Checking option " + optionKey, expectedValues.get( i ), currentValues.get( i ) );
          }
        }
        else
        {
          assertEquals( "Checking option " + optionKey, expectedValue, currentValue );
        }
      }
    }
  }

}
