package ch.ige.ipcloadvalidity.client.process;

import java.text.DecimalFormat;

/**
 * Darstellung eines IPC-Symbols
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcSymbol.java,v 1.5 2007/01/09 10:30:32 ruegsegger Exp $
 */
public class IpcSymbol
{
  private String section  = null;
  private String ipcclass = null;
  private String subclass = null;
  private String maingroup = "";
  private String subgroup = "";

  private DecimalFormat dfMain = new DecimalFormat("0000");


  /**
   * Ein gültiges IPC-Symbol beinhaltet zwingend section, ipcclass und subclass
   * @param section String gültige Werte sind A-H
   * @param ipcclass String gültige Werte sind 01-99
   * @param subclass String gültiger Werte sind A-Z
   */
  public IpcSymbol(String section, String ipcclass, String subclass)
  {
    this.section  = section;
    this.ipcclass = ipcclass;
    this.subclass = subclass;
  }

  /**
   * Ein IPC-Symbol kann einen group part beinhalten.
   * Group part ist optional und beinhaltet einen maingroup sowie eine subgroub
   * Maingroup wird rechtsbündig mit führenden Nullen abgespeichert
   * Subgroup wird linksbündig abgespeichert
   * @param maingroup String gültige Werte sind 1-9999
   * @param subgroup String gültige Werte sind 00-999999
   */
  public void setGroupPart(String maingroup, String subgroup)
  {
    try
    {
      if (maingroup != null && !maingroup.equals(""))
      {
        this.maingroup = dfMain.format( dfMain.parse(maingroup) );
      }
        /** TODO abklären wie subgroup behandelt werden soll, falls maingroup leer */
      if (subgroup != null && !subgroup.equals(""))
      {
        this.subgroup = subgroup;
      }
    }
    catch ( Exception ex )
    {
      System.out.println(ex);
    }
  }

  /**
   * Gibt das Ipc-Symbol zurück, wie es in Bagis gespeichert wird
   * @return String Ipc-Symbol
   */
  public String getIpcSymbolAsString()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append(section);
    buf.append(ipcclass);
    buf.append(subclass);
    buf.append(maingroup);
    buf.append(subgroup);

    return buf.toString();
  }

  /**
   * Gibt das Ipc-Symbol als String zurück
   * @return String Ipc-Symbol als String
   */
  public String toString()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("  section          = <"  + this.section            + ">\n");
    buf.append ("  ipcclass         = <"  + this.ipcclass           + ">\n");
    buf.append ("  subclass         = <"  + this.subclass           + ">\n");
    buf.append ("  maingroup        = <"  + this.maingroup          + ">\n");
    buf.append ("  subgroup         = <"  + this.subgroup           + ">\n");

    return buf.toString();
  }

}
