package ch.ige.ipcloadvalidity.util;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.Date;

import org.apache.log4j.Logger;

import ch.ige.ipcloadvalidity.BuildInfo;


/**
 * <p>Überschrift: Europäische Patente (EP) einlesen</p>
 * <p>Beschreibung: Mit dieser Klasse wird eine Fehlerliste geführt.<br>
 * Name der verarbeiteten Zip-Datei muss im Name ersichtlich sein Epband_st320306
 * In dieses File werden benutzerrelevante Informationen geschrieben.
 * File wird nach der Verarbeitung dem Benutzer als Fehlerliste via Mail zugestellt
 * Es wurde ein RandomAccessFile gewaehlt, damit bei einem Restart der Applikation
 * die eingetragenen Information nicht ueberschrieben werden</p>
 * <p>Copyright: Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * <p>Organisation: IGE</p>
 * @author A. Rueegsegger
 * @version $Id: ValidityLogFile.java,v 1.5 2008/07/01 06:36:20 kuenzli Exp $
 */

public class ValidityLogFile extends File
{

  private static final String DEFAULT_PATH = "c:\\data\\log\\";
  private static final String DEFAULT_NAME = "Validity_temp";
  private static final Logger LOG = Logger.getLogger( ValidityLogFile.class );
  private static final String EXT = ".log";

  private static RandomAccessFile f = null;
  private static ValidityLogFile validityLogFile;


  public ValidityLogFile( )
  {
    this( DEFAULT_PATH, DEFAULT_NAME + EXT );
  }

  public ValidityLogFile( String filename )
  {
    this(ValidityProperties.getInstance().getProperty( ValidityProperties.LOGFILE_PATH ), filename + EXT);
  }


  /**
   * File öffnen, bzw erstellen
   * File-Pointer ans Ende der Datei setzte, damit geschriebene Daten nicht ueberschrieben werden
   *
   * @param filename String
   */
  public ValidityLogFile( String parent, String child )
  {
    super( parent, child);

    if( !this.getParentFile().exists() )
    {
      this.getParentFile().mkdirs();
    }

    try
    {
      f = new RandomAccessFile( this, "rw" );
      f.seek( f.length() );
    }
    catch( Exception ex )
    {
      LOG.error( ex );
    }

    writeHeader();
  }


  private void writeHeader()
  {
    write("Verarbeitung IPC LOAD VALIDITY gestartet " + new Date().toString() + "\n");
    write("CVS Tag  = " + BuildInfo.CVS_TAG );
    write("Build Id = " + BuildInfo.BUILD_ID + "\n") ;
  }

  private void writeFooter()
  {
    write( "\nVerarbeitung IPC LOAD VALIDITY beendet -------------------------------------------------------\n");
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * @return EpbandDelegate
   * @throws EpbandException
   */
  public static ValidityLogFile getInstance()
  {
    if( validityLogFile == null )
    {
      validityLogFile = new ValidityLogFile();
    }
    return validityLogFile;
  }


  /**
   * File schliessen
   * @throws Exception
   */
  public void close()
  {
    try
    {
      writeFooter();
      f.close();
    }
    catch( Exception ex )
    {
      LOG.error( ex );
    }
  }


  /**
   * Eintrag ins File schreiben
   * @throws IOException
   */
  public void write( String line )
  {
    try
    {
      f.writeBytes( line + "\r\n" );
    }
    catch( IOException ex )
    {
      LOG.error( "Eintrag kann nicht in Fehlerliste geschrieben werden: " + line );
      LOG.error( ex );
    }
  }
}

