package ch.ige.jipps;

/**
 * Gets configuration values for JIPPS. 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: JIPPSPropertiesServices.java,v 1.1 2007/04/26 12:58:21 lewis_lee Exp $
 */
public interface JIPPSPropertiesServices
{
  /**
   * Holt den Drucker aus Property_Jipps
   * 
   * @param propertyKey String Property-Key für den Drucker
   * @return nummer des Druckers auf Jipps
   * @throws Exception
   */
  public int getDrucker( String propertyKey ) throws Exception;

  /**
   * Holt den Directory-Pfad aus Property_Jipps
   * 
   * @param propertyKey String Property-Key für den Directory-Path
   * @return Directory-Pfad
   * @throws Exception
   */
  public String getDirectoryPath( String propertyKey ) throws Exception;

  /**
   * Stellt die Meldungen zusammen und versendet das Mail
   * 
   * @param betreffText Text der in der Betreffzeile des Mails steht
   * @param meldung String formatierte Meldung die übergeben wird
   * @param propertyKey Property-Key für das Ablage-Verzeichnis und den Receiver
   */
  public void doMail( String betreffText, String meldung, String propertyKey );

  /**
   * Stellt die Meldungen zusammen und versendet das Mail
   * 
   * @param batchId Batch-Nummer
   * @param betreffText Text der in der Betreffzeile des Mails steht
   * @param meldung String formatierte Meldung die übergeben wird
   * @param errorMsg String Fehler-Meldung
   * @param propertyKey Property-Key für das Ablage-Verzeichnis und den Receiver
   * @param anzahl Zahl der erstellten Bestätigungen
   * @param fehler Anzahl Fehlerhafte Bestätigungen
   * @param drucker Drucker-Namen
   * @param path Pfad wo die PDF-Dateien gespeichert werden <!-- begin-xdoclet-definition -->
   * @ejb.interface-method <!-- end-xdoclet-definition -->
   */
  public void doMail( int batchId, String betreffText, String meldung, String errorMsg, String propertyKey, int anzahl, int fehler, int drucker, String path );

  /**
   * Erstellt eine Meldung, wenn keine Daten in der Datenbank für die Verarbeitung vorhanden waren. Dies kann vorkommen
   * und ist kein Applikationsfehler.
   * 
   * @param betreffText String Betreff-Text
   * @param meldung String formatierte Meldung die übergeben wird
   * @param errorMsg String Fehler-Meldung
   * @param propertyKey Property-Key für den Receiver
   * @param fehler int Anzahl-Fehler
   */
  public void doMailNoData( String betreffText, String meldung, String errorMsg, String propertyKey, int fehler );

  /**
   * @param key key
   * @return String property
   */
  public String getProperty( String key );
}
