package ch.ige.jipps;

import java.sql.Date;
import java.util.List;

import javax.ejb.EJBException;

import ch.ige.jipps.vo.DruckerVO;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * Administrates print jobs and batch runs.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Esther Bandschapp
 * @author   Eric Lewis
 * @version  $Id: PrintJobStoreServices.java,v 1.10 2008/05/08 13:49:48 lang Exp $
 */
public interface PrintJobStoreServices
{

  /**
   * Fügt eine neue Zeile in die Tabelle PDF ein.
   * @param printJobVO ValueObject mit Daten zum PDF-File
   * @throws EJBException Insert konnte nicht erfolgreich ausgeführt werden.
   */
  public abstract void insPrintJob( PrintJobVO printJobVO ) throws EJBException;

  /**
   * Methode zum holen eines Druckes auf der JIPPSDB
   * @param printjob_Id id des Drucks
   * @return VO mit dem Druck
   * @throws EJBException Printjob konnte nicht geholt werden
   */
  public abstract PrintJobVO selectPrintJob( int printjob_Id ) throws EJBException;

  /**
   * Der DruckStatus in Pdf wird neu gesetzt
   * @param listPrintId Id des zu druckenden Files
   * @param status Status der gesetzt werden soll
   * @param druckerId int mit der ausgewählten Drucker-Nr.
   * @throws EJBException Update konnte nicht erfolgreich ausgeführt werden.
   */
  public abstract void updPrintJob( List listPrintId, int status, int druckerId ) throws EJBException;

  /**
   * Printjobs, die älter sind als loeschDate werden gelöscht
   * @param loeschDate Datum zum Vergleich
   * @throws EJBException Update konnte nicht erfolgreich ausgeführt werden.
   */
  public abstract Integer deleteJobs( Date loeschDate ) throws EJBException;

  /**
   * delete printjobs with the corresponding printjobIds 
   * @param printjobIds
   * @throws EJBException
   */
  public abstract void deletePrintJobs( List<Integer> printjobIds ) throws EJBException;

  /**
   * delete printjobs having the given batch-id
   * @param batchId int
   * @throws EJBException
   */
  public abstract void deletePrintJobs( int batchId ) throws EJBException;

  /**
   * TODO who uses this method??
   * @param reportId reportId
   * @param batchId batchId
   * @throws EJBException
   */
  public abstract void deletePrintJob( int reportId, int batchId ) throws EJBException;

  /**
   * @param printJobId reportId
   * @throws EJBException
   */
  public abstract void deletePrintJob( int printJobId ) throws EJBException;

  /**
   * Die nächste IdentNumber wird geholt und der Zähler heraufgesetzt
   * @return id Der aktuelle Schlüssel
   * @throws EJBException
   */
  public abstract int getNextBatchId() throws EJBException;

  /**
   * get the next available PrintjobId 
   * @return the next available PrintjobId
   * @throws EJBException
   */
  public abstract int getNextPrintJobId() throws EJBException;

  /**
   * Gets all JIPPS printers.
   * @return All available JIPPS printers
   * @throws EJBException if the selection fails
   */
  public abstract List<DruckerVO> selectPrinter() throws EJBException;

  /**
   * Gets all print jobs for a given bfaktId.
   * @param bfaktId The unique identifier of the batch run
   * @return A <code>List</code> of print jobs for the batch run
   * @throws EJBException if the print jobs can't be fetched
   */
  public List<Integer> getPrintJobsByBfaktId( final Integer bfaktId ) throws EJBException;

}