package ch.ige.jipps.currentstate;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.address.Address;
import ch.ige.ipobjects.address.comparator.AddressRoleComparator;
import ch.ige.ipobjects.address.vo.AddressWithCountryVO;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.country.dao.CountryDAO;
import ch.ige.ipobjects.country.vo.CountryVO;
import ch.ige.ipobjects.report.dao.RegInhaltDAO;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.vo.StandDatVO;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.format.AddressFormatter;

/**
 * Builds addresses for the current state.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: AddressBuilder.java,v 1.14 2007/12/13 13:09:44 lewis_lee Exp $
 */
class AddressBuilder
{
  /**
   * Gets the IP's addresses.
   * 
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  @SuppressWarnings( "unused" )
  public static List<CurrentStateElement> getAddresses( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException
  {
    RegInhaltDAO reportContentDAO = new RegInhaltDAO();
    CountryDAO countryDAO = new CountryDAO();
    ArrayList<CurrentStateElement> formattedAddresses = new ArrayList<CurrentStateElement>();

    TimePeriodCollection<Address> addresses = ip.getRegAdr().getPeriodsContainingTime( currentState );
    Collections.sort( addresses, new AddressRoleComparator<Address>() );

    for ( Address baseAddress: addresses )
    {
      CountryVO country = null;
      if ( baseAddress.getBland_cd() != null )
      {
        // Note: The countries are cached, so this call isn't that expensive...
        country = countryDAO.findCountry( baseAddress.getBland_cd() );
      }
      AddressWithCountryVO addressWithCountry = new AddressWithCountryVO( baseAddress, country );

      CurrentStateElement address = new StandDatVO();
      Integer reportTextNumber = reportContentDAO.findReportTextNumber( ip.getBAGISType(), addressWithCountry.getAddress().getAddressRole() );
      address.setReportTextNumber( reportTextNumber );

      // Some addresses don't even have a country code...
      String countryName = null;
      if ( addressWithCountry.getAddress().getBland_cd() != null )
      {
        // Note: The country name is in the language of the IP, not in the given language!
        countryName = addressWithCountry.getCountry().getLand_bez( ip.getLocale() );
      }

      List<String> textLines = AddressFormatter.format( addressWithCountry.getAddress().getReg_name(), addressWithCountry.getAddress().getReg_strasse(),
          addressWithCountry.getAddress().getReg_ort(), addressWithCountry.getAddress().getBland_cd(), countryName, addressWithCountry.getAddress()
              .getLizenz_txt(), false );
      address.setTextLines( textLines );

      formattedAddresses.add( address );
    }

    return formattedAddresses;
  }
}
