package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.FormTextVO;
import ch.ige.util.enums.BAGISLanguage;
import ch.ige.util.validation.Check;

/**
 * Accesses form texts (for letters) in the table <code>FORMTEXT</code>.
 * 
 * <p>Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: FormTextDAO.java,v 1.1 2008/07/23 14:54:35 lewis_lee Exp $
 */
public class FormTextDAO extends AbstractBagisDAO
{
  /**
   * The type of value object to return.
   */
  private static final Class<FormTextVO> VO_TYPE = FormTextVO.class;

  /**
   * Selects the texts.
   */
  private static final SQLStatement<FormTextVO> SELECT = SQL
      .select( "SELECT TRIM(sprach_cd) AS language, TO_NUMBER(TRIM(formtext_zeile)) AS line_number, text " + "FROM formtext "
          + "WHERE TRIM(sprach_cd) = ? AND TRIM(formtext_berei) = ?" );

  /**
   * Selects all form texts for a certain area in a given language.
   * @param language The language
   * @param formArea The form area containing form lines
   * @return The texts in the given language for the form area
   * @throws SQLException if the query fails
   */
  public List<FormTextVO> select( BAGISLanguage language, String formArea ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notEmpty( formArea, "form area" );

    return selectVOList( 50, VO_TYPE, SELECT, language.toDB(), formArea );
  }
}
