package ch.ige.jipps.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.handlers.TypedResultSetHandler;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;

/**
 * Accesses potential payers in <code>POTZAHL</code>.
 * 
 * <p>Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author  Marc Bouquet
 * @version $Id: PotZahlDAO.java,v 1.22 2008/07/23 14:16:35 lewis_lee Exp $
 */
public class PotZahlDAO extends AbstractBagisDAO
{
  /**
   * Selects the address numbers.
   */
  private static final SQLStatement<Integer> SELECT = SQL.select( "SELECT adr_nr FROM pot_zahl WHERE bfakt_id = ? " );

  /**
   * Gets the address numbers for all potential payers of an invoice.
   * @param invoiceId The unique ID of the invoice
   * @return The potential payers' address numbers
   * @throws SQLException if the query fails
   */
  public List<Integer> select( int invoiceId ) throws SQLException
  {
    return getQueryRunner().query( SELECT, new AddressNumberHandler(), Integer.valueOf( invoiceId ) );
  }

  /**
   * The <code>ResultSetHandler</code> which collects address numbers.
   */
  class AddressNumberHandler implements TypedResultSetHandler<List<Integer>>
  {
    public List<Integer> handle( ResultSet rs ) throws SQLException
    {
      List<Integer> list = new ArrayList<Integer>();
      while ( rs.next() )
      {
        list.add( new Integer( rs.getInt( "adr_nr" ) ) );
      }
      return list;
    }
  }
}
