package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.util.Collection;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.jipps.dao.PropertyJippsDAO;
import ch.ige.jipps.vo.PropertyJippsVO;

/**
 * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: PropertyJippsDAOTest.java,v 1.4 2007/07/23 12:44:17 lang Exp $
 */
public class PropertyJippsDAOTest extends BaseDAOWithCRUDTestTemplate<PropertyJippsDAO, PropertyJippsVO, String>
{
  /**
   * An existing property key
   */
  private final static String PATH_XSL_KEY = "path_xsl";

  /**
   * An existing property value
   */
  private final static String PATH_XSL_VALUE = "/ige/jipps/xsl/";

  /**
   * The description of the existing property
   */
  private static final String PATH_XSL_DESCRIPTION = "Der Pfad, wo die XSL-Dateien zu finden sind.";

  /**
   * The number of properties in the database
   */
  private final static int NUMBER_OF_PROPERTIES = 44;

  /**
   * The only constructor
   * @param name The test's name
   */
  public PropertyJippsDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    // This may have been set by other tests and would override the value from the DB
    System.setProperty( PATH_XSL_KEY, "" );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#select()}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    defaultTestReadCollection( getExpectedVO(), NUMBER_OF_PROPERTIES );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#selectVO(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testSelectVO() throws Exception
  {
    defaultTestRead( getExpectedVO(), "", "gugus", PATH_XSL_KEY );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#select(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testSelectWithKey() throws Exception
  {
    try
    {
      getDAO().select( null );
      fail( "Reading with null ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    assertNull( "Checking read for unknown ID", getDAO().select( "gugus" ) );

    assertEquals( "Checking known property", PATH_XSL_VALUE, getDAO().select( PATH_XSL_KEY ) );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#insert(ch.ige.jipps.vo.PropertyJippsVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( PropertyJippsVO property )
  {
    property.value = "A totally different value";
    property.description = "Yes, this has indeed changed";
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#update(ch.ige.jipps.vo.PropertyJippsVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#delete(java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( null, "gugus" );
  }

  /**
   * Tests the cache.
   * @throws Exception if the test fails
   */
  public void testCache() throws Exception
  {
    defaultTestCache( PATH_XSL_KEY );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#getCacheSize()}.
   * @throws Exception if the test fails
   */
  public void testGetCacheSize() throws Exception
  {
    defaultTestCacheSize( PATH_XSL_KEY, NUMBER_OF_PROPERTIES );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.PropertyJippsDAO#describeCachedContent()}.
   * @throws Exception if the test fails
   */
  public void testDescribeCachedContent() throws Exception
  {
    defaultTestDescribeCachedContent( PATH_XSL_KEY, PATH_XSL_KEY, PATH_XSL_DESCRIPTION, PATH_XSL_VALUE );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<PropertyJippsDAO> getDAOClass()
  {
    return PropertyJippsDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected String insert( PropertyJippsVO property )
  {
    getDAO().insert( property );
    return property.getIdentifier();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected PropertyJippsVO read( String key ) throws SQLException
  {
    return getDAO().selectVO( key );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<PropertyJippsVO> read( Object... parameters ) throws SQLException
  {
    return getDAO().select();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( PropertyJippsVO property )
  {
    getDAO().update( property );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( String key )
  {
    getDAO().delete( key );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( PropertyJippsVO property, PropertyJippsVO loadedProperty )
  {
    assertEquals( "Checking key", property.key, loadedProperty.key );
    assertEquals( "Checking value", property.value, loadedProperty.value );
    assertEquals( "Checking description", property.description, loadedProperty.description );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( PropertyJippsVO originalVO, PropertyJippsVO serializedVO )
  {
    return originalVO.key.equals( serializedVO.key ) && originalVO.value.equals( serializedVO.value )
        && originalVO.description.equals( serializedVO.description );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected PropertyJippsVO getSerializableVO()
  {
    PropertyJippsVO property = new PropertyJippsVO();
    property.key = "the_key";
    property.value = "The value";
    property.description = "The description";
    return property;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected PropertyJippsVO getExpectedVO()
  {
    PropertyJippsVO property = new PropertyJippsVO();
    property.key = PATH_XSL_KEY;
    property.value = PATH_XSL_VALUE;
    property.description = PATH_XSL_DESCRIPTION;
    return property;
  }
}
