package ch.ige.jipps.event;

import java.sql.Date;
import java.util.List;

import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;

/**
 * Filters events based on different criteria.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: EventFilter.java,v 1.2 2007/05/10 12:21:12 lewis_lee Exp $
 */
public class EventFilter
{

  /**
   * Gets the highest date of all events with the required event number.
   * @param events All events to check
   * @param eventNumber The event number to look for
   * @return The required event or <code>null</code> if it isn't found
   */
  public static Date getHighestEventDate( List<Event> events, Integer eventNumber )
  {
    EreigIDat eventDate = null;
    for ( Event event: events )
    {
      if ( event != null && event.getNumericArt_nr() != null && event.getNumericArt_nr().equals( eventNumber ) )
      {
        if ( eventDate == null || event.getEreig_idat().after( eventDate ) )
        {
          eventDate = event.getEreig_idat();
        }
      }
    }
    if ( eventDate != null )
    {
      return eventDate.getDate();
    }
    else
    {
      return null;
    }
  }
}
