package ch.ige.jipps.ftp.web.test;

import java.sql.SQLException;

import junit.framework.TestCase;
import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ftp.FTPServerSystem;
import ch.ige.ftp.fileselector.util.FileType;
import ch.ige.jipps.ftp.web.AbstractService;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfigDAO;
import ch.ige.jipps.ftp.web.util.FTPServletConfigVO;

/**
 * Tests for PatDocDAO.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author fabrizioli
 * @version $Id: FTPServletConfigDAOTest.java,v 1.13 2009/03/03 22:00:02 fabrizioli Exp $
 */
public class FTPServletConfigDAOTest extends TestCase
{

  private static final Object EPA_FILE_TYPE_DOWNLOAD = FileType.AUTOMATIC;

  private static final Object EPA_FILE_TYPE_UPLOAD = FileType.AUTOMATIC;

  private static final Object FTP_FILE_TYPE_DOWNLOAD = FileType.AUTOMATIC;

  private static final Object FTP_FILE_TYPE_UPLOAD = FileType.AUTOMATIC;

  private static String FTP_SERVER_HOST = "ipiebgs012.ipie.ch";

  private static String FTP_SERVER_PATH = "\\EPAUpload\\Entwicklung";

  private static String FTP_SERVER_USER = "anonymous";

  private static String FTP_SERVER_PASS = "none";

  private static String FTP_SERVER_URL = "\\\\ipiebgs012.ipie.ch\\ftp\\EPAUpload\\Entwicklung";

  private static boolean FTP_SERVER_USESPAGEING = true;

  private static int FTP_SERVER_RWTIMEOUT = 60;

  private static int FTP_SERVER_CONNECTTIMEOUT = 60;

  private static FTPServerSystem FTP_SERVER_SYSTEM = FTPServerSystem.NT_SYSTEM;

  private static String EPA_SERVER_HOST = "ipiebgs012.ipie.ch";

  private static String EPA_SERVER_PATH = "\\EPAUpload\\Entwicklung\\TEST_EPA_SERVER";

  private static String EPA_SERVER_USER = "anonymous";

  private static String EPA_SERVER_PASS = "none";

  private static boolean EPA_SERVER_USESPAGEING = false;

  private static int EPA_SERVER_RWTIMEOUT = 240;

  private static int EPA_SERVER_CONNECTTIMEOUT = 240;

  private static FTPServerSystem EPA_SERVER_SYSTEM = FTPServerSystem.UNIX_SYSTEM;

  private static final String TEMPORARY_FOLDER_UPLOAD = "/ige/ftpservlet/epaupload";

  private static final String TEMPORARY_FOLDER_DOWNLOAD = "/ige/ftpservlet/download";

  public FTPServletConfigDAOTest( String args )
  {
    super( args );
  }

  @Override
  public void setUp() throws Exception
  {
    // super.setUp();
    ClientInitializer.initStandaloneClient();
    AbstractDAO.setRunningInJ2EEServer( false );

  }

  @Override
  public void tearDown() throws Exception
  {
    //super.tearDown();

  }

  protected FTPServletConfigDAO getDAO()
  {
    return new FTPServletConfigDAO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  protected FTPAccessConfigVO getSerializableVO()
  {
    return this.createDummy();
  }

  public void testSelect() throws SQLException
  {
    FTPAccessConfigVO selected = this.getDAO().selectIntranetConfig();
    FTPAccessConfigVO selectedEPA = this.getDAO().selectEPAConfig();
    assertTrue( selected != null );
    assertTrue( selectedEPA != null );
    validateIntranetParam( selected );
    validateEPAParam( selectedEPA );

    FTPServletConfigVO servletConfig = this.getDAO().selectServletConfig();
    assertTrue( servletConfig != null );
    validateServletConfig( servletConfig );
  }

  private void validateServletConfig( FTPServletConfigVO servletConfig )
  {
    assertEquals( TEMPORARY_FOLDER_DOWNLOAD, servletConfig.getPathToTemporaryDownloadDir() );
    assertEquals( TEMPORARY_FOLDER_UPLOAD, servletConfig.getPathToTemporaryUploadDir() );
    assertTrue( servletConfig.getPathToTemporaryDownloadDir().contains( AbstractService.TEMPORARY_FOLDER_MARKUP ) );
    assertTrue( servletConfig.getPathToTemporaryUploadDir().contains( AbstractService.TEMPORARY_FOLDER_MARKUP ) );
    assertEquals( null, servletConfig.getDownloadFileType() ); // remove or set default config, set in AccessConfig as specific for each ftp server
    assertEquals( null, servletConfig.getUploadFileType() );

  }

  private void validateIntranetParam( FTPAccessConfigVO candidates )
  {
    assertEquals( FTP_SERVER_HOST, candidates.getFTPHost() );
    assertEquals( FTP_SERVER_USER, candidates.getFTPUser() );
    assertEquals( FTP_SERVER_PASS, candidates.getFTPPasswort() );
    assertEquals( FTP_SERVER_URL, candidates.getFTPUrl() );
    assertEquals( FTP_SERVER_PATH, candidates.getFTPServerPath() );
    assertEquals( FTP_SERVER_CONNECTTIMEOUT, candidates.getConnectionTimeout() );
    assertEquals( FTP_SERVER_RWTIMEOUT, candidates.getRWTimeout() );
    assertEquals( FTP_SERVER_USESPAGEING, candidates.useListFiles() );
    assertEquals( FTP_FILE_TYPE_DOWNLOAD, candidates.getFileTypeForDownload() );
    assertEquals( FTP_FILE_TYPE_UPLOAD, candidates.getFileTypeForUpload() );
    assertEquals( null, candidates.getRecentDateFormat() );
    assertEquals( null, candidates.getServerLanguageCode() );
    assertEquals( null, candidates.getServerTimeZone() );
    assertEquals( null, candidates.getDefaultDateFormat() );
    assertEquals( FTP_SERVER_SYSTEM, candidates.getSystemKey() );

  }

  private void validateEPAParam( FTPAccessConfigVO candidates )
  {
    assertEquals( EPA_SERVER_HOST, candidates.getFTPHost() );
    assertEquals( EPA_SERVER_USER, candidates.getFTPUser() );
    assertEquals( EPA_SERVER_PASS, candidates.getFTPPasswort() );
    assertEquals( null, candidates.getFTPUrl() );
    assertEquals( EPA_SERVER_PATH, candidates.getFTPServerPath() );
    assertEquals( EPA_SERVER_CONNECTTIMEOUT, candidates.getConnectionTimeout() );
    assertEquals( EPA_SERVER_RWTIMEOUT, candidates.getRWTimeout() );
    assertEquals( EPA_SERVER_USESPAGEING, candidates.useListFiles() );
    assertEquals( EPA_FILE_TYPE_DOWNLOAD, candidates.getFileTypeForDownload() );
    assertEquals( EPA_FILE_TYPE_UPLOAD, candidates.getFileTypeForUpload() );
    assertEquals( null, candidates.getRecentDateFormat() );
    assertEquals( null, candidates.getServerLanguageCode() );
    assertEquals( null, candidates.getServerTimeZone() );
    assertEquals( null, candidates.getDefaultDateFormat() );
    assertEquals( EPA_SERVER_SYSTEM, candidates.getSystemKey() );
  }

  private FTPAccessConfigVO createDummy()
  {
    FTPAccessConfigVO vo = new FTPAccessConfigVO( FTP_SERVER_HOST, FTP_SERVER_USER, FTP_SERVER_PASS, FTP_SERVER_PATH, FTP_SERVER_URL, FTP_SERVER_SYSTEM,
        FTP_SERVER_RWTIMEOUT, FTP_SERVER_CONNECTTIMEOUT, FTP_SERVER_USESPAGEING, FileType.NOT_DEFINED, FileType.NOT_DEFINED );
    return vo;
  }

}
