package ch.ige.jipps.header;

import java.sql.Date;
import java.text.ParseException;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.util.constants.DeadlineTimeConstants;
import ch.ige.util.constants.DeadlineTypeConstants;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.date.DateConverter;
import ch.ige.util.date.LanguageDateFormatter;
import ch.ige.util.deadline.DeadlineCalculator;
import ch.ige.util.format.ReportIpNumberFormatter;

/**
 * Builds the special cases for the IP header in the report for trademarks.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: TrademarkHeaderBuilder.java,v 1.17 2007/12/18 12:32:57 lewis_lee Exp $
 */
class TrademarkHeaderBuilder extends BaseIpHeaderBuilder
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( TrademarkHeaderBuilder.class );

  /**
   * Only oppositions after this date are printed.
   */
  private static final String OPPOSITION_START_DATE = "01.04.1993";

  // See interface
  @Override
  public String getFormattedIpNumber( GenericIP ip )
  {
    Trademark trademark = (Trademark) ip;
    return ReportIpNumberFormatter.formatTrademarkWithDefault( trademark.getIPNumber(), trademark.getBAGISType(), trademark.getProlongationCount() );
  }

  // See interface
  @Override
  public String getOppositionDeadline( GenericIP ip, Date reportDate, LanguageDateFormatter dateFormatter )
  {
    try
    {
      java.util.Date oppositionStartDate = DateConverter.stringToDate( OPPOSITION_START_DATE );
      Event event = ip.getFirstPublication( EreigIDat.max( reportDate ) );
      if ( event != null )
      {
        java.util.Date deadline = DeadlineCalculator.getDeadline( ip.getBAGISType(), event.getPubl_dat(), ip.getProtectionStart(), null,
            DeadlineTypeConstants.DEADLINE_WITHOUT_GRACE_PERIOD_INCLUDING_SHIPPING, DeadlineTimeConstants.TIME_UNIT_MONTH, 3, 1, true );
        if ( deadline.before( oppositionStartDate ) )
        {
          return "";
        }
        else
        {
          return Header.formatDate( new java.sql.Date( deadline.getTime() ), dateFormatter );
        }
      }
      else
      {
        return "";
      }
    }
    catch ( ParseException ex )
    {
      LOG.error( "Couldn't parse opposition start date", ex );
      return "";
    }
  }

  // See interface
  @Override
  public PublicationVO getPublication( GenericIP ip, LanguageDateFormatter dateFormatter, Date reportDate, boolean isFirstPublication )
  {
    PublicationVO publication = getDefaultPublicationResults( ip, reportDate, isFirstPublication );

    Event publicationEvent = getPublicationEvent( ip, reportDate, isFirstPublication );
    if ( publicationEvent != null && publicationEvent.getDate() != null )
    {
      publication.formattedPublicationDate = Header.formatDate( publicationEvent.getPubl_dat(), dateFormatter );
      publication.publicationYear = NumericLiterals.ZERO;
      publication.publicationNumber = publicationEvent.getPubl_nr();
      // Trademark publication events are always published
      publication.publishPublicationEvent = true;
    }
    return publication;
  }
}
