package ch.ige.jipps.header.test;

import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;

import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;

/**
 * Gets the reference XML documents and compares their header to what is generated on the fly.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlIpHeaderTest.java,v 1.16 2007/03/23 16:04:03 lewis_lee Exp
 */
public class FacadeXmlIpHeaderTest extends AbstractTestFacadeXml
{
  /**
   * The default constructor
   * @param args Any argument
   */
  public FacadeXmlIpHeaderTest( String args )
  {
    super( args );

    // Force the images to be written.
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.PROCESS_IMAGES );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.checkAllIpTypes = true;
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  /**
   * Compares the expected header with the actual header.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   */
  @Override
  protected void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    Document expectedDocument = XMLUnit.buildTestDocument( expectedXml );
    Document actualDocument = XMLUnit.buildTestDocument( actualXml );

    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/auszugsTyp" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/sprache" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/auszugsTitel" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/schutitKey" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/schutitTyp" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/schutitRegisterStatus" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/anmeldDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/schutzEnde" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/eintrNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/schutitNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/stammBeginn" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/eintrDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/gesuchNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/loeschDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/erstPublJahr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/erstPublDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/ePublikation" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/erstPublNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/erstPublPflicht" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/letztPublJahr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/letztPublDat" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/letztPublNr" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/letztPublPflicht" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/widersprFrist" );
    compareXPath( expectedDocument, actualDocument, "/auszug/kopf/standDat" );

    // The IP name is optional, because it may be an image trademark
    compareOptionalXPath( expectedDocument, actualDocument, "/auszug/kopf/schutitel" );

    compareImages( expectedDocument, actualDocument, "/auszug/kopf/bild", true );

    // Just to be 100% sure...
    checkNode( expectedXml, actualXml, "kopf", referenceXmlVo );
  }
}
