package ch.ige.jipps.history.description;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.patent.Patent;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.patent.PatentPrioritiesDetail;
import ch.ige.util.enums.report.ReportType;

/**
 * Configures how the history part of the report is built for a patent.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: PatentHistoryDescription.java,v 1.8 2007/12/20 08:24:54 lewis_lee Exp $
 */
public class PatentHistoryDescription extends BaseHistoryDescription
{
  /**
   * Creates this with all required data.
   * @param ip The patent
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param includeWaitListEvents If this is <code>true</code>, events that are on the wait list are also shown
   * @param startFromApplication If this is <code>true</code>, events starting from the application are shown,
   * otherwise from the registration
   * @param allowedReportGroups The report groups, a <code>Map</code> where the keys show whether an event must be
   * included on a report. The values can be ignored.
   * @param reportType The report type
   */
  public PatentHistoryDescription( GenericIP ip, PointInTime currentState, Locale language, boolean includeWaitListEvents, boolean startFromApplication,
      Map<Integer, String> allowedReportGroups, ReportType reportType )
  {
    super( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
  }

  // @see ch.ige.jipps.history.description.BaseHistoryDescription#getIpSpecificDetailDescriptions()
  @Override
  public List<BaseHistoryItemDetail<? extends TimePeriod>> getIpSpecificDetailDescriptions()
  {
    Patent patent = (Patent) this.ip;
    List<BaseHistoryItemDetail<? extends TimePeriod>> descriptions = new ArrayList<BaseHistoryItemDetail<? extends TimePeriod>>();
    descriptions.add( new PatentPrioritiesDetail( patent.getPrios(), patent.getType(), this.ip ) );
    return descriptions;
  }
}
