package ch.ige.jipps.history.detail.design;

import java.util.ArrayList;

import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.history.ValueHistory;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a design's title.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignTitleDetail.java,v 1.2 2007/12/19 08:57:08 lewis_lee Exp $
 */
public class DesignTitleDetail extends BaseHistoryItemDetail<HistoryItem<String>>
{
  /**
   * Creates this with all the data.
   * @param designTitle The design title
   */
  public DesignTitleDetail( ValueHistory<HistoryItem<String>> designTitle )
  {
    super( designTitle, null, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<HistoryItem<String>, Detail> createBuilder()
  {
    return new DesignDescriptionDetailBuilder();
  }

  // See superclass
  @Override
  public boolean shouldGroupEndDates()
  {
    return true;
  }

  /**
   * The class that actually formats the description.
   */
  public class DesignDescriptionDetailBuilder extends AbstractHistoryItemDetailBuilder<HistoryItem<String>, Detail>
  {
    /**
     * Creates the builder.
     */
    public DesignDescriptionDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<HistoryItem<String>> detailData )
    {
      HistoryItem<String> designDescription = detailData.get( 0 );
      if ( designDescription.getValue() != null )
      {
        Detail detail = new Detail( designDescription, DesignTitleDetail.this );
        detail.reportTextNumber = ReportTextConstants.DESIGN_DESCRIPTION_INTEGER;
        detail.textLines = new ArrayList<String>();
        detail.textLines.add( designDescription.getValue() );
        return createListSingleDataItemSingleRepresentation( designDescription, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
