package ch.ige.jipps.imageconv;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.fop.apps.MimeConstants;
import org.apache.log4j.Logger;

import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.ImageTypeConstants;

/**
 * Acts as a remote interface to command line image conversion.
 * <p>
 * Copyright: Copyright (c) 2006 IGE
 * </p>
 * 
 * @author Johannes Lang
 * @version $Id: ImageConverterServlet.java,v 1.18 2009/05/05 15:48:41 lewis_lee Exp $
 */
public class ImageConverterServlet extends HttpServlet
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( ImageConverterServlet.class );

  // @see javax.servlet.http.HttpServlet#doGet(HttpServletRequest, HttpServletResponse)
  @Override
  public void doGet( HttpServletRequest request, HttpServletResponse response ) throws IOException
  {
    String imagePath = request.getParameter( "imagePath" );
    LOG.debug( "converting image " + imagePath + " with ImageMagick ..." );

    PrintWriter out = null;
    try
    {
      // Note that JPEGs are converted to JPEGs: This doesn't seem to make sense,
      // but there are images which can't be shown using FOP, they are only shown
      // after being converted by ImageMagick.
      String destinationImageType = null;
      if ( imagePath.endsWith( FileConstants.EXTENSION_PNG ) )
      {
        LOG.debug( "Image not converted, stays " + ImageTypeConstants.IMAGE_TYPE_PNG );
        destinationImageType = ImageTypeConstants.IMAGE_TYPE_PNG;
      }
      else
      {
        ImageMagickConverter.convert( imagePath, ImageTypeConstants.IMAGE_TYPE_JPG );
        LOG.debug( "Image converted to " + ImageTypeConstants.IMAGE_TYPE_JPG );
        destinationImageType = ImageTypeConstants.IMAGE_TYPE_JPG;
      }
      response.setContentType( MimeConstants.MIME_PLAIN_TEXT );
      out = response.getWriter();
      out.write( destinationImageType );
      out.flush();
    }
    catch ( Exception e )
    {
      LOG.error( "Couldn't convert " + imagePath, e );
      response.sendError( HttpServletResponse.SC_INTERNAL_SERVER_ERROR, "Couldn't convert " + imagePath + ": " + e.getMessage() );
    }
    finally
    {
      if ( out != null )
      {
        out.close();
      }
    }
  }
}
