package ch.ige.jipps.signature.test;

import java.util.List;

import org.w3c.dom.Node;

import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;

/**
 * Gets the reference XML documents and compares their state to what is generated on the fly. Note that some data
 * contains spaces at the end, so <code>String</code>s are always trimmed.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlSignatureTest.java,v 1.3 2006/09/06 13:14:46 lewis_lee Exp
 */
public class FacadeXmlSignatureTest extends AbstractTestFacadeXml
{

  /**
   * The default constructor
   * @param name The test's name
   */
  public FacadeXmlSignatureTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.checkAllIpTypes = true;
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  /**
   * Checks all state nodes for the given report date.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   */
  @Override
  public void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    String xPathRows = "/auszug/signature/zeile";
    // The XML is really simple, no need to get into individual nodes.
    // Just make sure that whitespace is removed.
    List<Node> expectedNodesRows = getNodes( expectedXml, xPathRows );
    List<Node> actualNodesRows = getNodes( actualXml, xPathRows );

    if ( expectedNodesRows.size() != actualNodesRows.size() )
    {
      fail( "Expected " + expectedNodesRows.size() + " signature lines, but got " + actualNodesRows.size(), expectedXml, actualXml,
          getInformation( referenceXmlVo ) );
    }
    checkNodesSuccess( expectedNodesRows, actualNodesRows, expectedXml, actualXml, referenceXmlVo );

    String xPathStamp = "/auszug/signature/stamp";
    List<Node> expectedNodesStamp = getNodes( expectedXml, xPathStamp );
    List<Node> actualNodesStamp = getNodes( actualXml, xPathStamp );
    if ( expectedNodesStamp.size() != actualNodesStamp.size() )
    {
      fail( "Expected " + expectedNodesStamp.size() + " stamps, but got " + actualNodesStamp.size(), expectedXml, actualXml, getInformation( referenceXmlVo ) );
    }
    checkNodesSuccess( expectedNodesStamp, actualNodesStamp, expectedXml, actualXml, referenceXmlVo );
  }

  /**
   * Checks the nodes whether the output matches with what is expected.
   * @param expectedNodes The expected nodes
   * @param actualNodes The actual nodes
   * @param expectedXml The expected XML
   * @param actualXml The actual XML
   * @param referenceXmlVo The data from ReferenceXML
   */
  private void checkNodesSuccess( List<Node> expectedNodes, List<Node> actualNodes, String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo )
  {
    for ( int i = 0; i < expectedNodes.size(); i++ )
    {
      Node expectedNode = expectedNodes.get( i );
      String expectedLine = getFirstText( expectedNode );

      Node actualNode = actualNodes.get( i );
      String actualLine = getFirstText( actualNode );

      if ( ( expectedLine == null && actualLine != null ) || ( expectedLine != null && !expectedLine.equals( actualLine ) ) )
      {
        fail( "Line " + i + ": Expected " + expectedLine + ", but got " + actualLine, expectedXml, actualXml, getInformation( referenceXmlVo ) );
      }
    }
  }
}
