package ch.ige.jipps.test;

import java.sql.Timestamp;
import java.util.List;

import junit.framework.TestCase;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.LetterBatchControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.PrintjobDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.printer.ReportPaper;

/**
 * Tests {@link ch.ige.jipps.LetterBatchControl#printAMVChanges(boolean)}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: LetterBatchControlTest.java,v 1.4 2008/09/22 08:31:39 lewis_lee Exp $
 */
public class LetterBatchControlTest extends TestCase
{
  static
  {
    ServicesFactory.initStandaloneClient();
  }

  /**
   * The only constructor.
   * @param name The test's name
   */
  public LetterBatchControlTest( String name )
  {
    super( name );
    PathConfiguration.setPathsForLocalTest();
  }

  /**
   * Tests {@link ch.ige.jipps.LetterBatchControl#printAMVChanges(boolean)}.
   * @throws Exception if the test fails
   */
  public void testPrintAMVChanges() throws Exception
  {
    // Lose the milliseconds and go back a minute, the times may differ between this server and the DB server
    Timestamp testStart = new Timestamp( ( System.currentTimeMillis() / 1000 * 1000 ) - 60 * 1000 );

    int status = new LetterBatchControl().printAMVChanges( false );
    assertEquals( "Checking that printing is ok", StatusConstants.OK, status );

    // It's not like this is especially intelligent...  ;-)
    // But it's all that we have... Just pray that there wasn't a batch run in-between!
    PrintjobDAO printJobDAO = new PrintjobDAO();
    int batchIdForTest = printJobDAO.getNextBatchId() - 1;
    List<PrintJobVO> printJobs = printJobDAO.readPrintJobs( batchIdForTest );
    assertTrue( "Checking that print jobs exist", !printJobs.isEmpty() );
    assertTrue( "Checking number of print jobs", printJobs.size() > 0 );
    for ( PrintJobVO printJob: printJobs )
    {
      assertEquals( "Checking batch ID", batchIdForTest, printJob.batchId );
      assertTrue( "Checking printer ID", printJob.druckerId > 0 );
      assertEquals( "Checking number of copies", 0, printJob.anzKopien );
      assertEquals( "Checking print status", PrintStatusConstants.READY_FOR_PRINTING, printJob.druckStatus );
      assertEquals( "Checking sender application", PrintJobVO.SYSTEM, printJob.senderApp );
      assertEquals( "Checking sender user", PrintJobVO.SYSTEM, printJob.senderUser );
      assertEquals( "Checking print job type", PrintJobVO.LETTER, printJob.formTyp );
      assertEquals( "Checking paper type", ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB().intValue(), printJob.paperMode );
      assertEquals( "Checking cover", false, printJob.coverCd );
      assertTrue( "Checking PDF file", printJob.printJobFile.length > 5400 );
      assertTrue( "Checking creation date", printJob.erstDatum.after( testStart ) );
    }

    // Clean up the mess...
    printJobDAO.deletePrintjobs( batchIdForTest );
  }
}
