package ch.ige.pctedi.client;

import java.util.Map;

import ch.ige.pctedi.server.PctDelegate;

/**
 * <p>Beschreibung: class for replacing unicode values not included inside
 * database character set ISO8859-15
 * <p>Copyright: Copyright (c) 2006</p>
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: MappingUnicode.java,v 1.6 2008/03/11 14:11:01 ruegsegger Exp $
 */
public class MappingUnicode
{
  private static MappingUnicode mappingUnicode;

  private static Map<String, String> mapUnicode = null;

  /**
   * loading all values from table mapping_unicode into hashMap
   */
  private MappingUnicode() throws Exception
  {
    mapUnicode = PctDelegate.getInstance().getAllMappingUnicode();
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * @return PctDelegate
   * @throws Exception
   */
  public static MappingUnicode getInstance() throws Exception
  {
    if ( mappingUnicode == null )
    {
      mappingUnicode = new MappingUnicode();
    }
    return mappingUnicode;
  }

  /**
   * replaces unicode values not included inside ISO8859-15
   * mapping with database table mapping_unicode
   * @param str String to replace
   * @return String replaced with mapping unicode values 
   */
  public String replaceUnicodeValues( String str )
  {
    String key = null;
    String value = null;

    for ( Map.Entry<String, String> entry: mapUnicode.entrySet() )
    {
      key = entry.getKey();
      value = entry.getValue();

      str = str.replaceAll( key, ( value == null ? "" : value ) );
    }
    return str;
  }
}
