package ch.ige.pctedi.client;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.log4j.Logger;

import ch.ige.pctedi.BuildInfo;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: PctLogFile.java,v 1.8 2008/03/11 14:11:01 ruegsegger Exp $
 */

public class PctLogFile
{
  private static final Logger LOG = Logger.getLogger( PctLogFile.class );

  public String version = BuildInfo.CVS_TAG + " - " + BuildInfo.BUILD_ID;

  private static RandomAccessFile f = null;

  private File logFile = null;

  private static final String DEFAULT_PATH = "c:\\data\\log\\pctedi_temp.log";

  public PctLogFile()
  {
  }

  /**
   * File öffnen, bzw erstellen
   * File-Pointer ans Ende der Datei setzte, damit geschriebene Daten nicht ueberschrieben werden
   * @param pathname Fielpfad des Files
   */
  public void setLogFile( File file )
  {
    this.logFile = file;
    if ( !file.getParentFile().exists() )
    {
      file.getParentFile().mkdirs();
    }

    try
    {
      f = new RandomAccessFile( file, "rw" );
      f.seek( f.length() );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
    }
  }

  public File getLogFile()
  {
    return logFile;
  }

  /**
   * File schliessen 
   */
  public void close()
  {
    try
    {
      f.close();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * Eintrag ins File schreiben
   * @param line Zeile, die ins Logfile geschrieben wird
   */
  public void write( String line )
  {
    try
    {
      if ( f == null )
      {
        createDefaultLogFile();
      }

      f.writeBytes( line + "\r\n" );
    }
    catch ( IOException ex )
    {
      LOG.error( "Eintrag kann nicht in Fehlerliste geschrieben werden: " + line );
      LOG.error( ex );
    }
  }

  /**
   * Erstellt ein Default-Logfile
   */
  private void createDefaultLogFile()
  {
    setLogFile( new File( PctLogFile.DEFAULT_PATH ) );
    LOG.info( "Fehlerliste befindet sich auf: " + DEFAULT_PATH );
  }

}
