package ch.ige.pctedi.client;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;

/**
 * wo-published-application.xml
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: XMLDoc.java,v 1.12 2008/07/01 07:09:02 kuenzli Exp $
 */

public class XMLDoc
{
  private Document doc;

  private XPath x;

  private Element e;

  private Attribute att;

  private String schutitNr = null;

  private String gesuchNr = null;

  public XMLDoc( String xmlString ) throws Exception
  {
    this( new ByteArrayInputStream( xmlString.getBytes() ) );
  }

  public XMLDoc( InputStream stream ) throws Exception
  {
    // Build the document with SAX and Xerces, no validation
    SAXBuilder builder = new SAXBuilder( "org.apache.xerces.parsers.SAXParser" );

    // system pfad wo referenzierte DTD's abgelegt sind
    // momentan nicht erforderlich, da im xml kein dtd referenziert wird
    String systemId = PctProperties.getInstance().getProperty( PctProperties.CONFIG_PATH );

    // Create the document
    doc = builder.build( stream, systemId );
  }

  public String getSchutitNr() throws Exception
  {
    if ( schutitNr == null )
    {
      schutitNr = this.getValueOf( "publication-reference", "document-id/doc-number" ).replaceAll( "/", "" );
    }
    return schutitNr;
  }

  public String getGesuchNr() throws Exception
  {
    if ( gesuchNr == null )
    {
      gesuchNr = this.getValueOf( "application-reference", "document-id/doc-number" ).replaceAll( "/", "" );
    }
    return gesuchNr;
  }

  public String getSchutitGesuchNr()
  {
    try
    {
      return getSchutitNr() + "/" + getGesuchNr();
    }
    catch ( Exception ex )
    {
      return "";
    }
  }

  public String getValueOf( String element ) throws Exception
  {
    return getValueOf( element, null );
  }

  /**
   *
   * @param parent String
   * @param element String
   * @return String falls Element nicht existiert wird "" zurückgegeben
   */
  public String getValueOf( String parent, String element ) throws Exception
  {
    x = XPath.newInstance( getXPath( parent, element ) );
    e = (Element) x.selectSingleNode( doc );
    return ( e == null ? "" : e.getText() );
  }

  public List<String> getValuesOf( String element ) throws Exception
  {
    return getValuesOf( element, null );
  }

  @SuppressWarnings( "unchecked" )
  public List<String> getValuesOf( String parent, String element ) throws Exception
  {
    List<Element> elementList;
    List<String> valueList = new ArrayList<String>();

    x = XPath.newInstance( getXPath( parent, element ) );
    elementList = x.selectNodes( doc );

    for ( int i = 0; i < elementList.size(); i++ )
    {
      Element ele = elementList.get( i );
      valueList.add( ele.getText() );
    }

    return valueList;
  }

  public String getAttribute( String element, String attribute ) throws Exception
  {
    return getAttribute( element, null, attribute );
  }

  public String getAttribute( String parent, String element, String attribute ) throws Exception
  {
    try
    {
      x = XPath.newInstance( getXPath( parent, element, attribute ) );
      att = (Attribute) x.selectSingleNode( doc );
      return ( att == null ? "" : att.getValue() );
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method getAttribute parent=<" + parent + "> element=<" + element + "> attribute=<" + attribute + "> message: " + ex );
    }
  }

  public List<String> getAttributes( String element, String attribute ) throws Exception
  {
    return getAttributes( element, null, attribute );
  }

  @SuppressWarnings( "unchecked" )
  public List<String> getAttributes( String parent, String element, String attribute ) throws Exception
  {
    List<Attribute> attList;
    List<String> valueList = new ArrayList<String>();

    try
    {
      x = XPath.newInstance( getXPath( parent, element, attribute ) );
      attList = x.selectNodes( doc );
      for ( int i = 0; i < attList.size(); i++ )
      {
        att = attList.get( i );
        valueList.add( att.getValue() );
      }
      return valueList;
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method getAttributes parent=<" + parent + "> element=<" + element + "> attribute=<" + attribute + "> message: " + ex );
    }
  }

  public Map<String, String> getAttributesOfElement( String element ) throws Exception
  {
    return getAttributesOfElement( element, null );
  }

  @SuppressWarnings( "unchecked" )
  public Map<String, String> getAttributesOfElement( String parent, String element ) throws Exception
  {
    Map<String, String> map = new HashMap<String, String>();

    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      e = (Element) x.selectSingleNode( doc );

      List<Attribute> attList = e.getAttributes();

      for ( int i = 0; i < attList.size(); i++ )
      {
        att = attList.get( i );
        map.put( att.getName(), att.getValue() );
      }

      return map;
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method getAttributesOfElement parent=<" + parent + "> element=<" + element + "> message: " + ex );
    }
  }

  @SuppressWarnings( "unchecked" )
  public List<Element> getElements( String element ) throws Exception
  {
    try
    {
      x = XPath.newInstance( getXPath( element ) );
      return x.selectNodes( doc );
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method getElements element=<" + element + "> message: " + ex );
    }
  }

  public String getValueOfElement( Object element, String child )
  {
    String value = null;

    if ( element instanceof Element )
    {
      value = getValueOf( (Element) element, child );
    }

    return value;
  }

  @SuppressWarnings( "unchecked" )
  public List<String> getValuesOfElement( Object element, String child )
  {
    List<String> values = new ArrayList<String>();

    if ( element instanceof Element )
    {
      List<Element> elements = ( (Element) element ).getChildren( child );
      for ( int i = 0; i < elements.size(); i++ )
      {
        values.add( ( (Element) element ).getText() );
      }
    }

    return values;
  }

  public String getAttributeOfElement( Object element, String attribute )
  {
    String value = null;

    if ( element instanceof Element )
    {
      value = this.getAttributeOf( (Element) element, attribute );
    }

    return value;
  }

  public String getValueOf( Element element, String child )
  {
    return element.getChildText( child );
  }

  public String getValueOfElement( Object obj, String[] child )
  {
    Element element = null;

    if ( obj instanceof Element )
    {
      element = (Element) obj;
      for ( int i = 0; i < child.length; i++ )
      {
        if ( element != null )
        {
          element = element.getChild( child[i] );
        }
      }
    }

    return ( element != null ? element.getValue() : "" );
  }

  @SuppressWarnings( "unchecked" )
  public List<String> getValuesOfElement( Object obj, String[] child, String children )
  {
    List<String> list = new ArrayList<String>();
    Element element = null;

    if ( obj instanceof Element )
    {
      element = (Element) obj;
      for ( int i = 0; i < child.length; i++ )
      {
        if ( element != null )
        {
          element = element.getChild( child[i] );
        }
      }
    }

    if ( element != null )
    {
      List<Element> l = element.getChildren();
      for ( int i = 0; i < l.size(); i++ )
      {
        Element ele = l.get( i );
        String value = ele.getValue();
        if ( value != null & !value.equals( "" ) )
        {
          list.add( value );
        }
      }
    }

    return list;
  }

  public String getAttributeOf( Element element, String attribute )
  {
    return element.getAttributeValue( attribute );
  }

  public int getNumberOf( String element ) throws Exception
  {
    try
    {
      x = XPath.newInstance( getXPath( element ) );
      return x.selectNodes( doc ).size();
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method getNumberOf element=<" + element + "> message: " + ex );
    }
  }

  public boolean exists( String element ) throws Exception
  {
    return exists( element, null );
  }

  public boolean exists( String parent, String element ) throws Exception
  {
    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      e = (Element) x.selectSingleNode( doc );
      return ( e == null ? false : true );
    }
    catch ( JDOMException ex )
    {
      throw new Exception( "method exists parent=<" + parent + "> element=<" + element + "> message: " + ex );
    }
  }

  private String getXPath( String element )
  {
    return getXPath( element, null, null );
  }

  private String getXPath( String parent, String element )
  {
    return getXPath( parent, element, null );
  }

  private String getXPath( String parent, String element, String attribute )
  {
    StringBuffer xpath = new StringBuffer();
    xpath.append( ".//" );
    xpath.append( parent );
    if ( element != null && !element.equals( "" ) )
    {
      xpath.append( "/" );
      xpath.append( element );
    }
    if ( attribute != null && !attribute.equals( "" ) )
    {
      xpath.append( "/@" );
      xpath.append( attribute );
    }
    return xpath.toString();
  }

}
