package ch.ige.pctedi.client.gazette;

import java.io.File;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.pctedi.BuildInfo;
import ch.ige.pctedi.client.PctZipFile;
import ch.ige.pctedi.client.batch.Process;
import ch.ige.pctedi.server.PctDelegate;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class Gazette extends Process
{
  private static final Logger LOG = Logger.getLogger( "ch.ige.pctedi.client.gazette" );

  private GazetteLogFile logFile = null;

  private String titel = " Logfile Gazette Section II";

  @Override
  protected String getAttachement()
  {
    return logFile != null ? logFile.getLogFile().getPath() : null;
  }

  @Override
  protected String getSubject()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( BuildInfo.BUILD_ENV_NAME );
    buf.append( " - " );
    buf.append( "PCT Verarbeitung " + zipFile.extractProcessWeek() );
    buf.append( titel );

    return buf.toString();
  }

  @Override
  protected void process( PctZipFile pctZipFile ) throws Exception
  {
    LOG.debug( "gazette process started... " );

    String processWeek = pctZipFile.extractProcessWeek();

    GazetteFile startDir = new GazetteFile( pctZipFile.getUnzipDirectory() );
    List<File> listGazette = startDir.listFiles( new GazetteFileFilter( processWeek ) );

    if ( !listGazette.isEmpty() )
    {
      for ( int i = 0; i < listGazette.size(); i++ )
      {
        process( listGazette.get( i ), pctZipFile );
      }
    }
    else
    {
      error = true;
      logAndAppendMsg( "no gazette section II file found to process in " + pctZipFile.getPath() );
    }

    LOG.debug( "gazette process finished..." );
  }

  public void process( File gazetteFile, PctZipFile pctZipFile ) throws Exception
  {
    LOG.debug( "process file: " + gazetteFile.getPath() );

    String processWeek = pctZipFile.extractProcessWeek();

    logFile = GazetteLogFile.getInstance();
    logFile.setLogFile( processWeek );

    GazetteProcess process = new GazetteProcess( gazetteFile.getPath() );
    Date publDat = process.extractPublDat();

    logFile.writeHeader( publDat, gazetteFile.getPath(), processWeek );

    List<String> listSchutitNr = process.extractSchutitNr();
    logFile.write( "Anzahl PCT's : " + listSchutitNr.size() + "\r\n" );

    save( listSchutitNr, publDat );

    logFile.writeFooter();
  }

  public void save( List<String> listSchutitNr, Date publDat ) throws Exception
  {
    int anzFound = 0;
    int anzNotFound = 0;

    for ( int i = 0; i < listSchutitNr.size(); i++ )
    {
      String schutitNr = listSchutitNr.get( i );
      int schutitKey = PctDelegate.getInstance().selectSchutitKey( schutitNr );

      if ( schutitKey != 0 )
      {
        anzFound++;
        String message = PctDelegate.getInstance().createExtension( schutitKey, publDat );
        logFile.write( "PCT: " + schutitNr + " " + message );
      }
      else
      {
        anzNotFound++;
      }
    }

    logFile.write( "\r\nAnzahl gefundene PCT's: " + anzFound );
    logFile.write( "Anzahl nicht gefundene PCT's: " + anzNotFound );
  }
}
