package ch.ige.pctedi.client.gazette;

import java.io.File;
import java.io.FileInputStream;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jdom.Element;

import ch.ige.pctedi.client.XMLDoc;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: GazetteProcess.java,v 1.3 2008/03/11 14:55:12 ruegsegger Exp $
 */
public class GazetteProcess
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "ddMMyyyy" );

  private DecimalFormat df = new DecimalFormat( "000000" );

  private GazetteLogFile logFile;

  private XMLDoc xml;

  public GazetteProcess( String fileName ) throws Exception
  {
    FileInputStream fi = new FileInputStream( new File( fileName ) );
    xml = new XMLDoc( fi );
    sdf.setLenient( false );
  }

  public void setLogFile( GazetteLogFile logFile )
  {
    this.logFile = logFile;
  }

  /**
   * extracts production date from xml-file
   * @throws ParseException
   * @return Date production date
   */
  public Date extractPublDat() throws Exception
  {
    try
    {
      String date = xml.getAttribute( "wo-gazette-section-ii", "date-produced" );
      Date publDat = sdf.parse( date );
      return publDat;
    }
    catch ( ParseException ex )
    {
      throw new Exception( "Verarbeitung abgebrochen - Fehler beim Extrahieren des Publikations-Datums: " + ex.getMessage() );
    }
  }

  /**
   * extract all documents numbers of demands for international preliminary examination
   * where CH/LI is not excluded for
   * @return List documents numbers of demands for international preliminary examination
   */
  public List<String> extractSchutitNr() throws Exception
  {
    List<String> listSchutitNr = new ArrayList<String>();

    String docNumber = null;
    String schutitNr = null;

    String demands = "wo-chapter-ii-demands/wo-chapter-ii-demand";

    List<Element> elements = xml.getElements( demands );
    for ( int i = 0; i < elements.size(); i++ )
    {
      String filedAfter = xml.getAttributeOfElement( elements.get( i ), "wo-filed-after-expiration-19months" );
      if ( filedAfter == null || !filedAfter.equals( "yes" ) )
      {
        try
        {
          if ( !isCHexcluded( elements.get( i ) ) )
          {
            String[] documentNumber = { "publication-reference", "document-id", "doc-number" };
            docNumber = xml.getValueOfElement( elements.get( i ), documentNumber );
            schutitNr = getFormattedSchutit( docNumber );
            listSchutitNr.add( schutitNr );
          }
        }
        catch ( Exception ex )
        {
          if ( logFile != null )
          {
            logFile.write( "PCT : " + docNumber + " not processed invalid format of PCT Number" );
          }
        }
      }
    }
    return listSchutitNr;
  }

  /**
   * controls whether CH or LI ist excluded  from demand
   * @param demand String element to control
   * @return boolean true if CH or LI is excluded from demand
   */
  private boolean isCHexcluded( Object element ) throws Exception
  {
    boolean isExcluded = false;

    String[] excludedStates = { "wo-excluded-states", "designated-states" };
    List<String> excludedCountries = xml.getValuesOfElement( element, excludedStates, "country" );

    if ( excludedCountries != null && excludedCountries.size() > 0 )
    {
      isExcluded = excludedCountries.contains( "CH" ) || excludedCountries.contains( "LI" );
    }

    return isExcluded;
  }

  /**
   * formats doc number to year of four position and schutit number of six positions
   * year and schutit number is expected to be separated by a slash
   * 05/0654231     -> 20050654231
   * 2005/0654231   -> 20050654231
   * 05/654231      -> 20050654231
   * @param documentNr String doc number to format
   * @throws Exception invalid format of schutit number
   * @return String formatted schutit number
   */
  private String getFormattedSchutit( String documentNr ) throws Exception
  {
    String schutitNr = documentNr.substring( documentNr.indexOf( "/" ) + 1 );
    String year = documentNr.substring( 0, documentNr.indexOf( "/" ) );

    try
    {
      schutitNr = df.format( df.parse( schutitNr ) );

      if ( year.length() == 2 )
      {
        int y = Integer.parseInt( year );
        year = String.valueOf( y += y < 70 ? 2000 : 1900 );
      }
    }
    catch ( Exception ex )
    {
      logFile.write( "PCT : " + documentNr + " nicht verarbeitet ungültiges format der PCT Number" );
      throw ex;
    }

    return ( year + schutitNr );
  }

}
