package ch.ige.pctedi.client.wpa;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.vo.EreignisVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: Ereignis.java,v 1.14 2008/03/11 14:55:12 ruegsegger Exp $
 */

public class Ereignis
{
  private XMLDoc xml;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  private WpaLogFile logFile = WpaLogFile.getInstance();

  public Ereignis()
  {
    sdf.setLenient( false );
  }

  public List<EreignisVO> extract( XMLDoc xmlDoc ) throws Exception
  {
    this.xml = xmlDoc;
    EreignisVO ereignisVO;
    List<EreignisVO> listEreignis = new ArrayList<EreignisVO>();

    ereignisVO = this.getEreigErsterfassung();
    ereignisVO.ereigIdat = getAppEreigIdat();
    listEreignis.add( ereignisVO );

    boolean chPctFound = xml.exists( "designation-of-states/designation-pct", "regional[country='CH'|country='LI'" );
    // Ereignis 30010
    if ( chPctFound )
    {
      ereignisVO = this.getEreigNationalePhase();
      ereignisVO.ereigIdat = this.getPubEreigIdat();
      ereignisVO.fristablDat = this.getFristAblDat();

      listEreignis.add( ereignisVO );
    }
    return listEreignis;
  }

  /**
   * Fristablauf für Ereignis 30010 (nationale Phase) ermitteln
   * Falls Prioritaet vorhanden das älteste prio_datum + 30 Monate sonst Anmeldedatum + 30 Monate
   *
   * @return Date
   */
  private Date getFristAblDat() throws Exception
  {
    Prioritaet prio = new Prioritaet();

    Date fristAblDate = prio.getMinPrioDat( xml );

    if ( fristAblDate == null )
    {
      fristAblDate = this.getFillingDate();
    }

    GregorianCalendar cal = new GregorianCalendar();
    cal.setTime( fristAblDate );
    cal.add( Calendar.MONTH, 30 );

    return cal.getTime();
  }

  private int getPubEreigIdat() throws Exception
  {
    StringBuffer buf = new StringBuffer();
    buf.append( sdf.format( this.getPublicationDate() ) );
    buf.append( "05" );
    return new Integer( buf.toString() ).intValue();
  }

  private int getAppEreigIdat() throws Exception
  {
    StringBuffer buf = new StringBuffer();
    buf.append( sdf.format( getFillingDate() ) );
    buf.append( "05" );
    return new Integer( buf.toString() ).intValue();
  }

  /**
   * Anmeldedatum ist erforderlich - Teils des PrimaryKeys
   * @throws Exception
   * @return Date
   */
  private Date getFillingDate() throws Exception
  {
    String fillingDate = null;
    try
    {
      fillingDate = xml.getValueOf( "application-reference", "document-id/date" );
      return sdf.parse( fillingDate );
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Kein oder ungueltiges Anmeldedatum <" + fillingDate
          + "> vorhanden. Ereignis 30000 kann nicht erstellt werden" );
      throw ex;
    }
  }

  /**
   * Publikationsdatum ist erforderlich - Teil des PrimaryKeys
   * @throws Exception
   * @return Date
   */
  private Date getPublicationDate() throws Exception
  {
    String publicationDate = null;
    try
    {
      publicationDate = xml.getValueOf( "publication-reference", "document-id/date" );
      return sdf.parse( publicationDate );
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Kein oder ungueltiges Publlikationsdatum <" + publicationDate
          + "> vorhanden. Ereignis 30010 kann nicht erstellt werden" );
      throw ex;
    }
  }

  /**
   * Generiert Ereignis für Ersterfassung 30000
   * @return Ereignis "Ersterfassung"
   */
  public EreignisVO getEreigErsterfassung()
  {
    EreignisVO ereignisVO = new EreignisVO();

    ereignisVO.artNr = EreignisVO.ERSTERFASSUNG;
    ereignisVO.regStat = EreignisVO.REGSTAT_GESUCH;
    ereignisVO.ereigStatNr = EreignisVO.EREIG_STAT_KEINE_FRIST;

    return ereignisVO;
  }

  /**
   * Generiert Ereignis für Eintritt in die nationale Phase 30010
   * @return Ereignis "Eintritt in die nationale Phase"
   */
  public EreignisVO getEreigNationalePhase()
  {
    EreignisVO ereignisVO = new EreignisVO();

    ereignisVO.artNr = EreignisVO.NATIONALE_PHASE;
    ereignisVO.regStat = EreignisVO.REGSTAT_GESUCH;
    ereignisVO.ereigStatNr = EreignisVO.EREIG_STAT_FRIST;

    return ereignisVO;
  }
}
