package ch.ige.pctedi.client.wpa;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import org.jdom.Element;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.server.PctDelegate;
import ch.ige.pctedi.vo.PrioritaetVO;

//import ch.ige.util.MinimalDate;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: Prioritaet.java,v 1.13 2008/03/11 14:55:12 ruegsegger Exp $
 */

public class Prioritaet
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  private List<PrioritaetVO> listPrioVO;

  private WpaLogFile logFile = WpaLogFile.getInstance();

  private List<String> listBland;

  public Prioritaet() throws Exception
  {
    if ( listBland == null )
    {
      listBland = PctDelegate.getInstance().getAllBland();
    }
    sdf.setLenient( false );
  }

  public Prioritaet( List<String> listBland )
  {
    this.listBland = listBland;
    sdf.setLenient( false );
  }

  /**
   * <!ELEMENT wo-bibliographic-data (..., wo-priority-info?, ...)>
   * <!ELEMENT wo-priority-info (priority-claim , priority-claim-ib-info?)+>
   * <!ELEMENT priority-claim (country , doc-number? , date , office-of-filing? , (priority-doc-requested | priority-doc-attached)?)>
   * <!ATTLIST priority-claim     id       ID     #IMPLIED
   *                              sequence CDATA  #REQUIRED
   *                              kind            (national | regional | international )  #REQUIRED >
   *
   * @param xml WPAXml
   * @return List
   */
  public List<PrioritaetVO> extract( XMLDoc xml ) throws Exception
  {
    listPrioVO = new ArrayList<PrioritaetVO>();
    PrioritaetVO vo = null;

    List<Element> elementList = xml.getElements( "priority-claim" );

    for ( int i = 0; i < elementList.size(); i++ )
    {
      vo = new PrioritaetVO();
      String sequence = xml.getAttributeOfElement( elementList.get( i ), "sequence" );
      vo.prioLnr = Integer.parseInt( sequence );
      vo.prioNr = xml.getValueOfElement( elementList.get( i ), "doc-number" );

      // blandCd (prio-land) darf nicht null sein
      vo.blandCd = xml.getValueOfElement( elementList.get( i ), "country" );
      if ( vo.blandCd == null || !listBland.contains( vo.blandCd ) )
      {
        logFile.appendMessage( WpaLogFile.WARN, "PCT " + xml.getSchutitGesuchNr() + " Bei Prioritaet " + vo.prioNr + " ungueltiges Land <" + vo.blandCd
            + "> CH eingefügt" );
        vo.blandCd = "CH";
      }

      // prioDat darf nicht null sein
      String date = xml.getValueOfElement( elementList.get( i ), "date" );
      vo.prioDat = this.getPrioDate( date );
      if ( vo.prioDat == null )
      {
        String fillingDate = xml.getValueOf( "application-reference", "document-id/date" );
        vo.prioDat = sdf.parse( fillingDate );
        logFile.appendMessage( WpaLogFile.ERROR, "PCT " + xml.getSchutitGesuchNr() + " Bei Prioritaet " + vo.prioNr + " ungueltiges Datum <" + date
            + "> Anmeldedatum <" + fillingDate + "> eingefügt " );
      }

      listPrioVO.add( vo );
    }

    return listPrioVO;
  }

  /**
   * Parsed das Datum anhand des Patterns "yyyyMMdd"
   * Funktion parse schneidet ab, deshalb muss vorgängig die Länge überprüft werden
   *
   * @param date String zu parsendes Datum im Format "yyyyMMdd"
   * @return Date geparstes Datum falls ungültig wird null zurückgegeben
   */
  private Date getPrioDate( String date )
  {
    Date prioDate = null;

    if ( date != null && date.length() == sdf.toPattern().length() )
    {
      try
      {
        prioDate = sdf.parse( date );
      }
      catch ( ParseException ex )
      {
        // return null bei ungültigem Datum
      }
    }
    return prioDate;
  }

  /**
   * Ältestet Prioritäts-Datum ermitteln
   * Wird benötigt für Erstellung Ereignis 30010 (20Monatefrist nat. Phase)
   * Attribute Fristablaufdatum = ältestes Prioritätsdatum + 20 Monate
   *
   * @param xml WPAXml
   * @return Date ältestes Prioritätsdatum null falls keine Prioritätsdaten vorhanden
   */
  public Date getMinPrioDat( XMLDoc xml ) throws Exception
  {
    Date minPrioDat = null;
    Date prioDate = null;
    SortedSet<Date> set = new TreeSet<Date>();

    List<String> listPrioDate = xml.getValuesOf( "priority-claim", "date" );

    for ( String date: listPrioDate )
    {
      prioDate = getPrioDate( date );

      if ( prioDate != null )
      {
        set.add( prioDate );
      }
      else
      {
        logFile.appendMessage( WpaLogFile.WARN, "PCT " + xml.getSchutitGesuchNr() + " Beim Ermitteln Fristablauf Ereignis 30010 ungueltiges Prio-Datum <"
            + date + "> nicht beruecksichtigt" );
      }
    }

    if ( set.size() > 0 )
    {
      minPrioDat = set.iterator().next();
    }

    return minPrioDat;
  }

}
