package ch.ige.pctedi.client.wpa;

import java.io.File;
import java.util.Date;
import org.apache.log4j.Logger;
import ch.ige.pctedi.client.*;
import ch.ige.pctedi.vo.StatistikVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: WpaLogFile.java,v 1.11 2008/07/01 07:44:22 kuenzli Exp $
 */

public class WpaLogFile extends PctLogFile
{
  private static final Logger LOG = Logger.getLogger( WpaLogFile.class );
  private static WpaLogFile logFile;
  private StringBuffer buf = new StringBuffer();

  public static final int DEBUG = 0;
  public static final int WARN = 1;
  public static final int INFO = 2;
  public static final int ERROR = 3;

  private WpaLogFile()
  {
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   */
  public static WpaLogFile getInstance()
  {
    if ( logFile == null )
    {
      logFile = new WpaLogFile();
    }
    return logFile;
  }

  /**
   * File öffnen, bzw erstellen
   * File-Pointer ans Ende der Datei setzte, damit geschriebene Daten nicht ueberschrieben werden
   * @param publDat Publikationsdatum im format wwyyyy 
   */ 
  public void setLogFileName( String processWeek )
  {  
    File file = new File( PctProperties.getInstance().getProperty( PctProperties.LOGFILE_PATH ), "PCT_" + processWeek + ".log" );
    super.setLogFile( file );
  }

  public void appendMessage( int loglevel, String line )
  {
    if ( buf.toString().trim().length() > 0 )
    {
      buf.append( "\r\n" );
    }

    switch ( loglevel )
    {
      case DEBUG:
        if ( LOG.isDebugEnabled() )
        {
          buf.append( "DEBUG \t" );
          buf.append( line );
        }
        break;
      case WARN:
        buf.append( "WARN  \t" );
        buf.append( line );
        break;
      case INFO:
        buf.append( "INFO  \t" );
        buf.append( line );
        break;
      case ERROR:
        buf.append( "ERROR \t" );
        buf.append( line );
        break;
    }
  }


  public void clearMessage()
  {
    buf = new StringBuffer();
  }

  
  public void writeMessage()
  {
    if ( !buf.toString().equals( "" ) )
    {
      this.write( buf.toString() );
      clearMessage();
    }
  }

  public void writeHeader( PctZipFile file )
  {
    write( "Verarbeitung PCT " + file.extractProcessWeek() + "\r\n" );
    write( "Version            : " + version + "\r\n" );
    write( "Gestartet am       : " + new Date() );
    write( "Verarbeiten von    : " + file.getPath() );
    write( "Verarbeitungswoche : " + file.extractProcessWeek() );
    write( "=======================================================\r\n" );
  }

  public void writeStatistik( StatistikVO statistikVO )
  {
    write( "\r\n" );
    write( "Statistik : \r\n" );
    write( "CH-PCT            : " + statistikVO.chCnt );
    write( "EURO-PCT          : " + statistikVO.euroCnt );
    write( "CH&EURO-PCT       : " + statistikVO.chEuroCnt );
    write( "Nicht CH-PCT      : " + statistikVO.notChCnt );
    write( "Total verarbeitet : " +
           ( statistikVO.chCnt + statistikVO.euroCnt + statistikVO.chEuroCnt + statistikVO.notChCnt ) );
  }

  public void writeFooter()
  {
    write( "\r\n" );
    write( "Beendet am        : " + new Date() );
    write( "=======================================================" );
    write( "\r\n" );
  }

  /**
   * Schreibt eine Debug-Message ins Logfile, sofern der Log-Level im Config-File auf Debug gesetzt ist
   * @param line String Logmeldung
   */
  public void writeDebugMessage( String line )
  {
    if ( LOG.isDebugEnabled() )
    {
      write( line );
    }
  }

}
