package ch.ige.pctedi.server.dao;

import java.sql.*;
import java.util.Date;
import ch.ige.pctedi.vo.IpcVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für die Tabelle IPC
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Martin Trachsel
 * @version $Id: IpcDAO.java,v 1.9 2006/12/11 14:34:59 ruegsegger Exp $
 */

public class IpcDAO
{

  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle IPC
   *
   * @param  ipcSymb
   * @return IpcVO ValueObject
   *
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public IpcVO select( String ipcSymb, Date gueltigVon ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    IpcVO ipcVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "SELECT trim(ipc_symb) as ipc_symb, zusatz, gueltig_von, gueltig_bis, class_level, core_pred, version " +
           "FROM ipc WHERE ipc_symb = RPAD(?,14,' ') AND gueltig_von = ?";
      ps = con.prepareStatement( query );
      ps.setString( 1, ipcSymb );
      if( gueltigVon == null )
      {
        ps.setDate( 2, null );
      }
      else
      {
        ps.setDate( 2, new java.sql.Date( gueltigVon.getTime() ) );
      }

      rs = ps.executeQuery();
      if( rs.next() )
      {
        ipcVO = new IpcVO();
        ipcVO.ipcSymb = rs.getString( "ipc_symb" );
        ipcVO.zusatz = rs.getString( "zusatz" );
        ipcVO.gueltigVon = rs.getDate( "gueltig_von" );
        ipcVO.gueltigBis = rs.getDate( "gueltig_bis" );
        ipcVO.classLevel = rs.getString( "class_level" );
        ipcVO.corePred = rs.getString( "core_pred" );
        ipcVO.version = rs.getInt( "version" );
      }

      return ipcVO;
    }
    catch( SQLException ex )
    {
      LogLevelHelper.log( ex, IpcDAO.class, query + " ipcSymb: " + ipcSymb );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle IPC
   *
   * @param  ipcSymb
   * @return IpcVO ValueObject
   *
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public IpcVO selectYoungest( String ipcSymb ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    IpcVO ipcVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "SELECT trim(ipc_symb) as ipc_symb, zusatz, gueltig_von, gueltig_bis, class_level, core_pred, version " +
           "FROM ipc WHERE ipc_symb = RPAD(?,14,' ') ORDER BY gueltig_von DESC";
      ps = con.prepareStatement( query );
      ps.setString( 1, ipcSymb );

      rs = ps.executeQuery();
      if( rs.next() )
      {
        ipcVO = new IpcVO();
        ipcVO.ipcSymb = rs.getString( "ipc_symb" );
        ipcVO.zusatz = rs.getString( "zusatz" );
        ipcVO.gueltigVon = rs.getDate( "gueltig_von" );
        ipcVO.gueltigBis = rs.getDate( "gueltig_bis" );
        ipcVO.classLevel = rs.getString( "class_level" );
        ipcVO.corePred = rs.getString( "core_pred" );
        ipcVO.version = rs.getInt( "version" );
      }

      return ipcVO;
    }
    catch( SQLException ex )
    {
      LogLevelHelper.log( ex, IpcDAO.class, query + " ipcSymb: " + ipcSymb  );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}


