package ch.ige.pctedi.server.dao;

import java.sql.*;
import ch.ige.pctedi.vo.PctAdrVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für die Tabelle PCTADR
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Martin Trachsel
 * @version $Id: PctAdrDAO.java,v 1.8 2006/12/11 14:34:59 ruegsegger Exp $
 */

public class PctAdrDAO
{
  /**
   * Fügt einen Datensatz in die Tabelle PCTADR ein
   *
   * @param pctAdrVO  ValueObject
   * @exception SQLException Unerwarteter SQL Fehler
   */

  public int insert( PctAdrVO pctAdrVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "INSERT INTO pctadr(" +
          "schutit_key, " +
          "pct_adr_lnr, " +
          "bland_cd, " +
          "schutit_typ, " +
          "adr_rolle_cd, " +
          "pct_name, " +
          "pct_adresse, " +
          "version) " +
          "VALUES (" +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?, " +
          "?) ";

      ps = con.prepareStatement(query);

      ps.setInt( 1, pctAdrVO.schutitKey );
      ps.setInt( 2, pctAdrVO.pctAdrLnr );
      ps.setString( 3, pctAdrVO.blandCd );
      ps.setInt( 4, pctAdrVO.schutitTyp );
      ps.setInt( 5, pctAdrVO.adrRolleCd );
      ps.setString( 6, pctAdrVO.pctName );
      ps.setString( 7, pctAdrVO.pctAdresse );
      ps.setInt( 8, pctAdrVO.version );

      return ps.executeUpdate();

    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PctAdrDAO.class, query + pctAdrVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}

