package ch.ige.pctedi.server.dao.test;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Date;

import ch.ige.pctedi.server.dao.PatIpcDAO;
import ch.ige.pctedi.vo.PatIpcVO;
import ch.ige.pctedi.vo.test.Testdata;

/**
 * Test der Klasse PatIpcDAO
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: PatIpcDAOTest.java,v 1.7 2008/03/11 14:11:01 ruegsegger Exp $
 */

public class PatIpcDAOTest extends DAOTestCase
{

  private PatIpcDAO patIpcDAO = new PatIpcDAO();

  private PatIpcVO patIpcVO = null;

  private PatIpcVO selectPatIpcVO = null;

  private int schutitKey = 2100393404;

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  public PatIpcDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {

    try
    {
      patIpcVO = Testdata.createPatIpc();
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle PAT_IPC ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );

    try
    {
      int retVal = patIpcDAO.insert( patIpcVO );
      assertEquals( "Anzahl eingefügte Datensätze ", 1, retVal );

      selectPatIpcVO = patIpcDAO.selectPatIpcVO( patIpcVO.schutitKey, patIpcVO.patIpcCd, patIpcVO.patIpcLnr );

      assertEquals( patIpcVO.ipcSymb, selectPatIpcVO.ipcSymb );
      assertEquals( patIpcVO.classLevel, selectPatIpcVO.classLevel );
      assertEquals( patIpcVO.classValue, selectPatIpcVO.classValue );
      assertEquals( patIpcVO.geschichte, selectPatIpcVO.geschichte );
      assertEquals( patIpcVO.source, selectPatIpcVO.source );
      assertEquals( patIpcVO.generatingOffice, selectPatIpcVO.generatingOffice );
      assertEquals( sdf.format( new Date() ), sdf.format( selectPatIpcVO.erfassDat ) );
      assertEquals( patIpcVO.erfassMa, selectPatIpcVO.erfassMa );
      assertEquals( patIpcVO.version, selectPatIpcVO.version );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle PAT_IPC ein
   * Lokal kann dieser Test nicht durchgeführt werden, da Log einen Null-Pointer verursacht (JMSException)
   */
  public void test02Insert()
  {
    System.out.println( "INSERT TEST CONSTRAINT IPC_SYMB" );

    try
    {
      patIpcVO.ipcSymb = "A01B003426";

      int retVal = patIpcDAO.insert( patIpcVO );
      if ( retVal == 1 )
      {
        fail( "Kein Constraint - Ungültiges Symbol eingefügt " );
      }
    }
    catch ( SQLException e )
    {
      assertTrue( e.getSQLState().equals( "23000" ) );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle PAT_IPC ein
   * Lokal kann dieser Test nicht durchgeführt werden, da Log einen Null-Pointer verursacht (JMSException)
   */
  public void test03Insert()
  {
    System.out.println( "INSERT TEST CONSTRAINT IPC-GUELTIG-VON" );

    try
    {
      patIpcVO.ipcGueltigVon = sdf.parse( "2000101" );

      int retVal = patIpcDAO.insert( patIpcVO );
      if ( retVal == 1 )
      {
        fail( "Kein Constraint - Ungültiges Symbol eingefügt " );
      }
    }
    catch ( SQLException e )
    {
      assertTrue( e.getSQLState().equals( "23000" ) );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   *  Löscht einen Datensatz aus der Tabelle PAT_IPC
   */
  public void test04Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      int retVal = patIpcDAO.delete( schutitKey, PatIpcVO.IPC_CD_N, 1 );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
