package ch.ige.rbbagisexport.server.config;

import java.sql.SQLException;

import org.apache.log4j.Logger;

import ch.ige.rbbagisexport.common.RBBagisExportConfiguration;
import ch.ige.rbbagisexport.server.dao.XxSysparamsDAO;

/**
 * Provides externally configurable values and reads them, if necessary, from DB.
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author M. Künzli
 * @version $Id: RBBagisExportConfigReader.java,v 1.6 2008/10/03 11:31:13 kuenzli Exp $
 */
public class RBBagisExportConfigReader
{
  //----------------------------------------------------------------------------
  // Private Members
  //----------------------------------------------------------------------------
  private static final Logger LOG = Logger.getLogger( RBBagisExportConfigReader.class );
  private static RBBagisExportConfigReader instance = null;

  private RBBagisExportConfiguration config = null;

  private static final Integer EXPORT_SERVER_NAME_PARAM_ID = Integer.valueOf( 40 );

  private static final Integer EXPORT_DIRECTORY_PARAM_ID = Integer.valueOf( 41 );

  private static final Integer EXPORT_FTP_LOGIN_PARAM_ID = Integer.valueOf( 42 );

  private static final Integer EXPORT_FTP_PASSWORD_PARAM_ID = Integer.valueOf( 43 );

  private static final Integer EXPORT_SMTP_HOST_PARAM_ID = Integer.valueOf( 44 );

  private static final Integer EXPORT_MAIL_TO_PARAM_ID = Integer.valueOf( 45 );

  private static final Integer EXPORT_MAIL_CC_PARAM_ID = Integer.valueOf( 46 );

  private static final Integer EXPORT_MAIL_FROM_PARAM_ID = Integer.valueOf( 47 );

  private static final Integer EXPORT_EREIGNIS_LIST_ID = Integer.valueOf( 48 );

  private static final Integer EXPORT_FILE_PATH = Integer.valueOf( 49 );
  //----------------------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------------------
  private RBBagisExportConfigReader()
  {
  }
  //----------------------------------------------------------------------------
  // Public Members
  //----------------------------------------------------------------------------
  public static RBBagisExportConfigReader instance(){
    if(RBBagisExportConfigReader.instance == null){
      RBBagisExportConfigReader.instance = new RBBagisExportConfigReader();
    }
    return RBBagisExportConfigReader.instance;
  }
  public RBBagisExportConfiguration readConfigurationFromDb()
  {
    XxSysparamsDAO sysparamsDAO = null;
    config = new RBBagisExportConfiguration();
    try
    {
      sysparamsDAO = new XxSysparamsDAO();
      config.setExportServerName( sysparamsDAO.getParameter( EXPORT_SERVER_NAME_PARAM_ID ) );
      config.setExportDirectory( sysparamsDAO.getParameter( EXPORT_DIRECTORY_PARAM_ID ) );
      config.setExportFtpLogin( sysparamsDAO.getParameter( EXPORT_FTP_LOGIN_PARAM_ID ) );
      config.setExportFtpPassword( sysparamsDAO.getParameter( EXPORT_FTP_PASSWORD_PARAM_ID ) );
      config.setExportMailTo( sysparamsDAO.getParameter( EXPORT_MAIL_TO_PARAM_ID ) );
      config.setExportMailCc( sysparamsDAO.getParameter( EXPORT_MAIL_CC_PARAM_ID ) );
      config.setExportMailFrom( sysparamsDAO.getParameter( EXPORT_MAIL_FROM_PARAM_ID ) );
      config.setExportSmtpHost( sysparamsDAO.getParameter( EXPORT_SMTP_HOST_PARAM_ID ) );
      config.setExportEreignisListe( sysparamsDAO.getParameter( EXPORT_EREIGNIS_LIST_ID ) );
      config.setExportFilePath( sysparamsDAO.getParameter( EXPORT_FILE_PATH ) );
    }
    catch ( SQLException e )
    {
      LOG.error( "readConfigurationFromDb(): Could not read from DB; " + e.getMessage() );
    }
    return config;
  }

  public RBBagisExportConfiguration getConfiguration(){
      readConfigurationFromDb();
      if(LOG.isDebugEnabled()){
      	  LOG.debug("getConfiguration(): Configuration: "+config); 
      }
    return config;
  }
}
