package ch.ige.rbbagisexport.server.dao;

import java.sql.Date;
import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.handlers.StringListHandler;
import ch.ige.dbutils.handlers.TemporalScalarHandler;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.rbbagisexport.server.vo.RBExportVO;

/**
 * Selects data for a RBExportVO from the data base.
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author M. Künzli
 * @version $Id: RBExportDAO.java,v 1.17 2008/04/24 14:16:31 kuenzli Exp $
 */
public class RBExportDAO extends AbstractBagisDAO
{
  //----------------------------------------------------------------------------
  // Private Members
  //----------------------------------------------------------------------------
  
  private static final long serialVersionUID = 1L;

  private static final Class<RBExportDAO.SchutitData> VO_SCHUTIT_DATA_TYPE = RBExportDAO.SchutitData.class;

  private static final SQLStatement<List<String>> SELECT_APPLICANT_NAME = SQL.select( " SELECT ra.reg_name AS applicant_name "
      + " FROM schutit_regadr sr, adr_rolle ar, schutit s, regadr ra " + " WHERE s.schutit_key = sr.schutit_key " + " AND s.schutit_typ = ar.schutit_typ "
      + " AND sr.adr_rolle_cd = ar.adr_rolle_cd " + " AND sr.regadr_nr = ra.regadr_nr " + " AND ar.adr_rolle_cd IN (1,2) " + " AND sr.bis_ereig_idat IS NULL "
      + " AND s.schutit_typ = 1 " + " AND sr.schutit_key = ? " );

  private static final SQLStatement<List<String>> SELECT_IPC_SYMBOLS = SQL.select( " SELECT ipc_symb " + " FROM pat_ipc pi " + " WHERE pat_ipc_cd = 1 "
      + " AND pi.schutit_key = ? " );

  private static final SQLStatement<String> SELECT_INVESTIGATOR = SQL.select( " SELECT m.ma_name ||' '|| m.ma_vname as investigator " + " FROM pat p, ma m  "
      + " WHERE p.recherche_ma = m.ma_nr" + " AND schutit_key = ? " );

  private static final SQLStatement<Date> SELECT_OLDEST_PRIORITY = SQL.select( " SELECT MIN(prio_dat) as min_prio_dat " + " FROM prioritaet  "
      + " WHERE schutit_key = ? " );

  private static final SQLStatement<RBExportDAO.SchutitData> SELECT_IP_DATA_BY_IP_KEY = SQL
  .select( "SELECT gesuch_nr as app_nr, schutz_beginn as application_date, beschreibung as title, sprache as language" + " FROM schutit " + " WHERE schutit_key = ? " );


  //----------------------------------------------------------------------------
  // Public Methods
  //----------------------------------------------------------------------------
  // *** Select RBExportVO
  //     -----------------
  public RBExportVO selectRBExportData( Integer schutitKey ) throws SQLException
  {
    RBExportVO rbExportData = new RBExportVO();
    // Oldest Priority
    rbExportData.setPriorityDate( this.selectOldestPriority( schutitKey ) );
    // Applicant Names
    rbExportData.setApplicantNames( this.selectApplicantNames( schutitKey ) );
    // IPC Classes
    rbExportData.setIpcClasses( this.selectIpcClasses( schutitKey ) );
    // ApplicationDate, Title, Language
    SchutitData schutitData = this.selectSchutitData( schutitKey );
    rbExportData.setApplicationDate( schutitData.applicationDate );
    rbExportData.setTitle( schutitData.title );
    rbExportData.setLanguage( schutitData.language );
    rbExportData.setAppNr( schutitData.appNr );
    // Investigator
    rbExportData.setInvestigator( this.selectInvestigator( schutitKey ) );
    return rbExportData;
  }

  //----------------------------------------------------------------------------
  // Private Methods
  //----------------------------------------------------------------------------
  // *** Select Application Nr, Application Date, Title, Language
  //     --------------------------------------------------------
  public static class SchutitData extends AbstractVO
  {
    private static final long serialVersionUID = 1L;

    public String appNr;

    public Date applicationDate;

    public String title;

    public Integer language; 

  }

  // *** Select Application Nr, Application Date, Title, Language
  //     --------------------------------------------------------
  private RBExportDAO.SchutitData selectSchutitData( Integer ipKey ) throws SQLException
  {
    RBExportDAO.SchutitData schutitData = selectVO( VO_SCHUTIT_DATA_TYPE, SELECT_IP_DATA_BY_IP_KEY, ipKey );
    return schutitData;
  }
  
  // *** Select oldest Priority
  //     ----------------------
  private Date selectOldestPriority( Integer schutitKey ) throws SQLException
  {
    return this.selectTemporal( getQueryRunner(), TemporalScalarHandler.DATE, SELECT_OLDEST_PRIORITY, schutitKey );
  }

  // *** Select Applicant Names
  //     ----------------------
  private List<String> selectApplicantNames( Integer schutitKey ) throws SQLException
  {
    return getQueryRunner().query( SELECT_APPLICANT_NAME, new StringListHandler(), schutitKey );
  }

  // *** Select IPC Classes
  //     ------------------
  private List<String> selectIpcClasses( Integer schutitKey ) throws SQLException
  {
    return getQueryRunner().query( SELECT_IPC_SYMBOLS, new StringListHandler(), schutitKey );
  }

  // *** Select Investigator
  //     -------------------
  private String selectInvestigator( Integer schutitKey ) throws SQLException
  {
    return selectString( getQueryRunner(), SELECT_INVESTIGATOR, schutitKey );
  }

}
