package ch.ige.rbbagisimport.client.batch;

import java.util.ArrayList;
import java.util.Map;

import org.apache.commons.cli.CommandLine;

import ch.ige.rbbagisimport.client.FileHelper;
import ch.ige.rbbagisimport.client.ImportProperties;
import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.common.vo.XxSysparamsVO;
import ch.ige.rbbagisimport.server.RBDelegate;
import ch.ige.rbbagisimport.BuildInfo;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReportPossibleMail;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;
import ch.ige.util.client.batch.BatchReportPossibleMail.MailSendingDecider;

/**
 * Main Class for RBBagisImport.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: RBBagisImport.java,v 1.15 2008/06/30 15:03:37 kuenzli Exp $
 */
public class RBBagisImport extends NewAbstractBatch
{
  /**
   * The name for this.
   */
  private static final String APPLICATION_NAME = "rbbagisimport";

  /**
   * Shown when any Exception occurs.
   */
  private static final BatchReturnCode UNEXPECTED_EXCEPTION = new BatchReturnCode( 1, "Eine unerwartete Exception ist aufgetreten" );

  /**
   * Shown when this can't initialize.
   */
  private static final BatchReturnCode INITIALIZATION_FAILED = new BatchReturnCode( 2, "Fehler bei der Initialisierung des Batch Job: {0}" );

  private FileHelper helper = new FileHelper();

  private ArrayList<SearchReportFileVO> srFiles;

  private ArrayList<SearchReportFileVO> srFilesProcessed;

  /**
   * The only constructor.
   */
  public RBBagisImport()
  {
    super( APPLICATION_NAME, null );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getPossibleReturnCodes()
  @Override
  protected BatchReturnCode[] getPossibleReturnCodes()
  {
    return BatchReturnCode.getBatchReturnCodes( RBBagisImport.class );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#checkCommandLine(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( "unused" )
  protected void checkCommandLine( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws IllegalArgumentException
  {
    // Nothing to check
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getBatchReports()
  @Override
  public BatchReport[] getBatchReports() throws BatchException
  {
    try
    {
      String smtpServer = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_SMTP_HOST_PARAM_ID );
      String from = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_MAIL_FROM_PARAM_ID );
      String to = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_MAIL_TO_PARAM_ID );
      String cc = ImportProperties.getInstance().getValue( XxSysparamsVO.RB_IMPORT_MAIL_CC_PARAM_ID );
      //TODO Use this subject also in case of error?
      String subject = BuildInfo.BUILD_ENV_NAME + " - " + APPLICATION_NAME + ": Rechercheberichte wurden verarbeitet!";

      BatchReportPossibleMail mailReport = new BatchReportPossibleMail( smtpServer, from, to, cc, subject, new MailSendingDecider()
      {
        // Send the mail only if there were results or if an error occured.
        public boolean sendMail()
        {
          boolean sendMail = false;
          if ( RBBagisImport.this.getCurrentReturnCode().equals( BatchReturnCode.OK ) )
          {
            int numberOfProcessedFiles = 0;
            if ( RBBagisImport.this.srFiles != null )
            {
              numberOfProcessedFiles = RBBagisImport.this.srFiles.size();
            }
            LOG.debug( "Processed " + numberOfProcessedFiles + " files" );
            sendMail = numberOfProcessedFiles > 0;
          }
          else
          {
            sendMail = true;
          }
          return sendMail;
        }
      } );
      return new BatchReport[] { mailReport };
    }
    catch ( Exception e )
    {
      throw new BatchException( INITIALIZATION_FAILED.formatMessage( e.getMessage() ) );
    }
  }

  private String getReportBody()
  {
    return new StatistikGenerator().generateStatics( this.srFilesProcessed );
  }

  private void init() throws Exception
  {
    LOG.debug( "init - begin" );
    LOG.debug( "Properties \n" + ImportProperties.getInstance().toString() );
    LOG.debug( "init - done" );
  }

  private void processReports() throws Exception
  {
    this.srFilesProcessed = new ArrayList<SearchReportFileVO>();
    RBDelegate delegate = RBDelegate.getInstance();

    for ( SearchReportFileVO sr: this.srFiles )
    {
      sr.setProcessState( SearchReportFileVO.PROCESS_STATE_IN_PROCESS );
      sr = delegate.insertIntoBAGIS( sr );
      this.srFilesProcessed.add( sr );
    }
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#run(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( "unused" )
  protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
  {
    try
    {
      init();

      this.srFiles = this.helper.readXMLFiles();

      processReports();

      // clean mal versuchen, ob es vorher geht...
      this.helper.cleanup( this.srFilesProcessed );

      writeBatchReportLine( getReportBody() );
    }
    catch ( Throwable t )
    {
      LOG.error( "main - Exception", t );
      throw new BatchException( UNEXPECTED_EXCEPTION, t );
    }
  }

  /**
   * Runs this batch.
   * @param args Ignored
   */
  public static void main( String[] args )
  {
    LOG.debug( "### RBBagisImport - main started ###" );
    NewAbstractBatch.start( new RBBagisImport(), args );
    LOG.debug( "### RBBagisImport - main successfully done ###" );
  }

  @Override
  protected String getBuildInfo()
  {
    return BuildInfo.BUILD_ID;
  }

  @Override
  protected String getCvsTag()
  {
    return BuildInfo.CVS_TAG;
  }
}