package ch.ige.rbbagisimport.server;

import java.rmi.RemoteException;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;

import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.common.vo.XxSysparamsVO;
import ch.ige.util.ServiceLocator;

/**
 * Delegate-Class for the RBWorkBean
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: RBDelegate.java,v 1.9 2008/04/23 13:28:09 wild Exp $
 */
public class RBDelegate
{
  //
  // fields
  //
  private static RBDelegate myself;

  private RBWork work;

  private RBDelegate() throws Exception
  {
//    localUse(); // Nur für Lokalen Gebrauch!
    getBean();
  }

  /**
   * Singelton 
   * 
   * @return RBDelegate
   * @throws Exception
   */
  public static RBDelegate getInstance() throws Exception
  {
    if ( myself == null )
    {
      myself = new RBDelegate();
    }
    return myself;
  }

  /**
   * Inserts the Xml in BAGIS
   * @param srFile SearchReport as XML 
   * @return Errorcode
   * @throws RemoteException
   */
  public SearchReportFileVO insertIntoBAGIS( SearchReportFileVO srFile ) throws RemoteException
  {
    return work.insertIntoBAGIS( srFile );
  }

  /**
   * Laden der Konfiguration und Rückgabe in einem einzigen VO
   * @return XxSysparamsVO gefüllt mit den nötigen Werten.
   * 
   * @throws RemoteException
   */
  public XxSysparamsVO getConfiguration() throws RemoteException
  {
    try
    {
      return work.getConfiguration();
    }
    catch ( RemoteException e )
    {
      throw e;
    }

  }

  @SuppressWarnings( "unused" )
  private void localUse() throws Exception
  {
    Hashtable<String, String> env = new Hashtable<String, String>();
    env.put( javax.naming.Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory" );
    env.put( javax.naming.Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces" );
    env.put( javax.naming.Context.PROVIDER_URL, "jnp://localhost:1099" );

    Context context = new InitialContext( env );
    RBWorkHome home = (RBWorkHome) context.lookup( RBWorkHome.JNDI_NAME );
    work = home.create();
  }

  private void getBean() throws Exception
  {
    RBWorkHome home = (RBWorkHome) ServiceLocator.getInstance().getHome( RBWorkHome.JNDI_NAME, RBWorkHome.class );
    work = home.create();
  }

} // end class