package ch.ige.rbbagisimport.server.test;

import java.sql.Date;
import java.util.ArrayList;

import junit.framework.TestCase;
import ch.ige.chsearchreport.ClassificationIpcr;
import ch.ige.chsearchreport.SearchReportData;
import ch.ige.rbbagisimport.server.XMLHelper;
import ch.ige.rbbagisimport.server.vo.IPCClassVO;

/**
 * Tests the XML-Helper Class
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: XMLTest.java,v 1.9 2008/05/06 13:54:36 wild Exp $
 */
public class XMLTest extends TestCase
{
  // minimum!
  private final static String xmlString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
      + "<search-report-data xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
      + "xsi:noNamespaceSchemaLocation=\"http://schema.ige.ch/ch-search-report-v1-2-ige0-4.xsd\" "
      + "id=\"srep\" lang=\"de\" status=\"draft\" srep-office=\"CH\" date-produced=\"2007-09-10\" " + "srep-type=\"ch-declaration-sr\">" + "<srep-for-pub>"
      + "<srep-fields-searched><minimum-documentation><classifications-ipcr>" + "<classification-ipcr sequence=\"1\"><text>C07K</text></classification-ipcr>"
      + "</classifications-ipcr></minimum-documentation></srep-fields-searched>" + "<srep-citations><text>blabla</text></srep-citations>" + "</srep-for-pub>"
      + "</search-report-data>";

  public XMLTest( String s )
  {
    super( s );
  }

  public void testXMLHelper() throws Exception
  {
    try
    {
      XMLHelper helper = new XMLHelper( xmlString );
      SearchReportData srdi = helper.getXMLData();
      String str = srdi.getStatus();
      helper.printXML( true, false );
      assertEquals( "draft", str );
    }
    catch ( Exception e )
    {
      System.out.println( e.getMessage() );
      throw e;
    }
  }

  public void testAddClasses() throws Exception
  {
    try
    {
      XMLHelper helper = new XMLHelper( xmlString );
      ArrayList<IPCClassVO> clazzes = new ArrayList<IPCClassVO>();
      clazzes
          .add( new IPCClassVO( "F41H001116", Date.valueOf( "2007-11-09" ), IPCClassVO.SYMBOL_POSITION_FIRST, IPCClassVO.CLASSIFICATION_VALUE_INVENTION, 2 ) );

      helper.addIPCClasses( clazzes );
      SearchReportData srdi = helper.getXMLData();
      helper.printXML( true, false );
      ClassificationIpcr ipc = (ClassificationIpcr) srdi.getSrepForPub().getClassificationsIpcr().getClassificationIpcr().get( 0 );
      assertEquals( "F", ipc.getSection() );
    }
    catch ( Exception e )
    {
      System.out.println( e.getMessage() );
      throw e;
    }

  }

  public void testHasMainClass() throws Exception
  {
    IPCClassVO class1 = new IPCClassVO( "F41H001116", Date.valueOf( "2007-11-09" ), IPCClassVO.SYMBOL_POSITION_FIRST,
        IPCClassVO.CLASSIFICATION_VALUE_INVENTION, 2 );
    IPCClassVO class2 = new IPCClassVO( "F41H001116", Date.valueOf( "2007-11-09" ), IPCClassVO.SYMBOL_POSITION_FIRST,
        IPCClassVO.CLASSIFICATION_VALUE_INVENTION, 2 );
    ArrayList<IPCClassVO> al = new ArrayList<IPCClassVO>();
    al.add( class1 );
    al.add( class2 );
    assertTrue( IPCClassVO.hasMainClass( al ) );
  }
}
