package ch.ige.rbbagisimport.server.vo;

import java.sql.Date;
import java.util.ArrayList;

/**
 * Represents an IPC-Class
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: IPCClassVO.java,v 1.2 2008/02/29 12:32:37 wild Exp $
 */
public class IPCClassVO
{
  /**
   * selbstredend
   */
  public static final String CLASSIFICATION_LEVEL_ADVANCED = "ipcr-advanced";

  /**
   * selbstredend
   */
  public static final String SYMBOL_POSITION_FIRST = "first-symbol-position";

  /**
   * selbstredend
   */
  public static final String SYMBOL_POSITION_LATER = "later-symbol-position";

  /**
   * selbstredend
   */
  public static final String CLASSIFICATION_VALUE_INVENTION = "invention";

  /**
   * selbstredend
   */
  public static final String CLASSIFICATION_VALUE_NON_INVENTION = "non-invention";

  /**
   * selbstredend
   */
  public static final int IPC_CD_MAINCLASS = 2;

  /**
   * selbstredend
   */
  public static final int IPC_CD_ADDITIONALCLASS = 3;

  /**
   * selbstredend
   */
  public static final int IPC_CD_FAKCLASS = 4;

  /**
   * selbstredend
   */
  public static final int IPC_CD_INDEXEDCLASS = 5;

  private String section;

  private String clazz; // can't use class :-(

  private String subClass;

  private String mainGroup;

  private String subGroup;

  private String symbolPosition;

  private Date versionDate;

  private String classificationLevel;

  private String classificationValue;

  private int ipcCd;

  /**
   * Constructor
   */
  public IPCClassVO()
  {
    this.classificationLevel = CLASSIFICATION_LEVEL_ADVANCED; // always!
  }

  /**
   * Constructor
   * @param ipcSymb 
   * @param versionDate 
   * @param symbolPosition 
   * @param classificationValue 
   * @param ipcCd 
   */
  public IPCClassVO( String ipcSymb, Date versionDate, String symbolPosition, String classificationValue, int ipcCd )
  {
    this();
    extractIPCData( ipcSymb );
    this.versionDate = versionDate;
    this.symbolPosition = symbolPosition;
    this.classificationValue = classificationValue;
    this.ipcCd = ipcCd;
  }

  /**
   * Gets the classificationValue.
   * @return the classificationValue
   */
  public String getClassificationValue()
  {
    return this.classificationValue;
  }

  /**
   * Sets the classificationValue.
   * @param classificationValue The classificationValue to set
   */
  public void setClassificationValue( String classificationValue )
  {
    this.classificationValue = classificationValue;
  }

  /**
   * Gets the ipcClass.
   * @return the ipcClass
   */
  public String getClazz()
  {
    return this.clazz;
  }

  /**
   * Sets the ipcClass.
   * @param clazz The ipcClass to set
   */
  public void setClazz( String clazz )
  {
    this.clazz = clazz;
  }

  /**
   * Gets the mainGroup.
   * @return the mainGroup
   */
  public String getMainGroup()
  {
    return this.mainGroup;
  }

  /**
   * Sets the mainGroup.
   * @param mainGroup The mainGroup to set
   */
  public void setMainGroup( String mainGroup )
  {
    this.mainGroup = mainGroup;
  }

  /**
   * Gets the section.
   * @return the section
   */
  public String getSection()
  {
    return this.section;
  }

  /**
   * Sets the section.
   * @param section The section to set
   */
  public void setSection( String section )
  {
    this.section = section;
  }

  /**
   * Gets the subClass.
   * @return the subClass
   */
  public String getSubClass()
  {
    return this.subClass;
  }

  /**
   * Sets the subClass.
   * @param subClass The subClass to set
   */
  public void setSubClass( String subClass )
  {
    this.subClass = subClass;
  }

  /**
   * Gets the subGroup.
   * @return the subGroup
   */
  public String getSubGroup()
  {
    return this.subGroup;
  }

  /**
   * Sets the subGroup.
   * @param subGroup The subGroup to set
   */
  public void setSubGroup( String subGroup )
  {
    this.subGroup = subGroup;
  }

  /**
   * Gets the symbolPosition.
   * @return the symbolPosition
   */
  public String getSymbolPosition()
  {
    return this.symbolPosition;
  }

  /**
   * Sets the symbolPosition.
   * @param symbolPosition The symbolPosition to set
   */
  public void setSymbolPosition( String symbolPosition )
  {
    this.symbolPosition = symbolPosition;
  }

  /**
   * Gets the versionDate.
   * @return the versionDate
   */
  public Date getVersionDate()
  {
    return this.versionDate;
  }

  /**
   * Sets the versionDate.
   * @param versionDate The versionDate to set
   */
  public void setVersionDate( Date versionDate )
  {
    this.versionDate = versionDate;
  }

  /**
   * Gets the classificationLevel.
   * @return the classificationLevel
   */
  public String getClassificationLevel()
  {
    return this.classificationLevel;
  }

  /**
   * Gets the ipcCd.
   * @return the ipcCd
   */
  public int getIpcCd()
  {
    return this.ipcCd;
  }

  /**
   * Sets the ipcCd.
   * @param ipcCd The ipcCd to set
   */
  public void setIpcCd( int ipcCd )
  {
    this.ipcCd = ipcCd;
  }

  /**
   * The Values as String 
   * @return String Field Values
   */
  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append( "section: " + this.section + "\n" );
    sb.append( "clazz: " + this.clazz + "\n" );
    sb.append( "subClass: " + this.subClass + "\n" );
    sb.append( "mainGroup: " + this.mainGroup + "\n" );
    sb.append( "subGroup: " + this.subGroup + "\n" );
    sb.append( "symbolPosition: " + this.symbolPosition + "\n" );
    sb.append( "versionDate: " + this.versionDate + "\n" );
    sb.append( "classificationLevel: " + this.classificationLevel + "\n" );
    sb.append( "classificationValue: " + this.classificationValue );
    return sb.toString();
  }

  /**
   * Does the list contain a Mainclass?
   * @param ipcClasses a list of IPCClasses
   * @return boolean true if a main class has been found
   */
  public static boolean hasMainClass( ArrayList<IPCClassVO> ipcClasses )
  {
    boolean hasMainClass = false;
    for ( IPCClassVO ipc: ipcClasses )
    {
      if ( ipc.getIpcCd() == IPCClassVO.IPC_CD_MAINCLASS )
      {
        hasMainClass = true;
      }
    }
    return hasMainClass;
  }

  private void extractIPCData( String ipcSymb )
  {
    this.section = ipcSymb.substring( 0, 1 );
    this.clazz = ipcSymb.substring( 1, 3 );
    this.subClass = ipcSymb.substring( 3, 4 );
    this.mainGroup = ipcSymb.substring( 4, 8 );
    this.subGroup = ipcSymb.substring( 8 );
  }

} // end class