package ch.ige.st36xmlgenerator.batch;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import ch.ige.st36xmlgenerator.XMLDocumentWithMetadata;

/**
 * Generates Statistics about the job
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: StatistikGenerator.java,v 1.9 2008/06/27 09:18:43 fabrizioli Exp $
 */
public class StatistikGenerator
{
  // constants
  private static final String mainTitle = "Folgende Daten wurden generiert:";

  private static final String titleOk = "ERFOLGREICH GENERIERT";

  private static final String titleNOk = "FEHLER BEIM GENERIEREN";

  private static final String titleDetailsOK = "DETAILS GENERIERTE ST.36 DATEN";

  private static final String titleDetailsNOK = "DETAILS FEHLERFAELLE";

  private static final String subTitleFinal = "ST.36 bibliographische Daten:";

  // fields

  private ArrayList<XMLDocumentWithMetadata> finalsOk = new ArrayList<XMLDocumentWithMetadata>();

  private ArrayList<XMLDocumentWithMetadata> finalsNok = new ArrayList<XMLDocumentWithMetadata>();

  public String generateStatics( Collection<List<XMLDocumentWithMetadata>> docListCollection, String dataSoreFolder )
  {
    for ( List<XMLDocumentWithMetadata> docList: docListCollection )
    {
      for ( XMLDocumentWithMetadata doc: docList )
      {
        evaluate( doc );
      }

    }
    return getBodyText( dataSoreFolder );
  }

  private void evaluate( XMLDocumentWithMetadata doc )
  {

    if ( isFailure( doc ) )
    {
      finalsNok.add( doc );
    }
    else
    {
      finalsOk.add( doc );
    }

  }

  private String getBodyText( String dataStoreFolder )
  {
    StringBuilder sb = new StringBuilder();
    sb.append( mainTitle + "\n\n" );
    sb.append( titleOk + "\n" );
    sb.append( subTitleFinal + "\t" );
    sb.append( finalsOk.size() + "\n\n\n" );
    sb.append( titleNOk + "\n" );
    sb.append( subTitleFinal + "\t" );
    sb.append( finalsNok.size() );

    if ( !finalsOk.isEmpty() )
    {
      sb.append( "\n\n\n" );
      sb.append( titleDetailsOK + "\n" );

      for ( XMLDocumentWithMetadata sr: finalsOk )
      {
        sb.append( "\n" );
        sb.append( dataStoreFolder + "  \\st36biblgenerator\\" + sr.getFilePath() );
      }
    }

    if ( !finalsNok.isEmpty() ) // There are Failures...
    {
      sb.append( "\n\n\n" );
      sb.append( titleDetailsNOK + "\n" );

      for ( XMLDocumentWithMetadata sr: finalsNok )
      {
        sb.append( "\n" );
        sb.append( sr.getFilePath() + "\t" + sr.getErrorMsg() );
      }
    }
    sb.append( "\n" );
    return sb.toString();
  }

  private boolean isFailure( XMLDocumentWithMetadata file )
  {
    boolean isFailure = true;
    if ( file.getErrorMsg().equals( "" ) )
    {
      isFailure = false;
    }
    return isFailure;
  }
} // end class
