package ch.ige.st36xmlgenerator.test;

import java.io.IOException;
import java.sql.Date;
import java.util.Set;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.iso_relax.verifier.VerifierConfigurationException;
import org.xml.sax.SAXException;

import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.st36xmlgenerator.builder.Factory.ST36Factory;
import ch.ige.st36xmlgenerator.exception.MissingMandatoryValueException;
import ch.ige.st36xmlgenerator.metadata.JAXBMetadataExtractorFactory;
import ch.ige.st36xmlgenerator.metadata.XMLAccessor;
import ch.ige.st36xmlgenerator.metadata.XMLGeneratorMetadataExtractor;
import ch.ige.util.constants.NumericLiterals;

/**
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author locher
 * @version $Id: JAXBMetadataExtractorFactoryTest.java,v 1.12 2008/09/25 07:29:27 fabrizioli Exp $
 */
public class JAXBMetadataExtractorFactoryTest extends AbstractXMLTestCase
{
  XMLGeneratorMetadataExtractor extractor;
  
  ST36Factory factory;
  
  private static final Date DATE_VALUE = BAGISTime.convertIDatToDate( 2008050500 );
 
  public JAXBMetadataExtractorFactoryTest( String name )
  {
    super( name, "C:\\jp\\projects\\st36xmlgenerator\\xml\\test" );
  }

  // @see junit.framework.TestCase#setUp()
  protected void setUp() throws Exception
  {
    JAXBWellKnownSchemas.initializeMetadataExtractorFactoryFromWellKnownFiles();
    extractor = JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-2-ige1-1.xsd" );
    factory = new ST36Factory();
  }

  protected void tearDown() throws Exception
  {

  }

  public void testMethods()
  {
    assertTrue( !JAXBMetadataExtractorFactory.getInstance().getAvailableSchemas().isEmpty() );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "gaga" ) );
  }

  public void testAvailableSchemas()
  {
    Set<String> schemas = JAXBMetadataExtractorFactory.getInstance().getAvailableSchemas();
    assertEquals( 1, schemas.size() );
    assertTrue( schemas.contains( "ch-patent-doc-v1-2-ige1-1.xsd" ) );
  }

  public void testSchemaIds()
  {
    assertEquals( NumericLiterals.SEVEN, JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-2-ige1-1.xsd" ).getSchemaIdentifier() );

  }

  public void testExtractorLookup() throws Exception
  {
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-0-ige2-0.xsd" ) );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-0-ige2-1.xsd" ) );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-0-ige2-2.xsd" ) );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-0-ige2-3.xsd" ) );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-1-ige1-0.xsd" ) );
    assertNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-2-ige1-0.xsd" ) );
    assertNotNull( JAXBMetadataExtractorFactory.getInstance().getExtractor( "ch-patent-doc-v1-2-ige1-1.xsd" ) );
  }
  
  public void testAccessors() throws Exception
  {
    XMLAccessor acc88 =  extractor.getXMLAccessors( 88, "A3" );
    assertNotNull(acc88);
    assertTrue(" Creation of path was not successful for " + acc88, acc88.createPathAndSetValue( factory, new Object[] { DATE_VALUE, DATE_VALUE, "" } ));

    XMLAccessor acc30 =  extractor.getXMLAccessors( 30, "B1" );
    assertNotNull(acc30);
    assertTrue(" Creation of path was not successful for " + acc30, acc30.createPathAndSetValue( factory, new Object[] { "CH", "1111", DATE_VALUE } ));
    assertTrue(" Creation of path was not successful for " + acc30, acc30.createPathAndSetValue( factory, new Object[] { "A", "2222", DATE_VALUE } ));

    XMLAccessor acc11 =  extractor.getXMLAccessors( 11, "B1" );
    assertNotNull(acc11);
    assertTrue(" Creation of path was not successful for " + acc11, acc11.createPathAndSetValue( factory, new Object[] { "1111", "CH", "B1" } ));

    XMLAccessor acc21 =  extractor.getXMLAccessors( 21, "B1" );
    assertNotNull(acc21);
    assertTrue(" Creation of path was not successful for " + acc21, acc21.createPathAndSetValue( factory, new Object[] { "1111", "CH" } ));

    XMLAccessor acc22 =  extractor.getXMLAccessors( 22, "B1" );
    assertNotNull(acc22);
    assertTrue(" Creation of path was not successful for " + acc22, acc22.createPathAndSetValue( factory, DATE_VALUE ));

    XMLAccessor acc24 =  extractor.getXMLAccessors( 24, "B1" );
    assertNotNull(acc24);
    assertTrue(" Creation of path was not successful for " + acc24, acc24.createPathAndSetValue( factory, DATE_VALUE ));

    XMLAccessor acc43 =  extractor.getXMLAccessors( 43, "A3" );
    assertNotNull(acc43);
    assertTrue(" Creation of path was not successful for " + acc43, acc43.createPathAndSetValue( factory, new Object[] { "CH", "", DATE_VALUE } ));

    XMLAccessor acc45 =  extractor.getXMLAccessors( 45, "B1" );
    assertNotNull(acc45);
    assertTrue(" Creation of path was not successful for " + acc45, acc45.createPathAndSetValue( factory, new Object[] { DATE_VALUE } )); // special type

    XMLAccessor acc51 =  extractor.getXMLAccessors( 51, "B1" );
    assertNotNull(acc51);
    assertTrue(" Creation of path was not successful for " + acc51, acc51.createPathAndSetValue( factory, new Object[] { "A", "01", "C", 1, "022", DATE_VALUE, "ipcr-advanced", "later-symbol-position",
        "invention" } ));

    XMLAccessor acc54 =  extractor.getXMLAccessors( 54, "B1" );
    assertNotNull(acc54);
    assertTrue(" Creation of path was not successful for " + acc54, acc54.createPathAndSetValue( factory, "test" ));

    XMLAccessor acc60 =  extractor.getXMLAccessors( 60, "B1" );
    assertNotNull(acc60);
    assertTrue(" Creation of path was not successful for " + acc60, acc60.createPathAndSetValue( factory, new Object[] { "CH", "test" } ));

    XMLAccessor acc62 =  extractor.getXMLAccessors( 62, "B1" );
    assertNotNull(acc62);
    assertTrue(" Creation of path was not successful for " + acc62, acc62.createPathAndSetValue( factory, new Object[] { "CH", "test" } ));

    XMLAccessor acc71 =  extractor.getXMLAccessors( 71, "A1" );
    assertNotNull(acc71);
    assertTrue(" Creation of path was not successful for " + acc71, acc71.createPathAndSetValue( factory, new Object[] { "A", "B", "C", "D", "E", "F", "G", "H", "I", "J" } ));

    XMLAccessor acc72 =  extractor.getXMLAccessors( 72, "B1" );
    assertNotNull(acc72);
    assertTrue(" Creation of path was not successful for " + acc72, acc72.createPathAndSetValue( factory, new Object[] { "A", "B", "C", "D", "E", "F", "G", "H" }) );
    
     XMLAccessor acc72W =  extractor.getXMLAccessors( 172, "B1" );
    assertNotNull(acc72W);
    assertTrue(" Creation of path was not successful for " + acc72W, acc72W.createPathAndSetValue( factory, new Object[] { "Der Erfinder hat auf Nennung verzichtet." } ));

    XMLAccessor acc74 =  extractor.getXMLAccessors( 74, "B1" );
    assertNotNull(acc74);
    assertTrue(" Creation of path was not successful for " + acc74, acc74.createPathAndSetValue( factory, new Object[] { "A", "B", "C", "D", "E", "F", "G", "H" } ));

    XMLAccessor acc86 =  extractor.getXMLAccessors( 86, "A3" );
    assertNotNull(acc86);
    assertTrue(" Creation of path was not successful for " + acc86, acc86.createPathAndSetValue( factory, new Object[] { "CH", "1111" } ));

    XMLAccessor acc87 =  extractor.getXMLAccessors( 87, "A3" );
    assertNotNull(acc87);
    assertTrue(" Creation of path was not successful for " + acc87, acc87.createPathAndSetValue( factory, new Object[] { "CH", "1111", DATE_VALUE } ) );

    factory.setRootAttributes( "de", DATE_VALUE, DATE_VALUE, 1111 );

    //System.out.println( factory.getMarshalledDocument().getContent() );
    this.compareWithStored( factory.getMarshalledDocument().getDOM(), "test" );
  }
  
  public void testAccessorErroneousKindcode() throws MissingMandatoryValueException, ClassNotFoundException
  {
    XMLAccessor acc88 = extractor.getXMLAccessors( 88, "A3" );
    assertTrue(" Creation of path was not successful for " + acc88, acc88.createPathAndSetValue( factory, new Object[] { DATE_VALUE, DATE_VALUE, "" }) );

    try
    {
      XMLAccessor acc88b = extractor.getXMLAccessors( 88, "B1" );
      assertNotNull(acc88b);
      acc88b.createPathAndSetValue( factory, new Object[] { DATE_VALUE, DATE_VALUE, "" } );

      fail();
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expected type " + e, e instanceof IllegalArgumentException );
    }
  }
  
  public void testAccessorMissingKindcode()
  {
     XMLAccessor acc88b = extractor.getXMLAccessors( 88, null );
     assertNull("Accessor should not be created for kindcode null " ,acc88b);
  }
  
  public void testAccessorMissingValue() throws Exception
  {
    try 
    {
      XMLAccessor acc54 =  extractor.getXMLAccessors( 54, "B1" );
      assertNotNull(acc54);
      acc54.createPathAndSetValue( factory, new Object[]{});
      fail();
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expected type " + e, e instanceof IllegalArgumentException );
    }
    
    try 
    {
      XMLAccessor acc54 =  extractor.getXMLAccessors( 54, "B1" );
      assertNotNull(acc54);
      acc54.createPathAndSetValue( factory, new Object[]{null});
      fail();
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expected type " + e, e instanceof MissingMandatoryValueException );
    }
  }
  
  public void testAccessorMissingValues() throws Exception
  {

    try 
    {
      XMLAccessor acc51 =  extractor.getXMLAccessors( 51, "B1" );
      assertNotNull(acc51);
      acc51.createPathAndSetValue( factory, new Object[] { null, "01", "C", 1, "022", DATE_VALUE, "ipcr-advanced", "later-symbol-position",
          "invention" } );
      fail();
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expected type " + e, e instanceof MissingMandatoryValueException );
    }
    
    try
    {
      XMLAccessor acc51 =  extractor.getXMLAccessors( 51, "B1" );
      assertNotNull(acc51);
      assertTrue(" Creation of path was not successful for " + acc51, acc51.createPathAndSetValue( factory, new Object[] { "A", "01", "C", null, "022", DATE_VALUE, "ipcr-advanced", "later-symbol-position",
          "invention" } ));
      fail();
    }
    catch ( Exception e )
    {
      assertTrue( "Exception was not of expected type " + e, e instanceof MissingMandatoryValueException );
    }

    // not mandatory
      XMLAccessor acc74 =  extractor.getXMLAccessors( 74, "B1" );
      assertNotNull("Accessor must not be null ", acc74);
      assertTrue(" Creation of path was not successful for " + acc74, acc74.createPathAndSetValue( factory, new Object[] { "A", "B", "C", null, "E", null, "G", "H" } ));
   }



  public void testVerifierSchemas() throws VerifierConfigurationException, SAXException, IOException, TransformerException, ParserConfigurationException   
  {
    assertNotNull( "Verifier schema was not available", extractor.getVerifierSchema() );
  }

}
