package ch.ige.ses.client.batch;

import java.io.*;
import org.apache.log4j.*;
import ch.ige.util.client.*;
import ch.ige.ses.BuildInfo;
import ch.ige.ses.util.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.server.batchctrl.*;

/**
 * SES Batch Programm zur Starten der Controlling-Verarbeitung.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Patrick Bolzli
 * @version $Id: SesControllingStart.java,v 1.17 2006/12/11 12:43:34 iten Exp $
 */

class SesControllingStart extends BatchBasis
{
  //
  // const
  //
  private static final Logger LOG         = Logger.getLogger("ch.ige.ses.client.batch");
  private static final String JOB_NAME    =   "SesControllingStart";       // job name

  private static final int    JOB_CTRL_INTERVAL   = 300;              // 5 min =   300 sec
  private static final int    JOB_TIMEOUT         = 3600;             // 1 h   =  3600 sec

  private static final String ERR_NUM_EXCEPTION   = "-1";
  private static final String ERR_MSG_EXCEPTION   = "Eine unerwartete Exception ist aufgetreten: ";

  private static final String ERR_NUM_WRONG_ARG  = "-2";
  private static final String ERR_MSG_WRONG_ARG  = "Aufruf Programm mit ungültigen Parametern";

  private static final String ERR_NUM_MISSING_ID  = "-3";
  private static final String ERR_MSG_MISSING_ID  = "Fehler beim ermitteln der neuen KontrollId";

  private static final String ERR_NUM_STATE       = "-4";
  private static final String ERR_MSG_STATE       = "Fehler beim ermitteln des Status";

  private static final String ERR_NUM_JOB_STOPPED = "-5";
  private static final String ERR_MSG_JOB_STOPPED = "Der Controlling Job wurde unterbrochen";

  private static final String ERR_NUM_TIMEOUT     = "-6";
  private static final String ERR_MSG_TIMEOUT     = "Der JGB Job hat die Timeout Zeit überschritten";

  private static final String ERR_NUM_DISABLED   = "-7";
  private static final String ERR_MSG_DISABLED   = "Der Controlling Job ist nicht Enabled. " +
                                                   "Bitte auf Tabelle SES.Property den Eintrag " +
                                                   "<batch.job." + JOB_NAME + ".enabled> überprüfen.";

  private static final String ERR_NUM_ERRORS    = "-8";
  private static final String ERR_MSG_ERRORS    = "Der Controlling Job wurde abgeschlossen, weist jedoch Fehler auf!" +
                                                  " Bitte kontrollieren Sie die Error-Tabelle";

  private static final int PRINT_USAGE     = 10;     // Modus zeigt Usage - Meldung an

  //
  // fields
  //
  public int  timeRunning    = 0;        // time the job already is running in seconds

  private     int             modus;
  private     java.sql.Date   bisDatum;

  private     int             kontrollId;

  //
  // constructors
  //
  public SesControllingStart()
  {
    super(JOB_NAME);
  }

  //
  // public methods
  //
  public void startProd() throws Exception
  {
    LOG.debug("startProd - modus=" + modus + ", bisDatum=" +bisDatum);
    BatchDelegate.getInstance().controllingStart(modus, bisDatum);
  }

  public void startRestart() throws Exception
  {
    LOG.debug("startRestart - modus=" + modus + ", bisDatum=" +bisDatum);
    BatchDelegate.getInstance().controllingStart(modus, bisDatum);
  }

  public void startProcessStako() throws Exception
  {
    LOG.debug("startProcessStako - modus=" + modus + ", bisDatum=" +bisDatum);
    BatchDelegate.getInstance().controllingStart(modus, bisDatum);
  }

  public void startUpdate() throws Exception
  {
    LOG.debug("startUpdate - modus=" + modus + ", bisDatum=" +bisDatum);
    BatchDelegate.getInstance().controllingStart(modus, bisDatum);
  }

  /**
   * Ausgabe Programmhilfe
   * Zeigt mit welchen Parametern das Programm gestartet werden kann.
   */
  private void printUsage()
  {
    System.out.println("Usage: " + BuildInfo.CVS_TAG);
    System.out.println("SesControllingStart modus");
    System.out.println("            [-h] [-?] [help] [?]");
    System.out.println();
    System.out.println("  modus             Program Modus [" +
                                            "START | " +
                                            "NOUPDATE | " +
                                            "RESTART | " +
                                            "UPDATE");
    System.out.println("  bisDatum          per Default wird ein Monat genommen");
    System.out.println("  -h, -?, help, ?   Anzeige Hilfe (usage)");
  }

  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute
   * @param args              programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams (String[] args) throws BatchException
  {
    if (args.length > 0)
    {
      if (args[0].equalsIgnoreCase("START"))
      {
        modus = SAPStakoJobVO.STAKO_MODUS_START;
      }
      else if (args[0].equalsIgnoreCase("NOUPDATE"))
      {
        modus = SAPStakoJobVO.STAKO_MODUS_NOUPDATE;
      }
      else if (args[0].equalsIgnoreCase("RESTART"))
      {
        modus = SAPStakoJobVO.STAKO_MODUS_RESTART;
      }
      else if (args[0].equalsIgnoreCase("UPDATE"))
      {
        modus = SAPStakoJobVO.STAKO_MODUS_UPDATE;
      }
      else if (args[0].equals("-h") || args[0].equals("-?") ||
               args[0].equals("help") || args[0].equals("?"))
      {
        modus = PRINT_USAGE;
      }
      else
      {
        modus = SAPStakoJobVO.STAKO_MODUS_START;     //dies wird als default genommen
      }
    }
    if (args.length == 2)
    {
      bisDatum = java.sql.Date.valueOf(args[1]);
    }

    if (args.length > 2 || args.length < 1)
    {
      setNotifyConsole(true);
      throw new BatchException (ERR_NUM_WRONG_ARG, ERR_MSG_WRONG_ARG);
    }
  } // end method

  /**
   * sleep for the given JOB_CTRL_INTERVAL and increment timeRunning variable
   */
  public void waitInterval()
  {
    try
    {
      LOG.debug("waitInterval - sleep for " + JOB_CTRL_INTERVAL + " seconds...");
      Thread.sleep(JOB_CTRL_INTERVAL * 1000);
      timeRunning = timeRunning + JOB_CTRL_INTERVAL;
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
  } // end method

  public void getNewKontrollId() throws BatchException
  {
    try
    {
      LOG.debug("getNewKontrollId");
      kontrollId = BatchDelegate.getInstance().controllingGetNewId();
      LOG.debug("getNewKontrollId - received kontrollId=" + kontrollId);
    }
    catch (Exception ex)
    {
      throw new BatchException ( ERR_NUM_MISSING_ID,
                                 ERR_MSG_MISSING_ID + "\n" + ex.getMessage() );
    }
  }


  /**
   * control job state
   * @return true if job is finished successfully, flas if job is still running
   * @throws BatchException in case job is stopped or an error is occured
   */
  public boolean ctrlState() throws BatchException
  {
    int       state     = 0;
    boolean   finished  = false;

    try
    {
      LOG.debug("ctrlState - getting actual state..." );
      state = BatchDelegate.getInstance().controllingCtrlJobState(kontrollId);
      LOG.debug("ctrlState - state=" + state);
    }
    catch (Exception ex)
    {
      throw new BatchException (ERR_NUM_STATE,
                                ERR_MSG_STATE + "\n" + ex.getMessage());
    }

    // control state
    if (state == StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED)
    {
      LOG.debug("ctrlState - STATUS_VERARBEITUNG_FINISHED received");
      finished = true;
    }
    else if (state == StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED_WITH_ERRORS)
    {
      LOG.debug("ctrlState - STATUS_VERARBEITUNG_FINISHED_WITH_ERRORS reseived");
      throw new BatchException(ERR_NUM_ERRORS, ERR_MSG_ERRORS);
    }
    else if (state == StakoKontrolleVO.STATUS_VERARBEITUNG_CANCELLED)
    {
      LOG.debug("ctrlState - STATUS_VERARBEITUNG_CANCELLED received");
      throw new BatchException(ERR_NUM_JOB_STOPPED, ERR_MSG_JOB_STOPPED);
    }
    else if (timeRunning > JOB_TIMEOUT)
    {
      LOG.debug("ctrlState - JOB_TIMEOUT occured");
      throw new BatchException( ERR_NUM_TIMEOUT,
                                ERR_MSG_TIMEOUT + " (Timeout=" + JOB_TIMEOUT + "sec)");
    }

    // all done
    return finished;
  }

//***********************************************************************
//  main
//***********************************************************************
  public static void main (String[] args)
  {
    SesControllingStart job = new SesControllingStart();

    try
    {
      //
      // init
      //
      ClientLogInitalizer.initalizeLogging(ClientLogInitalizer.BATCH_CONFIG);
      job.loadParams(args);

      if (job.modus == SAPStakoJobVO.STAKO_MODUS_START)
      {
        // START
        if (! job.enabled())
        {
          throw new BatchException(ERR_NUM_DISABLED, ERR_MSG_DISABLED);
        }

        job.startProd();              // start job
        job.waitInterval();           // check state after wait
        job.getNewKontrollId();       // neueste KontrollId holen

        while (! job.ctrlState() )    // check state
        {
          job.waitInterval();
        }
        job.notifyMessage();          // notify that job successfully finished
      }
      // RESTART
      else if (job.modus == SAPStakoJobVO.STAKO_MODUS_RESTART)
      {
        if (!job.enabled())
        {
          throw new BatchException(ERR_NUM_DISABLED, ERR_MSG_DISABLED);
        }
        job.startRestart();           // start job
        job.waitInterval();           // check state after wait
        job.getNewKontrollId();       // neueste KontrollId holen

        while (! job.ctrlState() )       // check state
        {
          job.waitInterval();
        }
        job.notifyMessage();          // notify that job successfully finished

      }
      // PROCESS_STAKO
      else if (job.modus == SAPStakoJobVO.STAKO_MODUS_NOUPDATE)
      {
        if (!job.enabled())
        {
          throw new BatchException(ERR_NUM_DISABLED, ERR_MSG_DISABLED);
        }

        job.startProcessStako();
        job.waitInterval();           // check state after wait
        job.getNewKontrollId();       // neueste KontrollId holen

        while (! job.ctrlState() )       // check state
        {
          job.waitInterval();
        }
        job.notifyMessage();          // notify that job successfully finished
      }

      // UPDATE
      else if (job.modus == SAPStakoJobVO.STAKO_MODUS_UPDATE)
      {
        job.startUpdate();
      }
      // USAGE
      else if (job.modus == PRINT_USAGE)
      {
        job.printUsage();
      }

      //
      // End
      //
      System.exit(0);
    }


    catch (BatchException bex)
    {
      LOG.debug("main - BatchException occured: " + bex.getErrorMsg());
      job.notifyMessage(MSG_TYPE_NOK, bex.getErrorNr(), bex.getErrorMsg());
      System.exit( Integer.parseInt(bex.getErrorNr()) );
    }
    catch (Exception ex)
    {
      LOG.error("main - exception occured: " +ex.getMessage());
      job.notifyMessage(MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION + ex.toString());
      System.exit( Integer.parseInt(ERR_NUM_EXCEPTION) );
    }
  } // end method



} // end class



