package ch.ige.ses.client.batch;

import java.io.*;
import java.util.*;
import org.apache.log4j.*;

import ch.ige.util.client.*;
import ch.ige.ses.BuildInfo;
import ch.ige.ses.util.*;
import ch.ige.ses.server.batchctrl.*;
import ch.ige.ses.server.bagisoutbound.SESDelegate;


/**
 * SES Batch Programm zum Überwachen der Fristen von BAGIS-Fakturen und
 * allfälliges Auslösung von Annullationen von Fakturaanforderungen in SAP
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter, Thomas Iten
 * @version $Id: SesFristAblauf.java,v 1.16 2007/02/07 13:39:23 iten Exp $
 */
public class SesFristAblauf extends BatchBasis
{
  //
  // const
  //
  private static final Logger LOG      = Logger.getLogger("ch.ige.ses.client.batch");

  private static final String JOB_NAME            = "SesFristAblauf";    // job name

  private static final String OK_NUM_SUCCESS      = "0";
  private static final String OK_MSG_SUCCESS      = "Batch Job erfolgreich beendet: ";

  private static final String ERR_NUM_EXCEPTION   = "-1";
  private static final String ERR_MSG_EXCEPTION   = "Eine unerwartete Exception ist aufgetreten: ";

  private static final String ERR_NUM_WRONG_ARG   = "-2";
  private static final String ERR_MSG_WRONG_ARG   = "Aufruf Program mit ungültigen Parametern";

  private static final String ERR_NUM_ANNULATE    = "-3";
  private static final String ERR_MSG_ANNULATE    = "Es konnten nicht alle bfakt_id's annuliert werden:";

  //
  // fields
  //
  public String  modus          = null;     // Mögliche Werte
                                            // - SESConstants.MODUS_PROD (program param)
                                            // - SESConstants.MODUS_TEST (program Param)
                                            // - "SHOW"                  (program param)
                                            // - "USAGE"                 (program intern)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SesFristAblauf()
  {
    super(JOB_NAME);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Ausgabe Programmhilfe
   * Zeigt mit welchen Parametern das Program gestartet werden kann.
   */
  private void printUsage()
  {
    System.out.println("Usage: " + BuildInfo.CVS_TAG);
    System.out.println("SesJgbStart modus");
    System.out.println("            [-h] [-?] [help] [?]");
    System.out.println();
    System.out.println("  modus             Program Modus [" + SESConstants.MODUS_PROD + "," +
                                                               SESConstants.MODUS_TEST + "," +
                                                               "SHOW]" );
    System.out.println("  -h, -?, help, ?   Anzeige Hilfe (usage)");
  }


  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args             programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams(String[] args) throws BatchException
  {
    if (args.length ==1)
    {
      if ( args[0].equals(SESConstants.MODUS_PROD) )
      {
        modus = SESConstants.MODUS_PROD;
      }
      else if ( args[0].equals(SESConstants.MODUS_TEST) )
      {
        modus = SESConstants.MODUS_TEST;
      }
      else if ( args[0].equals("SHOW") )
      {
        modus = "SHOW";
      }
      else if ( args[0].equals("-h")   || args[0].equals("-?") ||
                args[0].equals("help") || args[0].equals("?") )
      {
        modus = "USAGE";
      }
      else 
      {
        setNotifyConsole(true);
        throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
      }
    }
    else
    {
      setNotifyConsole(true);
      throw new BatchException (ERR_NUM_WRONG_ARG,ERR_MSG_WRONG_ARG);
    }
  
  }


  /**
   * Start Test Modus
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startTest() throws Exception
  {
    int     testBFaktId         = 0;
    List    bfaktIdList         = null;
    List    bfaktIdListReduced  = null;
    String  resultMsg           = null;


    //
    // init
    //
    System.out.println("");
    System.out.println(JOB_NAME + " - Testmodus" );

    System.out.println("");
    System.out.println("Eingabe gewünschte testBFaktId:");
    System.out.println("Beachte: Falls testBFaktId=0, so werden die 10 ersten");
    System.out.println("         Selektionen der Produktiven Abfrage ausgeführt.");

    //
    // get bfakt id's
    //
    testBFaktId = readInt("  testBFaktId = ");

    if(testBFaktId > 0)
    {
      bfaktIdList = new java.util.ArrayList();
      bfaktIdList.add(new Integer(testBFaktId));
    }
    else
    {
      // select bfakt id's
      System.out.println("Select List of bfakt id's ...");
      bfaktIdList = BatchDelegate.getInstance().fristSelectBfaktIdList();

      System.out.println("Select List of bfakt id's - " + bfaktIdList.size() + " entries found");

      // reduce list to max. 10 entires
      bfaktIdListReduced = new ArrayList();
      for ( int i = 0; i < bfaktIdList.size() && i < 10; i++)
      {
        bfaktIdListReduced.add( bfaktIdList.get(i) );
      }
      bfaktIdList = bfaktIdListReduced;
    }

    //
    // annualte
    //
    try
    {
      System.out.println("Annulate List of bfakt id's ...");
      resultMsg = annulateBfaktIdList(bfaktIdList);

      System.out.println(JOB_NAME + " wurde erfolgreich beendet");
      System.out.println(resultMsg);
    }
    catch (Exception ex)
    {
      System.out.println(JOB_NAME + " wurde mit Fehler beendet");
      System.out.println(ex.getMessage());
    }
  }


  /**
   * Start Test Show
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startShow() throws Exception
  {
    List      bfaktIdList = null;
    Integer   bfaktId     = null;


    System.out.println("");
    System.out.println(JOB_NAME + " - Show Modus" );

    System.out.println("Select List of bfakt id's ...");
    bfaktIdList = BatchDelegate.getInstance().fristSelectBfaktIdList();

    System.out.println("List of bfakt id's - the following entries where found");
    for ( int i = 0; i < bfaktIdList.size(); i++)
    {
      bfaktId = (Integer) bfaktIdList.get(i);
      System.out.println(" - " + bfaktId);
    }
    System.out.println("Total " + bfaktIdList.size() + " records found");
  }


  /**
   * Start Produktiv Modus.
   * Dabei wird die JgbLaufId des letzten Jobs ermittelt. Diese wird beim ersten
   * Control Aufruf verwendet um die aktuelle Job Id zu ermitteln.
   *
   * @throws Exception  unerwarteter Fehler
   */
  public void startProd() throws Exception
  {
    List    bfaktIdList = null;
    String  resultMsg   = null;


    // select list
    LOG.debug("startProd - BatchDelegate.getInstance().fristSelectBfaktIdList()");

    bfaktIdList = BatchDelegate.getInstance().fristSelectBfaktIdList();

    LOG.debug("startProd - " + bfaktIdList.size() + " entries found");


    // annualte
    try
    {
      resultMsg = annulateBfaktIdList(bfaktIdList);

      setMessage ( MSG_TYPE_OK, OK_NUM_SUCCESS, OK_MSG_SUCCESS + "\n" + resultMsg );

      LOG.debug("startProd - successfully done");
    }
    catch (Exception ex)
    {
      LOG.error("startProd - terminated with error's");
      throw new BatchException ( ERR_NUM_ANNULATE,
                                 ERR_MSG_ANNULATE + "\n" + ex.getMessage() );
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Annulation Liste mit bfakt_id's
   * Wichtig: Aufruf muss vom Client sein, wegen Transaction Handling im
   *          Zusammenhang mit dem Failure Recovery der Bagis DB Aufrufe.
   *          Nach DB Migration auf Oracle wurde das Failure Recovery ent-
   *          fernt. Diese
   *
   * @param bfaltIdList   liste mit bfakt_id's (Integer)
   * @return              log message falls alle aufrufe ok
   * @throws Exception    Exception mit Fehlerliste falls nicht alle
   *                      Aufrufe erfolgreich
   */
  private String annulateBfaktIdList(List bfaltIdList) throws Exception
  {
    Integer       bfaktId     = null;
    int           cntErr      = 0;
    int           cntOk       = 0;
    StringBuffer  buf         = new StringBuffer();


    buf.append("fristAnnulateBfaktIdList - Log: \n");

    for ( int i = 0; i < bfaltIdList.size(); i++)
    {
      bfaktId = (Integer) bfaltIdList.get(i);

      LOG.debug("fristAnnulateBfaktIdList - start annulate bfakt_id " + bfaktId);

      try
      {
        SESDelegate.getInstance().AnnullationRechnung ( bfaktId.intValue(),
                                                        SESConstants.HERKUNFT_BATCH );

        cntOk++;
        buf.append(" - annulate bfakt_id " + bfaktId + " ok \n");
        LOG.debug("fristAnnulateBfaktIdList - bfakt_id " + bfaktId + " ok");
      }
      catch (Exception ex)
      {
        cntErr++;
        buf.append(" - annulate bfakt_id " + bfaktId + " NOK \n");
        LOG.debug("fristAnnulateBfaktIdList - bfakt_id " + bfaktId + " NOK");
      }

    } // end for

    // statistics
    buf.append("fristAnnulateBfaktIdList - Statistics: \n");
    buf.append("  Total Annulation OK  : " + cntOk  + "\n");
    buf.append("  Total Annulation NOK : " + cntErr + "\n");

    // throw exception in case of errors
    if (cntErr > 0)
    {
      throw new Exception (buf.toString());
    }

    // return log & statistics
    return buf.toString();
  }


  /**
   * Ausgabe Textmeldung und einlesen Eingabe als int
   *
   * @param prompt        Textmeldung
   * @return              int Wert
   * @throws Exception    Unerwarteter Fehler
   */
  private int readInt(String prompt) throws Exception
  {
    BufferedReader  stdin = new BufferedReader(new InputStreamReader(System.in));

    System.out.print (prompt);
    return( Integer.parseInt( stdin.readLine() ) );
  }


  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    SesFristAblauf job = new SesFristAblauf();

    try
    {
      //
      // init
      //
      ClientLogInitalizer.initalizeLogging(ClientLogInitalizer.BATCH_CONFIG);
      job.loadParams(args);


      //
      // run job produktion
      //
      if ( job.modus.equals(SESConstants.MODUS_PROD) )
      {
        job.startProd();                      // start job
        job.notifyMessage();                  // notify that job successfully finished
      }

      //
      // run job test
      //
      else if ( job.modus.equals(SESConstants.MODUS_TEST) )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only
        job.startTest();                      // start test
      }

      //
      // run job test
      //
      else if ( job.modus.equals("SHOW") )
      {
        job.setNotifyAll(false,false,true);   // set notif options to console only
        job.startShow();                      // start test
      }

      //
      // Usage
      //
      else if ( job.modus.equals("USAGE") )
      {
        job.printUsage();
      }


      //
      // End
      //
      System.exit(0);

    }

    catch (BatchException bex)
    {
      LOG.debug("main - BatchException occured: " + bex.getErrorMsg());
      job.notifyMessage ( MSG_TYPE_NOK, bex.getErrorNr(), bex.getErrorMsg() );
      System.exit( Integer.parseInt(bex.getErrorNr()) );
    }
    catch (Exception ex)
    {
      LOG.error("main - exception occured: " + ex.getMessage());
      job.notifyMessage ( MSG_TYPE_NOK, ERR_NUM_EXCEPTION, ERR_MSG_EXCEPTION + ex.toString() );
      System.exit( Integer.parseInt(ERR_NUM_EXCEPTION) );
    }

  } // end main


} // end class

