package ch.ige.ses.client.sapinbound;

import java.io.*;
import java.util.*;

import ch.ige.util.vo.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.client.saputil.StopWatch;
import com.sap.mw.jco.JCO;


/**
 * SAP Remote Function Call's for manual Mode.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: ManualRFC.java,v 1.32 2006/12/11 12:43:32 iten Exp $
 */
public class ManualRFC extends RFC
{

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public ManualRFC()
  {
    super ();
  }

  //---------------------------------------------------------------------------
  // test methods
  //---------------------------------------------------------------------------

  public void testCreateBillingReq(SAPRechnungVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_ZVL_CREATE_BILLING_REQ:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);

      rfcOpen();
      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Rechnung:");
      System.out.println( rfm.toStringRechnung() );

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }

    finally
    {
      rfcClose();
    }
  }


  public void testChangeBillingReq(SAPRechnungVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_ZVL_CHANGE_BILLING_REQ:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ(this.fJCoRepository);

      rfcOpen();
      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Rechnung:");
      System.out.println( rfm.toStringRechnung() );

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }

    finally
    {
      rfcClose();
    }
  }


  public void testCreateBillingReqLoadBatchCommit(ArrayList voList) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm = null;
    Iterator  it     = voList.iterator();
    int       cntTot = 0;
    int       cntMsg = 0;


    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      while ( it.hasNext() )
      {
        rfm.addRechnung((SAPRechnungVO)it.next());
        cntTot++; cntMsg++;
        if (cntMsg == 100)
        {
          cntMsg = 0;
          System.out.println("  " + (new Date()) + " : total loaded records : " + cntTot);
        }
      }

      System.out.println("  " + (new Date()) + " : execute rfm");
      rfcExec(rfm.fJCoFunc);

      if ( rfm.isReturnOk() )
      {
        System.out.println("  " + (new Date()) + " : rfm return successfully !");
      }
      else
      {
        System.out.println("  " + (new Date()) + " : rfm return not successfully !");
        System.out.println( rfm.toStringReturnNOK());
      }
    }

    finally
    {
      rfcClose();
    }

  }


  public void testCreateBillingReqLoadSingleCommit(SAPRechnungVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm = null;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
        System.out.println("  " + (new Date()) + " : rfm return not successfully for " + vo.bfakt_id);
    }

    finally
    {
      rfcClose();
    }
  }


  /**
   * jgb create billing request
   *
   * @param  vo       in:  sap billing data
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int testJgbCreateBillingReq(SAPRechnungVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("testJgbCreateBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        System.out.println(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("testJgbCreateBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("testJgbCreateBillingReq - hint (next step): " + rfm.fNextStep + "\n");

      System.out.println(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method




  /**
   * jgb create billing request
   *
   * @param  list     in:  list of SAPRechnungVO's
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int testJgbCreateBillingReq(ArrayList list,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;
    Iterator                          it      = null;


    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      it  = list.iterator();
      rfcOpen();

      while ( it.hasNext() )
      {
        rfm.addRechnung( (SAPRechnungVO)it.next() );
      }

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("testJgbCreateBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        System.out.println(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("testJgbCreateBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("testJgbCreateBillingReq - hint (next step): " + rfm.fNextStep + "\n");

      System.out.println(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method

  /**
   * InsertStako
   *
   * @param vo in:    ValueObject mit Stakodaten
   * @return          rfc return code (siehe rfc.java)
   * @autor           Patrick Bolzli
   */
  public int testInsertStako(StakoVO vo)
  {
    RFM_Z_BAPI_STAKO_INSERT   rfm     = null;
    int                       retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_STAKO_INSERT:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_STAKO_INSERT(this.fJCoRepository);

      rfcOpen();
      rfm.addStako(vo);

      rfcExec(rfm.fJCoFunc);

      if (rfm.isReturnOk())
      {
        System.out.println("### RFM Return successfully!");
        System.out.println("### Export Parameter:");
        System.out.println(rfm.toStringReturn());
      }
      else
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;
        System.out.println("### RFM Return not successfully!");
        System.out.println(rfm.toStringReturn());
      }
    }
    finally
    {
      rfcClose();
    }
    return retCode;
  } // end method


  /**
   * commit annual fee run
   *
   * @param jgbLaufNr in: sequence number of run
   * @param retMsg    out: return message (only set in case of a failure)
   * @return          rfc return code (siehe rfc.java)
   */
  public int testCommitAnnualFee(String jgbLaufNr, StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_COMMIT_ANNUAL_FEE  rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_COMMIT_ANNUAL_FEE(this.fJCoRepository);

      rfcOpen();
      rfm.addSeqNumber(jgbLaufNr);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("commitAnnualFee - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        System.out.println(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("commitAnnualFee - exception: " + ex.toString() + "\n");
      System.out.println(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * commit tm (trademark) extension
   *
   * @param amvLaufNr in: sequence number of run
   * @param retMsg    out: return message (only set in case of a failure)
   * @return          rfc return code (siehe rfc.java)
   */
  public int testCommitTmExtension(String amvLaufNr, StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_COMMIT_TM_EXTENSION  rfm     = null;
    int                                 retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_COMMIT_TM_EXTENSION(this.fJCoRepository);

      rfcOpen();
      rfm.addSeqNumber(amvLaufNr);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("commitTmExtension - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        System.out.println(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("commitTmExtension - exception: " + ex.toString() + "\n");
      System.out.println(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  public void testRejectBillingReq(SAPRechnungAnnullationVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_REJECT_BILLING_REQ rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - RFM_Z_BAPI_ZVL_REJECT_BILLING_REQ:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_ZVL_REJECT_BILLING_REQ(this.fJCoRepository);

      rfcOpen();
      rfm.addAnnullation(vo);
      rfcExec(rfm.fJCoFunc);

      System.out.println("### Annulation:");
      System.out.println( rfm.toStringAnnullation() );

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }

    finally
    {
      rfcClose();
    }
  }


  public void testDeleteBillingReq(SAPRechnungVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_DELETE_BILLING_REQ rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_ZVL_DELETE_BILLING_REQ:");
      System.out.println("### bfakt_id: <" + vo.bfakt_id + ">");

      rfm = new RFM_Z_BAPI_ZVL_DELETE_BILLING_REQ(this.fJCoRepository);

      rfcOpen();
      rfm.delRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Delete Rechnung:");
      System.out.println( rfm.toStringDeleteRechnung() );

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }

    finally
    {
      rfcClose();
    }
  }


  /**
   * Customer Create
   * @param  vo       in:  Value Object mit Adresse
   * @throws JCO.Exception JCO Fehler
   */
  public void testCustomerCreate(VersadrVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_CUSTOMER_CREATE rfm = null;

   try
   {
     System.out.println("");
     System.out.println("### SAP Test RFC - Z_BAPI_CUSTOMER_CREATE:");
     System.out.println("### VO:");
     System.out.println(vo.toString());

     rfm = new RFM_Z_BAPI_CUSTOMER_CREATE(this.fJCoRepository);

     rfcOpen();
     rfm.addCustomer(vo);

     rfcExec(rfm.fJCoFunc);

     System.out.println("### Adresse:");
     System.out.println( rfm.toStringCustomer());

     if ( rfm.isReturnOk() )
     {
       System.out.println("### RFM Return successfully !");
     }
     else
     {
       System.out.println("### RFM Return not successfully !");
     }
     System.out.println( rfm.toStringReturn() );
   }
   finally
   {
     rfcClose();
   }
  } //Method

  /**
   * Customer Delete
   * @param  vo       in:  Value Object mit Adressen-Stati Informationen
   * @throws JCO.Exception JCO Fehler
   */
  public void testCustomerDelete(SAPAdrStatusVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_CUSTOMER_DELETE rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_CUSTOMER_DELETE:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_CUSTOMER_DELETE(this.fJCoRepository);

      rfcOpen();
      rfm.deleteCustomer(vo);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Adresse-Stati:");
      System.out.println( rfm.toStringCustomer());

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }
    finally
    {
      rfcClose();
    }
  } //Method


  /**
   * Customer Block
   *
   * @param  vo       in:  Value Object mit Adressen-Stati Informationen
   * @throws JCO.Exception JCO Fehler
   */
  public void testCustomerBlock(SAPAdrStatusVO vo) throws JCO.Exception
  {
    RFM_Z_BAPI_CUSTOMER_BLOCK rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_CUSTOMER_BLOCK:");
      System.out.println("### VO:");
      System.out.println(vo.toString());

      rfm = new RFM_Z_BAPI_CUSTOMER_BLOCK(this.fJCoRepository);

      rfcOpen();
      rfm.blockCustomer(vo);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Adresse-Stati:");
      System.out.println( rfm.toStringCustomer());

      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );
    }
    finally
    {
      rfcClose();
    }
  }


  public void testCustomerCheck(ArrayList list) throws JCO.Exception
  {
    RFM_Z_BAPI_CUSTOMER_CHECK rfm = null;
    StopWatch                 sw  = new StopWatch();
    int                       idx = 0;


    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - RFM_Z_BAPI_CUSTOMER_CHECK:");
      System.out.println("start time: " + sw.start());
      System.out.println("### List:");
      for (Iterator iter = list.iterator(); iter.hasNext(); )
      {
        VersadrCheckVO item = (VersadrCheckVO)iter.next();
        idx++;
        System.out.println("Element #" + idx);
        System.out.println(item.toString());
      }

      rfm = new RFM_Z_BAPI_CUSTOMER_CHECK(this.fJCoRepository);

      rfcOpen();
      rfm.addVersadrList(list);

      rfcExec(rfm.fJCoFunc);

      // check return
      if ( rfm.isReturnOk() )
      {
        System.out.println("### RFM Return successfully !");
      }
      else
      {
        System.out.println("### RFM Return not successfully !");
      }
      System.out.println( rfm.toStringReturn() );

      // get & show results
      System.out.println("### VersadrCheck - Results:");
      rfm.checkVersadrList(list);
      idx = 0;
      for (Iterator iter = list.iterator(); iter.hasNext(); )
      {
        VersadrCheckVO item = (VersadrCheckVO)iter.next();
        idx++;
        System.out.println("Element #" + idx);
        System.out.println(item.toString());
      }

      // time used
      System.out.println("end time  : " + sw.stop());
      System.out.println("time used : " + sw.toStringDeltaTimeShort());

    }
    finally
    {
      rfcClose();
    }
  }


  public void testCustomerExistence(int adrNr) throws JCO.Exception
  {
    RFM_BAPI_CUSTOMER_CHECKEXISTENCE rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - RFM_BAPI_CUSTOMER_CHECKEXISTENCE:");
      System.out.println("  adrNr: " + adrNr);
      System.out.println("");

      rfm = new RFM_BAPI_CUSTOMER_CHECKEXISTENCE (this.fJCoRepository);

      rfcOpen();
      rfm.addCustomer(adrNr);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Result:");
      System.out.println("  RFM Return Message: " + rfm.getReturnMessage());
      System.out.println("  Missing Customer: " + rfm.missingCustomer());

    }
    finally
    {
      rfcClose();
    }
  } // end method

  public void testCustomerExistenceFile(String pfadIn, String pfadOut) throws JCO.Exception
  {
    RFM_BAPI_CUSTOMER_CHECKEXISTENCE rfm = null;
    String        zeile = "";
    int           adrNr = 0;
    StringBuffer  missing = new StringBuffer();
    try
    {
      rfm = new RFM_BAPI_CUSTOMER_CHECKEXISTENCE(this.fJCoRepository);
      BufferedReader in = new BufferedReader(new FileReader(pfadIn));
      BufferedWriter out = new BufferedWriter(new FileWriter(pfadOut));
      rfcOpen();
      while ((zeile = in.readLine()) != null)
      {
        adrNr = Integer.parseInt(zeile);
        rfm.addCustomer(adrNr);
        rfcExec(rfm.fJCoFunc);

        if (rfm.missingCustomer())
        {
          System.out.println("adrNr: " + adrNr + "\t\tfehlt!");
          missing.append("Adresse " + adrNr + " ist auf SAP nicht vorhanden!\n");
        }
        else
        {
          System.out.println("adrNr: " + adrNr + "\t\tOK!");
        }
      }
      // fehlende Adressen in File speichern
      System.out.println("------------------------");
      System.out.println("Die Resultate finden Sie unter: " + pfadOut);
      out.write(missing.toString());
      in.close();
      out.close();
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
    finally
    {
      rfcClose();
    }
  } // end method




  /**
   * customerBalance
   *
   * @param  ktoNr Kontonummer (bzw. Adressnummer) des DepKto-Inhabers
   * @throws JCO.Exception JCO Fehler
   */
  public void testCustomerBalance(int ktoNr) throws JCO.Exception
  {
    RFM_Z_BAPI_ZVL_GET_CUST_BALANCE rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - Z_BAPI_ZVL_GET_CUST_BALANCE:");
      System.out.println("### ktoNr: " + ktoNr);

      rfm = new RFM_Z_BAPI_ZVL_GET_CUST_BALANCE(fJCoRepository);

      rfcOpen();
      rfm.getBalance(ktoNr);
      rfcExec(rfm.fJCoFunc);

      if (rfm.isReturnOk())
      {
        System.out.println("### RFM Return successfully!");
        System.out.println("### Export Parameter:");
        System.out.println(rfm.toStringExport());
      }
      else
      {
        System.out.println("### RFM Return not successfully!");
        System.out.println(rfm.toStringReturn());
      }

    }
    finally
    {
      rfcClose();
    }
  }


  /**
   * getArtikel
   * @return ArrayList der stakorelevanten Artikel
   * @throws JCO.Exception JCO Fehler
   */
  public ArrayList testGetArtikel() throws JCO.Exception
  {
    RFM_Z_BAPI_STAKO_GET_ARTIKEL rfm = null;
    ArrayList al = null;

    try
    {
      System.out.println("");
      System.out.println("## SAP Test RC - Z_BAPI_STAKO_GET_ARTIKEL:");
      System.out.println("");

      rfm = new RFM_Z_BAPI_STAKO_GET_ARTIKEL(fJCoRepository);

      rfcOpen();
      rfcExec(rfm.fJCoFunc);
      al = rfm.getArtikel();

      if (rfm.isReturnOk())
      {
        System.out.println("### RFM Return successfully!");
      }
      else
      {
        System.out.println("### RFM Return not successfully!");
        System.out.println(rfm.toStringReturn());
      }
    }
    finally
    {
      rfcClose();
    }
    return al;
  } // end method

  /**
   * ExistenceCheck
   * @param  art_nr Artikelnummer
   * @throws JCO.Exception JCO Fehler
   */
  public void testExistenceCheck(String art_nr) throws JCO.Exception
  {
    RFM_BAPI_MATERIAL_EXISTENCECHECK rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC - BAPI_MATERIAL_EXISTENCECHECK:");
      System.out.println("### art_nr: " + art_nr);

      rfm = new RFM_BAPI_MATERIAL_EXISTENCECHECK(fJCoRepository);

      rfcOpen();
      rfm.existenceCheck(art_nr);
      rfcExec(rfm.fJCoFunc);

      if (rfm.isReturnOk())
      {
        System.out.println("### RFM Return successfully!");
        System.out.println("### Export Parameter:");
        System.out.println(rfm.toStringReturn());
      }
      else
      {
        System.out.println("### RFM Return not successfully!");
        System.out.println(rfm.toStringReturn());
      }

    }
    finally
    {
      rfcClose();
    }

  }


  public void testStfcConnection(String txt) throws JCO.Exception
  {
    RFM_STFC_CONNECTION rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test STFC_CONNECTION - Results:");

      rfm = new RFM_STFC_CONNECTION(this.fJCoRepository);
      rfcOpen();
      rfm.add(txt);

      rfcExec(rfm.fJCoFunc);

      System.out.println("### Attributes:");
      System.out.println(this.fJCoClient.getAttributes().toString());
      System.out.println("### Export:");
      System.out.println(rfm.toStringExport());
    }
    finally
    {
      rfcClose();
    }
  }


  public void testRfcSystemInfo() throws JCO.Exception
  {
    RFM_RFC_SYSTEM_INFO rfm = null;

    try
    {
      System.out.println("");
      System.out.println("### SAP Test RFC_SYSTEM_INFO - Results:");

      rfm = new RFM_RFC_SYSTEM_INFO(this.fJCoRepository);
      rfcOpen();
      rfcExec(rfm.fJCoFunc);

      System.out.println("### Export Structure RFCSI:");
      System.out.println(rfm.toStringRFCSI());
      System.out.println("### Export:");
      System.out.println(rfm.toStringExport());
    }
    finally
    {
      rfcClose();
    }
  }
} // end class

