package ch.ige.ses.client.sapinbound;

import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;
import com.sap.mw.jco.*;

/**
 * SAP Remote Function Module BAPI_CUSTOMER_CHECKEXISTENCE
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Thomas Iten
 * @version $Id: RFM_BAPI_CUSTOMER_CHECKEXISTENCE.java,v 1.7 2006/12/11 12:43:32 iten Exp $
 */
public class RFM_BAPI_CUSTOMER_CHECKEXISTENCE extends RFM
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger(RFM_BAPI_CUSTOMER_CHECKEXISTENCE.class);

  //
  // fields
  //
  public  String  fNextStep;              // next step in process (trace info)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public RFM_BAPI_CUSTOMER_CHECKEXISTENCE (IRepository repository)
  {
    super("BAPI_CUSTOMER_CHECKEXISTENCE",repository);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void addCustomer(int adrNr)
  {
    try
    {
      this.setImportParam(adrNr);
    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }


  //---------------------------------------------------------------------------
  // important note:  do NOT USE the rfm methods
  //                  isReturnOk, toStringReturn and toStringReturnNOK
  //---------------------------------------------------------------------------

  /**
   * get return message as a single string
   *
   * @return  return message or empty string in case of an error
   */
  public String getReturnMessage()
  {
    String msg = "";

    try
    {
      JCO.ParameterList output = this.fJCoFunc.getExportParameterList();
      JCO.Structure     struct = output.getStructure("RETURN");

      String  type    = struct.getString("TYPE");
      String  code    = struct.getString("CODE");
      String  message = struct.getString("MESSAGE");

      msg = "Type=" + type + ", Code=" + code + ", Message=" + message;

    }
    catch (Exception ex)
    {
      // ignore
    }

    return msg;
  }


  /**
   * check if customer is missing within sap
   *
   * @return  true if customer is missing,
   *          false if customer exists or in case of an error
   */
  public boolean missingCustomer()
  {
    boolean   ret = false;

    try
    {
      JCO.ParameterList output = this.fJCoFunc.getExportParameterList();
      JCO.Structure     struct = output.getStructure("RETURN");
      String            code   = struct.getString("CODE");

      if ( code.equals("S>522") )
      {
        ret = true;
      }
    }
    catch (Exception ex)
    {
      // ignore
    }

    return ret;
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void setImportParam(int adrNr)
  {
    JCO.ParameterList input     = this.fJCoFunc.getImportParameterList();

    fNextStep = "setImportParam - CUSTOMERNO";
    input.setValue(Convert.toSapNr(adrNr,10),"CUSTOMERNO");

    fNextStep = "setImportParam - SALES_ORGANIZATION";
    input.setValue("1000","SALES_ORGANIZATION");

    fNextStep = "setImportParam - DISTRIBUTION_CHANNEL";
    input.setValue("00","DISTRIBUTION_CHANNEL");

    fNextStep = "setImportParam - DIVISION";
    input.setValue("00","DIVISION");
  }

} // end class


