package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.lang.*;
import java.math.BigDecimal;
import org.apache.log4j.*;
import com.sap.mw.jco.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.client.saputil.*;

/**
 * SAP Remote Function Module RFM_Z_BAPI_CUSTOMER_BLOCK
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Stefan Nydegger
 * @version $Id: RFM_Z_BAPI_CUSTOMER_BLOCK.java,v 1.6 2006/12/11 12:43:32 iten Exp $
 */
public class RFM_Z_BAPI_CUSTOMER_BLOCK extends RFM
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger(RFM_Z_BAPI_CUSTOMER_BLOCK.class);

  //
  // fields
  //
  private boolean fImportParamIsSet  = false;     // set by setImportParam
  public  String  fNextStep          = "";        // next step in process (trace info)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public RFM_Z_BAPI_CUSTOMER_BLOCK (IRepository repository)
  {
    super("Z_BAPI_CUSTOMER_BLOCK",repository);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * setzt oder entfernt Buchungssperre
   * @param vo Value Object mit Adressen Statusinformationen
   */
  public void blockCustomer(SAPAdrStatusVO vo)
  {
    try
    {
      if (! this.fImportParamIsSet)                 // set only once per call
        this.setImportParam(vo);
    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }

  /**
   * Gibt alle Informationen der Structure CUSTOMER als String zurück
   * @return Structure als String
   */
  public String toStringCustomer()
  {
    JCO.ParameterList in  = this.fJCoFunc.getImportParameterList();
    JCO.Structure strucCust = in.getStructure("CUSTOMER");
    StringBuffer      buf = new StringBuffer();

    buf.append(strucCust.toString());

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Setzt die Werte der Structure CUSTOMER
   * @param vo ValueObject mit Adresse
   */
  private void setImportParam(SAPAdrStatusVO vo)
  {
    JCO.ParameterList input = this.fJCoFunc.getImportParameterList();
    JCO.Structure strucCust = input.getStructure("CUSTOMER");

    fNextStep = "setImportParam - CUSTOMER";
    strucCust.setValue(Convert.toSapNr(vo.adrNr,10),"CUSTOMER");

    fNextStep = "setImportParam - COMP_CODE";
    strucCust.setValue(vo.compCode,"COMP_CODE");

    fNextStep = "setImportParam - SALESORG";
    strucCust.setValue(vo.salesOrg,"SALESORG");

    fNextStep = "setImportParam - DISTR_CHAN";
    strucCust.setValue(vo.distrChan,"DISTR_CHAN");

    fNextStep = "setImportParam - DIVISION";
    strucCust.setValue(vo.distrChan,"DIVISION");

    //Wenn der Modus auf Buchungssperre ist, flag setzen
    fNextStep = "setImportParam - PSTG_BLK_C";
    if(vo.modus == false)
    {
      input.setValue(vo.action,"PSTG_BLK_C");
    }
    this.fImportParamIsSet = true;
  }
} // end class

