package ch.ige.ses.client.sapinbound;

import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.util.vo.*;
import com.sap.mw.jco.*;

/**
 * SAP Remote Function Module RFM_Z_BAPI_CUSTOMER_CREATE
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Stefan Nydegger
 * @version $Id: RFM_Z_BAPI_CUSTOMER_CREATE.java,v 1.12 2006/12/11 12:43:32 iten Exp $
 */
public class RFM_Z_BAPI_CUSTOMER_CREATE extends RFM
{
  //
  // const
  //
  private static final Logger  LOG = Logger.getLogger(RFM_Z_BAPI_CUSTOMER_CREATE.class);

  //
  // fields
  //
  public  String  fNextStep;      // next step in process (trace info)

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public RFM_Z_BAPI_CUSTOMER_CREATE (IRepository repository)
  {
    super("Z_BAPI_CUSTOMER_CREATE",repository);
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Fügt Adresse ein
   * @param versadrVO Value Object
   */
  public void addCustomer(VersadrVO versadrVO)
  {
    try
    {
      this.setImportParam(versadrVO);
    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }


  /**
   * Gibt alle Informationen der Structure CUSTOMER als String zurück
   * @return Structure als String
   */
  public String toStringCustomer()
  {
    JCO.ParameterList in        = this.fJCoFunc.getImportParameterList();
    JCO.Structure     strucCust = in.getStructure("CUSTOMER");
    StringBuffer      buf       = new StringBuffer();

    buf.append(strucCust.toString());

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Setzt die Werte der Structure CUSTOMER
   * @param vo ValueObject mit Adresse
   */
  private void setImportParam(VersadrVO vo)
  {
    JCO.ParameterList input     = this.fJCoFunc.getImportParameterList();
    JCO.Structure     strucCust = input.getStructure("CUSTOMER");

    fNextStep = "setImportParam - CUSTOMER";
    strucCust.setValue(Convert.toSapNr(vo.getAdrNr(),10),"CUSTOMER");

    fNextStep = "setImportParam - COMP_CODE";
    strucCust.setValue("1000","COMP_CODE");

    fNextStep = "setImportParam - SALESORG";
    strucCust.setValue("1000","SALESORG");

    fNextStep = "setImportParam - DISTR_CHAN";
    strucCust.setValue("00","DISTR_CHAN");

    fNextStep = "setImportParam - DIVISION";
    strucCust.setValue("00","DIVISION");

    fNextStep = "setImportParam - ACCNT_GRP";
    strucCust.setValue("0001","ACCNT_GRP");

    // TITLE is not available

    fNextStep = "setImportParam - NAME";
    strucCust.setValue(vo.getNameOrEmptyStr(),"NAME");

    fNextStep = "setImportParam - NAME_2";
    if(vo.getZeile1() != null && !vo.getZeile1().equals(""))
      strucCust.setValue(vo.getZeile1(),"NAME_2");

    fNextStep = "setImportParam - NAME_3";
    if(vo.getZeile2() != null && !vo.getZeile2().equals(""))
      strucCust.setValue(vo.getZeile2(),"NAME_3");

    fNextStep = "setImportParam - NAME_4";
    if(vo.getZeile3() != null && !vo.getZeile3().equals(""))
      strucCust.setValue(vo.getZeile3(),"NAME_4");

    fNextStep = "setImportParam - SORT1";
    if ( (vo.getSuchbegr() == null) || vo.getSuchbegr().equals("") )
    {
      strucCust.setValue(vo.getNameOrEmptyStr().toUpperCase(),"SORT1");
    }
    else
    {
      strucCust.setValue(vo.getSuchbegr(),"SORT1");
    }

    // SORT2 is not available

    fNextStep = "setImportParam - STREET";
    if(vo.getZeile4() != null && !vo.getZeile4().equals(""))
      strucCust.setValue(vo.getZeile4(),"STREET");

    // HOUSE_NO is not available

    fNextStep = "setImportParam - POSTL_COD1";
    if(vo.getPlz() != null && !vo.getPlz().equals(""))
      strucCust.setValue(vo.getPlz(),"POSTL_COD1");

    fNextStep = "setImportParam - CITY";
    strucCust.setValue(vo.getOrtOrEmptyStr(),"CITY");

    fNextStep = "setImportParam - COUNTRY";
    strucCust.setValue(ConvertCountry.toSAPCountry(vo.getLandCdOrEmptyStr()),"COUNTRY");

    // PO_BOX is not available
    // PO_BOX_PCD is not available

    fNextStep = "setImportParam - LANGU";
    strucCust.setValue(Convert.toSapLangu(vo.getSprachCdOrEmptyStr()),"LANGU");

    fNextStep = "setImportParam - TEL1_NUMBR";
    if(vo.getTelefon() != null && !vo.getTelefon().equals(""))
      strucCust.setValue(vo.getTelefon(),"TEL1_NUMBR");

    // TEL1_EXT is not available

    fNextStep = "setImportParam - FAX_NUMBER";
    if(vo.getFax() != null && !vo.getFax().equals(""))
      strucCust.setValue(vo.getFax(),"FAX_NUMBER");

    // FAX_EXTENS is not available

    fNextStep = "setImportParam - E_MAIL";
    if(vo.getEmailNameDomain() != null && !vo.getEmailNameDomain().equals(""))
      strucCust.setValue(vo.getEmailNameDomain(),"E_MAIL");

    fNextStep = "setImportParam - CONTROL_ACCOUNT";
    strucCust.setValue(vo.isIsNew() ? "1100" : "/" ,"CONTROL_ACCOUNT");

    fNextStep = "setImportParam - OLD_ACCOUNT";
    strucCust.setValue("/","OLD_ACCOUNT");

    fNextStep = "setImportParam - PMNTTRMS";
    strucCust.setValue(vo.isIsNew() ? "Z004" : "/" ,"PMNTTRMS");

    fNextStep = "setImportParam - CUST_GROUP";
    strucCust.setValue(vo.getCustGroupOrEmptyStr(),"CUST_GROUP");

    fNextStep = "setImportParam - CURRENCY";
    strucCust.setValue("CHF","CURRENCY");

    fNextStep = "setImportParam - PRC_PROCED";
    strucCust.setValue("1","PRC_PROCED");

    fNextStep = "setImportParam - CUSTSTATGR";
    strucCust.setValue("1","CUSTSTATGR");

    fNextStep = "setImportParam - ACCNT_ASGN";
    strucCust.setValue("01","ACCNT_ASGN");

    fNextStep = "setImportParam - TAX_CLASS";
    strucCust.setValue(vo.getTaxClassOrEmptyStr(),"TAX_CLASS");

    fNextStep = "setImportParam - CUST_GRP1";
    strucCust.setValue(Convert.toSapCustGrp1(vo.getCustGrp1OrEmptyStr()),"CUST_GRP1");

    // CUST_GRP2 is not available
    // CUST_GRP3 is not available
    // CUST_GRP4 is not available
    // CUST_GRP5 is not available

    fNextStep = "setImportParam - ALLOC_KEY";
    strucCust.setValue(vo.isIsNew() ? "001" : "/","ALLOC_KEY");

    fNextStep = "setImportParam - PAYMENT_METHODS";
    strucCust.setValue(vo.isIsNew() ? "OPZF" : "/","PAYMENT_METHODS");

    fNextStep = "setImportParam - DUNN_PROC";
    strucCust.setValue(vo.isIsNew() ? "0001" : "/","DUNN_PROC");

    fNextStep = "setImportParam - CLERK";
    strucCust.setValue(vo.isIsNew() ? "01" : "/","CLERK");
  }

} // end class

