package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.lang.*;
import com.sap.mw.jco.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;

/**
 * SAP Remote Function Module Z_BAPI_ZVL_GET_CUST_BALANCE
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * @author Patrick Bolzli
 * @version $Id: RFM_Z_BAPI_ZVL_GET_CUST_BALANCE.java,v 1.7 2006/12/11 12:43:32 iten Exp $
 */

public class RFM_Z_BAPI_ZVL_GET_CUST_BALANCE extends RFM
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(RFM_Z_BAPI_ZVL_GET_CUST_BALANCE.class);
  private static final String COMP_CODE = "1000";

  //
  // fields
  //
  public String fNextStep = "";  // next step in process

  //------------------------------------------------------------------
  // constructors
  //------------------------------------------------------------------
  public RFM_Z_BAPI_ZVL_GET_CUST_BALANCE (IRepository repository)
  {
    super ("Z_BAPI_ZVL_GET_CUST_BALANCE" ,repository);
  }

  //------------------------------------------------------------------
  // public methods
  //------------------------------------------------------------------

  public void getBalance(int ktoNr)
  {
    try
    {
      setParams(ktoNr);
    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }

  /**
   * Gibt den Kontosaldo als double zurück
   * @return retval Kontosaldo als double
   */
  public double getRetval()
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("BALANCE");
    double retval = 0;

    if (tbl.getNumRows() > 0)     // Annahme, dass das Resultat immer 1 Row ist.
    {
      retval = tbl.getDouble("BALANCE");
    }
    return retval;
  }



  /**
   * Gibt die Returnwerte als String zurück
   * @return Returnwerte als String
   */
  public String toStringExport()
  {
    JCO.Table tbl = fJCoFunc.getTableParameterList().getTable("BALANCE");
    StringBuffer      buf    = new StringBuffer();

    if (tbl.getNumRows() > 0)
    {
      do
      {
        buf.append("CUSTOMER:\t" + tbl.getString("CUSTOMER") + "\n");
        buf.append("COMP_CODE:\t" + tbl.getString("COMP_CODE") + "\n");
        buf.append("BALANCE:\t" + tbl.getString("BALANCE") + "\n");
        buf.append("CURRENCY:\t" + tbl.getString("CURRENCY") + "\n");
      } while(tbl.nextRow());
    }
    return buf.toString();
  }

  //------------------------------------------------------------------
  // private methods
  //------------------------------------------------------------------
  private void setParams(int ktoNr)
  {
    JCO.ParameterList input = this.fJCoFunc.getImportParameterList();

    fNextStep = "setParams - CUSTOMER";
    input.setValue(Convert.toSapCustomer(ktoNr), "CUSTOMER");

    fNextStep = "setParams - COMP_CODE";
    input.setValue(COMP_CODE, "COMP_CODE");  // Konstanter Wert

  }

} // end class


