package ch.ige.ses.client.sapinbound.test;

import ch.ige.ses.client.saprequest.SapAdresse;
import ch.ige.ses.exception.*;
import ch.ige.util.vo.VersadrVO;
import ch.ige.ses.client.sapinbound.TestData;

/**
 * Load Test für Sap Adressen anlegen im Request-Reply Modus
 * <p>
 * Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Stefan Nydegger
 * @version $Id: SapAdrLoadTestThread.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class SapAdrLoadTestThread extends Thread
{
  int repeats     = 100;
  int delay       = 1000;
  int startAdr    = 9000000;
  int errorCount  = 0;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public SapAdrLoadTestThread()
  {
    // lazy bone
  }

  /**
   * @param repeats  Anzahl Wiederholungen pro Thread
   * @param delay    Pause in [ms] zwischen dem Anlegen der Adressen
   * @param startAdr Beginn der AdresseRange
   */
  public SapAdrLoadTestThread(int repeats, int delay, int startAdr)
  {
    this.repeats = repeats;
    this.delay   = delay;
    this.startAdr = startAdr;
  }


  //---------------------------------------------------------------------------
  //  methods
  //---------------------------------------------------------------------------

  /**
   * Generiert oder aktualisiert eine bestimmte Anzahl Adressen.<b>
   * Zwischen dem Anlegen einer Adresse wird eine Pause eingelegt
   */
  public void run()
  {
    SapAdresse adr = new SapAdresse();
    VersadrVO vo = TestData.createAdresse(startAdr);

    for (int i = 0; i < repeats; i++)
    {
      vo.setAdrNr(startAdr + i);
      vo.setIsNew(true);

      try
      {
        adr.create(vo);
      }
      catch (SapCallException ex)
      {
        errorCount++;
        System.out.println("SapCallException Message: " + ex.getMessage());
      }

      try
      {
        sleep(delay);
      }
      catch (InterruptedException ex)
      {
        System.out.println("Thread Interrupted Message: " + ex.getMessage());
      }

    }
  }

} // end class

