package ch.ige.ses.client.saprequest;

import ch.ige.ses.exception.SapCallException;
import ch.ige.ses.util.Jms;
import ch.ige.ses.client.sapinbound.RFC;

/**
 * <p>SapArtikel:</p>
 * <p>Diese Klasse handelt die Requests zur Abfrage eines Artikelstatus'</p>
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                           ACHTUNG                           !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                                                             !
 *    !  Diese Klasse verwendet J2EE Restrictions und darf nur      !
 *    !  von Java Client Programmen oder Servlets verwendet werden. !
 *    !                                                             !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * <p>Copyright: Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Patrick Bolzli
 * @version $Id: SapArtikel.java,v 1.6 2007/01/10 18:21:18 iten Exp $
 */

public class SapArtikel extends SapRequest
{
  /**
   * Die Aufrufe des SAP Systems via SapDepKto verlaufen im Reply / Request
   * Modus. Das heisst, der Aufrufer wartet auf die Antwort des SAP Systems.
   */
  public SapArtikel()
  {
    super();
  }

  //-------------------------------------------------------
  // public methods
  //-------------------------------------------------------

  /**
   * Prüft via JMS auf SAP ob der Mitgegebene Artikel existiert
   * @param art_nr zu prüfende Artikelnummer
   * @return status (true = Artikel vorhanden, false = Artikel nicht vorhanden
   * @throws SapCallException
   */
  public boolean existenceCheck(String art_nr) throws SapCallException
  {
    boolean     status;
    SapReturn   reply;

    try
    {
      connect();

      reply = request(  art_nr,
                        Jms.MSG_TYPE_SAP_MATERIAL,
                        Jms.MSG_DESC_SAP_MATERIAL);

      if (reply.rfcCode == RFC.RET_CODE_RETURN_FAILURE)
      {
        status = false;
      }
      else if (reply.rfcCode == RFC.RET_CODE_EXCEPTION)
      {
        throw new SapCallException( "Fehler beim existenceCheck",
                                    reply.rfcMessage.toString());
      }
      else
      {
        status = true;
      }
    }
    finally
    {
      disconnect();
    }

    return status;
  }

} // end class

