package ch.ige.ses.server.bagisoutbound;

import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;

import org.apache.log4j.Logger;

import ch.ige.ses.vo.bagis.SAPRechnungVO;

/**
 * Session Fascade für SES
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Pirmin Suter
 * @version  $Id: SESFacadeBean.java,v 1.48 2007/01/10 14:46:37 iten Exp $
 *
 * @ejb.bean 
 *  name="SESFacade"  
 *  display-name="SESFacade"
 *  view-type="both"
 *  jndi-name="SESFacade"
 *  local-jndi-name="SESFacadeLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESRechnung"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESRechnungAnnullation"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESJGBStart"
 *   view-type="local"
 */
public class SESFacadeBean implements SessionBean {

  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(SESFacadeBean.class);

  //
  // fields
  //
  SessionContext          sessionContext;

  SESRechnungLookup       rechnung        = new SESRechnungLookup();

  private SESJGBStartLocalHome            sesJGBStartLocalHome  = null;
  private SESJGBStartLocal                sesJGBStartLocal      = null;

  private SESRechnungAnnullationLocalHome sesRechnungAnnullationLocalHome = null;
  private SESRechnungAnnullationLocal     sesRechnungAnnullationLocal     = null;

  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb 
   */
  public void ejbCreate()
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext) {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  //  methods SESRechnungBean
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public void createRechnung (int bfaktId)
  {
    rechnung.getLocalInterface().createRechnung(bfaktId);
  }

  /**
   * @ejb.interface-method
   */
  public void changeRechnung (int bfaktId)
  {
    rechnung.getLocalInterface().changeRechnung(bfaktId);
  }

  /**
   * @ejb.interface-method
   */
  public SAPRechnungVO previewRechnung(int bfaktId)
  {
    return rechnung.getLocalInterface().previewRechnung(bfaktId);
  }


  //---------------------------------------------------------------------------
  //  methods SESRechnungAnnullationBean
  //---------------------------------------------------------------------------

  /**
   * Die Methode AnnullationRechnung wird einerseits von der Komponente BAEX
   * aufgerufen (bei der ONLINE-Annullierung einer Rechnung) und andererseits
   * von einem Batch-Programm, welches die Fristen der BAGIS-Rechnung
   * überwacht.
   *
   * @param   bfakt_id  Identifikation einer BFaktur
   * @param   herkunft  ONLINE oder BATCH
   * @return  boolean
   * 
   * @ejb.interface-method
   */
  public boolean AnnullationRechnung(int bfakt_id, String herkunft)
  {
    LOG.debug( "AnnullationRechnung(bfakt_id=" + bfakt_id +
               " herkunft=" + herkunft + ") starting ..." );

    getBeanAnnullation().AnnullationRechnung(bfakt_id, herkunft);
    return true;
  }


  /**
   * Die Methode AnnullationRechnungJGB wird von einem Batch-Programm
   * aufgerufen, welches die Fristen der JGB-Rechnungen überwacht.
   *
   * @param   alSAPRechnungAnnullationList    Liste mit JGB-Fakturen
   * @return  boolean
   * 
   * @ejb.interface-method
   */
  public boolean AnnullationRechnungJGB(java.util.ArrayList alSAPRechnungAnnullationList)
  {
    LOG.debug( "AnnullationRechnungJGB(list) starting ..." );

    getBeanAnnullation().AnnullationRechnungJGB(alSAPRechnungAnnullationList);
    return true;
  }


  /**
   * Get bean reference with lazy loading
   *
   * @return SESRechnungAnnullationLocal
   */
  private SESRechnungAnnullationLocal getBeanAnnullation()
  {
    try
    {
      if (sesRechnungAnnullationLocal == null)
      {
        Context ctx = new InitialContext();
        Object  ref = ctx.lookup(sesRechnungAnnullationLocalHome.COMP_NAME);

        sesRechnungAnnullationLocalHome = (SESRechnungAnnullationLocalHome) ref;
        sesRechnungAnnullationLocal = sesRechnungAnnullationLocalHome.create();
      }

      return sesRechnungAnnullationLocal;
    }
    catch (Exception ex)
    {
      sesRechnungAnnullationLocalHome = null;
      sesRechnungAnnullationLocal     = null;

      LOG.error("getBeanAnnullation - Exception: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //---------------------------------------------------------------------------
  //  methods SESJGBStartBean
  //---------------------------------------------------------------------------

  /**
   * Start JGB Verarbeitung (asynchron) via JMS Queue.
   *
   * @param   modus         Verarbeitungsmodus
   *                        - "PROD"  Start normale Verarbeitung
   *                        - "TEST"  Start mit wählbarer bfakt_id Range
   *                        - "STOP"  Stoppen der JGB-Vearbeitung
   * @param   minBFaktId    Begin bfakt_id range
   * @param   maxBFaktId    Ende bfakt_id range
   * @throws  EJBException  Unerwarteter Fehler
   * @return  boolean       Erfolgreich gestartet (true)
   *  
   * @ejb.interface-method
   */
  public boolean JGBStartSendToJMS (String modus, int minBFaktId, int maxBFaktId)
      throws EJBException
  {
    try
    {
      LOG.debug( "JGBStartSendToJMS(modus=" + modus +
                 " minBFaktId=" + minBFaktId +
                 " maxBFaktId=" + maxBFaktId + ") starting ..." );

      return getBeanJgbStart().JGBStartSendToJMS(modus, minBFaktId, maxBFaktId);
    }
    catch (Exception ex)
    {
      LOG.error("JGBStartSendToJMS - Exception:" + ex.getMessage());
      throw new EJBException(ex);
    }
  }


  /**
   * Get bean reference with lazy loading
   *
   * @return SESRechnungAnnullationLocal
   */
  private SESJGBStartLocal getBeanJgbStart()
  {
    try
    {
      if (sesJGBStartLocal == null)
      {
        Context ctx = new InitialContext();
        Object  ref = ctx.lookup(SESJGBStartLocalHome.COMP_NAME);

        sesJGBStartLocalHome = (SESJGBStartLocalHome) ref;
        sesJGBStartLocal = sesJGBStartLocalHome.create();
      }

      return sesJGBStartLocal;
    }
    catch (Exception ex)
    {
      sesJGBStartLocalHome  = null;
      sesJGBStartLocal      = null;

      LOG.error("getBeanJgbStart - Exception: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }


} // end class

