package ch.ige.ses.server.bagisoutbound.test;
import java.io.*;
import java.util.*;

import ch.ige.ses.server.bagisoutbound.*;
import ch.ige.ses.vo.bagis.*;

/**
 * test application (console) for bagis outbound functions.
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $ Id: $
 */
public class BagisOutboundTestConsole {

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public BagisOutboundTestConsole()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void run() throws Exception
  {
    boolean  go   = true;
    int      cmd;


    while (go)
    {
      try
      {
        cmd = showMenu();

        if (cmd == 99)
        {
          go = false;
          System.out.println("Exit Bagis Outbound Test, good by...");
          System.out.println("");
        }
        else
        {
          execMenu(cmd);
        }
      }
      catch (Exception ex)
      {
        System.out.println("");
        System.out.println("### Achtung folgende Exception ist aufgetreten:");
        System.out.println(ex.getMessage());
        System.out.println("Stack Trace:");
        ex.printStackTrace();
      }

    } // end while

  } // end run


  //---------------------------------------------------------------------------
  //  private methods
  //---------------------------------------------------------------------------

  private int showMenu()
  {
    String         cmd;

    try
    {
      System.out.println("Bagis Outbound Test - Menu");
      System.out.println("   1  SESDelegate.NewRechnung       - Neue Rechnung anlegen");
      System.out.println("   2  SESDelegate.ChangeRechnung    - Bestehende Rechnung (auf SAP) mutieren");
      System.out.println("   3  SESDelegate.getDruckvorschau  - Druckvorschau für Rechnung");
      System.out.println("   4  SESDelegate.getDruckvorschau  - Load Test");
      System.out.println("  99  exit");

      cmd = readLine("   Command: ");
      System.out.println();

      return (Integer.parseInt(cmd));
    }
    catch (Exception ex)
    {
      return -1;
    }
  }


  private void execMenu(int cmd) throws Exception
  {
    switch (cmd)
    {
      case 1:
      {
        // SESDelegate.NewRechnung - Neue Rechnung anlegen
        int bfaktId = Integer.parseInt(readLine("bfakt_id = "));
        SESDelegate.getInstance().NewRechnung(bfaktId);
        break;
      }

      case 2:
      {
        // SESDelegate.ChangeRechnung - Bestehende Rechnung (auf SAP) mutieren
        int bfaktId = Integer.parseInt(readLine("bfakt_id = "));
        SESDelegate.getInstance().ChangeRechnung(bfaktId);
        break;
      }

      case 3:
      {
        // SESDelegate.getDruckvorschau - Druckvorschau für Rechnung
        int           bfaktId = Integer.parseInt(readLine("bfakt_id = "));
        SAPRechnungVO vo      = SESDelegate.getInstance().getDruckvorschau(bfaktId);
        System.out.println(vo.toString());
        break;
      }

      case 4:
      {
        // SESDelegate.getDruckvorschau - Loadtest
        ArrayList     list = new ArrayList();
        String        buf  = null;
        int           rep  = 0;
        SAPRechnungVO vo   = null;

        System.out.println("Eingabe bfakt_id's bis Liste mit Eingabe \"0\" abgeschlossen wird:");
        buf = readLine("  bfakt_id = ");
        while (! buf.equals("0"))
        {
          list.add(buf);
          buf = readLine("  bfakt_id = ");
        }

        System.out.println("Eingabe Wiederholungen:");
        rep = Integer.parseInt(readLine("  Anzahl = "));

        System.out.println("Start Load Test...");
        for (int i=0; i<rep; i++)
        {
          for (Iterator iter = list.iterator(); iter.hasNext(); )
          {
            int item = Integer.parseInt((String)iter.next());
            vo = SESDelegate.getInstance().getDruckvorschau(item);
            System.out.println("### \n" + vo.toString());
          }
        }
        break;
      }

      default:
        System.out.println("execMenu - command <" + cmd + "> nicht bekannt !");
        break;
    }
  }


  private String readLine(String prompt)
  {
    try
    {
      BufferedReader stdin  = new BufferedReader(new InputStreamReader(System.in));
      System.out.print(prompt);
      return stdin.readLine();
    }
    catch (Exception ex)
    {
      return ("-1");
    }
  }


  //***************************************************************************
  // main
  //***************************************************************************

  public static void main(String[] args)
  {
    try
    {
      new BagisOutboundTestConsole().run();
    }
    catch (Exception ex)
    {
      System.out.println("");
      System.out.println("### Achtung folgende Exception ist aufgetreten:");
      System.out.println(ex.getMessage());
      System.out.println("Stack Trace:");
      ex.printStackTrace();
    }
  }

} // end class

