package ch.ige.ses.server.dao.bagis;

import java.util.*;
import java.sql.*;
import java.io.IOException;
import ch.ige.util.server.DBHelper;
import java.util.GregorianCalendar;


/**
 * Basic Bagis DAO. Stellt gemeinsame Grundfunktionen zur Verfügung.
 * Wird von allen DAO's welche auf Bagis zugreiffen erweitert.
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Thomas Iten
 * @version $Id: BagisDAO.java,v 1.14 2006/12/28 07:44:42 iten Exp $
 */
public class BagisDAO
{
  //
  // const
  //

  /** Bagis Datasource Name (für die neue Oracle DB) */
  public static final String DSN = DBHelper.BAGIS;

  /**
   * Die bfakt_id Basis kann zur Einschränkung der Suchmenge von grossen
   * Abfragen auf die Tabelle BFAKTUR verwendet werden.
   *
   * Der Wert ist so gesetzt,  dass alte (bereits verarbeitete) Rechnungen
   * nicht mehr berücksichtigt werden.
   */
  public static final int BFAKTID_BASE = 4000000;

  //
  // fields
  //
  public StringBuffer query = new StringBuffer();

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------


  public BagisDAO()
  {
    // lazy bone
  }

  public Connection getConnection()
  {
    return ch.ige.util.server.DBHelper.getConnection(DSN);
  }

  //---------------------------------------------------------------------------
  // methods - getter/setter
  //---------------------------------------------------------------------------

  public void resetQuery()
  {
    query.delete(0,query.length());
  }

  public void setQuery(String query)
  {
    resetQuery();
    this.query.append(query);
  }

  public void setQuery(StringBuffer query)
  {
    setQuery(query.toString());
  }

  public String getQuery()
  {
    return query.toString();
  }

  //---------------------------------------------------------------------------
  // methods - some helpers used for db migration from ingres to oracle
  //---------------------------------------------------------------------------

  public java.sql.Date toSqlDate(java.util.Date date)
  {
    if (date == null) return null;

    return new java.sql.Date( date.getTime() );
  }

  public java.sql.Date toSqlDatePlusOneDay(java.util.Date date)
  {
    if (date == null) return null;
    return toSqlDate( addDays(date,1) );
  }

  public java.util.Date addDays (java.util.Date date, int days)
  {
    Calendar cal = new GregorianCalendar();

    if (date == null) return null;

    cal.setTime(date);
    cal.add(Calendar.DAY_OF_MONTH,days);

    return cal.getTime();
  }

  public String getClobAsString(ResultSet rs, String colName) throws SQLException, IOException
  {
    Clob clob = rs.getClob(colName);

    if(clob != null)
    {
      char[] buf = new char[(int)clob.length()];
      clob.getCharacterStream().read(buf);

      return new String(buf).trim();
    }
    else
    {
      return null;
    }
  }

} // end class
