package ch.ige.ses.server.dao.bagis;

import java.sql.*;
import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.util.server.*;

/**
 * DataAccessObject für die Tabelle bformtxt_druck
 * <p>
 * Copyright (c) 2002-2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger, Thomas Iten
 * @version $Id: BformtxtDruckDAO.java,v 1.6 2006/12/11 12:43:32 iten Exp $
 */
public class BformtxtDruckDAO  extends BagisDAO
{

  /**
   * Selektiert einen Datensatz aus der Tabelle bformtxt_druck
   *
   * @param   formartCd       Formart Code (PK)
   * @param   textart         Textart (PK)
   * @param   sprachCd        Sprachcode (PK)
   * @throws  SQLException    Unerwarteter SQL Fehler
   * @return  bformtxtDruckVO Value Object oder null
   */
  public BformtxtDruckVO select(String formartCd, String textart, String sprachCd)
          throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    BformtxtDruckVO   vo  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append( getSelectFields() );
      query.append("FROM  bformtxt_druck ");
      query.append("WHERE textart=? ");                 // char(1)
      query.append("AND   trim(formart_cd)=? ");        // char(3)
      query.append("AND   trim(sprach_cd)=? ");         // char(3)

      ps = con.prepareStatement(getQuery());
      ps.setString(1, textart);
      ps.setString(2, formartCd.trim());
      ps.setString(3, sprachCd.trim());

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillSelectFields(rs);
      }

      return vo;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektiert alle Attribute eines Datensatzes aus der Tabelle bformtxt_druck
   * für ein Schreiben mit gegebenem Formatcode und Sprachcode.
   *
   * @param      formartCd     Primary Key
   * @param      sprachCd      Sprachcode
   * @exception  SQLException  Unerwarteter SQL-Fehler
   * @return     ArrayList     Liste mit BformtxtDruckVO's
   */
  public ArrayList select(String formartCd, String sprachCd) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;

    BformtxtDruckVO   vo    = null;
    ArrayList         list  = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append( getSelectFields() );
      query.append("FROM  bformtxt_druck ");
      query.append("WHERE trim(formart_cd)=? ");        // char(3)
      query.append("AND   trim(sprach_cd)=? ");         // char(3)

      ps = con.prepareStatement(getQuery());
      ps.setString(1, formartCd.trim());
      ps.setString(2, sprachCd.trim());

      rs = ps.executeQuery();

      while (rs.next())
      {
        vo = fillSelectFields(rs);
        list.add(vo);
      }

      return list;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert alle Datensätze eines Schreibens aus der Tabelle bformtxt_druck
   * für den gegebenben Fromatcode.$
   *
   * @param     formartCd     Formatcode
   * @exception SQLException  Unerwarteter SQL-Fehler
   * @return    ArrayList     ArrayList mit BformtxtDruckVO's
   */
  public ArrayList select(String formartCd) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;

    BformtxtDruckVO   vo    = null;
    ArrayList         list  = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append( getSelectFields() );
      query.append("FROM  bformtxt_druck ");
      query.append("WHERE trim(formart_cd)=? ");        // char(3)

      ps = con.prepareStatement(getQuery());
      ps.setString(1, formartCd.trim());

      rs = ps.executeQuery();

      while (rs.next())
      {
        vo = fillSelectFields(rs);
        list.add(vo);
      }

      return list;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert alle Textarten, welche für die Patent Jahresgebühren
   * Verarbeitung benötigt werden.
   *
   * @throws SQLException   Unerwarteter SQL-Fehler
   * @return ArrayList      Liste mit BformtxtDruckVO's
   */
  public ArrayList selectForJGB() throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;

    BformtxtDruckVO   vo    = null;
    ArrayList         list  = new ArrayList();

    try
    {
      con = getConnection();

      resetQuery();
      query.append( getSelectFields() );
      query.append("FROM  bformtxt_druck ");
      query.append("WHERE (textart BETWEEN '0' AND '4' OR textart='8') ");  // char(1)
      query.append("AND   formart_cd IN ");                                 // char(3)
      query.append("('115','117', '121', '122', '140', '150', '151', '190', '191',");
      query.append(" '192', '193', '198', '200', '201', '240', '290', '291', '405',");
      query.append(" '425', '428', '430', '448', '450', '451', '460', '495', '620',");
      query.append(" '633', '635', '640', '650', '651', '652', '664', '810') ");

      ps = con.prepareStatement(getQuery());
      rs = ps.executeQuery();

      while (rs.next())
      {
        vo = fillSelectFields(rs);
	list.add(vo);
      }

      return list;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Fügt einen Datensatz in die Tabelle bformtxt_druck ein.
   *
   * @param     vo            BformtxtDruckVO
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public void insert(BformtxtDruckVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("INSERT INTO bformtxt_druck (formart_cd,textart,sprach_cd,text) ");
      query.append("VALUES( ?,?,?,?)");

      ps = con.prepareStatement(getQuery());
      ps.setString (1, vo.formartCd);
      ps.setString (2, vo.textart);
      ps.setString (3, vo.sprachCd);
      ps.setString (4, vo.text);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert einen Datensatz in der Tabelle bformtxt_druck
   *
   * @param     vo            BformtxtVO
   * @exception SQLException  Unerwarteter SQL Fehler
   * @return    int           Anzahl aktualisierter Datensätze
   */
  public int update(BformtxtDruckVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("UPDATE bformtxt_druck SET text=?, version=? ");
      query.append("WHERE textart=? ");                 // char(1)
      query.append("AND   trim(formart_cd)=? ");        // char(3)
      query.append("AND   trim(sprach_cd)=? ");         // char(3)

      ps = con.prepareStatement(getQuery());
      ps.setString(1, vo.text);
      ps.setInt   (2, vo.version);
      ps.setString(3, vo.textart);
      ps.setString(4, vo.formartCd.trim());
      ps.setString(5, vo.sprachCd.trim());

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


 /**
   * Löscht einen Datensatz aus der Tabelle bformtxt_druck
   *
   * @param     formartCd     Primary Key
   * @param     textart       Primary Key
   * @param     sprachCd      Primary Key
   * @exception SQLException  Unerwarteter SQL Fehler
   * @return    int           Anzahl gelöschter Datensätze
   */
  public int delete(String formartCd, String textart, String sprachCd) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = getConnection();

      resetQuery();
      query.append("DELETE FROM bformtxt_druck ");
      query.append("WHERE textart=? ");                 // char(1)
      query.append("AND   trim(formart_cd)=? ");        // char(3)
      query.append("AND   trim(sprach_cd)=? ");         // char(3)

      ps = con.prepareStatement(getQuery());
      ps.setString(1, textart);
      ps.setString(2, formartCd.trim());
      ps.setString(3, sprachCd.trim());

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private String getSelectFields()
  {
    return "SELECT  textart, trim(formart_cd) as formart_cd, trim(sprach_cd) as sprach_cd, text, version ";
  }

  private BformtxtDruckVO fillSelectFields(ResultSet rs) throws SQLException
  {
    BformtxtDruckVO vo  = new BformtxtDruckVO();

    vo.textart    = rs.getString("textart");
    vo.formartCd  = rs.getString("formart_cd");
    vo.sprachCd   = rs.getString("sprach_cd");
    vo.text       = rs.getString("text");
    vo.version    = rs.getInt("version");

    return vo;
  }


} // end class
