package ch.ige.ses.server.dao.bagis.test;

import java.util.*;
import ch.ige.ses.server.dao.bagis.*;
import ch.ige.ses.vo.bagis.*;
import junit.framework.*;

/**
 * JUNIT-Test für JoinSAPRechnungDAO
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Pirmin Suter
 * @version $Id: JoinSAPRechnungDAOTest.java,v 1.22 2007/12/28 09:19:27 iten Exp $
 */

public class JoinSAPRechnungDAOTest extends BagisTestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public JoinSAPRechnungDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(JoinSAPRechnungDAOTest.class);
  }

  protected void setUp()
  {
    // not used
  }

  protected void tearDown()
  {
    // not used
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void testReadRechnungIds()
  {
    JoinSAPRechnungDAO  dao = new JoinSAPRechnungDAO();
    SAPRechnungVO       vo  = null;
    int[]               ids = {5936062}; //{5048705, 5000107, 5213386, 5565443, 5586101, 5832803, 5833232};

    try
    {
      for (int i = 0; i < ids.length; i++)
      {
        System.out.println();
        System.out.println("testReadRechnungIds (bfakt_id = " + ids[i] + "):");

        vo = dao.readRechnung(ids[i]);
        System.out.println("  query = " + dao.getQuery());
        System.out.println(vo.toString());
        assertEquals(ids[i],vo.bfakt_id);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }



  public void testReadRechnungIdsWithCompare()
  {
    JoinSAPRechnungDAO  dao = new JoinSAPRechnungDAO();

    int                 id  = -1;                           // test data
    SAPRechnungVO       vo  = null;


    try
    {
      for (int i = 0; i < JoinSAPRechnungDAOTestHelper.bfaktIds.length; i++)
      {
        id = JoinSAPRechnungDAOTestHelper.bfaktIds[i];
        vo = dao.readRechnung(id);

        System.out.println();
        System.out.println("testReadRechnungIdsWithCompare (bfakt_id = " + id + "):");
        System.out.println("### SOLL ###");
        System.out.println(JoinSAPRechnungDAOTestHelper.bfaktStrings[i]);
        System.out.println("### IST ###");
        System.out.println(JoinSAPRechnungDAOTestHelper.flatten(vo.toString()));
        System.out.println("### END ###");

        assertEquals( JoinSAPRechnungDAOTestHelper.bfaktStrings[i],
                      JoinSAPRechnungDAOTestHelper.flatten(vo.toString()) );
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testReadJgbSingle()
  {
    JoinSAPRechnungDAO  dao     = new JoinSAPRechnungDAO();
    boolean             success = false;

    int                 id        = 4612259;                  // test data
    Map                 origState = null;
    Map                 newState  = null;
    ArrayList           al        = null;
    SAPRechnungVO       vo        = null;

    try
    {
      // set up
      System.out.println("");
      System.out.println("testReadJgbSingle (" + id + "," + id + "):");

      origState = JoinSAPRechnungDAOTestHelper.getJGBState(id);

      newState = new HashMap();
      newState.put("status_druck","3");
      newState.put("status_zahlg","1");
      JoinSAPRechnungDAOTestHelper.setJGBState(id,newState);

      // test
      al = dao.readJGB(id,id);

      for (Iterator iter = al.iterator(); iter.hasNext(); )
      {
        vo = (SAPRechnungVO)iter.next();

        System.out.println();
        System.out.println(vo.toString());
        success = true;
      }

      // tear down
      JoinSAPRechnungDAOTestHelper.setJGBState(id,origState);

      // check
      assertTrue(success);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testReadJgbRangeA()
  {
    List result = null;

    try
    {
      System.out.println();
      System.out.println("testReadJgbRangeA (" + JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeA[0] + "," + JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeA[1] + "):");

      result = readJgb2 (
          JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeA[0],
          JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeA[1] );

      checkJgb2(result, JoinSAPRechnungDAOTestHelper.bfaktJgbStringsRangeA);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testReadJgbRangeB()
  {
    List result = null;

    try
    {
      System.out.println();
      System.out.println("testReadJgbRangeB (" + JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeB[0] + "," + JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeB[1] + "):");

      result = readJgb2 (
          JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeB[0],
          JoinSAPRechnungDAOTestHelper.bfaktJgbIdRangeB[1] );

      checkJgb2(result, JoinSAPRechnungDAOTestHelper.bfaktJgbStringsRangeB);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private List readJgb2 (int idVon, int idBis)
  {
    JoinSAPRechnungDAO  dao = new JoinSAPRechnungDAO();
                                                        // test data
    List                origStateList = null;           // - original states
    Map                 newState      = null;           // - state for jgb test
    ArrayList           result        = null;           // - result list

    try
    {
      // init needed states
      newState = new HashMap();
      newState.put("status_druck","3");
      newState.put("status_zahlg","1");

      // change state of test data
      origStateList = JoinSAPRechnungDAOTestHelper.getJGBStateList(idVon,idBis);
      JoinSAPRechnungDAOTestHelper.setJGBStateList(idVon,idBis,newState);

      // execute test
      result = dao.readJGB(idVon,idBis);

      // restore state
      JoinSAPRechnungDAOTestHelper.setJGBStateList(idVon,origStateList);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }

    // all done
    return result;
  }


  private void checkJgb2 (List result, String[] expectedResult)
  {
    int i = 0;

    for (Iterator iter = result.iterator(); iter.hasNext(); )
    {
      SAPRechnungVO vo = (SAPRechnungVO)iter.next();

      System.out.println("Element #" + i + ":");
      System.out.println(vo.toString());

      assertEquals( expectedResult[i],
                    JoinSAPRechnungDAOTestHelper.flatten(vo.toString()) );
      i++;
    }
  }

} // end class
