package ch.ige.ses.server.dao.bagis.test;

import java.sql.*;
import java.util.*;
import ch.ige.ses.server.dao.bagis.*;
import ch.ige.util.server.*;
import junit.framework.*;
import ch.ige.ses.vo.bagis.StakoVO;

/**
 * JUNIT-Test für JoinStakoArtikel
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: JoinStakoArtikelDAOTest.java,v 1.13 2007/01/04 15:58:08 iten Exp $
 */
public class JoinStakoArtikelDAOTest extends BagisTestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public JoinStakoArtikelDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------


  public static Test suite()
  {
     return new TestSuite(JoinStakoArtikelDAOTest.class);
  }

  protected void setUp()
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void testDummy()
  {
    // dummy for nightly build tests, since the real tests are exceeding
    // the timeout used during the nightly build.
    assertTrue(true);
  }

  public void _testResetUpdateStakoArtikel()
  {
    JoinStakoArtikelDAO dao       = new JoinStakoArtikelDAO();
    ArrayList           origList  = null;
    int                 origSize  = -1;
    int                 resetSize = -1;
    int                 updSize   = -1;

    try
    {
      System.out.println();
      System.out.println("testUpdateStakoArtikel:");

      // set up
      origList = selectArtNrWithStandpreis();
      origSize = origList.size();
      System.out.println("  list size before update = " + origSize);

      // test reset
      dao.resetStakoArtikel();
      resetSize = selectArtNrWithStandpreis().size();
      System.out.println("  list size after reset   = " + resetSize);

      // test update
      dao.updateStakoArtikel(origList);
      updSize = selectArtNrWithStandpreis().size();
      System.out.println("  list size after update  = " + updSize);
      assertEquals(origSize,updSize);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testSelectBfakturBfaktPos()
  {
    JoinStakoArtikelDAO dao     = new JoinStakoArtikelDAO();

    // test data
    java.util.Date      vonA    = dao.addDays(new java.util.Date(), - 15);        // last 15 days
    java.util.Date      vonB    = new GregorianCalendar(2003,1,1).getTime();      // year, month (0=january), day
    java.util.Date      bisB    = new GregorianCalendar(2003,1,28).getTime();     // year, month (0=january), day
    String              exclMa  = "333,568";
    ArrayList           res1    = null;
    ArrayList           res2    = null;
    ArrayList           res3    = null;
    ArrayList           res4    = null;


    try
    {
      System.out.println();
      System.out.println("testSelectBfakturBfaktPos:");

      // CAUTION: bis date !
      res1 = dao.selectBfakturBfaktPos(vonA,null,null);
      System.out.println("  test #1 (" + vonA + ",null,null)");
      System.out.println("  - result size = " + res1.size());
      System.out.println("  - query  = " + dao.getQuery());

      // CAUTION: bis date !
      res2 = dao.selectBfakturBfaktPos(vonA,null,exclMa);
      System.out.println("  test #2 (" + vonA + ",null,[" + exclMa + "]):");
      System.out.println("  - result size = " + res2.size());
      System.out.println("  - query  = " + dao.getQuery());

      res3 = dao.selectBfakturBfaktPos(vonB,bisB,null);
      System.out.println("  test #3 (von,bis,null):");
      System.out.println("  - result size = " + res3.size());
      System.out.println("  - query  = " + dao.getQuery());
      assertTrue(res3.size() > 0);

      res4 = dao.selectBfakturBfaktPos(vonB,bisB,exclMa);
      System.out.println("  test #4 (von,bis,exclMa):");
      System.out.println("  - result size = " + res4.size());
      System.out.println("  - query  = " + dao.getQuery());
      assertTrue(res3.size() > res4.size());
    }
    catch(Exception e)
    {
     e.printStackTrace();
     fail(e.toString());
    }
  }


  public void _testSelectEreignisMA()
  {
    JoinStakoArtikelDAO dao     = new JoinStakoArtikelDAO();

    // test data
    java.util.Date      vonA    = dao.addDays(new java.util.Date(), - 15);        // last 15 days
    java.util.Date      vonB    = new GregorianCalendar(2003,1,1).getTime();      // year, month (0=january), day
    java.util.Date      bisB    = new GregorianCalendar(2003,1,28).getTime();     // year, month (0=january), day
    String              exclMa  = "333,568";
    ArrayList           res1    = null;
    ArrayList           res2    = null;
    ArrayList           res3    = null;
    ArrayList           res4    = null;

    try
    {
      System.out.println();
      System.out.println("testSelectEreignisMA:");

      // CAUTION: bis date !
      res1 = dao.selectEreignisMA(vonA,null,null);
      System.out.println("  test #1 (" + vonA + ",null,null)");
      System.out.println("  - result size = " + res1.size());
      System.out.println("  - query  = " + dao.getQuery());

      // CAUTION: bis date !
      res2 = dao.selectEreignisMA(vonA,null,exclMa);
      System.out.println("  test #2 (" + vonA + ",null,[" + exclMa + "]):");
      System.out.println("  - result size = " + res2.size());
      System.out.println("  - query  = " + dao.getQuery());

      res3 = dao.selectEreignisMA(vonB,bisB,null);
      System.out.println("  test #3 (von,bis,null):");
      System.out.println("  - result size = " + res3.size());
      System.out.println("  - query  = " + dao.getQuery());
      assertTrue(res3.size() > 0);

      res4 = dao.selectEreignisMA(vonB,bisB,exclMa);
      System.out.println("  test #4 (von,bis,exclMa):");
      System.out.println("  - result size = " + res4.size());
      System.out.println("  - query  = " + dao.getQuery());
      assertTrue(res3.size() > res4.size());
    }
    catch(Exception e)
    {
     e.printStackTrace();
     fail(e.toString());
    }
  }

  //---------------------------------------------------------------------------
  // test helpers
  //---------------------------------------------------------------------------

  public void _testPrintArtNrList4Backup()
  {
    try
    {
      System.out.println();
      System.out.println("testPrintArtNrList4Backup:");

      System.out.println("  selectArtNrWithStandpreis:");
      printArtNrList4Backup( selectArtNrWithStandpreis() );
      assertTrue(true);

      System.out.println("  selectArtNrWithStandpreisBackup:");
      printArtNrList4Backup( selectArtNrWithStandpreisBackup() );
      assertTrue(true);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  //---------------------------------------------------------------------------
  // private methods - helper
  //---------------------------------------------------------------------------

  private ArrayList selectArtNrWithStandpreis() throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    ResultSet   rs      = null;
    ArrayList   result  = new ArrayList();

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select art_nr from artikel where standpreis > 0 order by art_nr");

      while (rs.next())
      {
        result.add( rs.getString("art_nr") );
      }

      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }

  private ArrayList selectArtNrWithStandpreisBackup() throws SQLException
  {
    /**
     * Backup vom 08.12.2005 / Bagis Entwicklung (Ingres).
     * Kann alternativ zur Methode selectArtNrWithStandpreis verwendet werden,
     * wenn alle Artikel auf standpreis=0 gesetzt sind.
     */
    String artNrStr =
      "10010,10050,10060,11001,110350,110365,110371,110372,110375,110395," +
      "110397,110400,110401,110410,110411,110412,110413,110425,110441,110443," +
      "110444,110445,110446,110447,110448,110449,110450,110451,110455,110456," +
      "110533,110535,110537,110539,110541,110543,110545,11500,11505,11510," +
      "11520,11525,11550,11560,12002,12025,12026,12028,12032,12033," +
      "12035,12037,12040,12050,12060,12070,12080,12100,12130,12160," +
      "12170,12180,12187,12190,12200,12210,13010,13020,13150,13200," +
      "13201,14041,14050,14071,14073,14081,14090,14110,14115,14140," +
      "14165,14175,14185,14195,14210,14260,14345,14500,14600,14700," +
      "14710,14800,14810,18010,18020,18030,18040,18045,18050,18060," +
      "18070,18080,18090,18095,18100,18110,18120,18125,18130,18140," +
      "18150,18170,18190,18200,18210,18220,18230,18240,18245,18246," +
      "18250,18261,18262,18263,18265,18270,18280,18290,18330,18801," +
      "20020,20040,20060,20080,20100,20120,20170,20171,20198,20200," +
      "20209,20280,20300,20310,20315,20320,20345,20430,20440,20500," +
      "20700,20710,27010,27030,27040,27045,27050,27060,27070,27080," +
      "27140,27250,27260,27300,27310,28010,28020,28040,28045,28050," +
      "28060,28070,28080,28095,28100,28110,28120,28125,28130,28140," +
      "28150,28161,28170,28190,28195,28200,28210,28220,28230,28240," +
      "28245,28246,28250,28261,28262,28263,28265,28270,28280,28290," +
      "28330,3000,3070,3071,40000,40007,40025,40050,40502,40574," +
      "43830,43900,44020,44030,44120,44130,44140,44150,44160,44205," +
      "44206,44210,44220,44230,44240,44250,44260,44270,44280,44510," +
      "44515,44520,44525,44565,44566,44570,45000,45060,45065,45066," +
      "45070,45080,45090,45110,46010,46020,46023,46030,46035,46042," +
      "46044,46510,46520,48000,48001,48002,48003,48010,94000,94030," +
      "94040,94050,94080,94085,94500,94510,94512,94520,94525,94530," +
      "94535,94540,94550,94555,94556,94580,94582,94584,94586,94588," +
      "94950,97100,97110,97140,97150,97160,97200,97210,97220,97230," +
      "97231,97310,97320,99100,99105,99110";

    StringTokenizer st        = new StringTokenizer(artNrStr,",");
    ArrayList       artNrList = new ArrayList();

    while( st.hasMoreTokens() )
    {
      artNrList.add( (String)st.nextToken() );
    }

    return artNrList;
  }

  private void printArtNrList4Backup(ArrayList list)
  {
    boolean   first = true;
    int       cnt   = 0;
    String    nose  = "       \"";
    String    tail  = "\" + ";


    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      String item = ((String)iter.next()).trim();

      if (first)
      {
        first = false;
        System.out.print(nose);
      }
      else
      {
        System.out.print(",");
      }

      if (cnt == 10)
      {
        cnt = 0;
        System.out.println(tail);
        System.out.print(nose);
      }

      System.out.print(item);
      cnt++;

    }

    System.out.println("\";");
  }

} // end class
