package ch.ige.ses.server.dao.ses;

import java.sql.*;
import java.util.*;
import java.util.Date;

import ch.ige.util.server.*;
import ch.ige.ses.vo.bagis.*;
import org.apache.log4j.*;

/**
 * DataAccessObject für Tabelle bfaktjgb_ses
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BfaktJGBSESDAO.java,v 1.7 2006/12/13 14:54:56 iten Exp $
 */

public class BfaktJGBSESDAO
{
	public BfaktJGBSESDAO()
	{
    // lazy bone
	}

	/**
	 * Selektiert einen Datensatz aus der Tabelle bfaktjgb_ses
	 *
	 * @param  bfaktId         BFaktur ID
	 * @param  adrNr           Adresse Nr
	 * @return SAPJGBParamVO   Value Object
	 * @exception SQLException    Unerwarteter SQL Fehler
	 */
	public SAPJGBParamVO select(int bfaktId, int adrNr) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    SAPJGBParamVO     vo  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE bfakt_id = ? AND adr_nr = ? " );

      ps.setInt(1, bfaktId);
      ps.setInt(2, adrNr);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillSAPJGBParamVO(rs);
      }

      return vo;                                // return valid vo or null

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektiert alle Positionen einer Rechnung aus der Tabelle bfaktjgb_ses
   *
   * @param  bfaktId       bfaktur ID
   * @return ArrayList     ArrayList mit Value Object or empty list
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public ArrayList select(int bfaktId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
            ( getSelectStatement() +
			        "WHERE bfakt_id = ?" );

      ps.setInt(1, bfaktId);
      rs = ps.executeQuery();

      while(rs.next())
      {
        al.add( fillSAPJGBParamVO(rs) );
      }

      return al;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektiert Anzahl Einträge pro ID der Tabelle bfaktjgb_ses
   *
   * @param  bfaktId        bfaktur ID
   * @return int            Anzahl gefundene Rechnungsempfänger
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int selectCount(int bfaktId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    int               anz = 0;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement( "SELECT COUNT(*) as anz from bfaktjgb_ses WHERE bfakt_id = ?" );

      ps.setInt(1, bfaktId);
      rs = ps.executeQuery();

      if( rs.next() )
      {
        anz = rs.getInt("anz");
      }

      return anz;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


 /**
   * Fügt einen Datensatz in die Tabelle bfaktjgb_ses ein
   *
   * @param vo  ValueObject      SAPJGBParamVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert(SAPJGBParamVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "INSERT INTO bfaktjgb_ses(" +
                "bfakt_id, " +
                "adr_nr, " +
                "druckperi, " +
                "listenart, " +
                "fach_nr, " +
								"druck_art, " +
                "adr_nr_vtr, " +
                "sort_txt) " +
          "VALUES( ?,?,?,?,?,?,?,?)"
      );

      ps.setInt (1, vo.bfakt_id);
			ps.setInt(2, vo.adr_nr);
      ps.setString (3, vo.druckperi);
      ps.setString (4, vo.listenart);
		  ps.setInt(5, vo.fach_nr);
			ps.setString(6, vo.druck_art);
			ps.setInt(7, vo.adr_nr_vtr);
			ps.setString(8,vo.sort_text);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Aktualisiert einen Datensatz in der Tabelle bfaktjgb_ses
   *
   * @param     vo            SAPJGBParamVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update(SAPJGBParamVO vo) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE bfaktjgb_ses SET " +
                "druckperi = ?, " +
                "listenart = ?, " +
                "fach_nr = ?, " +
								"druck_art = ?, " +
                "adr_nr_vtr = ?, " +
                "sort_txt = ?, " +
								"changed_dat = SYSDATE " +
          "WHERE bfakt_id = ? AND " +
                "adr_nr = ? "
      );

			ps.setString(1, vo.druckperi);
      ps.setString(2, vo.listenart);
      ps.setInt(3, vo.fach_nr);
      ps.setString(4, vo.druck_art);
			ps.setInt(5, vo.adr_nr_vtr);
			ps.setString(6, vo.sort_text);
			ps.setInt(7, vo.bfakt_id);
			ps.setInt(8, vo.adr_nr);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

 /**
   * Löscht einen Datensatz aus der Tabelle bfaktjgb_ses
   *
   * @param     bfaktId       Primary Key
   * @param     adrNr         Primary Key
   * @return    int           Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int bfaktId, int adrNr) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
        ( "DELETE FROM bfaktjgb_ses " +
          "WHERE bfakt_id = ? " +
          "AND   adr_nr = ? ");
      ps.setInt(1, bfaktId);
      ps.setInt(2, adrNr);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
    * delivers sql statement for the select methods WITHOUT the where part
    * note:
    * the field must correspondent with the fields used in the method
    * SAPJGBParamVO !
    *
    * @return select statement
    */
  private String getSelectStatement()
  {
    return
      ( "SELECT bfakt_id, adr_nr, druckperi, listenart, fach_nr, " +
        "druck_art, adr_nr_vtr, sort_txt, created_dat, changed_dat " +
        "FROM bfaktjgb_ses " );

  } // end method


  /**
    * fill SAPJGBParamVO with data from a valid result set
    *
    * @param     rs            valid result set
    * @return    SAPJGBParamVO value object
    * @exception SQLException  unexpected sql error
    */
  private SAPJGBParamVO fillSAPJGBParamVO (ResultSet rs) throws SQLException
  {
    SAPJGBParamVO vo = new SAPJGBParamVO();

    vo.bfakt_id      = rs.getInt("bfakt_id");
    vo.adr_nr        = rs.getInt("adr_nr");
    vo.druckperi     = rs.getString("druckperi");
    vo.listenart     = rs.getString("listenart");
    vo.fach_nr       = rs.getInt("fach_nr");
    vo.druck_art     = rs.getString("druck_art");
    vo.adr_nr_vtr    = rs.getInt("adr_nr_vtr");
    vo.sort_text     = rs.getString("sort_txt");
    vo.created_dat   = rs.getTimestamp("created_dat") == null ? null : new Date(rs.getTimestamp("created_dat").getTime());
    vo.changed_dat   = rs.getTimestamp("changed_dat") == null ? null : new Date(rs.getTimestamp("changed_dat").getTime());

    return vo;

  } // end method



} // end class

