package ch.ige.ses.server.dao.ses;

import oracle.sql.BLOB;
import oracle.jdbc.OracleResultSet;
import java.io.*;
import java.sql.*;
import java.util.ArrayList;

import org.apache.log4j.Logger;

import ch.ige.ses.server.batchctrl.BatchJobBean;
import ch.ige.ses.vo.ses.*;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle ERROR
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: ErrorDAO.java,v 1.8 2006/12/29 12:23:20 iten Exp $
 */

public class ErrorDAO
{
  private static final Logger LOG = Logger.getLogger(ErrorDAO.class);

  public ErrorDAO()
  {
    // lazy bone
  }

  /**
   * Selektierte alle Einträge der Tablle ERROR
   *
   * @return ArrayList  ArrayList mit Value Objects ErrorVO
   * @throws Exception  Unerwarteter SQL-Fehler oder I/O-Fehler(Stream)
   */
  public ArrayList selectAll() throws Exception
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();
    ErrorVO           vo  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT   error_id, source, employee_id, employee_notify, " +
                    "employee_msg_type, description, data_type_cd, data, error_stat, " +
                    "created_dat, changed_dat " +
          "FROM     ses.error " +
          "ORDER BY error_id");

      rs = ps.executeQuery();

      while(rs.next())
      {
        vo = this.fillVO(rs);
        al.add(vo);
      }

    }
    finally
    {
      DBHelper.close(ps, con);
    }

    return al;
  }

  /**
   * Selektiert eine Row der Tablle ERROR via ERROR_ID
   *
   * @param errorId      PK Fehler-Id
   * @return ErrorVO     Value Object
   * @throws Exception   Unerwarteter SQL-Fehler oder I/O-Fehler
   */
  public ErrorVO select(int errorId) throws Exception
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    byte[]            buf = null;
    ErrorVO           vo  = new ErrorVO();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT error_id, source, employee_id, employee_notify, " +
          "employee_msg_type, description, data_type_cd, data, error_stat, " +
          "created_dat, changed_dat " +
          "FROM ses.error " +
          "WHERE error_id = ?");

      ps.setInt(1, errorId);

      rs = ps.executeQuery();
      if( !rs.next())
      {
        return null;
      }

      vo = this.fillVO(rs);

    }
    finally
    {
      DBHelper.close(ps, con);
    }

    return vo;
  }

  /**
   * Selektiert folgende Attribute der Tabelle ERROR
   *  error_id
   *  source
   *  employee_id
   *  employee_notify
   *  data_type_cd
   *  error_stat
   *  created_dat
   *  changed_dat
   * @param errorSelKritVO enthält die Selektionskriterien
   * @return ArrayList von ErrorVO die den Selektionskriteren entsprechen
   * @throws Exception unerwartete Fehlersituation
   */
  public ArrayList select(ErrorSelKritVO errorSelKritVO) throws Exception
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    ArrayList al  = new ArrayList();

    StringBuffer select = new StringBuffer();
    StringBuffer where  = new StringBuffer();
    StringBuffer order  = new StringBuffer();
    String and = "";

    select.append("SELECT error_id, source, employee_id, " +
                  "employee_notify, data_type_cd, error_stat, " +
                  "created_dat, changed_dat " +
                  "FROM ses.error ");
    where.append("WHERE ");
    order.append(" ORDER BY error_id DESC ");

    if (errorSelKritVO.errorId.equals(""))
    {
      // keine ErrorId angegeben
      if (!errorSelKritVO.dataTypeCd.equals(""))
      {
        where.append(and);
        where.append("data_type_cd = " + errorSelKritVO.dataTypeCd);
        and = " AND ";
      }

      if (!errorSelKritVO.errorStat.equals(""))
      {
        where.append(and);
        where.append("error_stat = " + errorSelKritVO.errorStat);
        and = " AND ";
      }

      if (!errorSelKritVO.errorIdVon.equals("") && !errorSelKritVO.errorIdBis.equals(""))
      {
        where.append(and);
        where.append("error_id BETWEEN " + errorSelKritVO.errorIdVon + " AND " +  errorSelKritVO.errorIdBis);
        and = " AND ";
      }
    }
    else
    {
      // ErrorId wurde eingegeben
      // nur nach ErrorId suchen
      where.append("error_id = " + errorSelKritVO.errorId);
      and = " AND ";
    }

    String sql =  select.toString() +
                  ( where.toString().equals("WHERE ") ? "" : where.toString() ) +
                  order.toString();

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      stmt = con.createStatement();
      rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      {
        ErrorVO errorVO = new ErrorVO();

        errorVO.errorId          = rs.getInt("error_id");
        errorVO.source           = rs.getString("source");
        errorVO.employeeId       = rs.getString("employee_id");
        errorVO.employeeNotify   = rs.getInt("employee_notify");
        errorVO.dataTypeCd       = rs.getInt("data_type_cd");
        errorVO.errorStat        = rs.getInt("error_stat");
        errorVO.createdDat       = new Date(rs.getTimestamp("created_dat").getTime());
        errorVO.changedDat       = new Date(rs.getTimestamp("changed_dat").getTime());

        al.add(errorVO);
      }

      return al;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }


  /**
   * Fügt einen Datensatz in die Tabelle ERROR ein
   *
   * @param     errorVO    ValueObject ErrorVO
   * @exception Exception  Unerwarteter SQL Fehler oder Konvertierungsfehler
   */
  public void insert(ErrorVO errorVO) throws Exception
  {
    Connection           con  = null;
    PreparedStatement    ps0  = null;
    PreparedStatement    ps1  = null;
    PreparedStatement    ps2  = null;
    PreparedStatement    ps3  = null;
    ResultSet            rs   = null;
    BLOB                 blob = null;

    try
    {
      //
      // init
      // 
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("insert - start init...");
      }

      limitMaxFieldSize(errorVO);         // limit max field size
      con  = DBHelper.getConnection(DBHelper.SES);

      
      //
      // ps0 - insert values and preparing blob-column with empty-blob
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("insert - start ps0 - insert values and preparing blob-column with empty-blob...");
      }
      
      ps0 = con.prepareStatement(
          "INSERT INTO ses.error(" +
          "error_id, " +
          "source, " +
          "employee_id, " +
          "employee_notify, " +
          "employee_msg_type, " +
          "description, " +
          "data_type_cd, " +
          "data , " +
          "error_stat) " +
          "VALUES(SES.SEQ_ERROR_ID.nextval,?,?,?,?,?,?, EMPTY_BLOB(),?)"
          );

      ps0.setString(1, errorVO.source);
      ps0.setString(2, errorVO.employeeId);
      ps0.setInt   (3, errorVO.employeeNotify);
      ps0.setInt   (4, errorVO.employeeMsgType);
      ps0.setString(5, errorVO.description);
      ps0.setInt   (6, errorVO.dataTypeCd);
      ps0.setInt   (7, errorVO.errorStat);

      if( ps0.executeUpdate() != 1) 
      {
        throw new SQLException();
      }

      
      //
      // ps1 - get the current created error_id
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("insert - start ps1 - get the current created error_id...");
      }
      
      ps1 = con.prepareStatement("SELECT max(error_id) as error_id FROM error");
      rs = ps1.executeQuery();
      rs.next();
      errorVO.errorId = rs.getInt("error_id");


      //
      // ps2 - select blob column for update (an OracleResultSet will be used)
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("insert - start ps2 - select blob column for update (an OracleResultSet will be used)...");
      }
      
      ps2 = con.prepareStatement("SELECT error_id, data FROM error WHERE error_id = ? FOR UPDATE");
      ps2.setInt( 1, errorVO.errorId );

      rs = ps2.executeQuery();
      if( ! rs.next() )
      {
        // throw an exception if the blob reference was not retrieved.
        throw new SQLException( "Unable to retrieve BLOB reference.");
      }
      blob = (oracle.sql.BLOB)rs.getBlob("data");


      //
      // ps3 - update blob
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("insert - start ps3 - update blob...");
      }
            
      ps3 = con.prepareStatement("UPDATE error set data = ? WHERE error_id = ? ");
      ps3.setInt(2,errorVO.errorId);
      if(errorVO.data == null)
      {
        ps3.setNull(1, java.sql.Types.BLOB);
      }
      else
      {
        // Oracle 10g Treiber - putBytes mit setBytes ersetzt
        blob.setBytes(1, convDataToBlob(errorVO.data));
        ps3.setBlob(1, blob);
      }
      ps3.executeUpdate();

    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(ps3);
      DBHelper.close(ps2);
      DBHelper.close(ps1);
      DBHelper.close(ps0, con);
    }
  }


  /**
   * Aktualisiert Attribut DATA(Blob) der Tabelle ERROR
   *
   * @param errorVO      ValueObject mit Fehlerdaten
   * @return Anzahl aktualisierter Datensätze
   * @throws Exception   Unerwarteter SQL Fehler oder Konvertierungsfehler
   */
  public int update(ErrorVO errorVO) throws Exception
  {
    Connection           con       = null;
    PreparedStatement    ps        = null;
    ResultSet            rs        = null;
    BLOB                 blob      = null;

    try
    {
      //
      // init
      // 
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("update - start init...");
      }

      limitMaxFieldSize(errorVO);
      con  = DBHelper.getConnection(DBHelper.SES);


      //
      // update
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("update - start update...");
      }
            
      ps = con.prepareStatement(
          "UPDATE ses.error SET " +
          "SOURCE = ?, " +
          "EMPLOYEE_ID = ?, " +
          "EMPLOYEE_NOTIFY = ?, " +
          "EMPLOYEE_MSG_TYPE  = ?, " +
          "DESCRIPTION = ?, " +
          "DATA_TYPE_CD = ?, " +
          "DATA = EMPTY_BLOB(), " +
          "ERROR_STAT = ?, " +
          "CHANGED_DAT = SYSDATE " +
          "WHERE error_id = ? "
          );

      ps.setString(1, errorVO.source);
      ps.setString(2, errorVO.employeeId);
      ps.setInt   (3, errorVO.employeeNotify);
      ps.setInt   (4, errorVO.employeeMsgType);
      ps.setString(5, errorVO.description);
      ps.setInt   (6, errorVO.dataTypeCd);
      ps.setInt   (7, errorVO.errorStat);
      ps.setInt   (8, errorVO.errorId);

      if(ps.executeUpdate() != 1) throw new SQLException();


      //
      // select the blob column for update.
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("update - start select the blob column for update...");
      }
      
      ps = con.prepareStatement("SELECT error_id, data FROM error WHERE error_id = ? FOR UPDATE");
      ps.setInt( 1, errorVO.errorId );

      rs = ps.executeQuery();
      if( ! rs.next() )
      {
        // throw an exception if the blob reference was not retrieved.
        throw new SQLException( "Unable to retrieve BLOB reference.");
      }
      blob = (oracle.sql.BLOB)rs.getBlob("data");


      //
      // update blob
      //
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug("update - start update blob...");
      }
      
      ps = con.prepareStatement("UPDATE error set data = ? WHERE error_id = ? ");
      ps.setInt(2,errorVO.errorId);
      if(errorVO.data == null)
      {
        ps.setNull(1, java.sql.Types.BLOB);
      }
      else
      {
        // Oracle 10g Treiber - putBytes mit setBytes ersetzt
        blob.setBytes(1, convDataToBlob(errorVO.data));
        ps.setBlob(1, blob);
      }
      ps.executeUpdate();

      //
      // return
      //
      return 1;
      
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }



  /**
   * Löscht einen Datensatz aus der Tabelle ERROR
   *
   * @param     errorId    PK FehlerId
   * @return Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(int errorId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "DELETE FROM ses.error WHERE error_id = ? "
          );
      ps.setInt(1, errorId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Füllt einen Datensatz anhand des ResultSet<br>
   * Ist das data-Attribut mit einem empty_lob gefüllt, wird NULL eingesetzt
   *
   * @param     rs         Gültiges resultSet
   * @return    ErrorVO    Value Object
   * @exception Exception  Unerwarteter SQL Fehler oder Konvertierungsfehler
   */
  private ErrorVO fillVO(ResultSet rs) throws Exception
  {
    ErrorVO errorVO = new ErrorVO();

    errorVO.errorId          = rs.getInt("error_id");
    errorVO.source           = rs.getString("source");
    errorVO.employeeId       = rs.getString("employee_id");
    errorVO.employeeNotify   = rs.getInt("employee_notify");
    errorVO.employeeMsgType  = rs.getInt("EMPLOYEE_MSG_TYPE");
    errorVO.description      = rs.getString("description");
    errorVO.dataTypeCd       = rs.getInt("data_type_cd");


    Object obj = rs.getObject("data");
    if (obj == null)
    {
      errorVO.data = null;
    }
    else
    {
      ObjectInputStream ois = new ObjectInputStream(rs.getBinaryStream("data"));
      errorVO.data = ois.readObject();
      ois.close();
    }

    errorVO.errorStat        = rs.getInt("error_stat");
    errorVO.createdDat       = new Date(rs.getTimestamp("created_dat").getTime());
    errorVO.changedDat       = new Date(rs.getTimestamp("changed_dat").getTime());

    return errorVO;
  }

  /**
   * serialize object into byte array, to store as blob type
   *
   * @param     obj       object to serialize
   * @return    byte stream to store as blob
   * @exception Exception conversion exceptions
   */
  private byte[] convDataToBlob(Object obj) throws Exception
  {
    byte[]                ba   = null;
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    ObjectOutput          oo   = new ObjectOutputStream(baos);

    if (obj != null)
    {
      oo.writeObject(obj);
      ba = baos.toByteArray();
    }
    return ba;
  }

  /**
   * limit maximal field size because of SAP restrictions
   * @param errorVO Value Object
   */
  private void limitMaxFieldSize(ErrorVO errorVO)
  {
    if (errorVO.source != null)
    {
      if (errorVO.source.length() > 40)
        errorVO.source = errorVO.source.substring(0,37) + "...";
    }

    if (errorVO.employeeId != null)
    {
      if (errorVO.employeeId.length() > 40)
        errorVO.employeeId = errorVO.employeeId.substring(0,37) + "...";
    }

    if (errorVO.description != null)
    {
      if (errorVO.description.length() > 1000)
        errorVO.description = errorVO.description.substring(0,997) + "...";
    }
  }

} // end class


