package ch.ige.ses.server.dao.ses;

import java.util.*;
import java.sql.*;

import ch.ige.util.server.DBHelper;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.*;



/**
 * Datenbankzugriffe auf SES-Datenbanken für die Beschaffung der Rechnungsdaten
 * <p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rüegsegger
 * @version $Id: JoinSESRechnungDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */

public class JoinSESRechnungDAO
{

  public JoinSESRechnungDAO()
  {
    // lazy bone
  }

  /**
   * Selektiert Jahresgebühren-Rechnungen die den Selektionskriterien entsprechen.
   * Die Rechnungen werden in der Sortierreihenfolge der Jahresgebühren zurückgegeben
   * Selektiert folgende Attribute der Tabelle BFAKTUR_SES
   *  lauf_nr
   *  schutit_typ
   * Selektiert folgende Attribute der Tabelle BFAKTJGB_SES
   *  bfakt_id
   *  druckperi
   *  fach_nr
   *  adr_nr
   *  druck_art
   *  adr_nr_vtr
   *  status_druck
   *  sort_txt
   *  created_dat
   *  changed_dat
   * @param sapRechnungVO Enthält die Selektionskriterien
   * @return ArrayList von SAPRechnungVO die den Selektionskriteren entsprechen
   * @throws SQLException unerwartete Fehlersituation
   */
  public ArrayList selectJGBSort(SAPRechnungVO sapRechnungVO) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    ArrayList al  = new ArrayList();
    SAPJGBParamVO sapJGBParamVO = new SAPJGBParamVO();

    if (sapRechnungVO.zusatzRechEmpf.size() > 0)
    {
      sapJGBParamVO = (SAPJGBParamVO)sapRechnungVO.zusatzRechEmpf.get(0);
    }

    StringBuffer select = new StringBuffer();
    StringBuffer where  = new StringBuffer();
    StringBuffer order  = new StringBuffer();
    String and = " AND ";

    select.append("SELECT a.lauf_nr, " +
                    "a.jgb_lauf_id, " +
                    "b.schutit_typ, " +
                    "c.bfakt_id, c.druckperi, c.fach_nr, c.adr_nr, c.druck_art, " +
                    "c.adr_nr_vtr, c.sort_txt, c.changed_dat, c.created_dat " +
                 "FROM ses.jgb_lauf a, ses.bfaktur_ses b, ses.bfaktjgb_ses c ");
    where.append("WHERE a.jgb_lauf_id = b.jgb_lauf_id AND b.bfakt_id = c.bfakt_id");
    order.append(" ORDER BY a.lauf_nr, c.druckperi, c.fach_nr, " +
                    "c.adr_nr, c.druck_art, b.schutit_typ, c.adr_nr_vtr, c.sort_txt");

    if (sapRechnungVO.jgbLaufNr != null && !sapRechnungVO.jgbLaufNr.equals(""))
    {
      where.append(and);
      where.append("a.lauf_nr = " + sapRechnungVO.jgbLaufNr);
      and = " AND ";
    }

    if (sapRechnungVO.schutit_typ != 0)
    {
      where.append(and);
      where.append("b.schutit_typ = '" + sapRechnungVO.schutit_typ + "'");
      and = " AND ";
    }

    if (sapJGBParamVO.bfakt_id != 0)
    {
      where.append(and);
      where.append("c.bfakt_id = " + sapJGBParamVO.bfakt_id);
      and = " AND ";
    }

    if (sapJGBParamVO.druckperi != null && !sapJGBParamVO.druckperi.equals(""))
    {
      where.append(and);
      where.append("c.druckperi = " + sapJGBParamVO.druckperi);
      and = " AND ";
    }

    if (sapJGBParamVO.fach_nr != 0)
    {
      where.append(and);
      where.append("c.fach_nr = " + sapJGBParamVO.fach_nr);
      and = " AND ";
    }

    if (sapJGBParamVO.adr_nr != 0)
    {
      where.append(and);
      where.append("c.adr_nr = " + sapJGBParamVO.adr_nr);
      and = " AND ";
    }

    if (sapJGBParamVO.druck_art != null && !sapJGBParamVO.druck_art.equals(""))
    {
      where.append(and);
      where.append("c.druck_art = " + sapJGBParamVO.druck_art);
      and = " AND ";
    }

    if (sapJGBParamVO.adr_nr_vtr != 0)
    {
      where.append(and);
      where.append("c.adr_nr_vtr  = " + sapJGBParamVO.adr_nr_vtr);
      and = " AND ";
    }

    if (sapJGBParamVO.sort_text != null && !sapJGBParamVO.sort_text.equals(""))
    {
      where.append(and);
      where.append("c.sort_txt = '" + sapJGBParamVO.sort_text + "'");
      and = " AND ";
    }

    String sql =  select.toString() + where.toString() + order.toString();

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());

      while (rs.next())
      {
        sapRechnungVO = new SAPRechnungVO();
        sapRechnungVO.zusatzRechEmpf = new ArrayList();
        sapJGBParamVO = new SAPJGBParamVO();

        sapRechnungVO.jgbLaufNr = rs.getString("lauf_nr");
        sapRechnungVO.jgbLaufId = rs.getInt("jgb_lauf_id");
        sapRechnungVO.schutit_typ = rs.getInt("schutit_typ");
        sapJGBParamVO.bfakt_id = rs.getInt("bfakt_id");
        sapJGBParamVO.adr_nr = rs.getInt("adr_nr");
        sapJGBParamVO.druckperi = rs.getString("druckperi");
        sapJGBParamVO.fach_nr = rs.getInt("fach_nr");
        sapJGBParamVO.druck_art = rs.getString("druck_art");
        sapJGBParamVO.adr_nr_vtr = rs.getInt("adr_nr_vtr");
        sapJGBParamVO.sort_text = rs.getString("sort_txt");
        sapJGBParamVO.created_dat = rs.getTimestamp("created_dat");
        sapJGBParamVO.changed_dat = rs.getTimestamp("changed_dat");

        sapRechnungVO.zusatzRechEmpf.add(sapJGBParamVO);
        al.add(sapRechnungVO);
      }

      return al;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }


  /**
   * select list of SAPRechnungVO for a given bfaktId range, a given jgbLaufId
   * and a choosen state or all states, from the following tables:
   *   - bfaktur_ses
   *   - bfaktjgb_ses
   *   - bfaktpos_ses.
   *   - jgb_lauf
   *
   *  note:
   *  - if the param statusVerarbeitung is set to NULL then all vo in the
   *    choosen range are selected otherwise only the vo for the desired state.
   *
   * @param  bfaktIdFrom          start bfakt id range
   * @param  bfaktIdTo            end bfakt id range
   * @param  jgbLaufId            lauf id
   * @param  statusVerarbeitung   state (see SAPRechnungVO.STATUS_VERARBEITUNG_*) or null
   * @return ArrayList of SAPRechnungVO or empty list if nothing is found
   * @exception Exception  unexpected error
   */
  public ArrayList selectJGBRechnung
    (int bfaktIdFrom, int bfaktIdTo, int jgbLaufId, String statusVerarbeitung ) throws Exception
  {
    BfakturSESDAO   bfakturSESDAO   = null;       // data access objects
    BfaktposSESDAO  bfaktposSESDAO  = null;
    BfaktJGBSESDAO  bfaktJGBSESDAO  = null;
    JGBLaufDAO      jgbLaufDAO      = null;

    ArrayList       reList          = null;       // list of SAPRechnungVO's
    ArrayList       jgbList         = null;       // list of SAPJGBParamVO's
    SAPRechnungVO   reVO            = null;
    SAPJGBParamVO   jgbVO           = null;

    String          jgbLaufNr       = null;       // lauf number (same for all reVO's)
    String          hint            = null;       // hint in case of an exception


    try
    {
      // init
      hint = "step 1: init";
      bfakturSESDAO  = new BfakturSESDAO();
      bfaktposSESDAO = new BfaktposSESDAO();
      bfaktJGBSESDAO = new BfaktJGBSESDAO();
      jgbLaufDAO     = new JGBLaufDAO();

      reList  = new ArrayList();
      jgbList = new ArrayList();

      // get lauf number
      hint = "step 2: call jgbLaufDAO.select";
      jgbLaufNr = jgbLaufDAO.select(jgbLaufId).laufNr;

      // get bfaktur data
      hint = "step 3: call bfakturSESDAO.select";
      reList = bfakturSESDAO.select (bfaktIdFrom,bfaktIdTo,jgbLaufId,statusVerarbeitung);

      //
      // get position & jgb data
      //
      for (Iterator i = reList.iterator(); i.hasNext(); )
      {
        // cast vo data and set lauf number
        hint = "step 4: get SAPRechnungVO";
        reVO = (SAPRechnungVO) i.next();
        reVO.jgbLaufNr = jgbLaufNr;

        // get posList
        hint = "step 5: call bfaktposSESDAO.select with bfakt_id: " + reVO.bfakt_id;
        reVO.posList = bfaktposSESDAO.select(reVO.bfakt_id);

        // get jgbHauptRechEmpfParam, jgbZusatzRechEmpfParam
        // and zusatzRechEmpf
        hint = "step 6: call bfaktJGBSESDAO.select with bfakt_id: " + reVO.bfakt_id;
        jgbList = bfaktJGBSESDAO.select(reVO.bfakt_id);

        for (Iterator j = jgbList.iterator(); j.hasNext(); )
        {
          jgbVO = (SAPJGBParamVO) j.next();

          if ( jgbVO.adr_nr == reVO.hauptRechEmpf )
          {
            reVO.jgbHauptRechEmpfParam = jgbVO;
          }
          else
          {
            reVO.jgbZusatzRechEmpfParam.add(jgbVO);
            reVO.zusatzRechEmpf.add(new Integer(jgbVO.adr_nr));
          }
        } // end for

      } // end for

      //
      // return rechnung list or empty list if nothing is found
      //
      return reList;

    }
    catch (Exception ex)
    {
      throw new Exception ( "selectJGBRechnungList exception(" + ex.toString() +
                            ") occured, hint: " + hint );
    }

  } // end method


  /**
   * Gibt die Anzahl Rechnungsempfänger eines JGB-Laufes zurück
   * @param jgbLaufId Lauf-ID
   * @return Anzahl Rechnungsempfänger eines JGB-Laufes
   * @throws SQLException
   * @author Patrick Bolzli
   */
  public int countEmpfaenger(int jgbLaufId) throws SQLException
  {
    int               anz   = 0;
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    String            sql   = "Select count(*) As Anzahl " +
                              "From ses.bfaktjgb_ses " +
                              "Where bfakt_id in ( " +
                              "Select bfakt_id from ses.bfaktur_ses " +
                              "Where jgb_lauf_id = ?)";

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(sql);
      ps.setInt(1, jgbLaufId);
      rs = ps.executeQuery();

      if( rs.next() )
      {
        anz = rs.getInt("Anzahl");
      }
      return anz;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method

  /**
   * Gibt die Anzahl Rechnungsempfänger eines JGB-Laufes zurück
   * @param jgbLaufId Lauf-ID
   * @return Anzahl Rechnungsempfänger eines JGB-Laufes
   * @throws SQLException
   * @author Patrick Bolzli
   */
  public int getAnzPerioden(int jgbLaufId) throws SQLException
  {
    int               anz   = 0;
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    String            sql   = "Select count(*) As Anzahl " +
                              "From ses.bfaktjgb_ses " +
                              "Where bfakt_id in ( " +
                              "Select bfakt_id from ses.bfaktur_ses " +
                              "Where jgb_lauf_id = ? )" +
                              "Group by druckperi";
    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(sql);
      ps.setInt(1, jgbLaufId);
      rs = ps.executeQuery();

      int i;
      for (i = 0; rs.next(); i++)
      {
        anz++;
      }
      return anz;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method

  /**
   * Gibt die Anzahl Rechnungsempfänger eines JGB-Laufes zurück
   * @param  jgbLaufId  Lauf-ID
   * @param  status     SES-Verarbeitungsstatus
   * @return Anzahl     Rechnungsempfänger eines JGB-Laufes
   * @throws SQLException
   * @author Patrick Bolzli
   */
  public int countByStatus(int jgbLaufId, String status) throws SQLException
  {
    int               anz   = 0;
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;
    String            sql   = "Select count(*) As Anzahl " +
                              "From ses.bfaktjgb_ses " +
                              "Where bfakt_id in ( " +
                              "Select bfakt_id from ses.bfaktur_ses " +
                              "Where jgb_lauf_id = ? " +
                              "And status_verarbeitung = ? )";
    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(sql);
      ps.setInt(1, jgbLaufId);
      ps.setString(2, status);
      rs = ps.executeQuery();

      if (rs.next())
      {
        anz = rs.getInt("Anzahl");
      }
      return anz;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  } // end method


  /**
   * check end conditions to finish jgb run on sap system.
   * the end condition is reached, if the following check's are true
   *  a) sap_stat from table jgb_lauf is SAP_STAT_STARTED_LAST
   *  b) all entries in table bfaktur_ses (for the given jgbLaufId) have
   *     status_verarbeitung == STATUS_VERARBEITUNG_SAP
   *
   * @param jgbLaufId     id from jgb run
   * @return              true if conditions are reached
   * @throws SQLException unexpectetd error
   */
  public boolean checkJgbEndCondition (int jgbLaufId) throws SQLException
  {
    JGBLaufVO         jgbLaufVO = null;     // data from table jgb_lauf

    Connection        con       = null;     // data from table bfaktur_ses
    PreparedStatement ps        = null;
    ResultSet         rs        = null;

    boolean           ret       = false;    // true if end condition is reached

    try
    {
      //
      // check if entry in table JGB_LAUF has
      // SAP_STAT == SAP_STAT_STARTED_LAST
      //
      jgbLaufVO = new JGBLaufDAO().select(jgbLaufId);

      if (jgbLaufVO == null)
      {
        return ret;
      }
      else
      {
        if ( jgbLaufVO.sapStat != jgbLaufVO.SAP_STAT_STARTED_LAST)
        {
          return ret;
        }
      }

      //
      // check if all entries of BFAKTUR_SES have
      // STATUS_VERARBEITUNG == STATUS_VERARBEITUNG_SAP
      //
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
             ( "SELECT count(bfakt_id) as cnt " +
               "FROM   bfaktur_ses " +
               "WHERE  jgb_lauf_id = ? " +
               "AND    status_verarbeitung <> ? ");

      ps.setInt   (1,jgbLaufId);
      ps.setString(2,SAPRechnungVO.STATUS_VERARBEITUNG_SAP);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        if ( rs.getInt("cnt") == 0 )
          ret = true;
      }

      return ret;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * check stop condition of jgb run
   * the stop condition is reached, if the following check's are true
   *  a) sap_stat from table jgb_lauf is SAP_STAT_STARTED_LAST
   *  b) no more entries in table bfaktur_ses with
   *     status_verarbeitung == STATUS_VERARBEITUNG_NEW
   *  c) some entries in table bfaktur_ses with
   *     status_verarbeitung == STATUS_VERARBEITUNG_ERROR
   *
   * @param jgbLaufId     id from jgb run
   * @return              true if conditions are reached
   * @throws SQLException unexpectetd error
   */
  public boolean checkJgbStopCondition (int jgbLaufId) throws SQLException
  {
    JGBLaufVO         jgbLaufVO = null;     // data from table jgb_lauf

    Connection        con       = null;     // data from table bfaktur_ses
    PreparedStatement ps        = null;
    ResultSet         rs        = null;

    //
    // check if entry in table JGB_LAUF has
    // SAP_STAT == SAP_STAT_STARTED_LAST
    //
    jgbLaufVO = new JGBLaufDAO().select(jgbLaufId);

    if (jgbLaufVO == null)
    {
      return false;
    }
    else
    {
      if ( jgbLaufVO.sapStat != jgbLaufVO.SAP_STAT_STARTED_LAST)
      {
        return false;
      }
    }


    //
    // check if no more entries in table bfaktur_ses with
    // status_verarbeitung == STATUS_VERARBEITUNG_NEW
    //
    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
             ( "SELECT count(bfakt_id) as cnt " +
               "FROM   bfaktur_ses " +
               "WHERE  jgb_lauf_id = ? " +
               "AND    status_verarbeitung = ? ");

      ps.setInt   (1,jgbLaufId);
      ps.setString(2,SAPRechnungVO.STATUS_VERARBEITUNG_NEW);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        if ( rs.getInt("cnt") > 0 )
          return false;
      }
      else
      {
        return false;
      }

    }
    finally
    {
      DBHelper.close(ps, con);
    }


    //
    // check for some entries in table bfaktur_ses with
    // status_verarbeitung == STATUS_VERARBEITUNG_ERROR
    //
    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
             ( "SELECT count(bfakt_id) as cnt " +
               "FROM   bfaktur_ses " +
               "WHERE  jgb_lauf_id = ? " +
               "AND    status_verarbeitung = ? ");

      ps.setInt   (1,jgbLaufId);
      ps.setString(2,SAPRechnungVO.STATUS_VERARBEITUNG_ERROR);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        if ( rs.getInt("cnt") == 0 )
          return false;
      }
      else
      {
        return false;
      }

    }
    finally
    {
      DBHelper.close(ps, con);
    }

    //
    // stop condition reached
    //
    return true;

  } // end method


  /**
   * Liste mit allen Rechnungsempfängern, welche bereits erfolgreich nach
   * SAP übertragen wurden, d.h. den Status "verarbeitet" oder "gedruckt"
   * aufweisen.
   *
   * @param   jgbLaufId       Lauf Id
   * @throws  SQLException    Unerwarteter Fehler
   * @return  ArrayList       Liste mit Rechnungsempfängern
   */
  public ArrayList getStatistikCalcReVOList (int jgbLaufId) throws SQLException
  {
    Connection        con   = null;
    PreparedStatement ps    = null;
    ResultSet         rs    = null;

    StatistikCalcReVO vo    = null;              // Rechnungsempfänger Daten
    ArrayList         list  = new ArrayList();   // Liste mit Rechnungsempfängern

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
             ( "SELECT  j.bfakt_id,j.adr_nr,j.listenart " +
               "FROM    bfaktur_ses b, bfaktjgb_ses j   " +
               "WHERE   b.jgb_lauf_id =?                " +
               "AND     (b.status_verarbeitung =? or b.status_verarbeitung=?) " +
               "AND     b.bfakt_id = j.bfakt_id " +
               "ORDER BY j.bfakt_id ");

      ps.setInt   (1,jgbLaufId);
      ps.setString(2,SAPRechnungVO.STATUS_VERARBEITUNG_SAP);
      ps.setString(3,SAPRechnungVO.STATUS_VERARBEITUNG_PRINTED);

      rs = ps.executeQuery();

      while ( rs.next() )
      {
        vo = new StatistikCalcReVO();

        vo.bfaktId    = rs.getInt("bfakt_id");
        vo.adrNr      = rs.getInt("adr_nr");
        vo.listenArt  = rs.getString("listenart");

        list.add(vo);
      }

      return list;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

} // end class


