package ch.ige.ses.server.dao.ses;

import java.sql.*;
import java.util.ArrayList;

import ch.ige.ses.vo.ses.PropertyVO;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle PROPERTY
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: PropertyDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */

public class PropertyDAO
{

  public PropertyDAO()
  {
    // lazy bone
  }

  /**
   * Selektierte alle Einträge der Tablle PROPERTY
   *
   * @return ArrayList      ArrayList mit Value Objects PropertyVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList selectAll() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    ArrayList al = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT key, value, description " +
          "FROM ses.property ORDER BY key");

      rs = ps.executeQuery();

      while(rs.next())
      {
        PropertyVO  propertyVO = new PropertyVO();

        propertyVO.key = rs.getString("key");
        propertyVO.value = rs.getString("value");
        propertyVO.description = rs.getString("description");

        al.add(propertyVO);
      }
      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektierte eine Row der Tablle PROPERTY via KEY
   *
   * @param key             PK
   * @return JGBLaufVO      Value Object PropertyVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public PropertyVO select(String key) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement(
          "SELECT key, value, description " +
          "FROM ses.property " +
          "WHERE key = ?");

      ps.setString(1, key);

      rs = ps.executeQuery();
      if( !rs.next())
      {
        return null;
      }

      PropertyVO propertyVO = new PropertyVO();

      propertyVO.key = rs.getString("key");
      propertyVO.value = rs.getString("value");
      propertyVO.description = rs.getString("description");

      return propertyVO;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle PROPERTY ein
   *
   * @param     propertyVO     ValueObject PropertyVO
   * @exception SQLException   Unerwarteter SQL Fehler
   */
  public void insert(PropertyVO propertyVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "INSERT INTO ses.property(" +
            "key, " +
            "value, " +
            "description) " +
          "VALUES(?,?,?)"
          );

      ps.setString(1, propertyVO.key);
      ps.setString(2, propertyVO.value);
      ps.setString(3, propertyVO.description);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut VALUE der Tabelle PROPERTY
   *
   * @param propertyVO      Value Object
   * @return Anzahl         aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int update(PropertyVO propertyVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.property SET " +
            "value = ?, " +
            "description = ? " +
          "WHERE key = ? "
          );

      ps.setString(1, propertyVO.value);
      ps.setString(2, propertyVO.description);
      ps.setString(3, propertyVO.key);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle PROPERTY
   *
   * @param     key           PK
   * @return    int           Anzahl gelöschter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int delete(String key) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
      ( "DELETE FROM ses.property WHERE key = ? ");
      ps.setString(1, key);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }
}


