package ch.ige.ses.server.dao.ses.test;

import java.sql.*;
import java.util.*;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.vo.ses.*;
import junit.framework.*;

/**
* Testklasse für die Klasse AmvRechnungDAO
* <p>
* Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
* @author T. Iten
* @version $Id: AmvRechnungDAOTest.java,v 1.8 2007/11/13 12:35:02 nydegger Exp $
*/

public class AmvRechnungDAOTest extends SesTestCase
{
  //
  // const
  //
  private static final  String  LAUF_NR          = "2999062100";

  private static final  int     ERINNERUNG_1_MIN = 1;
  private static final  int     ERINNERUNG_1_MAX = 10;
  private static final  int     ERINNERUNG_2_MIN = 11;
  private static final  int     ERINNERUNG_2_MAX = 15;

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public AmvRechnungDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(AmvRechnungDAOTest.class);
  }

  /**
   * Setup Routine
   * Achtung wird vor JEDER einzelnen Testfunktion aufgerufen !
   */
  protected void setUp()
  {
    // lazy bone
  }

  /**
   * Teardown Routine
   * Achtung wird nach JEDER einzelnen Testfunktion aufgerufen !
   */
  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void test01CreateLauf()
  {
    try
    {
      System.out.println("CreateLauf...");

      AmvLaufDAO  laufDAO = new AmvLaufDAO();
      AmvLaufVO   laufVO  = new AmvLaufVO();

      System.out.println("  create lauf nr " + LAUF_NR);
      laufVO.laufNr   = LAUF_NR;
      laufVO.laufStat = AmvLaufVO.LAUF_STAT_STARTED;
      laufDAO.insert(laufVO);

      laufVO = laufDAO.select(LAUF_NR);
      System.out.println("  amv lauf id " + laufVO.amvLaufId + " generated");

      System.out.println("CreateLauf finished");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test02SelectEmpty()
  {
    try
    {
      ArrayList list = new AmvRechnungDAO().select(getAmvLaufId());
      assertTrue(list.isEmpty());
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test03Insert()
  {
    try
    {
      AmvRechnungDAO  dao = new AmvRechnungDAO();
      AmvRechnungVO   vo  = null;
      int             id  = getAmvLaufId();


      for (int i=ERINNERUNG_1_MIN; i<=ERINNERUNG_1_MAX; i++)
      {
        vo = new AmvRechnungVO();

        vo.amvLaufId    = id;
        vo.rechnungCd   = AmvRechnungVO.RE_CD_ERINNERUNG_1;
        vo.bfaktId      = i;
        vo.anzRechEmpf  = 1;
        vo.rechnungStat = AmvRechnungVO.RE_STAT_SELECTED;
        vo.errorMsg     = null;
        dao.insert(vo);
      }


      for (int i=ERINNERUNG_2_MIN; i<=ERINNERUNG_2_MAX; i++)
      {
        vo = new AmvRechnungVO();

        vo.amvLaufId    = id;
        vo.rechnungCd   = AmvRechnungVO.RE_CD_ERINNERUNG_2;
        vo.bfaktId      = i;
        vo.anzRechEmpf  = 1;
        vo.rechnungStat = AmvRechnungVO.RE_STAT_SAP_ERROR;
        vo.errorMsg     = "Test Fehlermeldung";
        dao.insert(vo);
      }
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }

  public void test04Select()
  {
    try
    {
      int             id    = getAmvLaufId();
      int             size  = ERINNERUNG_1_MAX - ERINNERUNG_1_MIN + 1 +
                              ERINNERUNG_2_MAX - ERINNERUNG_2_MIN + 1;
      ArrayList       list  = null;
      AmvRechnungVO   vo    = null;

      list = new AmvRechnungDAO().select(id);
      assertEquals("Anzahl Einträge", size, list.size() );

      // check first elem
      vo = (AmvRechnungVO) list.get(0);

      assertEquals("amvLaufId == " + id, true, (vo.amvLaufId == id));
      assertEquals("typ == " + AmvRechnungVO.RE_CD_ERINNERUNG_1,
                   AmvRechnungVO.RE_CD_ERINNERUNG_1, vo.rechnungCd);
      assertEquals("bfaktId == 1", 1, vo.bfaktId);
      assertEquals("status == " + AmvRechnungVO.RE_STAT_SELECTED,
                   AmvRechnungVO.RE_STAT_SELECTED, vo.rechnungStat);
      assertEquals("errorMsg == null", null, vo.errorMsg);
      assertNotNull("createdDat",vo.createdDat);
      assertNotNull("changedDat",vo.changedDat);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test05SelectByRechnungStat()
  {
    try
    {
      ArrayList list = null;
      int       size = ERINNERUNG_1_MAX - ERINNERUNG_1_MIN + 1;

      list = new AmvRechnungDAO().selectByRechnungStat
              (getAmvLaufId(),AmvRechnungVO.RE_STAT_SELECTED);
      assertEquals("Anzahl Einträge mit Status selected", size, list.size() );
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void test06SelectByRechnungCd()
  {
    try
    {
      ArrayList list = null;
      int       size = ERINNERUNG_2_MAX - ERINNERUNG_2_MIN + 1;

      list = new AmvRechnungDAO().selectByRechnungCd
              (getAmvLaufId(),AmvRechnungVO.RE_CD_ERINNERUNG_2);
      assertEquals("Anzahl Einträge mit Typ erinnerung 2", size, list.size() );
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void _test07Update()
  {
    try
    {
      ArrayList       list          = null;
      AmvRechnungVO   voBefore      = null;
      AmvRechnungVO   voAfter       = null;
      AmvRechnungVO   vo            = new AmvRechnungVO();
      int             affectedRows  = 0;

      //
      // init
      //

      // get existing value (first elem form list)
      list = new AmvRechnungDAO().select(getAmvLaufId());
      voBefore = (AmvRechnungVO) list.get(0);
      System.out.println("Update - voBefore: \n" + voBefore.toString());

      // save old values
      vo.amvLaufId    = voBefore.amvLaufId;
      vo.rechnungCd   = voBefore.rechnungCd;
      vo.bfaktId      = voBefore.bfaktId;
      vo.anzRechEmpf  = voBefore.anzRechEmpf;
      vo.rechnungStat = voBefore.rechnungStat;
      vo.errorMsg     = voBefore.errorMsg;
      vo.createdDat   = voBefore.createdDat;
      vo.changedDat   = voBefore.changedDat;

      //
      // do update (with error_msg=null)
      //

      // change some data
      vo.anzRechEmpf  = voBefore.anzRechEmpf + 10;
      vo.rechnungStat = AmvRechnungVO.RE_STAT_SAP_OK;

      affectedRows = new AmvRechnungDAO().update(vo);
      assertEquals("affectedRows == 1", true, (affectedRows == 1));

      // get updated value
      list = new AmvRechnungDAO().select(getAmvLaufId());
      voAfter = (AmvRechnungVO) list.get(0);
      System.out.println("Update - voAfter (error_id=null): \n" + voAfter.toString());

      // check values
      assertEquals(voBefore.amvLaufId,voAfter.amvLaufId);
      assertEquals(voBefore.rechnungCd,voAfter.rechnungCd);
      assertEquals(voBefore.bfaktId,voAfter.bfaktId);
      assertEquals(voAfter.anzRechEmpf,voBefore.anzRechEmpf + 10);
      assertEquals(voBefore.rechnungStat,AmvRechnungVO.RE_STAT_SELECTED);
      assertEquals(voAfter.rechnungStat,AmvRechnungVO.RE_STAT_SAP_OK);
      assertEquals(voBefore.errorMsg,voAfter.errorMsg);
      assertEquals(voBefore.createdDat,voAfter.createdDat);


      //
      // do update (with error_msg <> null)
      //

      // change some data
      vo.errorMsg = "Test Update - Fehlermeldung";

      affectedRows = new AmvRechnungDAO().update(vo);
      assertEquals("affectedRows == 1", true, (affectedRows == 1));

      // get updated value
      list = new AmvRechnungDAO().select(getAmvLaufId());
      voAfter = (AmvRechnungVO) list.get(0);
      System.out.println("Update - voAfter (error_id <> null): \n" + voAfter.toString());

      // check error message
      assertEquals(voAfter.errorMsg,"Test Update - Fehlermeldung");

    }
    catch (Exception ex)
    {
     ex.printStackTrace();
     fail(ex.toString());
    }
  }


  public void test08UpdateRechnungStat()
  {
    try
    {
      ArrayList       list          = null;
      AmvRechnungVO   voBefore      = null;
      AmvRechnungVO   voAfter       = null;
      AmvRechnungVO   vo            = new AmvRechnungVO();
      int             affectedRows  = 0;

      //
      // init
      //

      // get existing value (first elem form list)
      list = new AmvRechnungDAO().select(getAmvLaufId());
      voBefore = (AmvRechnungVO) list.get(1);
      System.out.println("Update RechnungStat - voBefore: \n" + voBefore.toString());

      // save old values
      vo.amvLaufId    = voBefore.amvLaufId;
      vo.rechnungCd   = voBefore.rechnungCd;
      vo.bfaktId      = voBefore.bfaktId;
      vo.anzRechEmpf  = voBefore.anzRechEmpf;
      vo.rechnungStat = voBefore.rechnungStat;
      vo.errorMsg     = voBefore.errorMsg;
      vo.createdDat   = voBefore.createdDat;
      vo.changedDat   = voBefore.changedDat;

      //
      // do update status
      //

      // change some data
      vo.rechnungStat = AmvRechnungVO.RE_STAT_SENT;

      affectedRows = new AmvRechnungDAO().update(vo);
      assertEquals("affectedRows == 1", true, (affectedRows == 1));

      // get updated value
      list = new AmvRechnungDAO().select(getAmvLaufId());
      voAfter = (AmvRechnungVO) list.get(1);
      System.out.println("Update RechnungStat - voAfter: \n" + voAfter.toString());

      // check values
      assertEquals(voAfter.rechnungStat,AmvRechnungVO.RE_STAT_SENT);

    }
    catch (Exception ex)
    {
     ex.printStackTrace();
     fail(ex.toString());
    }
  }


  public void test09Delete()
  {
    try
    {
      AmvRechnungDAO  dao     = new AmvRechnungDAO();
      AmvRechnungVO   vo      = null;
      ArrayList       list    = null;
      int             id      = getAmvLaufId();
      int             sizeE1  = ERINNERUNG_1_MAX - ERINNERUNG_1_MIN + 1;
      int             sizeE2  = ERINNERUNG_2_MAX - ERINNERUNG_2_MIN + 1;

      // select count E1/E2
      list = new AmvRechnungDAO().select(getAmvLaufId());
      assertEquals("Anzahl Einträge E1+E2", sizeE1 + sizeE2, list.size() );

      // delete E1
      for (int i=ERINNERUNG_1_MIN; i<=ERINNERUNG_1_MAX; i++)
      {
        vo = new AmvRechnungVO();

        vo.amvLaufId  = id;
        vo.rechnungCd = AmvRechnungVO.RE_CD_ERINNERUNG_1;
        vo.bfaktId    = i;
        dao.delete(vo);
      }

      // select count E2
      list = new AmvRechnungDAO().select(getAmvLaufId());
      assertEquals("Anzahl Einträge E2", sizeE2, list.size() );

    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }

  public void test10DestroyLauf()
  {
    try
    {
      System.out.println("DestroyLauf...");

      AmvLaufDAO  laufDAO   = new AmvLaufDAO();
      int         amvLaufId = getAmvLaufId();

      System.out.println("  delete lauf id " + amvLaufId);
      laufDAO.delete(amvLaufId);

      System.out.println("DestroyLauf finished");
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }

  //---------------------------------------------------------------------------
  // private functions
  //---------------------------------------------------------------------------

  private int getAmvLaufId() throws SQLException
  {
    AmvLaufDAO  laufDAO = new AmvLaufDAO();
    AmvLaufVO   laufVO  = laufDAO.select(LAUF_NR);

    return laufVO.amvLaufId;
  }

  //***************************************************************************
  // main
  //***************************************************************************

   /**
    * main function
    *
    * @param args programm arguments
    */
  public static void main(String[] args)
  {
    junit.textui.TestRunner.run(suite());
  }

} // end class

