package ch.ige.ses.server.dao.ses.test;

import junit.framework.*;
import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.server.dao.ses.BfaktposSESDAO;
import java.math.BigDecimal;

/**
 * Test der Klasse BfaktposSESDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BfaktposSESDAOTest.java,v 1.9 2007/01/04 15:58:08 iten Exp $
 */
public class BfaktposSESDAOTest extends SesTestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public BfaktposSESDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(BfaktposSESDAOTest.class);
  }

  protected void setUp() throws Exception
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void test00Dummy()
  {
    /** TODO junit - testfälle junit tauglich machen */
    assertEquals(true,true);
  }


  public void _test01Insert()
  {
    System.out.println("Test INSERT");
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    SAPRechnungsPositionVO sapRechnungsPositionVO = new SAPRechnungsPositionVO();
		sapRechnungsPositionVO.bfakt_id = 1000;
		sapRechnungsPositionVO.bfakt_posnr = 1000;
		sapRechnungsPositionVO.art_nr = "1000";
		sapRechnungsPositionVO.anzahl = 1000;
		sapRechnungsPositionVO.artKurztext = "test_insert";
		sapRechnungsPositionVO.artText = "test_insert";
		sapRechnungsPositionVO.artZusatzText = "test_insert";
		sapRechnungsPositionVO.dat_ereignis = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
		sapRechnungsPositionVO.frist = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
		sapRechnungsPositionVO.lageinh = "ANZ";
		sapRechnungsPositionVO.preis = new BigDecimal(10000.00);
		sapRechnungsPositionVO.schutit_nr = "15000";
		sapRechnungsPositionVO.wert = new BigDecimal(10000.00);

    try
    {

      bfaktposSESDAO.insert(sapRechnungsPositionVO);
      // testen ob Datensatz eingefügt wurde
      sapRechnungsPositionVO = bfaktposSESDAO.select(1000, 1000);
      assertEquals("1. Test: bfaktId: ", sapRechnungsPositionVO.bfakt_id, 1000);
      assertEquals("2. Test: bfakt_posnr: ", sapRechnungsPositionVO.bfakt_posnr, 1000);
      assertEquals("3. Test: art_nr: ", sapRechnungsPositionVO.art_nr, "1000");
			assertEquals("4. Test: anzahl: ", sapRechnungsPositionVO.anzahl, 1000);
			assertEquals("5. Test: artKurztext: ", sapRechnungsPositionVO.artKurztext, "test_insert");
			assertEquals("6. Test: artText ", sapRechnungsPositionVO.artText, "test_insert");
			assertEquals("7. Test: artZusatzText ", sapRechnungsPositionVO.artZusatzText, "test_insert");
			assertEquals("8. Test: lageinh: ", sapRechnungsPositionVO.lageinh, "ANZ");
			assertEquals("9. Test: preis: ", sapRechnungsPositionVO.preis, new BigDecimal(10000.00));
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _test02Update()
  {
    System.out.println("Test UPDATE ");
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    SAPRechnungsPositionVO sapRechnungsPositionVO = new SAPRechnungsPositionVO();

    try
    {
      sapRechnungsPositionVO = bfaktposSESDAO.select(1000, 1000);
      if (sapRechnungsPositionVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        // Datensatz ändern
        sapRechnungsPositionVO.artKurztext = "test_update";
        sapRechnungsPositionVO.artText = "test_update";
        int retVal = bfaktposSESDAO.update(sapRechnungsPositionVO);
        assertEquals("Anzahl geänderter Datensätze: ", retVal, 1);
        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        sapRechnungsPositionVO = bfaktposSESDAO.select(1000, 1000);
        assertEquals(sapRechnungsPositionVO.artKurztext, "test_update");
				assertEquals(sapRechnungsPositionVO.artText, "test_update");
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _test03SelectAll()
  {
    System.out.println("Test SELECT mehrere VO");
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    SAPRechnungsPositionVO[] sapRechnungsPositionVO;
    SAPRechnungsPositionVO sapPos = new SAPRechnungsPositionVO();
		sapPos.bfakt_id = 1000;
		sapPos.bfakt_posnr = 2;

		ArrayList al = new ArrayList();

    try
    {
			// ein zweiter Datensatz einfügen
		  bfaktposSESDAO.insert(sapPos);

			//selektieren
      al = bfaktposSESDAO.select(1000);
      sapRechnungsPositionVO = (SAPRechnungsPositionVO[])al.toArray(new SAPRechnungsPositionVO[0]);

      assertTrue("Anzahl = ", al.size()==2);
      for ( int i = 0; i < sapRechnungsPositionVO.length; i++)
      {
        assertTrue("bfakt_id: ", sapRechnungsPositionVO[i].bfakt_id == 1000);
      }
			//zweiten Datensatz wieder löschen

			int retVal = bfaktposSESDAO.delete(1000, 2);
      assertEquals("1 gelöschter Datensatz",retVal, 1);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _test04Select()
  {
    System.out.println("Test SELECT ");
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    SAPRechnungsPositionVO sapRechnungsPositionVO = new SAPRechnungsPositionVO();
    try
    {
      sapRechnungsPositionVO = bfaktposSESDAO.select(1000, 1000);

      System.out.println(sapRechnungsPositionVO.toString());

      assertEquals("bfakid: ", sapRechnungsPositionVO.bfakt_id, 1000);
      assertEquals("bfakt_posnr: ", sapRechnungsPositionVO.bfakt_posnr, 1000);
			assertEquals("artKurztext: ", sapRechnungsPositionVO.artKurztext, "test_update");
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _test05Delete()
  {
    System.out.println("Test DELETE");
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    try
    {
      int retVal = bfaktposSESDAO.delete(1000, 1000);
      assertEquals("1 gelöschter Datensatz",retVal, 1);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

} // end class


