package ch.ige.ses.server.dao.ses.test;

import junit.framework.TestCase;
import java.util.*;

import ch.ige.ses.server.dao.ses.StakoKontrolleDAO;
import ch.ige.ses.vo.ses.StakoKontrolleVO;

/**
 * <p>Test der Klasse StakoKontrolleDAO</p>
 * <p>Copyright: 2003, Copyright (c) Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Stefan Nydegger
 * @version $Id: StakoKontrolleDAOTest.java,v 1.8 2007/08/03 07:48:56 iten Exp $
 */


public class StakoKontrolleDAOTest extends SesTestCase
{

  public StakoKontrolleDAOTest(String s)
  {
    super(s);
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle STAKO_KONTROLLE ein
   */
  public void testInsert()
  {
    System.out.println("STAKO_KONTROLLE: Test INSERT");
    StakoKontrolleDAO stakoKontrolleDAO = new StakoKontrolleDAO();
    StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

    stakoKontrolleVO.vonDat                = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
    stakoKontrolleVO.bisDat                = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
    stakoKontrolleVO.anzahlEreignisMaGelesen = 1000;
    stakoKontrolleVO.anzahlBfakturGelesen    = 1000;
    stakoKontrolleVO.anzahlEreignisMaAnSap   = 1000;
    stakoKontrolleVO.anzahlBfakturAnSap      = 1000;
    stakoKontrolleVO.anzahlEreignisMaAnSapOK = 1000;
    stakoKontrolleVO.anzahlBfakturAnSapOK    = 1000;
    stakoKontrolleVO.stakoStat               = 1;
    stakoKontrolleVO.herkunftStat            = 9;

    try
    {
      stakoKontrolleDAO.insert(stakoKontrolleVO);
      // Testen ob eingefügt
      stakoKontrolleVO = stakoKontrolleDAO.selectMaxKontrollId();
      if (stakoKontrolleVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("StakoKontrolleVO eingefügt ", stakoKontrolleVO != null);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle STAKO_KONTROLLE
   */
  public void testSelect()
  {
    System.out.println("STAKO KONTROLLE : Test SELECT");
    StakoKontrolleDAO stakoKontrolleDAO = new StakoKontrolleDAO();
    StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();
    try
    {
      stakoKontrolleVO = stakoKontrolleDAO.selectMaxKontrollId();
      stakoKontrolleVO = stakoKontrolleDAO.select(stakoKontrolleVO.stakoKontrolleId);

      assertNotNull( stakoKontrolleVO );
      
      assertEquals("vonDatum: ", stakoKontrolleVO.vonDat, new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime()));
      assertEquals("bisDatum: ", stakoKontrolleVO.bisDat, new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime()));
      assertEquals("anzahlEreignisMaGelesenis: ", stakoKontrolleVO.anzahlEreignisMaGelesen, 1000);
      assertEquals("anzahlBfakturGelesen: ", stakoKontrolleVO.anzahlBfakturGelesen, 1000);
      assertEquals("anzahlEreignisMaAnSap: ", stakoKontrolleVO.anzahlEreignisMaAnSap, 1000);
      assertEquals("anzahlBfakturAnSap: ", stakoKontrolleVO.anzahlBfakturAnSap, 1000);
      assertEquals("anzahlEreignisMaAnSapOK: ", stakoKontrolleVO.anzahlEreignisMaAnSapOK, 1000);
      assertEquals("anzahlBfakturAnSapOK: ", stakoKontrolleVO.anzahlBfakturAnSapOK, 1000);
      assertEquals("stakoStat: ", stakoKontrolleVO.stakoStat, 1);
      assertEquals("herkunftStat: ", stakoKontrolleVO.herkunftStat, 9);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   *  Selektiert alle Datensätze aus der Tabelle JGB_LAUF
   */
  public void testSelectAll()
  {
    System.out.println("STAKO KONTROLLE: Test SELECT ALL ");
    StakoKontrolleDAO stakoKontrolleDAO = new StakoKontrolleDAO();
    ArrayList al = new ArrayList();
    try
    {
      al = stakoKontrolleDAO.selectAll();
      assertTrue("Select All: ", al.size() > 0);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
  *  Aktualisiert einen Datensatz der Tabelle STAKO_KONTROLLE
  */
  public void testUpdate()
  {
    System.out.println("STAKO KONTROLLE: TEST UPDATE ");
    StakoKontrolleDAO stakoKontrolleDAO = new StakoKontrolleDAO();
    StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

    try
    {
      stakoKontrolleVO = stakoKontrolleDAO.selectMaxKontrollId();
      if (stakoKontrolleVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        // Datensatz ändern
        stakoKontrolleVO.verarbeitungDat         = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
        stakoKontrolleVO.vonDat                = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
        stakoKontrolleVO.bisDat                = new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime());
        stakoKontrolleVO.anzahlEreignisMaGelesen = 3000;
        stakoKontrolleVO.anzahlBfakturGelesen    = 3000;
        stakoKontrolleVO.anzahlEreignisMaAnSap   = 3000;
        stakoKontrolleVO.anzahlBfakturAnSap      = 3000;
        stakoKontrolleVO.anzahlEreignisMaAnSapOK = 3000;
        stakoKontrolleVO.anzahlBfakturAnSapOK    = 3000;
        stakoKontrolleVO.stakoStat               = 1;
        stakoKontrolleVO.herkunftStat            = 8;

        int retVal = stakoKontrolleDAO.update(stakoKontrolleVO);
        assertEquals("Anzahl geänderter Datensätze: ", retVal, 1);
        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        stakoKontrolleVO = stakoKontrolleDAO.selectMaxKontrollId();
        assertEquals("verarbeitungDat: ", stakoKontrolleVO.verarbeitungDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
        assertEquals("vonDatum: ", stakoKontrolleVO.vonDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
        assertEquals("bisDatum: ", stakoKontrolleVO.bisDat, new java.sql.Date(new GregorianCalendar(2003,0,3,0,0,0).getTime().getTime()));
        assertEquals("anzahlEreignisMaGelesen: ", stakoKontrolleVO.anzahlEreignisMaGelesen, 3000);
        assertEquals("anzahlBfakturGelesen: ", stakoKontrolleVO.anzahlBfakturGelesen, 3000);
        assertEquals("anzahlEreignisMaAnSap: ", stakoKontrolleVO.anzahlEreignisMaAnSap, 3000);
        assertEquals("anzahlBfakturAnSap: ", stakoKontrolleVO.anzahlBfakturAnSap, 3000);
        assertEquals("anzahlEreignisMaAnSapOK: ", stakoKontrolleVO.anzahlEreignisMaAnSapOK, 3000);
        assertEquals("anzahlBfakturAnSapOK: ", stakoKontrolleVO.anzahlBfakturAnSapOK, 3000);
        assertEquals("herkunftStat: ", stakoKontrolleVO.herkunftStat, 8);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle STAKO_KONTROLLE
   */
  public void testDelete()
  {
    System.out.println("STAKO Kontrolle: Test DELETE");
    StakoKontrolleDAO stakoKontrolleDAO = new StakoKontrolleDAO();
    StakoKontrolleVO stakoKontrolleVO = new StakoKontrolleVO();

    try
    {
      stakoKontrolleVO = stakoKontrolleDAO.selectMaxKontrollId();
      if (stakoKontrolleVO == null)
      {
        System.out.println("DELETE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = stakoKontrolleDAO.delete(stakoKontrolleVO.stakoKontrolleId);
        assertEquals("Anzahl gelöschte Datensätze = 1", retVal, 1);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
}

