package ch.ige.ses.server.dao.ses.test;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Properties;

import junit.framework.Test;
import junit.framework.TestSuite;
import ch.ige.ses.client.sapinbound.SapInbound;
import ch.ige.ses.server.dao.ses.TraceDAO;
import ch.ige.ses.vo.ses.TraceSelKritVO;
import ch.ige.ses.vo.ses.TraceVO;
import ch.ige.util.server.DBHelper;

/**
 * Test der Klasse TraceDAO
 * <p>
 * Copyright (c) Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: TraceDAOTest.java,v 1.6 2008/01/18 10:01:35 iten Exp $
 */
public class TraceDAOTest extends SesTestCase
{
  //
  // fields
  //
  TraceVO vo1;

  TraceDAO dao;

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public TraceDAOTest( String s )
  {
    super( s );

    // init fields
    vo1 = new TraceVO(); // initialised by setUp()
    dao = new TraceDAO(); // used by methods
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
    return new TestSuite( TraceDAOTest.class );
  }

  protected void setUp()
  {
    // set up test data
    // traceId will be set after successfully insert

    // vo #1
    vo1.process = SapInbound.getProgramName();
    vo1.processNr = "01";
    vo1.action = "insert testrecord";
    vo1.data = "some data ...";

  }

  protected void tearDown()
  {
    // not used
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void test01Insert() throws Exception
  {
    try
    {
      dao.insert( vo1 );
      assertTrue( true ); // insert done
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void _test02Select()
  {
    SimpleDateFormat sdf = new SimpleDateFormat( "dd.MM.yyyy" );
    TraceSelKritVO vo = new TraceSelKritVO();
    ArrayList al = null;

    vo.process = "";
    vo.action = "";
    vo.data = "";
    vo.maxRow = 100;

    try
    {
      vo.createdDatVon = new Date( sdf.parse( "01.01.2000" ).getTime() );
      vo.createdDatBis = new Date( sdf.parse( "31.12.2099" ).getTime() );
      al = dao.select( vo );
      assertTrue( "Trace selektiert ", al.size() > 0 );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void test03SelectMaxId() throws Exception
  {
    int lastId = 0;

    // init
    System.out.println( "TRACE: Test INSERT" );
    lastId = this.selectMaxId();
    System.out.println( "last id = " + lastId );

    // insert
    dao.insert( vo1 );
    vo1.traceId = this.selectMaxId();

    // check
    assertTrue( "ErrorVO #1 eingefügt ", vo1.traceId != lastId );
  }

  public void test04SelectByCreatedDat() throws Exception
  {
    try
    {
      GregorianCalendar today = new GregorianCalendar();
      GregorianCalendar created = new GregorianCalendar();
      int months = 3;
      Properties prop = null;

      created.set( Calendar.MONTH, today.get( Calendar.MONTH ) - months );

      prop = dao.select( created.getTime() );

      if ( prop.getProperty( "rs" ).equalsIgnoreCase( "true" ) && !prop.getProperty( "cnt" ).equals( "0" ) )
      {
        System.out.println( "testSelectByCreatedDat - TRACE_ID " + prop.getProperty( "min" ) + " to " + prop.getProperty( "max" ) + " ("
            + prop.getProperty( "cnt" ) + " rows) found" );
      }
      else
      {
        System.out.println( "testSelectByCreatedDat - no entires found \n" );
      }

      assertTrue( true );

    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  public void _test05Delete()
  {
    try
    {
      GregorianCalendar today = new GregorianCalendar();
      GregorianCalendar created = new GregorianCalendar();
      int months = 0;
      int count = 0;

      created.set( Calendar.MONTH, today.get( Calendar.MONTH ) - months );
      count = dao.delete( created.getTime() );

      assertTrue( count > 0 );

    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private int selectMaxId()
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    int max_value = 0;

    try
    {
      con = DBHelper.getConnection( DBHelper.SES );

      ps = con.prepareStatement( "SELECT MAX(trace_id) max_value from ses.trace" );
      rs = ps.executeQuery();

      if ( rs.next() )
      {
        max_value = rs.getInt( "max_value" );
      }
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.toString() );
    }

    return max_value;
  }

} // end class
