package ch.ige.ses.server.error;

import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.BytesMessage;
import javax.jms.JMSException;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.util.Jms;
import ch.ige.ses.util.JmsQueue;
import ch.ige.ses.vo.ses.ErrorVO;

/**
 * Erstellt einen neuen Eintrag in die SES.errorQueue
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Patrick Bolzli
 * @version $Id: SESErrorWorkBean.java,v 1.20 2007/12/24 10:13:23 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESErrorWork"  
 *  display-name="SESErrorWork"
 *  view-type="local"
 *  jndi-name="SESErrorWork"
 *  local-jndi-name="SESErrorWorkLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SESErrorWorkBean implements SessionBean
{
  //TODO maintenance (optional) - jms handling umbauen

  // const
  private static final Logger LOG = Logger.getLogger(SESErrorWorkBean.class);

  private static final boolean JMS_SESSION_TRANSACTED = false;
  private static final int JMS_SESSION_ACKNOWLEDGE_MODE = Session.AUTO_ACKNOWLEDGE;

  // fields
  private JmsQueue fJmsQueue = null;
  private QueueSession fSession = null;
  private QueueSender fSender = null;

  SessionContext sessionContext;


  //---------------------------------------------------------------------------
  // bean methods 
  //---------------------------------------------------------------------------
  
  /**
   * ejb create
   * 
   * @throws CreateException
   */
  public void ejbCreate()  
  {
      fJmsQueue = new JmsQueue( 
          JmsQueue.RUN_ON_APPSRV,
          Jms.JNDI_JMS_QUEUE_FACTORY_NAME,
          Jms.JMS_ERROR_QUEUE_NAME,
          JMS_SESSION_TRANSACTED,
          JMS_SESSION_ACKNOWLEDGE_MODE );
  }
  
  public void ejbRemove()
  {
    // lazy bone    
  }
  
  public void ejbActivate()
  {
    // lazy bone    
  }
  
  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  //  methods 
  //---------------------------------------------------------------------------

  /**
   * Stellt ein errorVO in die JMS - Queue<br>
   * Da es im MDB mit einer ObjectMessage zu Problemen kam, musste ein anderer
   * Weg gefunden werden. Der Workaround besteht darin, dass das Objekt in ein
   * byte[] serialisiert und als BytesMessage verschickt wird. Der Listener (MDB)
   * auf der Gegenseite muss dieses byte[] anschliessend wieder deserialisieren und
   * kann das Objekt danach Problemlos zu einem errorVO zurück casten.
   * 
   * @param eVO Error - ValueObject
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendError (ErrorVO eVO) throws Exception {
    try
    {
      connect();

      // serialize ErrorVO to byte[]
      byte[] barr = null;
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream os = new ObjectOutputStream(baos);
      os.writeObject(eVO);
      barr = baos.toByteArray();

      // create new BytesMessage with the serialized ErrorVO and put into queue
      BytesMessage bm = fSession.createBytesMessage();
      bm.writeBytes(barr);
      fSender.send(bm);
    }
    catch (JMSException ex)
    {
      LOG.error("JMSException in Methode sendError von SESErrorWorkBean: " + ex.getMessage());
      throw new Exception (ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  
  private void connect() throws NamingException, JMSException 
  {
    fJmsQueue.connectSender();

    fSession = fJmsQueue.getSession();
    fSender  = fJmsQueue.getSender();
  }
  
  private void disconnect()
  {
    fJmsQueue.disconnect();
    fSession = null;        
    fSender  = null;
  }
  

} // end of class

