package ch.ige.ses.server.monitor;

import java.util.*;


import ch.ige.ses.vo.ses.*;
import ch.ige.util.*;
import ch.ige.ses.vo.bagis.*;


/**
 * BusinessDelegate für SESMONITOR
 * <p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rüegsegger
 * @version $Id: MonitorDelegate.java,v 1.26 2006/12/13 14:48:40 iten Exp $
 */

public class MonitorDelegate
{
  private static MonitorDelegate monitorDelegate = null;
  private MonitorCtrl monitorCtrl = null;
  private static final Class HOME_CLASS = MonitorCtrlHome.class;


  private MonitorDelegate()  throws Exception
  {
    MonitorCtrlHome home = (MonitorCtrlHome)
                           ServiceLocator.getInstance().getHome(MonitorCtrlHome.JNDI_NAME, HOME_CLASS);
    monitorCtrl = home.create();
  }


  /**
   * Gibt die einzige Instanz zurück (Singleton)
   *
   * @return MonitorDelegatelegate
   * @exception Exception
   */
  public static MonitorDelegate getInstance() throws Exception
  {
    if (monitorDelegate == null)
    {
      monitorDelegate = new MonitorDelegate();
    }
    return monitorDelegate;
  }


  /**
   * Gibt die Meta-Daten von der SES-DB zurück.
   * @return String URL
   * @throws Exception Unerwarteter Fehlersituation
   */
  public String getSESDbUrl() throws Exception
  {
    return monitorCtrl.getSESDbUrl();
  }

  /**
   * Gibt die Server-Version zurück(InfoDAO)
   * @return String Version
   * @throws Exception Unerwarteter Fehlersituation
   */
  public String getSESVersion() throws Exception
  {
    return monitorCtrl.getSESVersion();
  }



//-----------------------------------------------------------------------------
//  PROPERTY
//-----------------------------------------------------------------------------

  /**
  * Alle Datensätze der Tabelle Property selektieren
  * @return ArrayList von PropertyVOs
  * @throws Exception Fehler
  */
  public ArrayList findProperties() throws Exception
  {
    return monitorCtrl.findProperties();
  }

  /**
   * Neuen Datensatz in Tabelle Property einfügen
   * @param propertyVO Value Object mit den Property-Daten
   * @throws Exception Fehler
   */
  public void createProperty(PropertyVO propertyVO) throws Exception
  {
    monitorCtrl.createProperty(propertyVO);
  }

  /**
   * Datensatz der Tabelle Property via key selektieren
   * @param key PK
   * @return Value Object mit den Property-Daten
   * @throws Exception Fehler
   */
  public PropertyVO getProperty(String key) throws Exception
  {
    return monitorCtrl.getProperty(key);
  }

  /**
   * Datensatz der Tabelle Property updaten
   * @param propertyVO Value Object mit den Property-Daten
   * @throws Exception Fehler
   */
  public void setProperty(PropertyVO propertyVO) throws Exception
  {
    monitorCtrl.setProperty(propertyVO);
  }

  /**
   * Datensatz der Tabelle Property löschen
   * @param key PK
   * @throws Exception Fehler
   */
  public void deleteProperty(String key) throws Exception
  {
    monitorCtrl.deleteProperty(key);
  }


//-----------------------------------------------------------------------------
//  JGB_LAUF
//-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle JGB_Lauf selektieren
   * @return ArrayList von JGBLaufVOs
   * @throws Exception Fehler
   */
  public ArrayList findJGBLauf() throws Exception
  {
    return monitorCtrl.findJGBLauf();
  }

  /**
   * Einen Datensatz der Tabelle JGB_Lauf selektieren
   * @param jgbLaufNr JGB-Laufnummer
   * @return JGBLaufVO
   * @throws Exception
   */
  public JGBLaufVO getJGBLauf(String jgbLaufNr) throws Exception
  {
    return monitorCtrl.getJGBLauf(jgbLaufNr);
  }

  /**
   * Einen Datensatz der Tabelle JGB_Lauf updaten
   * @param jgbLaufVO beinhaltet die zu aktualisierenden Datensätze
   * @return Anzahl mutierte Datensätze
   * @throws Exception
   */
  public int setJGBLauf(JGBLaufVO jgbLaufVO) throws Exception
  {
    return monitorCtrl.setJGBLauf(jgbLaufVO);
  }

//-----------------------------------------------------------------------------
//  CONTROLLING
//-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle STAKO_KONTROLLE selektieren
   * @return ArrayList von StakoKontrolleVO's
   * @throws Exception Fehler
   */
  public ArrayList findStakoKontrolle() throws Exception
  {
    return monitorCtrl.findStakoKontrolle();
  }

  /**
   * Einen Datensatz der Tabelle STAKO_KONTROLLE selektieren
   * @param stakoKontrolleId Key StakoKontrolle
   * @return StakoKontrolleVO
   * @throws Exception Fehler
   */
  public StakoKontrolleVO getStakoKontrolle(int stakoKontrolleId) throws Exception
  {
    return monitorCtrl.getStakoKontrolle(stakoKontrolleId);
  }

  /**
   * Einen Datensatz der Tabelle STAKO_KONTROLLE updaten
   * @param stakoKontrolleVO beinhaltet die zu aktualisierenden Datensätze
   * @return Anzahl mutierte Datensätze
   * @throws Exception Fehler
   */
  public int setStakoKontrolle(StakoKontrolleVO stakoKontrolleVO) throws Exception
  {
    return monitorCtrl.setStakoKontrolle(stakoKontrolleVO);
  }


//-----------------------------------------------------------------------------
//  ERROR
//-----------------------------------------------------------------------------

  /**
   * Selektiert alle Einträge der Tabelle ERROR
   * die den Selektionskriterien entsprechen
   * @param errorSelKritVO ValueObject mit Selektionskriterien
   * @return ArrayList von ErrorVOs
   * @throws Exception Fehler
   */
  public ArrayList findErrors(ErrorSelKritVO errorSelKritVO) throws Exception
  {
    return monitorCtrl.findErrors(errorSelKritVO);
  }

  /**
   * Holt einen Fehlereintrag aus der Tabelle ERROR
   * @param errorId   PK
   * @return ErrorVO mit Fehlerinformationen
   * @throws Exception Fehler
   */
  public ErrorVO getError(int errorId) throws Exception
  {
    return monitorCtrl.getError(errorId);
  }

  /**
   * Einen Datensatz der Tabelle ERROR updaten
   * @param errorVO   ValueObject mit Fehlerinformationen
   * @throws Exception Fehler
   */
  public void setError(ErrorVO errorVO) throws Exception
  {
    monitorCtrl.setError(errorVO);
  }


//-----------------------------------------------------------------------------
//  BFAKTUR SES
//-----------------------------------------------------------------------------

  /**
   * Selektiert alle Rechnungen aus der Tabelle BFAKTUR_SES
   * die den Selektionskriterien entsprechen
   * @param sapRechnungVO ValueObject mit Selektionskriterien
   * @return ArrayList von SAPRechnungVOs
   * @throws Exception Fehler
   */
  public ArrayList findRechnungen(SAPRechnungVO sapRechnungVO) throws Exception
  {
    return monitorCtrl.findBfakturen(sapRechnungVO);
  }

  /**
   * Selektiert die grösste bfakt_id des gewünschtes Laufes aus der Tabelle bfaktur_ses
   * @param     jgbLaufId lauf_id eines Jahresgebühren Laufes
   * @return    grösste bfakt_id des ensprechenden Laufes
   * @exception Exception Unerwarteter Fehler
   */
  public int getMaxBfaktId(int jgbLaufId) throws Exception
  {
    return monitorCtrl.getMaxBfaktId(jgbLaufId);
  }

  /**
   * Selektiert die kleinste bfakt_id des gewünschten Laufes aus der Tabelle bfaktur_ses
   * @param     jgbLaufId lauf_id eines Jahresgebühren Laufes
   * @return    kleinste bfakt_id des entsprechenden Laufes
   * @exception Exception Unerwarteter Fehler
   */
  public int getMinBfaktId(int jgbLaufId) throws Exception
  {
    return monitorCtrl.getMinBfaktId(jgbLaufId);
  }

  /**
   * Selektiert alle Datensätze eines JGB-Laufes aus der Tabelle bfaktur_ses
   * mit einem bestimmten Verarbeitungsstatus
   *
   * @param jgbLaufId Lauf-Id des JGB-Laufes
   * @param statusVerarbeitung Verarbeitungsstatus
   * @return Liste von SapRechnungV0s
   * @throws Exception Unerwarteter Fehler
   */
   public ArrayList findErrorList(int jgbLaufId, String statusVerarbeitung) throws Exception
   {
     return monitorCtrl.findStatusVerarbList(jgbLaufId, statusVerarbeitung);
   }

   /**
    * Verarbeitungstatus wird zurückgesetzt von 'E' auf '0'
    * Die Zähler des JGB-Laufes werden pro Update wie folgt aktualisiert
    *    anzahl_bfaktur_an_sap:      -1
    *    anzahl_bfaktur_an_sap_nok:  -1
    *    anzahl_bfaktur_an_sap_ok:   nichts
    *    anzahl_bfaktjgb_an_sap_ok:  -anzahl Rechnungsempfänger
    *
    * @param sapRechnungVO Array von SAPRechnungVOs
    * @return Anzahl aktualisierter Datensätze
    * @throws Exception Fehler
    */
   public int updateStatusVerarbeitung(SAPRechnungVO[] sapRechnungVO) throws Exception
   {
     return monitorCtrl.updateStatusVerarbeitung(sapRechnungVO);
   }

   /**
    * Gibt die Anzahl Bfakturen pro Verarbeitungsstatus eines JGB - Laufes zurück
    * @param jgbLaufId Jahresgebühren - Lauf
    * @return Anzahl Datensätze pro Verarbeitungsstatus
    * @throws SQLException
    */
   public HashMap countStatusVerarbeitung(int jgbLaufId) throws Exception
   {
     return monitorCtrl.countStatusVerarbeitung(jgbLaufId);
   }

   /**
    * Gibt eine ArrayList von BfaktIds mit einem bestimmten Verarbeitungsstatus
    * eines Jahresgebühren - Laufes zurück
    *
    * @param jgbLaufId  JGB-Lauf
    * @param statusVerarbeitung Verarbeitungsstatus
    * @return ArrayList von bfaktIds
    * @throws Exception Unerwartete Fehlersituation
    */
   public ArrayList findBfaktIds(int jgbLaufId, String statusVerarbeitung) throws Exception
   {
     return monitorCtrl.findBfaktIds(jgbLaufId, statusVerarbeitung);
   }


//-----------------------------------------------------------------------------
//  BFAKTUR_SES, BFAKTJGB_SES, BFAKTPOS_SES
//-----------------------------------------------------------------------------

  /**
   * Selektiert diverse Attribute der Rechnungen die den Selektionskriterien entsprechen.
   * Rechnuungen werden in der Sortierreihenfolge der Jahresgebühren zurückgegeben
   * @param sapRechnungVO ValueObject mit Selektionskriterien Rechnungsdaten & Rechnungsempfänger
   * @return ArrayList von SAPRechnungVOs mit integrierten Rechnungempfänger(SAPJGBParamVO)
   * @throws Exception Fehler
   */
  public ArrayList findSESRechnungen(SAPRechnungVO sapRechnungVO) throws Exception
  {
    return monitorCtrl.findSESRechnungen(sapRechnungVO);
  }

  /**
   * Holt alle Daten einer Rechnung aus den entsprechenden Tabllen (BFAKTUR_SES, BFAKTJGB_SES, BFAKTPOS_SES)
   * @param bfaktId   PK
   * @param jgbLaufId PK
   * @return SAPRechnungVO mit Rechnungs-Daten
   * @throws Exception Fehler
   */
  public SAPRechnungVO getRechnung(int bfaktId, int jgbLaufId) throws Exception
  {
    return monitorCtrl.getRechnung(bfaktId, jgbLaufId);
  }


//-----------------------------------------------------------------------------
//  TRACE
//-----------------------------------------------------------------------------

  /**
   * Selektiert alle Einträge der Tabelle TRACE
   * die den Selektionskriterien entsprechen
   * @param traceSelKritVO ValueObject mit Selektionskriterien
   * @return ArrayList von TraceVOs
   * @throws Exception Fehler
   */
  public ArrayList findTraces(TraceSelKritVO traceSelKritVO) throws Exception
  {
    return monitorCtrl.findTraces(traceSelKritVO);
  }

  /**
   * Löscht alle Datensaetze die aelter als die übergebene Anzahl Tage sind
   * (Anzahl Tage werden in der Tabelle PROPERTY gespeichert trace.clear.numberOfDays
   * @param days Anzahl Tage
   * @return Anzahl gelöschter Datensätze
   * @throws Exception Fehler
   */
  public int deleteTraces(int days) throws Exception
  {
    return monitorCtrl.deleteTraces(days);
  }

//-----------------------------------------------------------------------------
//  Jahresgebühren - Auswertungen
//-----------------------------------------------------------------------------
/**
 * Gibt die Fehlerliste eines Jahresgebühren Laufes als String zurück
 * @param jgbLaufId Lauf-Id eines Jahresgebühren-Laufes
 * @return Fehlerliste als String
 * @throws Exception
 * @author Patrick Bolzli
 */
  public String getProtokoll(int jgbLaufId) throws Exception
  {
    return monitorCtrl.getProtokoll(jgbLaufId);
  }

  /**
   * Gibt die Statistik eines Jahresgebühren Laufes als String zurück
   * @param jgbLaufId Lauf-Id eines Jahresgebühren-Laufes
   * @param newcalc -> Neuberechnung durchführen
   * @return Statistik als String
   * @throws Exception
   * @author Patrick Bolzli
   */
  public String getStatistik(int jgbLaufId, boolean newcalc) throws Exception
  {
    return monitorCtrl.getStatistik(jgbLaufId, newcalc);
  }


//-----------------------------------------------------------------------------
//  JGB Bagis Druckstatus
//-----------------------------------------------------------------------------
  /**
   * Aktualisiert bei der JGB Verarbeitung den Status druck auf BAGIS von
   * 3 (bereit zum Druck JGB) auf 1 (bereit zum Druck), sobald der Datensatz
   * von BAGIS nach SES übertragen wurde.
   *
   * @param vo Rechnungsdaten
   */
    public void setBfakturStatusDruck (SAPRechnungVO vo) throws Exception
    {
      monitorCtrl.setBfakturStatusDruck(vo);
    }


//-----------------------------------------------------------------------------
//  AMV_LAUF
//-----------------------------------------------------------------------------

    /**
     * Alle Datensätze der Tabelle AMV_Lauf selektieren
     * @return ArrayList von AMVLaufVOs
     * @throws Exception Fehler
     */
    public ArrayList findAmvLauf() throws Exception
    {
      return monitorCtrl.findAmvLauf();
    }

    /**
     * Einen Datensatz der Tabelle AMV_Lauf selektieren
     * @param amvLaufNr JGB-Laufnummer
     * @return AmvLaufVO
     * @throws Exception Unerwarteter Fehler
     */
    public AmvLaufVO getAmvLauf(String amvLaufNr) throws Exception
    {
      return monitorCtrl.getAmvLauf(amvLaufNr);
    }

    /**
     * Einen Datensatz der Tabelle AMV_Lauf selektieren
     * @param amvLaufNr JGB-Laufnummer
     * @return AmvLaufVO
     * @throws Exception Unerwarteter Fehler
     */
    public AmvLaufVO getAmvLauf(int amvLaufId) throws Exception
    {
      return monitorCtrl.getAmvLauf(amvLaufId);
    }

    /**
    * Selektiert alle Einträge der Tabelle AMV_RECHNUNG
    * die den Selektionskriterien entsprechen
    * @param amvRechnungSelKritVO ValueObject mit Selektionskriterien
    * @return ArrayList von AmvRechnungVOs
    * @throws Exception Fehler
    */
   public ArrayList findAmvRechnungen(AmvRechnungSelKritVO amvRechnungSelKritVO) throws Exception
   {
     return monitorCtrl.findAmvRechnungen(amvRechnungSelKritVO);
   }

   /**
    * Holt alle Daten einer Amv-Rechnung aus der Tabllen AMV_RECHNUNG
    * @param amvLaufId    PK AmvLauf
    * @param rechnungCd   PK Rechnungstyp
    * @param bfaktId      PK Rechnungsnummer
    * @return AmvRechnungVO mit Rechnungs-Informationen
    * @throws Exception Fehler
    */
   public AmvRechnungVO getAmvRechnung(int amvLaufId, int rechnungCd, int bfaktId) throws Exception
   {
     return monitorCtrl.getAmvRechnung(amvLaufId, rechnungCd, bfaktId);
   }

    /**
    * Selektiert pro Verarbeitungsstatus die summierte Anzahl Rechnungsempfänger
    * @param   amvLaufId      Lauf Id
    * @param   rechnungCd     Erinnerungstyp
    * @return  Map            beinhaltet summierten Rechnungsempfänger pro Verarbeitungsstatus
    * @throws Exception       Unerwarteter Fehler
    */
    public Map getStatCounters(int amvLaufId, int rechnungCd) throws Exception
    {
      return monitorCtrl.getStatCounters(amvLaufId, rechnungCd);
    }
} // end class


