package ch.ige.ses.server.sapctrl;

import java.util.ArrayList;
import java.util.Date;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.controlling.StakoBagisReadLocal;
import ch.ige.ses.server.controlling.StakoBagisReadLocalHome;
import ch.ige.ses.server.controlling.StakoControlLocal;
import ch.ige.ses.server.controlling.StakoControlLocalHome;
import ch.ige.ses.vo.bagis.SAPRechnungAnnullationVO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.bagis.StakoVO;
import ch.ige.ses.vo.ses.AmvLaufVO;
import ch.ige.ses.vo.ses.AmvRechnungVO;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.JGBLaufVO;
import ch.ige.ses.vo.ses.SAPAdrStatusVO;
import ch.ige.ses.vo.ses.SAPJGBJobVO;
import ch.ige.ses.vo.ses.SAPStakoJobVO;
import ch.ige.ses.vo.ses.StakoKontrolleVO;
import ch.ige.ses.vo.ses.TraceVO;
import ch.ige.util.vo.VersadrVO;

/**
 * Controller für den SAP Inbound Service sowie alle Bean's und Clients's
 * die die SAP Inbound Service verwenden.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapInboundCtrlBean.java,v 1.43 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SapInboundCtrl"  
 *  display-name="SapInboundCtrl"
 *  view-type="both"
 *  jndi-name="SapInboundCtrl"
 *  local-jndi-name="SapInboundCtrlLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SapSend"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SapReceive"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SapRequest"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="StakoControl"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="StakoBagisRead"
 *   view-type="local"
 */
public class SapInboundCtrlBean implements SessionBean
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapInboundCtrlBean.class);

  //
  // fields
  //
  SessionContext          sessionContext;

  SapSendLocalHome        sapSendLocalHome        = null;
  SapSendLocal            sapSendLocal            = null;

  SapReceiveLocalHome     sapReceiveLocalHome     = null;
  SapReceiveLocal         sapReceiveLocal         = null;

  SapRequestLocalHome     sapRequestLocalHome     = null;
  SapRequestLocal         sapRequestLocal         = null;

  StakoControlLocalHome   stakoControlLocalHome   = null;
  StakoControlLocal       stakoControlLocal       = null;

  StakoBagisReadLocalHome stakoBagisReadLocalHome = null;
  StakoBagisReadLocal     stakoBagisReadLocal     = null;

  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate() 
  {
    Context   ctx;
    Object    ref;

    try
    {
      ctx = new InitialContext();

      ref                   = ctx.lookup(SapSendLocalHome.COMP_NAME);
      sapSendLocalHome      = (SapSendLocalHome)ref;
      sapSendLocal          = sapSendLocalHome.create();

      ref                   = ctx.lookup(SapReceiveLocalHome.COMP_NAME);
      sapReceiveLocalHome   = (SapReceiveLocalHome)ref;
      sapReceiveLocal       = sapReceiveLocalHome.create();

      ref                   = ctx.lookup(SapRequestLocalHome.COMP_NAME);
      sapRequestLocalHome   = (SapRequestLocalHome)ref;
      sapRequestLocal       = sapRequestLocalHome.create();

      ref                   = ctx.lookup(StakoControlLocalHome.COMP_NAME);
      stakoControlLocalHome = (StakoControlLocalHome)ref;
      stakoControlLocal     = stakoControlLocalHome.create();

      ref                     = ctx.lookup(StakoBagisReadLocalHome.COMP_NAME);
      stakoBagisReadLocalHome = (StakoBagisReadLocalHome)ref;
      stakoBagisReadLocal     = stakoBagisReadLocalHome.create();

    }
    catch (CreateException ex)
    {
      LOG.error("ejbCreate - CreateException occured: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("ejbCreate - NamingException occured: " + ex.getMessage());
    }

  } // end method


  public void ejbRemove()
  {
    // lazy bone
  }


  public void ejbActivate()
  {
    // lazy bone
  }


  public void ejbPassivate()
  {
    // lazy bone
  }


  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // receive
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public void receiveError(ErrorVO err)
  {
    sapReceiveLocal.receiveError(err);
  }


  /**
   * @ejb.interface-method
   */
  public void receiveTrace(TraceVO trace) throws Exception
  {
    sapReceiveLocal.receiveTrace(trace);
  }


  //---------------------------------------------------------------------------
  // request jgb
  //---------------------------------------------------------------------------
  
  /**
   * @ejb.interface-method
   */
  public ArrayList requestJgbRechnungJob (SAPJGBJobVO job) throws Exception
  {
    return sapRequestLocal.requestJgbRechnungJob(job);
  }


  /**
   * @ejb.interface-method
   */
  public void requestJgbRechnungUpdate (ArrayList reList) throws Exception
  {
    sapRequestLocal.requestJgbRechnungUpdate(reList);
  }


  /**
   * @ejb.interface-method
   */
  public JGBLaufVO requestJgbLaufSelect (int jgbLaufId) throws Exception
  {
    return sapRequestLocal.requestJgbLaufSelect(jgbLaufId);
  }


  /**
   * @ejb.interface-method
   */
  public void requestJgbLaufUpdate(int jgbLaufId, int sapStat) throws Exception
  {
    sapRequestLocal.requestJgbLaufUpdate(jgbLaufId,sapStat);
  }


  /**
   * @ejb.interface-method
   */
  public boolean requestJgbEndCondition(int jgbLaufId) throws Exception
  {
    return sapRequestLocal.requestJgbEndCondition(jgbLaufId);
  }


  /**
   * @ejb.interface-method
   */
  public boolean requestJgbStopCondition(int jgbLaufId) throws Exception
  {
    return sapRequestLocal.requestJgbStopCondition(jgbLaufId);
  }


  //---------------------------------------------------------------------------
  // request jgb
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public void requestAmvRechnungUpdate(AmvRechnungVO vo) throws Exception
  {
    sapRequestLocal.requestAmvRechnungUpdate(vo);
  }


  /**
   * @ejb.interface-method
   */
  public void requestAmvLaufStatUpdate(int amvLaufId, int laufStat) throws Exception
  {
    sapRequestLocal.requestAmvLaufStatUpdate(amvLaufId,laufStat);
  }


  //---------------------------------------------------------------------------
  // send
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public void sendRechnung (SAPRechnungVO vo) throws Exception
  {
    sapSendLocal.sendRechnung(vo);
  }


  /**
   * @ejb.interface-method
   */
  public void sendAnnulation (SAPRechnungAnnullationVO vo) throws Exception
  {
    sapSendLocal.sendAnnulation(vo);
  }


  /**
   * @ejb.interface-method
   */
  public void sendAnnulationList (ArrayList list) throws Exception
  {
    sapSendLocal.sendAnnulationList(list);
  }


  /**
   * @ejb.interface-method
   */
  public void sendJGBJob (SAPJGBJobVO vo) throws Exception
  {
    sapSendLocal.sendJGBJob(vo);
  }


  /**
   * @ejb.interface-method
   */
  public void sendCtrlStop() throws Exception
  {
    sapSendLocal.sendCtrlStop();
  }


  /**
   * @ejb.interface-method
   */
  public void sendCtrlJgbFinish (int jgbLaudId) throws Exception
  {
    sapSendLocal.sendCtrlJgbFinish(jgbLaudId);
  }

  /**
   * @ejb.interface-method
   */
  public void sendCtrlAmvFinish (AmvLaufVO vo) throws Exception
  {
    sapSendLocal.sendCtrlAmvFinish(vo);
  }

  /**
   * @ejb.interface-method
   */
  public void sendAdressCreate (VersadrVO vo) throws Exception
  {
    sapSendLocal.sendAdressCreate(vo);
  }


  /**
   * @ejb.interface-method
   */
  public void sendAdressStatus (SAPAdrStatusVO vo) throws Exception
  {
    sapSendLocal.sendAdressStatus(vo);
  }


  //---------------------------------------------------------------------------
  // Stako Controlling methods to BAGIS
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public boolean stakoUpdateArtikelStatus(ArrayList al)
  {
    return stakoBagisReadLocal.updateArtikelStatus(al);
  }

  /**
   * @ejb.interface-method
   */
  public ArrayList stakoSelectEreignisMA(Date von, Date bis, String exclMa)
  {
    return stakoBagisReadLocal.selectEreignisMA(von, bis, exclMa);
  }

  /**
   * @ejb.interface-method
   */
  public ArrayList stakoSelectBfakturBfaktPos(Date von, Date bis, String exclMa)
  {
    return stakoBagisReadLocal.selectBfakturBfaktPos(von, bis, exclMa);
  }


  //---------------------------------------------------------------------------
  // stako controlling methods to ses-db
  //---------------------------------------------------------------------------

  /**
   * @ejb.interface-method
   */
  public void stakoCreateStakoKontrolle(StakoKontrolleVO stakoKontrolleVO)
  {
    stakoControlLocal.createStakoKontrolle(stakoKontrolleVO);
  }

  /**
   * @ejb.interface-method
   */
  public void stakoSetStakoKontrolle(StakoKontrolleVO stakoKontrolleVO)
  {
    stakoControlLocal.setStakoKontrolle(stakoKontrolleVO);
  }

  /**
   * @ejb.interface-method
   */
  public StakoKontrolleVO stakoGetStakoKontrolle(int kontrollId)
  {
    return stakoControlLocal.getStakoKontrolle(kontrollId);
  }

  /**
   * @ejb.interface-method
   */
  public StakoKontrolleVO stakoGetLastStakoKontrolle()
  {
    return stakoControlLocal.getLastStakoKontrolle();
  }

  /**
   * @ejb.interface-method
   */
  public void sendStakoJob (SAPStakoJobVO vo) throws Exception
  {
    sapSendLocal.sendStakoJob(vo);
  }

  /**
   * @ejb.interface-method
   */
  public void sendStako (StakoVO vo) throws Exception
  {
    sapSendLocal.sendStako(vo);
  }

} // end class

