package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.ObjectUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.NiceTermVO;

/**
 * Tests {@link ch.ige.wdltool.dao.GeneralTermDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: AbstractGeneralTermDAOTestTemplate.java,v 1.6 2008/04/29 16:38:12 heinzer Exp $
 */
public abstract class AbstractGeneralTermDAOTestTemplate extends BaseDAOWithCRUDTestTemplate<GeneralTermDAO, AbstractTermVO, Integer>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( AbstractGeneralTermDAOTestTemplate.class );

  /**
   * The only constructor
   * @param name The test's name
   */
  public AbstractGeneralTermDAOTestTemplate( String name )
  {
    super( name );
  }

  /**
   * Gets the latest migration date; all change dates must be later than this. 
   * @return The latest migration date
   */
  public static Timestamp getLatestMigration()
  {
    try
    {
      return DateConverter.stringToTimestamp( "13.07.2007 11:00:00.000" );
    }
    catch ( ParseException e )
    {
      LOG.error( "Couldn't parse latest migration date: " + e.getMessage(), e );
      return null;
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<GeneralTermDAO> getDAOClass()
  {
    return GeneralTermDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  protected Collection<AbstractTermVO> read( Object... searchParameters ) throws SQLException
  {
    Map<Integer, List<AbstractTermVO>> termMap = getDAO().findPositiveTerms( (Locale) searchParameters[0], (Integer) searchParameters[1] );
    List<AbstractTermVO> terms = new ArrayList<AbstractTermVO>( termMap.size() );
    for ( Integer classNumber: termMap.keySet() )
    {
      List<AbstractTermVO> allTerms = termMap.get( classNumber );
      for ( AbstractTermVO oneTerm: allTerms )
      {
        terms.add( oneTerm );
      }
    }
    return terms;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( AbstractTermVO originalVO, AbstractTermVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking class number", originalVO.getClassNumber(), loadedVO.getClassNumber() );
    assertEquals( "Checking language", originalVO.getLanguage(), loadedVO.getLanguage() );
    assertEquals( "Checking term", originalVO.getTerm(), loadedVO.getTerm() );
    assertEquals( "Checking private comment", originalVO.getPrivateComment(), loadedVO.getPrivateComment() );
    assertEquals( "Checking public comment", originalVO.getPublicComment(), loadedVO.getPublicComment() );
    assertEquals( "Checking change user", originalVO.getChangeUser(), loadedVO.getChangeUser() );
    assertEquals( "Checking data source", originalVO.getDataSource(), loadedVO.getDataSource() );
    assertEquals( "Checking search term", originalVO.getSearchTerm(), loadedVO.getSearchTerm() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#validateVO(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void validateVO( AbstractTermVO loadedVO )
  {
    assertTrue( "Checking change date", loadedVO.getChangeDate().after( getLatestMigration() ) );
  }

  @Override
  protected boolean assertVOEqual( AbstractTermVO originalVO, AbstractTermVO serializedVO )
  {
    return super.assertVOEqual( originalVO, serializedVO ) && ObjectUtils.equals( originalVO.getChangeDate(), serializedVO.getChangeDate() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  @SuppressWarnings( "unused" )
  protected AbstractTermVO read( Integer termId ) throws SQLException
  {
    // This is a hack... but there's no substitute
    return DAOPool.get( NiceTermDAO.class ).find( termId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  /**
   * Attention: this assumes the abstractTermVO is a NiceTermVO.
   */
  protected final Integer insert( AbstractTermVO abstractTermVO )
  {
    NiceTermDAOTest niceTermDAOTest = new NiceTermDAOTest( this.getName() );
    return niceTermDAOTest.insert( (NiceTermVO) abstractTermVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void modifyVOBeforeUpdate( AbstractTermVO abstractTermVO )
  {
    // Nothing to do
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void update( AbstractTermVO abstractTermVO )
  {
    // Nothing to do
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  @SuppressWarnings( "unused" )
  protected void delete( Integer termId )
  {
    //Note: This DAO doesn't return any term type, so use the specific DAO.
    DAOPool.get( NiceTermDAO.class ).delete( termId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public AbstractTermVO getSerializableVO() throws SQLException
  {
    try
    {
      NiceTermVO term = new NiceTermVO();
      term.setIdentifier( 33 );
      term.setClassNumber( 42 );
      term.setLanguage( Locale.FRENCH );
      term.setTerm( "Don't use mee%" );
      term.setPrivateComment( "The private comment" );
      term.setPublicComment( "The public comment" );
      term.setChangeDate( DateConverter.stringToTimestamp( "03.02.2002 18:44:55.000" ) );
      term.setChangeUser( "billg" );
      term.setCreationUser( "dummy" );
      term.setGeneric( false );
      term.setSearchTerm( "this_is_not_tested" );
      term.setTermGroupId( null );
      return term;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  public AbstractTermVO getExpectedVO() throws ParseException
  {
    AbstractTermVO term = new NiceTermVO();
    term.setIdentifier( 38490 ); // This may change!
    term.setClassNumber( 45 );
    term.setLanguage( Locale.GERMAN );
    term.setTerm( "Hüten von Haustieren" );
    term.setPrivateComment( null );
    term.setPublicComment( null );
    term.setChangeDate( DateConverter.stringToTimestamp( "07.11.2007 14:37:31.000" ) );
    term.setChangeUser( "migration script" );
    term.setSearchTerm( "huten von haustiren" );
    return term;
  }
}
