package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.AbstractWdlDAO;
import ch.ige.dbutils.dao.CachingDAO;
import ch.ige.wdltool.vo.ClassVO;

/**
 * Accesses the table <code>CLASS</code>.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ClassDAO.java,v 1.9 2007/09/06 14:23:53 lewis_lee Exp $
 */
public class ClassDAO extends AbstractWdlDAO<ClassVO, Integer> implements CachingDAO
{
  /**
   * All the classes are cached in here
   */
  private static List<ClassVO> classes;

  /**
   * The only constructor.
   */
  public ClassDAO()
  {
    super();
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<ClassVO> getVOType()
  {
    return ClassVO.class;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getTableName()
  @Override
  protected String getTableName()
  {
    return "class";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getColumnNames()
  @Override
  protected String getColumnNames()
  {
    return null;
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( ClassVO classVO )
  {
    return new Object[] { classVO.getIdentifier() };
  }

  // @see ch.ige.dbutils.dao.CachingDAO#clearCache()
  public synchronized void clearCache()
  {
    classes = null;
  }

  // @see ch.ige.dbutils.dao.CachingDAO#describeCachedContent()
  public CharSequence describeCachedContent()
  {
    return classes == null ? "" : classes.toString();
  }

  // @see ch.ige.dbutils.dao.CachingDAO#getCacheSize()
  public int getCacheSize()
  {
    return classes == null ? 0 : classes.size();
  }

  /**
   * Gets the classes, which are cached after the first loading.
   * 
   * @return The classes
   * @throws SQLException if the query fails
   */
  protected synchronized List<ClassVO> getClasses() throws SQLException
  {
    if ( classes == null )
    {
      classes = super.findAll();
    }
    return classes;
  }

  /**
   * Finds all classes.
   * @return All classes, ordered by class number
   * @throws SQLException if the query fails
   */
  public List<ClassVO> findAll() throws SQLException
  {
    return getClasses();
  }
}
