package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.ClassVO;

/**
 * Tests {@link ch.ige.wdltool.dao.ClassDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ClassDAOTest.java,v 1.3 2008/04/29 16:38:12 heinzer Exp $
 */
public class ClassDAOTest extends BaseDAOWithCRUDTestTemplate<ClassDAO, ClassVO, Integer>
{
  /**
   * The number of Nice classes.
   */
  private static final int NUMBER_OF_CLASSES = 45;

  /**
   * A well-known class number
   */
  private static final Integer KNOWN_CLASS_NUMBER = 3;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public ClassDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.ClassDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    List<ClassVO> classes = getDAO().findAll();
    assertEquals( "Checking number of classes", NUMBER_OF_CLASSES, classes.size() );
  }

  /**
   * Tests the caching of the DAO.
   * @throws Exception
   */
  public void testCache() throws Exception
  {
    defaultTestCache( KNOWN_CLASS_NUMBER );
    defaultTestCacheSize( KNOWN_CLASS_NUMBER, NUMBER_OF_CLASSES );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<ClassDAO> getDAOClass()
  {
    return ClassDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( ClassVO originalVO, ClassVO loadedVO )
  {
    assertEquals( "Checking class number", originalVO.getClassNumber(), loadedVO.getClassNumber() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected ClassVO getSerializableVO()
  {
    ClassVO classVO = new ClassVO();
    classVO.setClassNumber( 3 );
    return classVO;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected ClassVO read( Integer id ) throws SQLException
  {
    // This is used only for testing
    return new ClassDAOWithId().find( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected Integer insert( ClassVO vo )
  {
    // Doesn't exist
    return null;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void modifyVOBeforeUpdate( ClassVO vo )
  {
    // Doesn't exist
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected void update( ClassVO vo )
  {
    // Doesn't exist
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  @SuppressWarnings( "unused" )
  protected void delete( Integer id )
  {
    // Doesn't exist
  }

  /**
   * Extends the DAO to test, because that DAO doesn't contain a method to find
   * a class by its ID (class number).
   */
  public static class ClassDAOWithId extends ClassDAO
  {
    /**
     * Finds one class.
     * @param classNumber The class number of the class to find
     * @return The class for the given class number or <code>null</code> if none is found
     * @throws SQLException if the query fails
     */
    public ClassVO find( Integer classNumber ) throws SQLException
    {
      List<ClassVO> classes = getClasses();
      for ( ClassVO classVO: classes )
      {
        if ( classVO.getClassNumber().equals( classNumber ) )
        {
          return classVO;
        }
      }
      return null;
    }
  }
}
