package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.Collection;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.vo.DataSourceVO;

/**
 * Tests {@link ch.ige.wdltool.dao.DataSourceDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: DataSourceDAOTest.java,v 1.6 2008/04/29 16:38:12 heinzer Exp $
 */
public class DataSourceDAOTest extends BaseDAOWithCRUDTestTemplate<DataSourceDAO, DataSourceVO, DataSource>
{
  /**
   * The old value before the update
   */
  private Integer oldTermConfirmationPeriod;

  /**
   * The old value before the update
   */
  private Integer oldTermValidityPeriod;

  /**
   * The old value before the update
   */
  private boolean oldIsCleanerActive;

  /**
   * The only constructor
   * @param name The test's name
   */
  public DataSourceDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<DataSourceDAO> getDAOClass()
  {
    return DataSourceDAO.class;
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.DataSourceDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    defaultTestReadCollection( getSerializableVO(), 3 );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.DataSourceDAO#find(DataSource)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    try
    {
      read( (DataSource) null );
      fail( "Reading with null ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    DataSourceVO loadedDataSource = read( DataSource.IPI );
    compareVOs( getSerializableVO(), loadedDataSource );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.DataSourceDAO#update(ch.ige.wdltool.vo.DataSourceVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    try
    {
      update( null );
      fail( "Null value object must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    DataSourceVO ipi = read( DataSource.IPI );
    this.oldTermConfirmationPeriod = ipi.getTermConfirmationPeriod();
    this.oldTermValidityPeriod = ipi.getTermValidityPeriod();
    this.oldIsCleanerActive = ipi.isCleanerActive();

    DataSourceVO dataSourceVO = getSerializableVO();
    modifyVOBeforeUpdate( dataSourceVO );
    update( dataSourceVO );
    DataSourceVO reloadedVO = read( dataSourceVO.getIdentifier() );
    compareVOs( dataSourceVO, reloadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected DataSourceVO read( DataSource dataSource ) throws SQLException
  {
    return getDAO().find( dataSource );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<DataSourceVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().findAll();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( DataSourceVO originalVO, DataSourceVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking name", originalVO.getName(), loadedVO.getName() );
    assertEquals( "Checking cleaner flag", originalVO.isCleanerActive(), loadedVO.isCleanerActive() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#validateVO(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void validateVO( DataSourceVO loadedVO )
  {
    if ( loadedVO.getName().equals( "IGE" ) )
    {
      // The configured values can change, so just test less specific data
      assertTrue( "Checking term confirmation period", loadedVO.getTermConfirmationPeriod() >= 1 );
      assertTrue( "Checking term validity period", loadedVO.getTermValidityPeriod() >= 1 );
    }
    else
    {
      assertNull( "Checking term confirmation period", loadedVO.getTermConfirmationPeriod() );
      assertNull( "Checking term validity period", loadedVO.getTermValidityPeriod() );
    }
    // Ignore change date
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( DataSourceVO originalVO, DataSourceVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  @SuppressWarnings( "unused" )
  protected DataSource insert( DataSourceVO vo )
  {
    // There's no insert
    return null;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( DataSourceVO dataSourceVO )
  {
    dataSourceVO.setCleanerActive( false );
    dataSourceVO.setTermConfirmationPeriod( 2341 );
    dataSourceVO.setTermValidityPeriod( 772 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( DataSourceVO dataSourceVO )
  {
    getDAO().update( dataSourceVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  @SuppressWarnings( "unused" )
  protected void delete( DataSource id )
  {
    // There's no delete
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected DataSourceVO getSerializableVO() throws SQLException
  {
    try
    {
      DataSourceVO dataSource = new DataSourceVO();
      dataSource.setIdentifier( DataSource.IPI );
      dataSource.setName( "IGE" );
      dataSource.setChangeDate( DateConverter.stringToTimestamp( "03.02.2005 18:22:33.332" ) );
      dataSource.setCleanerActive( true );
      dataSource.setTermValidityPeriod( 50 );
      dataSource.setTermConfirmationPeriod( 20 );
      return dataSource;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( DataSourceVO originalVO, DataSourceVO serializedVO )
  {
    compareVOs( originalVO, serializedVO );
    return originalVO.getChangeDate().equals( serializedVO.getChangeDate() )
        && originalVO.getTermValidityPeriod().equals( serializedVO.getTermValidityPeriod() )
        && originalVO.getTermConfirmationPeriod().equals( serializedVO.getTermConfirmationPeriod() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#tearDown()
  @Override
  public void tearDown() throws Exception
  {
    if ( this.oldTermConfirmationPeriod != null )
    {
      // Revert to old values
      DataSourceVO ipi = read( DataSource.IPI );
      ipi.setTermConfirmationPeriod( this.oldTermConfirmationPeriod );
      ipi.setTermValidityPeriod( this.oldTermValidityPeriod );
      ipi.setCleanerActive( this.oldIsCleanerActive );
      update( ipi );
    }
    super.tearDown();
  }
}