package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.Collection;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.IpiTermGroupVO;

/**
 * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Esther Bandschapp
 * @version  $Id: IpiTermGroupDAOTest.java,v 1.15 2008/08/11 11:43:09 lewis_lee Exp $
 */
public class IpiTermGroupDAOTest extends BaseDAOWithCRUDTestTemplate<IpiTermGroupDAO, IpiTermGroupVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public IpiTermGroupDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#insert(IpiTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#createNextFreeIpiTermGroup()}.
   * @throws Exception if the test fails
   */
  public void testCreateNextFreeIpiTermGroup() throws Exception
  {
    IpiTermGroupVO ipiTermGroup = getDAO().createNextFreeIpiTermGroup();
    assertNotNull( "Checking that IPI term group exists", ipiTermGroup );
    assertTrue( "Checking IPI term group number", ipiTermGroup.getIpiTermNumber() >= 1000000 && ipiTermGroup.getIpiTermNumber() <= 1999999 );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    IpiTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestRead( termGroupVO, -1, 999999999, termGroupVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#findByNumber(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindByNumber() throws Exception
  {
    try
    {
      getDAO().findByNumber( null );
      fail( "Null number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().findByNumber( -1 );
      fail( "Negative number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    IpiTermGroupVO ipiTermGroup = getDAO().findByNumber( 999999999 );
    assertNull( "Checking for unknown term group number", ipiTermGroup );

    ipiTermGroup = getDAO().findByNumber( 1000300 ); // This may change!
    assertNotNull( "Checking for known term group number", ipiTermGroup );
    assertEquals( "Checking ID for known term group number", 7933, ipiTermGroup.getIdentifier().intValue() ); // This may change!
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    IpiTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestReadCollection( termGroupVO, 1 );

    Collection<IpiTermGroupVO> termGroups = read( new Object[] {} );
    int previousTermGroupNumber = 0;
    for ( IpiTermGroupVO termGroup: termGroups )
    {
      assertTrue( "Checking that the term groups are ordered by term group number", termGroup.getIpiTermNumber() > previousTermGroupNumber );
      previousTermGroupNumber = termGroup.getIpiTermNumber().intValue();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#update(IpiTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.IpiTermGroupDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<IpiTermGroupDAO> getDAOClass()
  {
    return IpiTermGroupDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( IpiTermGroupVO termGroupVO )
  {
    return getDAO().insert( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( IpiTermGroupVO originalVO, IpiTermGroupVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking IPI term number", originalVO.getIpiTermNumber(), loadedVO.getIpiTermNumber() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( IpiTermGroupVO originalVO, IpiTermGroupVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public IpiTermGroupVO getSerializableVO()
  {
    IpiTermGroupVO termGroup = new IpiTermGroupVO();
    termGroup.setIdentifier( 44 );
    termGroup.setIpiTermNumber( 1999999 );
    return termGroup;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( IpiTermGroupVO termGroupVO )
  {
    termGroupVO.setIpiTermNumber( 1000987 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected IpiTermGroupVO read( Integer termGroupId ) throws SQLException
  {
    return getDAO().find( termGroupId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<IpiTermGroupVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().findAll();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( IpiTermGroupVO termGroupVO )
  {
    getDAO().update( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer termGroupId )
  {
    getDAO().delete( termGroupId );
  }
}
