package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.Collection;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.StandardWdlTermGroupVO;

/**
 * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Esther Bandschapp
 * @version  $Id: StandardWdlTermGroupDAOTest.java,v 1.8 2008/04/29 16:38:12 heinzer Exp $
 */
public class StandardWdlTermGroupDAOTest extends BaseDAOWithCRUDTestTemplate<StandardWdlTermGroupDAO, StandardWdlTermGroupVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public StandardWdlTermGroupDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO#insert(StandardWdlTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    StandardWdlTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestRead( termGroupVO, -1, 999999999, termGroupVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    StandardWdlTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestReadCollection( termGroupVO, 1 );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO#update(StandardWdlTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.StandardWdlTermGroupDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<StandardWdlTermGroupDAO> getDAOClass()
  {
    return StandardWdlTermGroupDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( StandardWdlTermGroupVO termGroupVO )
  {
    return getDAO().insert( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( StandardWdlTermGroupVO originalVO, StandardWdlTermGroupVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking name", originalVO.getName(), loadedVO.getName() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected StandardWdlTermGroupVO getSerializableVO()
  {
    StandardWdlTermGroupVO termGroup = new StandardWdlTermGroupVO();
    termGroup.setIdentifier( 44 );
    termGroup.setName( "test" );
    return termGroup;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( StandardWdlTermGroupVO termGroupVO )
  {
    termGroupVO.setName( "updated test" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected StandardWdlTermGroupVO read( Integer termGroupId ) throws SQLException
  {
    return getDAO().find( termGroupId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<StandardWdlTermGroupVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().findAll();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( StandardWdlTermGroupVO termGroupVO )
  {
    getDAO().update( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer termGroupId )
  {
    getDAO().delete( termGroupId );
  }
}
