package ch.ige.wdltool.ejb;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.RandomStringUtils;

import ch.ige.wdltool.ParameterConstants;
import ch.ige.wdltool.dao.NegativeIpiTermDAO;
import ch.ige.wdltool.dao.NegativeIpiTermDAOTest;
import ch.ige.wdltool.vo.NegativeIpiTermVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for negative IPI terms. <p>Copyright (c) 2007, Swiss Federal Institute
 * of Intellectual Property</p>
 * @author Eric Lewis
 * @version $Id: WdlServicesForNegativeIpiTermTest.java,v 1.12 2009/02/02 13:45:10 bandschapp Exp $
 */
public class WdlServicesForNegativeIpiTermTest extends BaseServiceTestTemplate<NegativeIpiTermDAO, NegativeIpiTermVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private NegativeIpiTermDAOTest negativeIpiTermDAOTest;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForNegativeIpiTermTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.negativeIpiTermDAOTest = new NegativeIpiTermDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findNegativeIpiTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindNegativeIpiTerm() throws Exception
  {
    NegativeIpiTermVO newTerm = createAndInsert( true );
    defaultTestRead( newTerm, MINUS_ONE, 999999999, newTerm.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertNegativeIpiTerm(NegativeIpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testInsertNegativeIpiTerm() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertNegativeIpiTerm(NegativeIpiTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testInsertNegativeIpiTermWithExistingStopword() throws Exception
  {
    defaultTestCreateTermWithExistingStopword( Locale.FRENCH, "une" );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateNegativeIpiTerm(NegativeIpiTermVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdateNegativeIpiTerm() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateNegativeIpiTerm(NegativeIpiTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testUpdateNegativeIpiTermWithExistingStopword() throws Exception
  {
    defaultTestUpdateTermWithExistingStopword( Locale.FRENCH, "pour" );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getInvalidPropertyValues(ch.ige.dbutils.vo.AbstractVO, boolean)
  @Override
  @SuppressWarnings( "unused" )
  public Map<String, Object[]> getInvalidPropertyValues( NegativeIpiTermVO valueObject, boolean isUpdate ) throws IllegalStateException
  {
    Map<String, Object[]> properties = new HashMap<String, Object[]>();

    if ( isUpdate )
    {
      properties.put( "identifier", new Integer[] { null, -1, 0 } );
    }
    else
    {
      properties.put( "identifier", new Integer[] { -1, 0 } );
    }

    properties.put( "classNumber", new Integer[] { 0, 46 } );

    properties.put( "language", new Locale[] { null, Locale.CHINESE } );

    properties.put( "term", new String[] { null, "", " ", " bla", "bla ", RandomStringUtils.randomAlphabetic( ParameterConstants.EXCEEDED_LIMIT_FIELD_TERM ),
        "´", "bl´a", "`", "bl`a", "^", "bl^a", "¨", "bl¨a", "bla" + '\u2014' + "bla" /* long dash, created automatically by Word */, "bla bla  " } );

    properties.put( "privateComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla  bla" } );

    properties.put( "publicComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla bla  " } );

    properties.put( "creationUser", new String[] { null, "", RandomStringUtils.randomAlphabetic( 26 ) } );

    properties.put( "changeUser", new String[] { null, "", RandomStringUtils.randomAlphabetic( 26 ) } );

    return properties;
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#deleteNegativeIpiTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testDeleteNegativeIpiTerm() throws Exception
  {
    defaultTestDelete( MINUS_ONE, 999999999 );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getSerializableVO()
  @Override
  protected NegativeIpiTermVO getSerializableVO() throws SQLException
  {
    return this.negativeIpiTermDAOTest.getSerializableVO();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( NegativeIpiTermVO originalVO, NegativeIpiTermVO loadedVO )
  {
    this.negativeIpiTermDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public NegativeIpiTermVO readWithService( Integer id ) throws Exception
  {
    return getServices().findNegativeIpiTerm( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( NegativeIpiTermVO vo )
  {
    vo.setIdentifier( null );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#insertWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insertWithService( NegativeIpiTermVO vo ) throws Exception
  {
    NegativeIpiTermVO newTerm = getServices().insertNegativeIpiTerm( vo );
    return newTerm.getIdentifier();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( NegativeIpiTermVO vo )
  {
    this.negativeIpiTermDAOTest.modifyVOBeforeUpdate( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#updateWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void updateWithService( NegativeIpiTermVO vo ) throws Exception
  {
    getServices().updateNegativeIpiTerm( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#deleteWithService(java.io.Serializable)
  @Override
  public void deleteWithService( Integer id ) throws Exception
  {
    getServices().deleteNegativeIpiTerm( id );
  }
}
