package ch.ige.wdltool.ejb;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.RandomStringUtils;

import ch.ige.wdltool.ParameterConstants;
import ch.ige.wdltool.dao.NiceTermDAO;
import ch.ige.wdltool.dao.NiceTermDAOTest;
import ch.ige.wdltool.vo.NiceTermVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for terms. <p>Copyright (c) 2007, Swiss Federal Institute of
 * Intellectual Property</p>
 * @author Eric Lewis
 * @version $Id: WdlServicesForNiceTermTest.java,v 1.11 2009/02/02 13:45:10 bandschapp Exp $
 */
public class WdlServicesForNiceTermTest extends BaseServiceTestTemplate<NiceTermDAO, NiceTermVO, Integer, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private NiceTermDAOTest niceTermDAOTest;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForNiceTermTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.niceTermDAOTest = new NiceTermDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findNiceTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindNiceTerm() throws Exception
  {
    defaultTestRead( getExpectedVO(), MINUS_ONE, 999999999, getExpectedVO().getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertNiceTermAndRemoveIpiTermCandidate(NiceTermVO)}.
   * @throws Exception if the test fails
   */
  public void testInsertNiceTerm() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#insertNiceTermAndRemoveIpiTermCandidate(NiceTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testInsertNiceTermWithExistingStopword() throws Exception
  {
    defaultTestCreateTermWithExistingStopword( Locale.FRENCH, "une" );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateNiceTermAndRemoveIpiTermCandidate(NiceTermVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdateNiceTerm() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#updateNiceTermAndRemoveIpiTermCandidate(NiceTermVO)}
   * with existing stopword.
   * @throws Exception if the test fails
   */
  public void testUpdateNiceTermWithExistingStopword() throws Exception
  {
    defaultTestUpdateTermWithExistingStopword( Locale.FRENCH, "pour" );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getInvalidPropertyValues(ch.ige.dbutils.vo.AbstractVO, boolean)
  @Override
  @SuppressWarnings( "unused" )
  public Map<String, Object[]> getInvalidPropertyValues( NiceTermVO valueObject, boolean isUpdate ) throws IllegalStateException
  {
    Map<String, Object[]> properties = new HashMap<String, Object[]>();

    if ( isUpdate )
    {
      properties.put( "identifier", new Integer[] { null, -1, 0 } );
    }
    else
    {
      properties.put( "identifier", new Integer[] { -1, 0 } );
    }

    properties.put( "classNumber", new Integer[] { null, 0, 46 } );

    properties.put( "termGroupId", new Integer[] { 0 } );

    properties.put( "language", new Locale[] { null, Locale.CHINESE } );

    properties.put( "term", new String[] { null, "", " ", " bla", "bla ", RandomStringUtils.randomAlphabetic( ParameterConstants.EXCEEDED_LIMIT_FIELD_TERM ),
        "´", "bl´a", "`", "bl`a", "^", "bl^a", "¨", "bl¨a", "bla" + '\u2014' + "bla" /* long dash, created automatically by Word */, "bla  bla" } );

    properties.put( "privateComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla  bla" } );

    properties.put( "publicComment", new String[] { "", RandomStringUtils.randomAlphabetic( 501 ), "bla bla bla  " } );

    properties.put( "changeUser", new String[] { null, "", RandomStringUtils.randomAlphabetic( 26 ) } );

    return properties;
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#deleteNiceTerm(Integer)}.
   * @throws Exception if the test fails
   */
  public void testDeleteNiceTermTerm() throws Exception
  {
    defaultTestDelete( MINUS_ONE, 999999999 );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getSerializableVO()
  @Override
  protected NiceTermVO getSerializableVO() throws SQLException
  {
    return this.niceTermDAOTest.getSerializableVO();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected NiceTermVO getExpectedVO() throws ParseException
  {
    return this.niceTermDAOTest.getExpectedVO();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( NiceTermVO originalVO, NiceTermVO loadedVO )
  {
    this.niceTermDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public NiceTermVO readWithService( Integer id ) throws Exception
  {
    return getServices().findNiceTerm( id );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( NiceTermVO vo )
  {
    vo.setIdentifier( null );
    vo.setTermGroupId( null ); // Otherwise it must exist
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#insertWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public Integer insertWithService( NiceTermVO vo ) throws Exception
  {
    NiceTermVO newTerm = getServices().insertNiceTermAndRemoveIpiTermCandidate( vo ).getNewTerm();
    return newTerm.getIdentifier();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( NiceTermVO vo )
  {
    this.niceTermDAOTest.modifyVOBeforeUpdate( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#updateWithService(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void updateWithService( NiceTermVO vo ) throws Exception
  {
    getServices().updateNiceTermAndRemoveIpiTermCandidate( vo );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#deleteWithService(java.io.Serializable)
  @Override
  public void deleteWithService( Integer id ) throws Exception
  {
    getServices().deleteNiceTerm( id );
  }
}
