package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.List;

import ch.ige.wdltool.dao.NiceTermGroupDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.NiceTermGroupVO;

/**
 * Implements services for Nice term groups.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: NiceTermGroupImpl.java,v 1.5 2008/01/18 12:08:09 lewis_lee Exp $
 */
public class NiceTermGroupImpl extends BaseServicesImpl<NiceTermGroupDAO, NiceTermGroupVO, Integer>
{
  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "Nice term group";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<NiceTermGroupDAO> getDAOClass()
  {
    return NiceTermGroupDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#findAll(ch.ige.dbutils.dao.DAO)
  @Override
  protected List<NiceTermGroupVO> findAll( NiceTermGroupDAO dao ) throws SQLException
  {
    return dao.findAll();
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected NiceTermGroupVO find( NiceTermGroupDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  /**
   * Finds a Nice term group.
   * @param niceTermNumber The term group's number (not the ID!)
   * @return The Nice term group containing the Nice term number
   * @throws WrappedRuntimeException 
   * @throws WdlObjectNotFoundException 
   * @see ch.ige.wdltool.ejb.WdlServices#findByNiceTermNumber(Integer)
   */
  protected NiceTermGroupVO findByNiceTermNumber( Integer niceTermNumber ) throws WrappedRuntimeException, WdlObjectNotFoundException
  {
    final String errorMessage = "Didn't find " + getObjectName() + " with term number " + niceTermNumber;
    NiceTermGroupVO termGroup = null;
    try
    {
      termGroup = getDAOInstance().findByNumber( niceTermNumber );
      if ( termGroup == null )
      {
        ExceptionThrower.throwObjectNotFoundException( errorMessage, niceTermNumber );
      }
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( errorMessage, e, niceTermNumber );
    }
    return termGroup;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.AbstractDAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected NiceTermGroupVO insert( NiceTermGroupDAO dao, NiceTermGroupVO vo )
  {
    dao.insert( vo );
    return vo;
  }

}
