package ch.ige.wdltool.exception;

import java.io.Serializable;

import javax.ejb.ObjectNotFoundException;

/**
 * Is thrown when an object should be found, but none exists.
 * Note that the caller must have been sure that the object can be found,
 * such as when the unique ID (for instance the primary key in the DB table)
 * has been specified as the search criteria.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlObjectNotFoundException.java,v 1.5 2008/02/05 13:59:54 lewis_lee Exp $
 */
public class WdlObjectNotFoundException extends ObjectNotFoundException
{
  /**
   * The identifier for the value object to find
   */
  private Serializable identifier;

  /**
   * Creates this without message.
   * @param cause The causing Exception
   * @param identifier The identifier of the object to find
   */
  public WdlObjectNotFoundException( Throwable cause, Serializable identifier )
  {
    this( null, cause, identifier );
  }

  /**
   * Creates this with message.
   * @param message The error message
   * @param cause The causing Exception
   * @param identifier The identifier of the object to find
   */
  public WdlObjectNotFoundException( String message, Throwable cause, Serializable identifier )
  {
    super( message );
    initCause( cause );
    this.identifier = identifier;
  }

  /**
   * Gets the identifier for the value object to find.
   * @return the identifier for the value object to find
   */
  public Serializable getIdentifier()
  {
    return this.identifier;
  }
}
