package ch.ige.wdltool.text;

import java.util.List;
import java.util.Locale;

import junit.framework.TestCase;

/**
 * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter}.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: UpperCaseCharacterConverterTest.java,v 1.5 2008/03/03 15:22:39 lewis_lee Exp $
 */
public class UpperCaseCharacterConverterTest extends TestCase
{
  /**
   * Creates this.
   * @param name The test's name
   */
  public UpperCaseCharacterConverterTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter#firstCharacterConverter(String, boolean)}
   * without text.
   */
  public void testFirstCharacterConverterForEmptyText()
  {
    try
    {
      new UpperCaseCharacterConverter( null );
      fail( "Null language must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    UpperCaseCharacterConverter converter = new UpperCaseCharacterConverter( Locale.GERMAN );
    assertNull( "Checking no text", converter.firstCharacterConverter( null, true ) );
    assertNull( "Checking no text", converter.firstCharacterConverter( "", true ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter#firstCharacterConverter(String, boolean)}
   * with German text.
   */
  public void testFirstCharacterConverterForGerman()
  {
    UpperCaseCharacterConverter converter = new UpperCaseCharacterConverter( Locale.GERMAN );
    assertNull( "Checking German text", converter.firstCharacterConverter( "Olé!", true ) );
    assertNull( "Checking German text", converter.firstCharacterConverter( "olé!", true ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter#firstCharacterConverter(String, boolean)}
   * with Italian text.
   */
  public void testFirstCharacterConverterForItalian()
  {
    UpperCaseCharacterConverter converter = new UpperCaseCharacterConverter( Locale.ITALIAN );
    assertNull( "Checking Italian text", converter.firstCharacterConverter( "è certo!", true ) );
    assertNull( "Checking Italian text", converter.firstCharacterConverter( "E certo!", true ) );
  }

  /**
   * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter#firstCharacterConverter(String, boolean)}
   * with French text.
   */
  public void testFirstCharacterConverterForFrench()
  {
    UpperCaseCharacterConverter converter = new UpperCaseCharacterConverter( Locale.FRENCH );
    checkCharacters( converter.firstCharacterConverter( "a la minute", true ), 'A' );
    checkCharacters( converter.firstCharacterConverter( "à la minute", true ), 'A', 'À' );
    checkCharacters( converter.firstCharacterConverter( "à la minute", false ), 'A' );
    checkCharacters( converter.firstCharacterConverter( "á la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "â la minute", true ), 'A', 'Â' );
    checkCharacters( converter.firstCharacterConverter( "â la minute", false ), 'A' );
    checkCharacters( converter.firstCharacterConverter( "ä la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "ã la minute", true ), null );

    checkCharacters( converter.firstCharacterConverter( "A la minute", true ), 'a', 'à', 'â' );
    checkCharacters( converter.firstCharacterConverter( "A la minute", false ), 'à', 'â' );
    checkCharacters( converter.firstCharacterConverter( "À la minute", true ), 'à' );
    checkCharacters( converter.firstCharacterConverter( "À la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Á la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Â la minute", true ), 'â' );
    checkCharacters( converter.firstCharacterConverter( "Â la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Ä la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Ã la minute", true ), null );

    checkCharacters( converter.firstCharacterConverter( "b la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "B la minute", true ), null );

    checkCharacters( converter.firstCharacterConverter( "ça la minute", true ), 'C', 'Ç' );
    checkCharacters( converter.firstCharacterConverter( "ça la minute", false ), 'C' );
    checkCharacters( converter.firstCharacterConverter( "Ca la minute", true ), 'c', 'ç' );
    checkCharacters( converter.firstCharacterConverter( "Ca la minute", false ), 'ç' );
    checkCharacters( converter.firstCharacterConverter( "Ça la minute", true ), 'ç' );
    checkCharacters( converter.firstCharacterConverter( "Ça la minute", false ), null );

    checkCharacters( converter.firstCharacterConverter( "et la minute", true ), 'E' );
    checkCharacters( converter.firstCharacterConverter( "et la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "èt la minute", true ), 'E', 'È' );
    checkCharacters( converter.firstCharacterConverter( "èt la minute", false ), 'E' );
    checkCharacters( converter.firstCharacterConverter( "ét la minute", true ), 'E', 'É' );
    checkCharacters( converter.firstCharacterConverter( "ét la minute", false ), 'E' );
    checkCharacters( converter.firstCharacterConverter( "êt la minute", true ), 'E', 'Ê' );
    checkCharacters( converter.firstCharacterConverter( "êt la minute", false ), 'E' );
    checkCharacters( converter.firstCharacterConverter( "ët la minute", true ), 'E', 'Ë' );
    checkCharacters( converter.firstCharacterConverter( "ët la minute", false ), 'E' );

    checkCharacters( converter.firstCharacterConverter( "Et la minute", true ), 'e', 'è', 'é', 'ê', 'ë' );
    checkCharacters( converter.firstCharacterConverter( "Et la minute", false ), 'è', 'é', 'ê', 'ë' );
    checkCharacters( converter.firstCharacterConverter( "Èt la minute", true ), 'è' );
    checkCharacters( converter.firstCharacterConverter( "Èt la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Ét la minute", true ), 'é' );
    checkCharacters( converter.firstCharacterConverter( "Ét la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Êt la minute", true ), 'ê' );
    checkCharacters( converter.firstCharacterConverter( "Êt la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Ët la minute", true ), 'ë' );
    checkCharacters( converter.firstCharacterConverter( "Ët la minute", false ), null );

    checkCharacters( converter.firstCharacterConverter( "ih la minute", true ), 'I' );
    checkCharacters( converter.firstCharacterConverter( "ih la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "ìh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "íh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "îh la minute", true ), 'I', 'Î' );
    checkCharacters( converter.firstCharacterConverter( "îh la minute", false ), 'I' );
    checkCharacters( converter.firstCharacterConverter( "ïh la minute", true ), 'I', 'Ï' );
    checkCharacters( converter.firstCharacterConverter( "ïh la minute", false ), 'I' );

    checkCharacters( converter.firstCharacterConverter( "Ih la minute", true ), 'i', 'î', 'ï' );
    checkCharacters( converter.firstCharacterConverter( "Ih la minute", false ), 'î', 'ï' );
    checkCharacters( converter.firstCharacterConverter( "Ìh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Íh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Îh la minute", true ), 'î' );
    checkCharacters( converter.firstCharacterConverter( "Îh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Ïh la minute", true ), 'ï' );
    checkCharacters( converter.firstCharacterConverter( "Ïh la minute", false ), null );

    checkCharacters( converter.firstCharacterConverter( "oh la minute", true ), 'O' );
    checkCharacters( converter.firstCharacterConverter( "oh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "òh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "óh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "ôh la minute", true ), 'O', 'Ô' );
    checkCharacters( converter.firstCharacterConverter( "ôh la minute", false ), 'O' );
    checkCharacters( converter.firstCharacterConverter( "öh la minute", true ), null );

    checkCharacters( converter.firstCharacterConverter( "Oh la minute", true ), 'o', 'ô' );
    checkCharacters( converter.firstCharacterConverter( "Oh la minute", false ), 'ô' );
    checkCharacters( converter.firstCharacterConverter( "òh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Óh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Ôh la minute", true ), 'ô' );
    checkCharacters( converter.firstCharacterConverter( "Ôh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Öh la minute", true ), null );

    checkCharacters( converter.firstCharacterConverter( "uh la minute", true ), 'U' );
    checkCharacters( converter.firstCharacterConverter( "uh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "ùh la minute", true ), 'U', 'Ù' );
    checkCharacters( converter.firstCharacterConverter( "ùh la minute", false ), 'U' );
    checkCharacters( converter.firstCharacterConverter( "úh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "ûh la minute", true ), 'U', 'Û' );
    checkCharacters( converter.firstCharacterConverter( "ûh la minute", false ), 'U' );
    checkCharacters( converter.firstCharacterConverter( "üh la minute", true ), 'U', 'Ü' );
    checkCharacters( converter.firstCharacterConverter( "üh la minute", false ), 'U' );

    checkCharacters( converter.firstCharacterConverter( "Uh la minute", true ), 'u', 'ù', 'û', 'ü' );
    checkCharacters( converter.firstCharacterConverter( "Uh la minute", false ), 'ù', 'û', 'ü' );
    checkCharacters( converter.firstCharacterConverter( "Ùh la minute", true ), 'ù' );
    checkCharacters( converter.firstCharacterConverter( "Ùh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Úh la minute", true ), null );
    checkCharacters( converter.firstCharacterConverter( "Ûh la minute", true ), 'û' );
    checkCharacters( converter.firstCharacterConverter( "Ûh la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "Üh la minute", true ), 'ü' );
    checkCharacters( converter.firstCharacterConverter( "Üh la minute", false ), null );

    checkCharacters( converter.firstCharacterConverter( "y'a la minute", true ), 'Y' );
    checkCharacters( converter.firstCharacterConverter( "y'a la minute", false ), null );
    checkCharacters( converter.firstCharacterConverter( "ÿ'a la minute", true ), 'Y', 'Ÿ' );
    checkCharacters( converter.firstCharacterConverter( "ÿ'a la minute", false ), 'Y' );
    checkCharacters( converter.firstCharacterConverter( "Y'a la minute", true ), 'y', 'ÿ' );
    checkCharacters( converter.firstCharacterConverter( "Y'a la minute", false ), 'ÿ' );
    checkCharacters( converter.firstCharacterConverter( "Ÿ'a la minute", true ), 'ÿ' );
    checkCharacters( converter.firstCharacterConverter( "Ÿ'a la minute", false ), null );
  }

  /**
   * Tests {@link ch.ige.wdltool.text.UpperCaseCharacterConverter#firstCharacterConverter(String, boolean)},
   * checking whether it runs in a thread-safe mode.
   */
  public void testFirstCharacterConverterForThreadSafety()
  {
    UpperCaseCharacterConverter converter = new UpperCaseCharacterConverter( Locale.FRENCH );
    List<Character> characters1 = converter.firstCharacterConverter( "A la minute", false );
    List<Character> characters2 = converter.firstCharacterConverter( "A la minute", false );
    assertTrue( "Checking for thread safety", characters1 != characters2 );
  }

  /**
   * Checks the characters.
   * @param actualCharacters The characters as created by the Converter
   * @param expectedCharacters The expected characters
   */
  private void checkCharacters( List<Character> actualCharacters, char... expectedCharacters )
  {
    if ( expectedCharacters == null )
    {
      assertNull( "Checking non-existing character", actualCharacters );
    }
    else
    {
      assertNotNull( "Checking returned characters exist", actualCharacters );
      assertEquals( "Checking number of characters", expectedCharacters.length, actualCharacters.size() );
      for ( char expectedCharacter: expectedCharacters )
      {
        assertTrue( "Checking '" + expectedCharacter + "'", actualCharacters.contains( Character.valueOf( expectedCharacter ) ) );
      }
    }
  }
}