package ch.ige.wdltool.vo;

import java.io.Serializable;

/**
 * Contains a part of a class text, which - unlike the class text group - has no logical meaning.
 * This is used for validating the WDL and marking certain parts of it as invalid, unknown etc.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ClassTextPartVO.java,v 1.2 2007/07/04 14:00:56 heinzer Exp $
 */
public class ClassTextPartVO implements Serializable
{
  /**
   * Contains the checked part of the class text
   */
  private String classTextPart;
  
  /**
   * Shows the result of the check
   */
  private CheckedClassTextPart checkFlag;

  /**
   * Creates this with everything.
   * @param classTextPart The checked part of the class text
   * @param checkFlag The result of the check
   */
  public ClassTextPartVO(String classTextPart, CheckedClassTextPart checkFlag)
  {
    this.classTextPart = classTextPart;
    this.checkFlag = checkFlag;
  }
  
  /**
   * Gets the checked part of the class text.
   * @return the checked part of the class text
   */
  public String getClassTextPart()
  {
    return this.classTextPart;
  }

  /**
   * Sets the checked part of the class text.
   * @param classTextPart The checked part of the class text to set
   */
  public void setClassTextPart( String classTextPart )
  {
    this.classTextPart = classTextPart;
  }

  /**
   * Gets the result of the check.
   * @return the result of the check
   */
  public CheckedClassTextPart getCheckFlag()
  {
    return this.checkFlag;
  }

  /**
   * Sets the result of the check.
   * @param checkFlag The result of the check
   */
  public void setCheckFlag( CheckedClassTextPart checkFlag )
  {
    this.checkFlag = checkFlag;
  }
}